@@ -22,7 +22,7 @@
 
 #include "bookmarks-context-menu-several.h"
 #include "bookmarks-clipboard.h"
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include "galeon-marshal.h"
 #include "gul-gobject-misc.h"
 #include "gul-gui.h"
@@ -31,8 +31,6 @@
 #include <gtk/gtkmain.h>
 #include <string.h>
 
-#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-
 /**
  * Private data
  */
