@@ -21,19 +21,18 @@
 #endif
 
 #include "bookmarks-context-menu.h"
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include "galeon-marshal.h"
 #include "gul-gobject-misc.h"
 #include "gul-gui.h"
 #include "bookmarks-single-editor.h"
 #include "bookmarks-clipboard.h"
+#include "pixbuf-cache.h"
 #include <gtk/gtkclipboard.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkmain.h>
 #include <string.h>
 
-#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-
 /**
  * Private data
  */
@@ -245,7 +244,7 @@ gb_context_menu_build_menu (GbContextMenu *cm)
 		if (p->location_source)
 		{
 			gul_gui_append_new_menuitem_stock_icon 
-				(p->menu, GTK_STOCK_ADD, _("Add _Bookmark Here"),
+				(p->menu, STOCK_ADD_BOOKMARK, _("Add _Bookmark Here"),
 				 G_CALLBACK (gb_context_menu_add_bookmark_here_cb), cm);
 		}
 		else
@@ -592,9 +591,10 @@ gb_context_menu_new_bookmark_cb (GtkWidget *menu_item, GbContextMenu *cm)
 	niu = gb_site_new (bm->set, _("New Site"), "http://");
 
 	gb_folder_add_child (GB_FOLDER (bm), GB_BOOKMARK (niu), 0);
+	gb_bookmark_set_time_added_now (GB_BOOKMARK (niu));
 
 	gb_activated_emit (cm, GbContextMenuSignals[GB_CONTEXT_MENU_BOOKMARK_ACTIVATED],
-		           GB_BOOKMARK (niu), niu, GB_BAF_EDITOR);
+		           GB_BOOKMARK (niu), niu->url, GB_BAF_EDITOR);
 
 	g_object_unref (niu);
 
@@ -625,6 +625,9 @@ gb_context_menu_add_bookmark_here_cb (GtkWidget *menu_item, GbContextMenu *cm)
 	gb_folder_add_child (GB_FOLDER (bm), GB_BOOKMARK (niu), -1);
 	gb_activated_emit (cm, GbContextMenuSignals[GB_CONTEXT_MENU_BOOKMARK_ACTIVATED],
 		           GB_BOOKMARK (niu), niu->url, GB_BAF_EDITOR);
+	gb_bookmark_set_time_added_now (GB_BOOKMARK (niu));
+	gb_site_set_time_visited_now (niu);
+
 	g_object_unref (niu);
 
 	g_object_unref (cm);
@@ -641,6 +644,7 @@ gb_context_menu_add_folder_here_cb (GtkWidget *menu_item, GbContextMenu *cm)
 
 	niu = gb_folder_new (bm->set, _("New folder"));
 	gb_folder_add_child (GB_FOLDER (bm), GB_BOOKMARK (niu), -1);
+	gb_bookmark_set_time_added_now (GB_BOOKMARK (niu));
 	gb_activated_emit (cm, GbContextMenuSignals[GB_CONTEXT_MENU_BOOKMARK_ACTIVATED],
 		           GB_BOOKMARK (niu), NULL, GB_BAF_EDITOR);
 	g_object_unref (niu);
