@@ -20,7 +20,7 @@
 #include "config.h"
 #endif
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include "gul-glade.h"
 #include "galeon-marshal.h"
 #include "bookmarks-editor.h"
@@ -35,6 +35,7 @@
 #include "bookmarks-export-druid-netscape.h"
 #include "bookmarks-import-druid-konqueror.h"
 #include "bookmarks-export-druid-konqueror.h"
+#include "bookmarks-import-druid-epiphany.h"
 #include "bookmarks-tree-view.h"
 #include "gul-gobject-misc.h"
 #include "gul-string.h"
@@ -42,6 +43,7 @@
 #include "gul-general.h"
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
+#include "galeon-debug.h"
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtklabel.h>
@@ -53,6 +55,7 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtktreeselection.h>
+#include <gtk/gtktextview.h>
 #include <bonobo/bonobo-window.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-ui-component.h>
@@ -60,8 +63,6 @@
 #include <string.h>
 
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
 
 /**
  * Private data
@@ -190,6 +191,8 @@ static void		gb_editor_bonobo_cmd_import_netscape	(BonoboUIComponent *uic, GbEdi
 								 const gchar *verbname);
 static void		gb_editor_bonobo_cmd_import_konqueror	(BonoboUIComponent *uic, GbEditor *e,
 								 const gchar *verbname);
+static void		gb_editor_bonobo_cmd_import_epiphany	(BonoboUIComponent *uic, GbEditor *e,
+								 const gchar *verbname);
 static void		gb_editor_bonobo_cmd_export_mozilla	(BonoboUIComponent *uic, GbEditor *e,
 								 const gchar *verbname);
 static void		gb_editor_bonobo_cmd_export_netscape	(BonoboUIComponent *uic, GbEditor *e,
@@ -270,6 +273,7 @@ BonoboUIVerb bookmarks_editor_verbs [] = {
 	BONOBO_UI_VERB ("ImportMozilla", (BonoboUIVerbFn) gb_editor_bonobo_cmd_import_mozilla),
 	BONOBO_UI_VERB ("ImportNetscape", (BonoboUIVerbFn) gb_editor_bonobo_cmd_import_netscape),
 	BONOBO_UI_VERB ("ImportKonqueror", (BonoboUIVerbFn) gb_editor_bonobo_cmd_import_konqueror),
+	BONOBO_UI_VERB ("ImportEpiphany", (BonoboUIVerbFn) gb_editor_bonobo_cmd_import_epiphany),
 	BONOBO_UI_VERB ("ExportMozilla", (BonoboUIVerbFn) gb_editor_bonobo_cmd_export_mozilla),
 	BONOBO_UI_VERB ("ExportNetscape", (BonoboUIVerbFn) gb_editor_bonobo_cmd_export_netscape),
 	BONOBO_UI_VERB ("ExportKonqueror", (BonoboUIVerbFn) gb_editor_bonobo_cmd_export_konqueror),
@@ -382,7 +386,7 @@ gb_editor_finalize_impl (GObject *o)
 
 	g_free (p);
 	
-	DEBUG_MSG (("GbEditor finalized\n"));
+	LOG ("GbEditor finalized");
 	
 	G_OBJECT_CLASS (g_object_class)->finalize (o);
 }
@@ -840,7 +844,8 @@ gb_editor_key_press_event_cb (GtkWidget *widget, GdkEventKey *event)
 	if ((event->state & GDK_CONTROL_MASK) || (event->state & GDK_MOD1_MASK) || (event->keyval == GDK_Delete) || (event->keyval == GDK_KP_Delete))
 	{
 		GtkWidget *entry = gtk_window_get_focus (GTK_WINDOW(widget));
-		if (GTK_IS_EDITABLE(entry) && gtk_widget_event (entry, (GdkEvent*)event))
+		if ((GTK_IS_EDITABLE(entry) || GTK_IS_TEXT_VIEW (entry)) &&
+		    gtk_widget_event (entry, (GdkEvent*)event))
 			return TRUE;
 	}
 	return FALSE;
@@ -941,6 +946,14 @@ gb_editor_init_widgets_impl (GbEditor *e)
 					 (BonoboUIListenerFn) gb_editor_bonobo_edit_pane_changed_cb, 
 					 e);
 
+	/* load default status */
+	{
+		gboolean edit_pane = eel_gconf_get_boolean (CONF_BOOKMARKS_EDITOR_EDIT_PANE);
+		gboolean split_view = eel_gconf_get_boolean (CONF_BOOKMARKS_EDITOR_SPLIT_VIEW);
+		bonobo_ui_component_set_prop (ui_component, SPLIT_VIEW_PATH, "state", split_view ? "1" : "0", NULL);
+		bonobo_ui_component_set_prop (ui_component, EDIT_PANE_PATH, "state", edit_pane ? "1" : "0", NULL);
+	}
+
 	/* sync ui with editor state */
 	gb_editor_bonobo_cmd_edit_pane (ui_component, e, "EditPane");
 	gb_editor_bonobo_cmd_split_view (ui_component, e, "SplitView");
@@ -1435,6 +1448,7 @@ gb_editor_add_several (GbEditor *e, GSList *l)
 	for (li = l; li; li = li->next)
 	{
 		gb_folder_add_child (parent, li->data, index++);
+		gb_bookmark_set_time_added_now (li->data);
 	}
 	
 	if (l && !l->next)
@@ -1443,7 +1457,7 @@ gb_editor_add_several (GbEditor *e, GSList *l)
 	}
 	else if (l)
 	{
-		// TODO: select all of them?
+		/* TODO: select all of them? */
 	}
 }
 
@@ -1583,7 +1597,7 @@ gb_editor_cmd_file_open (GbEditor *e)
 {
 	GbEditorPrivate *p = e->priv;
 	gchar *dir = eel_gconf_get_string (CONF_STATE_OPEN_DIR);
-	//GbBookmarkSet *currentset = p->set;
+	/* GbBookmarkSet *currentset = p->set; */
 	char *file;
 	
 	file = bonobo_file_selector_open 
@@ -1591,7 +1605,8 @@ gb_editor_cmd_file_open (GbEditor *e)
 		 FALSE, _("Select the bookmarks file to open"),
 		 "XBEL files: application/xbel"
 		 "|Mozilla bookmarks files:application/mozilla-bookmarks,text/html"
-		 "|Netscape bookmarks files:application/netscape-bookmarks,text/html", 
+		 "|Netscape bookmarks files:application/netscape-bookmarks,text/html"
+		 "|Epiphany RDF bookmarks files:application/xml,text/xml",
 		 dir);
 	g_free (dir);
 	
@@ -1809,6 +1824,22 @@ gb_editor_cmd_import_konqueror (GbEditor *e)
 	gb_import_druid_show (d);
 }
 
+static void
+gb_editor_bonobo_cmd_import_epiphany (BonoboUIComponent *uic, GbEditor *e,
+				      const gchar *verbname)
+{
+	gb_editor_cmd_import_epiphany (e);
+}
+
+void
+gb_editor_cmd_import_epiphany (GbEditor *e)
+{
+	GbEditorPrivate *p = e->priv;
+	GbImportDruid *d = GB_IMPORT_DRUID (gb_import_druid_epiphany_new ());
+	gb_import_druid_set_dest_bookmark_set (d, p->set);
+	gb_import_druid_show (d);
+}
+
 static void
 gb_editor_bonobo_cmd_sort_one_level (BonoboUIComponent *uic, GbEditor *e,
 				     const gchar *verbname)
@@ -1956,6 +1987,8 @@ void
 gb_editor_cmd_split_view (GbEditor *e, gboolean split)
 {
 	gb_editor_set_show_right_tree_view (e, split);
+
+	eel_gconf_set_boolean (CONF_BOOKMARKS_EDITOR_SPLIT_VIEW, split);
 }
 
 static void
@@ -1975,12 +2008,15 @@ gb_editor_bonobo_cmd_edit_pane (BonoboUIComponent *uic, GbEditor *e,
 	gchar *value = bonobo_ui_component_get_prop (uic, EDIT_PANE_PATH, "state", NULL);
 	gb_editor_cmd_edit_pane (e, atoi (value));
 	g_free (value);
+
 }
 
 void
 gb_editor_cmd_edit_pane (GbEditor *e, gboolean pane)
 {
 	gb_editor_set_show_edit_pane (e, pane);
+
+	eel_gconf_set_boolean (CONF_BOOKMARKS_EDITOR_EDIT_PANE, pane);
 }
 
 
@@ -2115,7 +2151,7 @@ gb_editor_search_timeout_cb (gpointer data)
 {
 	GbEditor *f = data;
 	GbEditorPrivate *p = f->priv;
-	DEBUG_MSG (("gb_editor_search_timeout_cb\n"));
+	LOG ("gb_editor_search_timeout_cb");
 
 	p->search_timeout = 0;
 
