@@ -20,7 +20,7 @@
 #include "config.h"
 #endif
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include "bookmarks-export-druid.h"
 #include "bookmarks-io.h"
 #include "bookmarks-tree-model.h"
@@ -223,6 +223,13 @@ gb_export_druid_init_location_widgets (GbExportDruid *d)
 
 	p->location_entry = gnome_file_entry_new ("galeon-bookamrks-export-locations",
 						  _("Choose a file to export"));
+	/* Use the new file chooser if we can */
+	if (g_object_class_find_property (G_OBJECT_GET_CLASS (p->location_entry), 
+					  "use_filechooser" ))
+	{
+		g_object_set (p->location_entry, "use_filechooser", TRUE, NULL);
+	}
+
 	gtk_widget_show (p->location_entry);
 	g_signal_connect (gnome_entry_gtk_entry (GNOME_ENTRY (gnome_file_entry_gnome_entry 
 							      (GNOME_FILE_ENTRY (p->location_entry)))),
