@@ -26,14 +26,14 @@
 #include "galeon-marshal.h"
 #include "bookmarks-dnd.h"
 #include "gul-string.h"
+#include "galeon-debug.h"
 #include "bookmarks-context-menu.h"
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkaccellabel.h>
-
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
+#include <gtk/gtktooltips.h>
+#include <string.h>
 
 /**
  * Private data
@@ -66,7 +66,8 @@ static void		gb_gtk_menu_item_menu_event_after_cb		(GtkWidget *wid, GdkEventAny
 static void		gb_gtk_menu_item_bookmark_activated_cb	 	(GObject *sender,
 									 GbBookmarkEventActivated *ev,
 									 GbGtkMenuItem *bmi);
-
+static void             gb_gtk_menu_item_style_set_impl                 (GtkWidget *widget,
+									 GtkStyle *previous_style);
 
 static gpointer gtk_image_menu_item_class;
 
@@ -75,6 +76,7 @@ enum GbGtkMenuItemSignalsEnum {
 	GB_GTK_MENU_ITEM_LAST_SIGNAL
 };
 static gint GbGtkMenuItemSignals[GB_GTK_MENU_ITEM_LAST_SIGNAL];
+static GtkTooltips *_gb_gtk_menu_item_tooltips = NULL;
 
 /**
  * GtkMenu object
@@ -89,6 +91,8 @@ gb_gtk_menu_item_class_init (GbGtkMenuItemClass *klass)
 	G_OBJECT_CLASS (klass)->finalize = gb_gtk_menu_item_finalize_impl;
 	gtk_image_menu_item_class = g_type_class_peek_parent (klass);
 
+	GTK_WIDGET_CLASS (klass)->style_set = gb_gtk_menu_item_style_set_impl;
+
 	GbGtkMenuItemSignals[GB_GTK_MENU_ITEM_BOOKMARK_ACTIVATED] = g_signal_new (
 		"bookmark-activated", G_OBJECT_CLASS_TYPE (klass),  
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP,
@@ -96,6 +100,7 @@ gb_gtk_menu_item_class_init (GbGtkMenuItemClass *klass)
 		NULL, NULL, 
 		g_cclosure_marshal_VOID__POINTER,
 		G_TYPE_NONE, 1, G_TYPE_POINTER);
+
 }
 
 static void 
@@ -103,6 +108,15 @@ gb_gtk_menu_item_init (GbGtkMenuItem *m)
 {
 	GbGtkMenuItemPrivate *p = g_new0 (GbGtkMenuItemPrivate, 1);
 	m->priv = p;
+
+	if (!_gb_gtk_menu_item_tooltips)
+	{
+		_gb_gtk_menu_item_tooltips = gtk_tooltips_new ();
+		g_object_add_weak_pointer (G_OBJECT (_gb_gtk_menu_item_tooltips),
+					   (gpointer)&_gb_gtk_menu_item_tooltips);
+	}
+	g_object_ref (_gb_gtk_menu_item_tooltips);
+	gtk_object_sink (GTK_OBJECT(_gb_gtk_menu_item_tooltips));
 }
 
 static void
@@ -111,7 +125,7 @@ gb_gtk_menu_item_finalize_impl (GObject *o)
 	GbGtkMenuItem *gm = GB_GTK_MENU_ITEM (o);
 	GbGtkMenuItemPrivate *p = gm->priv;
 
-	DEBUG_MSG (("Finalizing GbGtkMenuItem\n"));
+	LOG ("Finalizing GbGtkMenuItem");
 
 	gb_gtk_menu_item_set_location_source (gm, NULL);
 	g_signal_handlers_disconnect_matched (p->bookmark, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, gm);
@@ -129,9 +143,27 @@ gb_gtk_menu_item_finalize_impl (GObject *o)
 	g_object_unref (G_OBJECT (p->bookmark));
 	g_free (p);
 
+	g_object_unref (_gb_gtk_menu_item_tooltips);
+
 	G_OBJECT_CLASS (gtk_image_menu_item_class)->finalize (o);
 }
 
+static void    
+gb_gtk_menu_item_style_set_impl (GtkWidget *widget,
+				 GtkStyle *previous_style)
+{
+	/* Hack courtesy of Owen, see bug 111560 */
+	GtkStyle* style = gtk_rc_get_style_by_paths (gtk_widget_get_settings (widget),
+						     "GtkWindow.GtkImageMenuItem.GtkLabel",
+						     "GtkWindow.GtkImageMenuItem.GtkLabel",
+						     GTK_TYPE_LABEL);
+	
+	gtk_widget_set_style (GTK_BIN (widget)->child, style);
+
+	if (GTK_WIDGET_CLASS (gtk_image_menu_item_class)->style_set)
+		GTK_WIDGET_CLASS (gtk_image_menu_item_class)->style_set (widget, previous_style);
+}
+
 GbGtkMenuItem *
 gb_gtk_menu_item_new (GbBookmark *bookmark)
 {
@@ -158,6 +190,35 @@ gb_gtk_menu_item_new (GbBookmark *bookmark)
 	return ret;
 }
 
+static void
+gb_gtk_menu_item_set_tooltip (GbGtkMenuItem *bmi, GbBookmark *bm, const char *visible_name)
+{
+	char *parts[2+1] = {NULL};
+	int n = 0;
+
+	if (strcmp (visible_name, bm->name) != 0)
+	{
+		/* truncated for menu, show full name */
+		parts[n++] = bm->name;
+	}
+
+	if (bm->notes && bm->notes[0] != '\0')
+	{
+		parts[n++] = bm->notes;
+	}
+
+	if (n > 0)
+	{
+		char *tooltip;
+
+		tooltip = g_strjoinv("\n\n", parts);
+		gtk_tooltips_set_tip (_gb_gtk_menu_item_tooltips,
+				      GTK_WIDGET(bmi),
+				      tooltip, NULL);
+		g_free (tooltip);
+	}
+}
+
 static void
 gb_gtk_menu_item_build_menuitem (GbGtkMenuItem *bmi)
 {
@@ -180,14 +241,9 @@ gb_gtk_menu_item_build_menuitem (GbGtkMenuItem *bmi)
 		gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (accel_label), GTK_WIDGET (bmi));
 		gtk_widget_show (accel_label);
 	}
+
+	gb_gtk_menu_item_set_tooltip (bmi, bm, name);
 	g_free (name);
-		
-	if (GB_IS_SITE (bm))
-	{
-		// gchar *tip = misc_string_shorten (GB_SITE (bm)->url, GB_GTK_MENU_ITEM_TIP_MAX_LENGTH);
-		// TODO: do we really want the tooltip?
-		// g_free (tip);
-	}
 	
 	if (pixbuf)
 	{
@@ -268,7 +324,7 @@ gb_gtk_menu_item_rebuild_timeout_cb (gpointer data)
 {
 	GbGtkMenuItem *bmi = data;
 	GbGtkMenuItemPrivate *p = bmi->priv;
-	DEBUG_MSG (("GbGtkMenuItem rebuild timeout\n"));
+	LOG ("GbGtkMenuItem rebuild timeout");
 
 	p->rebuild_timeout = 0;
 
@@ -293,7 +349,7 @@ gb_gtk_menu_item_rebuild_with_timeout (GbGtkMenuItem *bmi)
 static void
 gb_gtk_menu_item_rebuild (GbGtkMenuItem *bmi)
 {
-	DEBUG_MSG (("gb_gtk_menu_item_rebuild called\n"));
+	LOG ("gb_gtk_menu_item_rebuild called");
 	gb_gtk_menu_item_build_menuitem (bmi);
 }
 
@@ -347,6 +403,9 @@ gb_gtk_menu_item_button_press_cb (GbGtkMenuItem *bmi, GdkEventButton *event)
 	else if (event->button == 2)
 	{
 		const gchar *url = GB_IS_SITE (p->bookmark) ? GB_SITE (p->bookmark)->url : NULL;
+
+		gtk_menu_shell_deactivate (GTK_MENU_SHELL (GTK_WIDGET (bmi)->parent));
+
 		gb_activated_event (bmi, p->bookmark, url, GB_BAF_NEW_TAB_OR_WINDOW, (GdkEvent *) event);
 
 		/* avoid activating this bookmark */
@@ -357,6 +416,19 @@ gb_gtk_menu_item_button_press_cb (GbGtkMenuItem *bmi, GdkEventButton *event)
 	return FALSE;
 }
 
+static void
+gb_gtk_menu_item_popdown_menus (GbGtkMenuItem *wid)
+{
+	GbGtkMenuItemPrivate *p = wid->priv;
+	GtkWidget *parent = GTK_WIDGET (wid)->parent;
+	if (parent && GTK_IS_MENU_SHELL (parent))
+	{
+		/* avoid activating this bookmark */
+		p->ignore_next_activation = TRUE;
+		gtk_menu_shell_activate_item (GTK_MENU_SHELL (parent), GTK_WIDGET (wid), TRUE);
+	}
+}
+
 static void
 gb_gtk_menu_item_menu_event_after_cb (GtkWidget *wid, GdkEventAny *event, 
 				      GbGtkMenuItem *bmi)
@@ -379,6 +451,8 @@ gb_gtk_menu_item_menu_event_after_cb (GtkWidget *wid, GdkEventAny *event,
 		 && !GTK_MENU_SHELL (wid)->active_menu_item)
 	{
 		const gchar *url = GB_IS_SITE (p->bookmark) ? GB_SITE (p->bookmark)->url : NULL;
+		
+		gb_gtk_menu_item_popdown_menus (bmi);
 		gb_activated_event (bmi, p->bookmark, url, GB_BAF_NEW_TAB_OR_WINDOW, (GdkEvent *) event);
 	}
 }
