@@ -20,21 +20,19 @@
 #include "config.h"
 #endif
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkimagemenuitem.h>
-#include <gtk/gtktearoffmenuitem.h>
+#include <gtk/gtkseparatormenuitem.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkimage.h>
 #include "bookmarks-gtk-menu.h"
 #include "bookmarks-gtk-menu-item.h"
 #include "gul-gobject-misc.h"
 #include "galeon-marshal.h"
-#include "eel-gconf-extensions.h"
 #include "gul-string.h"
+#include "gul-gui.h"
 #include "bookmarks-context-menu.h"
-
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
+#include "galeon-debug.h"
 
 /**
  * Private data
@@ -43,6 +41,7 @@ struct _GbGtkMenuPrivate {
 	GbFolder *root;
 	GtkMenuShell *ms;
 	guint rebuild_timeout;
+	gboolean context_only;
 
 	GbLocationSource *location_source;
 	GSList *menuitems;
@@ -65,6 +64,7 @@ static void		gb_gtk_menu_child_added_cb		(GbFolder *p, GbBookmark *c,
 								 gint position, GbGtkMenu *gm);
 static void		gb_gtk_menu_child_removed_cb	 	(GbFolder *p, GbBookmark *c, 
 								 gint position, GbGtkMenu *gm);
+static void		gb_gtk_menu_context_menu_cb		(GbBookmarkSet *set, GbGtkMenu *gm);
 static void		gb_gtk_menu_rebuild			(GbGtkMenu *gm);
 static void 		gb_gtk_menu_rebuild_with_timeout	(GbGtkMenu *gm);
 static gboolean		gb_gtk_menu_rebuild_timeout_cb		(gpointer data);
@@ -116,11 +116,16 @@ gb_gtk_menu_finalize_impl (GObject *o)
 	GbGtkMenuPrivate *p = gm->priv;
 	GSList *li;
 
-	DEBUG_MSG (("Finalizing GbGtkMenu\n"));
+	LOG ("Finalizing GbGtkMenu");
 
 	gb_gtk_menu_set_location_source (gm, NULL);
 
 	g_signal_handlers_disconnect_matched (p->root, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, gm);
+	if (GB_BOOKMARK (p->root)->set)
+	{
+		g_signal_handlers_disconnect_matched (GB_BOOKMARK (p->root)->set, 
+						      G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, gm);
+	}
 
 	if (p->rebuild_timeout)
 	{
@@ -140,8 +145,8 @@ gb_gtk_menu_finalize_impl (GObject *o)
 	G_OBJECT_CLASS (g_object_class)->finalize (o);
 }
 
-GbGtkMenu *
-gb_gtk_menu_new (GbFolder *root, GtkMenuShell *ms)
+static GbGtkMenu *
+gb_gtk_menu_new_internal (GbFolder *root, GtkMenuShell *ms, gboolean context_only)
 {
 	GbGtkMenu *ret = g_object_new (GB_TYPE_GTK_MENU, NULL);
 	GbGtkMenuPrivate *p = ret->priv;
@@ -149,12 +154,22 @@ gb_gtk_menu_new (GbFolder *root, GtkMenuShell *ms)
 	p->root = root;
 	g_object_ref (root);
 
-	g_signal_connect (root, "child-modified", 
-			  G_CALLBACK (gb_gtk_menu_child_modied_cb), ret);
-	g_signal_connect (root, "child-added", 
-			  G_CALLBACK (gb_gtk_menu_child_added_cb), ret);
-	g_signal_connect (root, "child-removed", 
-			  G_CALLBACK (gb_gtk_menu_child_removed_cb), ret);
+	p->context_only = context_only;
+
+	if (p->context_only)
+	{
+		g_signal_connect (GB_BOOKMARK (root)->set, "context-menu",
+				  G_CALLBACK (gb_gtk_menu_context_menu_cb), ret);
+	}
+	else
+	{
+		g_signal_connect (root, "child-modified", 
+				  G_CALLBACK (gb_gtk_menu_child_modied_cb), ret);
+		g_signal_connect (root, "child-added", 
+				  G_CALLBACK (gb_gtk_menu_child_added_cb), ret);
+		g_signal_connect (root, "child-removed", 
+				  G_CALLBACK (gb_gtk_menu_child_removed_cb), ret);
+	}
 
 	p->ms = ms;
 	g_object_ref (G_OBJECT (ms));
@@ -165,32 +180,56 @@ gb_gtk_menu_new (GbFolder *root, GtkMenuShell *ms)
 	return ret;
 }
 
+GbGtkMenu *
+gb_gtk_menu_new (GbFolder *root, GtkMenuShell *ms)
+{
+	GbGtkMenu *ret = gb_gtk_menu_new_internal (root, ms, FALSE);
+	return ret;
+}
+
+GbGtkMenu *
+gb_gtk_menu_new_context_only (GbBookmarkSet *set, GtkMenuShell *ms)
+{
+	GbGtkMenu *ret = gb_gtk_menu_new_internal (set->root, ms, TRUE);
+	return ret;
+}
+
 static void
 gb_gtk_menu_build_submenu (GbGtkMenu *gm)
 {
 	GbGtkMenuPrivate *p = gm->priv;
 	guint index = 0;
-	GbBookmark *i;
+	GSList *items;
+	GSList *li;
 
-	gboolean has_tearoff = eel_gconf_get_boolean ("/desktop/gnome/interface/menus_have_tearoff");
-	if (has_tearoff)
+	if (p->context_only)
 	{
-		GtkWidget *tearoff = gtk_tearoff_menu_item_new ();
-		gtk_widget_show (tearoff);
-		gtk_menu_shell_prepend (p->ms, tearoff);
+		items = g_slist_copy ((GSList *) gb_bookmark_set_get_context_bookmarks (GB_BOOKMARK (p->root)->set));
+		if (g_slist_length(items) != 0)
+			gul_gui_add_separator_conditional (p->ms);
+		else
+			gul_gui_remove_separator_conditional (p->ms);
+	}
+	else
+	{
+		gul_gui_setup_tearoff (p->ms);
+		items = gb_folder_list_children (p->root);
 	}
 
-	for (i = p->root->child; i != NULL; i = i->next)
+	for (li = items; li != NULL; li = li->next)
 	{
+		GbBookmark *i = li->data;
+		
 		if  (GB_IS_FOLDER (i) || GB_IS_SITE (i))
 		{
 			gb_gtk_menu_build_bookmark (gm, i, index);
 		}
 		else if (GB_IS_SEPARATOR (i))
 		{
-			GtkWidget *sep = gtk_menu_item_new ();
+			GtkWidget *sep = gtk_separator_menu_item_new ();
 			gtk_container_add (GTK_CONTAINER (p->ms), sep);
 			gtk_widget_show (sep);
+			p->menuitems = g_slist_prepend (p->menuitems, g_object_ref (sep));
 		}
 		else
 		{
@@ -198,6 +237,8 @@ gb_gtk_menu_build_submenu (GbGtkMenu *gm)
 		}
 		index++;
 	}
+	
+	g_slist_free (items);
 }
 
 static void
@@ -240,7 +281,10 @@ gb_gtk_menu_set_location_source (GbGtkMenu *gm, GbLocationSource *src)
 
 	for (li = p->menuitems; li; li = li->next)
 	{
-		gb_gtk_menu_item_set_location_source (li->data, src);
+		if (GB_IS_GTK_MENU_ITEM (li->data))
+		{
+			gb_gtk_menu_item_set_location_source (li->data, src);
+		}
 	}
 }
 
@@ -249,7 +293,7 @@ gb_gtk_menu_rebuild_timeout_cb (gpointer data)
 {
 	GbGtkMenu *gm = data;
 	GbGtkMenuPrivate *p = gm->priv;
-	DEBUG_MSG (("GbGtkMenu rebuild timeout\n"));
+	LOG ("GbGtkMenu rebuild timeout");
 
 	p->rebuild_timeout = 0;
 
@@ -275,15 +319,20 @@ static void
 gb_gtk_menu_rebuild (GbGtkMenu *gm)
 {
 	GbGtkMenuPrivate *p = gm->priv;
-	GList *children;
-	GList *li;
 	GSList *sli;
 
-	DEBUG_MSG (("gb_gtk_menu_rebuild called\n"));
+	LOG ("gb_gtk_menu_rebuild called");
 	
 	/* this could be more fine grained. For now, just destroy everything and 
 	   build it again */
 	
+	/* remove already created menuitems*/
+	for (sli = p->menuitems; sli; sli = sli->next)
+	{
+		g_assert (GTK_IS_WIDGET (sli->data) && GTK_WIDGET (sli->data)->parent == GTK_WIDGET (p->ms));
+		gtk_container_remove (GTK_CONTAINER (p->ms), sli->data);
+	}
+
 	/* free the list of menuitems */
 	for (sli = p->menuitems; sli; sli = sli->next)
 	{
@@ -292,13 +341,6 @@ gb_gtk_menu_rebuild (GbGtkMenu *gm)
 	g_slist_free (p->menuitems);
 	p->menuitems = NULL;
 
-	children = gtk_container_get_children (GTK_CONTAINER (p->ms));
-	for (li = children; li; li = li->next)
-	{
-		gtk_container_remove (GTK_CONTAINER (p->ms), li->data);
-	}
-	g_list_free (children);
-
 	gb_gtk_menu_build_submenu (gm);
 }
 
@@ -320,6 +362,16 @@ gb_gtk_menu_child_removed_cb (GbFolder *p, GbBookmark *c, gint position, GbGtkMe
 	gb_gtk_menu_rebuild_with_timeout (gm);
 }
 
+static void
+gb_gtk_menu_context_menu_cb (GbBookmarkSet *set, GbGtkMenu *gm)
+{
+	g_return_if_fail (GB_IS_GTK_MENU (gm));
+	g_return_if_fail (GB_IS_BOOKMARK_SET (set));
+	g_return_if_fail (GB_BOOKMARK (gm->priv->root)->set == set);
+
+	gb_gtk_menu_rebuild_with_timeout (gm);
+}
+
 static void
 gb_gtk_menu_bookmark_activated_cb (GObject *sender,
 				   GbBookmarkEventActivated *ev,
@@ -336,7 +388,10 @@ gb_gtk_menu_fill_children_submenus (GbGtkMenu *gm)
 
 	for (li = p->menuitems; li; li = li->next)
 	{
-		gb_gtk_menu_item_fill_submenu (li->data, FALSE);
+		if (GB_IS_GTK_MENU_ITEM (li->data))
+		{
+			gb_gtk_menu_item_fill_submenu (li->data, FALSE);
+		}
 	}
 }
 
