@@ -20,7 +20,7 @@
 #include "config.h"
 #endif
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include "bookmarks-import-druid.h"
 #include "bookmarks-io.h"
 #include "bookmarks-tree-model.h"
@@ -154,7 +154,6 @@ gb_import_druid_init_widgets (GbImportDruid *d)
 	g_return_if_fail (p->window == NULL);
 	
 	p->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_resizable (GTK_WINDOW (p->window), FALSE);
 	gtk_window_set_title (GTK_WINDOW (p->window), 
 			      gb_import_druid_get_string (d, "window title"));
 
@@ -305,6 +304,14 @@ gb_import_druid_init_location_widgets (GbImportDruid *d)
 
 	p->location_entry = gnome_file_entry_new ("galeon-bookamrks-import-locations",
 						  _("Choose a file to import"));
+
+	/* Use the new file chooser if we can */
+	if (g_object_class_find_property (G_OBJECT_GET_CLASS (p->location_entry), 
+					  "use_filechooser" ))
+	{
+		g_object_set (p->location_entry, "use_filechooser", TRUE, NULL);
+	}
+
 	gtk_widget_show (p->location_entry);
 	g_signal_connect (gnome_entry_gtk_entry (GNOME_ENTRY (gnome_file_entry_gnome_entry 
 							      (GNOME_FILE_ENTRY (p->location_entry)))),
