@@ -20,11 +20,12 @@
 #include "config.h"
 #endif
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include "gul-glade.h"
 #include "bookmarks-single-editor.h"
 #include "gul-gobject-misc.h"
 #include "gul-string.h"
+#include "gul-state.h"
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
 #include <gtk/gtktextbuffer.h>
@@ -32,9 +33,14 @@
 #include <gtk/gtktogglebutton.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtklabel.h>
+#include <gtk/gtkcomboboxentry.h>
 #include <libgnomeui/gnome-pixmap-entry.h>
 #include "bookmarks-tree-view.h"
 
+#include <gdk/gdkkeysyms.h>
+#include <string.h>
+#include <time.h>
+
 /**
  * Private data
  */
@@ -50,6 +56,10 @@ struct _GbSingleEditorPrivate {
 	GtkWidget *parents_scrolledwindow;
 	GbTreeView *parents_treeview;
 	GbTreeModel *parents_treemodel;
+	GtkWidget *alias_label;
+	GtkWidget *smart_entry_width_hbox;
+
+	GtkWidget *close_button;
 };
 
 /**
@@ -81,7 +91,7 @@ static void		gb_single_editor_create_toolbar_checkbox_toggled_cb (GtkWidget *edi
 static void		gb_single_editor_add_to_context_checkbox_toggled_cb (GtkWidget *edited,
 									     GbSingleEditor *e);
 static void 		gb_single_editor_image_pixmapentry_changed_cb (GtkWidget *edited,
-								       GbSingleEditor *e);
+								      GbSingleEditor *e);
 static void		gb_single_editor_parameter_encoding_combo_changed_cb (GtkWidget *edited,
 									      GbSingleEditor *e);
 
@@ -160,6 +170,26 @@ parents_treeview_bookmark_check_toggled (GbTreeView *tv, GbBookmark *b, GbSingle
 	}
 }
 
+static gboolean
+gb_single_editor_close_on_ctrl_enter (GtkWidget *widget, GdkEventKey *event,
+				      GbSingleEditor *e)
+{
+	if ((event->state & GDK_CONTROL_MASK) && 
+	    ((event->keyval == GDK_Return) ||  (event->keyval == GDK_KP_Enter)))
+	{
+		gtk_widget_destroy (e->priv->window);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
+static void
+gb_single_editor_move_to_next (GtkWidget *widget, GbSingleEditor *e)
+{
+	gtk_widget_child_focus (e->priv->window, GTK_DIR_TAB_FORWARD);
+}
+
 static void
 gb_single_editor_init_widgets_impl (GbSingleEditor *e)
 {
@@ -178,11 +208,11 @@ gb_single_editor_init_widgets_impl (GbSingleEditor *e)
 		{ "bookmarks_properties_date_visited_value_label", &e->date_visited_value_label },
 		{ "bookmarks_properties_smart_entry_width_label", &e->smart_entry_width_label },
 		{ "bookmarks_properties_smart_entry_width_spinbutton", &e->smart_entry_width_spinbutton },
+		{ "bookmarks_properties_smart_entry_width_hbox", &p->smart_entry_width_hbox },
 		{ "bookmarks_properties_parameter_encoding_label", &e->parameter_encoding_label },
 		{ "bookmarks_properties_parameter_encoding_combo", &e->parameter_encoding_combo },
 		{ "bookmarks_properties_location_label", &e->location_label },
 		{ "bookmarks_properties_location_entry", &e->location_entry },
-		{ "bookmarks_properties_notes_label", &e->notes_label },
 		{ "bookmarks_properties_notes_textview", &e->notes_textview },
 		{ "bookmarks_properties_nicks_label", &e->nicks_label },
 		{ "bookmarks_properties_nicks_entry", &e->nicks_entry },
@@ -191,6 +221,8 @@ gb_single_editor_init_widgets_impl (GbSingleEditor *e)
 		{ "bookmarks_properties_image_label", &e->image_label },
 		{ "bookmarks_properties_image_pixmapentry", &e->image_pixmapentry },
 		{ "bookmarks_properties_parents_scrolledwindow", &p->parents_scrolledwindow },
+		{ "bookmarks_properties_alias_label", &p->alias_label },
+		{ "bookmarks_properties_close_button", &p->close_button },
 		{ NULL, NULL }
 	};
 	GladeXML *gxml = gul_glade_widget_new ("bookmarks-editor.glade", "bookmarks_properties_window",
@@ -199,6 +231,7 @@ gb_single_editor_init_widgets_impl (GbSingleEditor *e)
 	g_object_unref (gxml);
 	
 	g_object_weak_ref (G_OBJECT (p->window), (GWeakNotify) g_object_unref, e);
+	gul_state_monitor_window (p->window, "Bookmark Properties", 370, 390);
 
 	p->parents_treeview = gb_tree_view_new ();
 	gb_tree_view_set_check_visible (p->parents_treeview, TRUE);
@@ -208,6 +241,41 @@ gb_single_editor_init_widgets_impl (GbSingleEditor *e)
 			  G_CALLBACK (parents_treeview_bookmark_check_toggled), e);
 	gtk_widget_show (GTK_WIDGET (p->parents_treeview));
 	gtk_container_add (GTK_CONTAINER (p->parents_scrolledwindow), GTK_WIDGET (p->parents_treeview));
+	gtk_label_set_mnemonic_widget (GTK_LABEL (p->alias_label), GTK_WIDGET (p->parents_treeview));
+
+	/* Hook up the close button */
+	g_signal_connect_swapped (p->close_button, "clicked",
+				  G_CALLBACK (gtk_widget_destroy), e->priv->window);
+
+	/* Setup CTRL+Enter to close the window */
+	g_signal_connect (e->name_entry, "key-press-event",
+			  G_CALLBACK (gb_single_editor_close_on_ctrl_enter), e);
+	g_signal_connect (e->smarturl_entry, "key-press-event",
+			  G_CALLBACK (gb_single_editor_close_on_ctrl_enter), e);
+	g_signal_connect (e->location_entry, "key-press-event",
+			  G_CALLBACK (gb_single_editor_close_on_ctrl_enter), e);
+	g_signal_connect (e->nicks_entry, "key-press-event",
+			  G_CALLBACK (gb_single_editor_close_on_ctrl_enter), e);
+	g_signal_connect (p->parents_treeview, "key-press-event",
+			  G_CALLBACK (gb_single_editor_close_on_ctrl_enter), e);
+	g_signal_connect (e->smart_entry_width_spinbutton, "key-press-event",
+			  G_CALLBACK (gb_single_editor_close_on_ctrl_enter), e);
+	g_signal_connect (e->notes_textview, "key-press-event",
+			  G_CALLBACK (gb_single_editor_close_on_ctrl_enter), e);
+	g_signal_connect (GTK_BIN (e->parameter_encoding_combo)->child, "key-press-event",
+			  G_CALLBACK (gb_single_editor_close_on_ctrl_enter), e);
+
+	/* Setup Enter to move to the next widget */
+	g_signal_connect (GTK_ENTRY (e->name_entry), "activate", 
+			  G_CALLBACK (gb_single_editor_move_to_next), e);
+	g_signal_connect (GTK_ENTRY (e->nicks_entry), "activate", 
+			  G_CALLBACK (gb_single_editor_move_to_next), e);
+	g_signal_connect (GTK_ENTRY (e->smarturl_entry), "activate", 
+			  G_CALLBACK (gb_single_editor_move_to_next), e);
+	g_signal_connect (GTK_ENTRY (e->location_entry), "activate", 
+			  G_CALLBACK (gb_single_editor_move_to_next), e);
+	g_signal_connect (GTK_ENTRY (GTK_BIN (e->parameter_encoding_combo)->child), "activate", 
+			  G_CALLBACK (gb_single_editor_move_to_next), e);
 }
 
 void 
@@ -277,11 +345,12 @@ gb_single_editor_init_widgets_and_signals (GbSingleEditor *e)
 	{
 		g_signal_connect (e->image_pixmapentry, "changed",
 				  G_CALLBACK (gb_single_editor_image_pixmapentry_changed_cb), e);
+		gtk_label_set_mnemonic_widget (GTK_LABEL (e->image_label), e->image_pixmapentry);
 	}
 	
-	if (GTK_COMBO (e->parameter_encoding_combo))
+	if (GTK_COMBO_BOX_ENTRY (e->parameter_encoding_combo))
 	{
-		g_signal_connect (GTK_COMBO (e->parameter_encoding_combo)->entry, "changed",
+		g_signal_connect (GTK_BIN (e->parameter_encoding_combo)->child, "changed",
 				  G_CALLBACK (gb_single_editor_parameter_encoding_combo_changed_cb), e);
 	}
 }
@@ -402,15 +471,39 @@ safe_gtk_widget_hide (GtkWidget *w)
 	}
 }
 
+static gchar *
+gb_single_editor_format_time (GTime t)
+{
+	if (t > 0)
+	{
+		gchar buffer[256];
+		int length;
+		struct tm stm;
+		time_t tt = t;
+
+		/* Convert to local time */
+		localtime_r (&tt, &stm);
+		length = strftime (buffer, sizeof(buffer), _("%a, %b %-d %Y at %-I:%M %p"),
+				   &stm);
+		buffer[length] = '\0';		
+
+		return g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
+	}
+	return NULL;
+}
+
+
 static void
 gb_single_editor_update_controls (GbSingleEditor *e)
 {
-	// TODO: blank all fields when disabling them.
+	/* TODO: blank all fields when disabling them. */
 	
 	GbSingleEditorPrivate *p = e->priv;
 
-	void (*disable) (GtkWidget *) = p->disable_policy == GB_SINGLE_EDITOR_DISABLE_POLICY_HIDE ? safe_gtk_widget_hide : safe_gtk_widget_disable;
-	void (*enable) (GtkWidget *) = p->disable_policy == GB_SINGLE_EDITOR_DISABLE_POLICY_HIDE ? safe_gtk_widget_show : safe_gtk_widget_enable;
+	void (*disable) (GtkWidget *) = p->disable_policy == GB_SINGLE_EDITOR_DISABLE_POLICY_HIDE 
+		? safe_gtk_widget_hide : safe_gtk_widget_disable;
+	void (*enable) (GtkWidget *) = p->disable_policy == GB_SINGLE_EDITOR_DISABLE_POLICY_HIDE
+		? safe_gtk_widget_show : safe_gtk_widget_enable;
 
 	gb_single_editor_init_widgets_and_signals (e);
 
@@ -476,17 +569,23 @@ gb_single_editor_update_controls (GbSingleEditor *e)
 		enable (e->name_label);
 	}
 
-	if (e->nicks_entry)
-	{
-		gtk_entry_set_text (GTK_ENTRY (e->nicks_entry), p->bm->nick);
-		enable (e->nicks_entry);
-		enable (e->nicks_label);
-	}
-
 	if (e->notes_textview)
 	{
 		GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (e->notes_textview));
-		gtk_text_buffer_set_text (buffer, p->bm->notes, -1);
+		GtkTextIter begin, end;
+		char * text;
+
+		/* Fix bug 97262, don't update the text if it won't change it, 
+		 * this stops the cursor being moved to the end */
+		gtk_text_buffer_get_start_iter (buffer, &begin);
+		gtk_text_buffer_get_end_iter (buffer, &end);
+		text = gtk_text_buffer_get_text (buffer, &begin, &end, TRUE);
+		if (strcmp (text, p->bm->notes))
+		{
+			gtk_text_buffer_set_text (buffer, p->bm->notes, -1);
+		}
+
+		g_free (text);
 		enable (e->notes_textview);
 		enable (e->notes_label);
 	}
@@ -500,23 +599,27 @@ gb_single_editor_update_controls (GbSingleEditor *e)
 
 	if (e->date_added_value_label)
 	{
-		char *s = gul_string_time_to_string (p->bm->time_added);
-		gtk_label_set_text (GTK_LABEL (e->date_added_value_label), s);
+		char *s = gb_single_editor_format_time (p->bm->time_added);
+		if (!s) s = g_strdup_printf( "<i>&lt;%s&gt;</i>", _("Not specified"));
+		gtk_label_set_markup (GTK_LABEL (e->date_added_value_label), s);
 		g_free (s);
 		enable (e->date_added_value_label);
 	}
 
 	if (e->date_modified_value_label)
 	{
-		gchar *s = gul_string_time_to_string (p->bm->time_modified);
-		gtk_label_set_text (GTK_LABEL (e->date_modified_value_label), s);
+		/* The entry was 'modified' when it was added */
+		GTime t = p->bm->time_modified ? p->bm->time_modified : p->bm->time_added;
+		gchar *s = gb_single_editor_format_time (t);
+		if (!s) s = g_strdup_printf( "<i>&lt;%s&gt;</i>", _("Not specified"));
+		gtk_label_set_markup (GTK_LABEL (e->date_modified_value_label), s);
 		g_free (s);
 		enable (e->date_modified_value_label);
 	}
 
 	if (e->image_pixmapentry)
 	{
-		gnome_file_entry_set_filename (GNOME_FILE_ENTRY (e->image_pixmapentry), 
+		gnome_file_entry_set_filename (GNOME_FILE_ENTRY (e->image_pixmapentry),
 					       p->bm->pixmap_file);
 		enable (e->image_pixmapentry);
 	}
@@ -532,11 +635,18 @@ gb_single_editor_update_controls (GbSingleEditor *e)
 
 		if (e->date_visited_value_label)
 		{
-			gchar *s = gul_string_time_to_string (GB_SITE (p->bm)->time_visited);
-			gtk_label_set_text (GTK_LABEL (e->date_visited_value_label), s);
+			gchar *s = gb_single_editor_format_time (GB_SITE (p->bm)->time_visited);
+			if (!s) s = g_strdup_printf( "<i>&lt;%s&gt;</i>", _("Not specified"));
+			gtk_label_set_markup (GTK_LABEL (e->date_visited_value_label), s);
 			g_free (s);
 			enable (e->date_visited_value_label);
 		}
+		if (e->nicks_entry)
+		{
+			gtk_entry_set_text (GTK_ENTRY (e->nicks_entry), p->bm->nick);
+			enable (e->nicks_entry);
+			enable (e->nicks_label);
+		}
 	}
 	else
 	{
@@ -544,10 +654,16 @@ gb_single_editor_update_controls (GbSingleEditor *e)
 		{
 			gtk_entry_set_text (GTK_ENTRY (e->location_entry), "");
 		}
+		if (e->nicks_entry)
+		{
+			gtk_entry_set_text (GTK_ENTRY (e->nicks_entry), "");
+		}
 		disable (e->location_entry);
 		disable (e->location_label);
 		disable (e->date_visited_value_label);
 		disable (e->date_visited_label);
+		disable (e->nicks_entry);
+		disable (e->nicks_label);
 	}
 
 	if (GB_IS_SMART_SITE (p->bm))
@@ -558,11 +674,14 @@ gb_single_editor_update_controls (GbSingleEditor *e)
 			gtk_spin_button_set_value (GTK_SPIN_BUTTON (e->smart_entry_width_spinbutton), width);
 			enable (e->smart_entry_width_spinbutton);
 			enable (e->smart_entry_width_label);
+			enable (p->smart_entry_width_hbox);
 		}
 
 		if (e->smarturl_entry)
 		{
-			gtk_entry_set_text (GTK_ENTRY (e->smarturl_entry), GB_SMART_SITE (p->bm)->smarturl);
+			gchar * smarturl = gb_smart_site_get_smarturl (GB_SMART_SITE (p->bm));
+			gtk_entry_set_text (GTK_ENTRY (e->smarturl_entry), smarturl);
+			g_free (smarturl);
 			enable (e->smarturl_entry);
 			enable (e->smarturl_label);
 		}
@@ -570,7 +689,7 @@ gb_single_editor_update_controls (GbSingleEditor *e)
 		if (e->parameter_encoding_combo)
 		{
 			gchar *encoding = gb_smart_site_get_encoding (GB_SMART_SITE (p->bm));
-			gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (e->parameter_encoding_combo)->entry), encoding);
+			gtk_entry_set_text (GTK_ENTRY (GTK_BIN (e->parameter_encoding_combo)->child), encoding);
 			g_free (encoding);
 			enable (e->parameter_encoding_combo);
 			enable (e->parameter_encoding_label);
@@ -580,6 +699,7 @@ gb_single_editor_update_controls (GbSingleEditor *e)
 	{
 		disable (e->smart_entry_width_spinbutton);
 		disable (e->smart_entry_width_label);
+		disable (p->smart_entry_width_hbox);
 		if (e->smarturl_entry)
 		{
 			gtk_entry_set_text (GTK_ENTRY (e->smarturl_entry), "");
@@ -628,6 +748,7 @@ gb_single_editor_name_entry_changed_cb	(GtkWidget *edited,
 
 	s = gtk_editable_get_chars (GTK_EDITABLE (e->name_entry), 0, -1);
 	gb_bookmark_set_name (p->bm, s);
+	gb_bookmark_set_time_modified_now (p->bm);
 	g_free (s);
 }
 
@@ -644,6 +765,7 @@ gb_single_editor_smarturl_entry_changed_cb (GtkWidget *edited,
 
 	s = gtk_editable_get_chars (GTK_EDITABLE (e->smarturl_entry), 0, -1);
 	gb_smart_site_set_smarturl (GB_SMART_SITE (p->bm), s);
+	gb_bookmark_set_time_modified_now (p->bm);
 	g_free (s);
 }
 
@@ -660,6 +782,7 @@ gb_single_editor_location_entry_changed_cb (GtkWidget *edited,
 
 	s = gtk_editable_get_chars (GTK_EDITABLE (e->location_entry), 0, -1);
 	gb_site_set_url (GB_SITE (p->bm), s);
+	gb_bookmark_set_time_modified_now (p->bm);
 	g_free (s);
 }
 
@@ -676,6 +799,7 @@ gb_single_editor_smart_entry_width_spinbutton_value_changed_cb (GtkWidget *spinb
 	width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (e->smart_entry_width_spinbutton));
 
 	gb_smart_site_set_entry_size (GB_SMART_SITE (p->bm), 0, width);
+	gb_bookmark_set_time_modified_now (p->bm);
 }
 
 static void
@@ -696,6 +820,7 @@ gb_single_editor_notes_text_buffer_changed_cb (GtkWidget *edited,
 	gtk_text_buffer_get_end_iter (b, &end);
 	notes = gtk_text_buffer_get_text (b, &begin, &end, TRUE);
 	gb_bookmark_set_notes (p->bm, notes);
+	gb_bookmark_set_time_modified_now (p->bm);
 	g_free (notes);
 
 }
@@ -713,6 +838,7 @@ gb_single_editor_nicks_entry_changed_cb	(GtkWidget *edited,
 
 	s = gtk_editable_get_chars (GTK_EDITABLE (e->nicks_entry), 0, -1);
 	gb_bookmark_set_nick (p->bm, s);
+	gb_bookmark_set_time_modified_now (p->bm);
 	g_free (s);
 
 }
@@ -730,6 +856,7 @@ gb_single_editor_create_toolbar_checkbox_toggled_cb (GtkWidget *edited,
 
 	val = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (e->create_toolbar_checkbox));
 	gb_folder_set_create_toolbar (GB_FOLDER (p->bm), val);
+	gb_bookmark_set_time_modified_now (p->bm);
 
 }
 
@@ -746,12 +873,13 @@ gb_single_editor_add_to_context_checkbox_toggled_cb (GtkWidget *edited,
 
 	val = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (e->add_to_context_checkbox));
 	gb_bookmark_set_add_to_context_menu (p->bm, val);
+	gb_bookmark_set_time_modified_now (p->bm);
 
 }
 
 static void
 gb_single_editor_image_pixmapentry_changed_cb (GtkWidget *edited,
-					       GbSingleEditor *e)
+						GbSingleEditor *e)
 {
 	GbSingleEditorPrivate *p = e->priv;
 	gchar *s;
@@ -762,6 +890,7 @@ gb_single_editor_image_pixmapentry_changed_cb (GtkWidget *edited,
 
 	s = gtk_editable_get_chars (GTK_EDITABLE (e->image_pixmapentry), 0, -1);
 	gb_bookmark_set_pixmap (p->bm, s);
+	gb_bookmark_set_time_modified_now (p->bm);
 	g_free (s);
 
 }
@@ -777,8 +906,9 @@ gb_single_editor_parameter_encoding_combo_changed_cb (GtkWidget *edited,
 	
 	g_return_if_fail (GB_IS_SMART_SITE (p->bm));
 
-	s = gtk_editable_get_chars (GTK_EDITABLE (GTK_COMBO (e->parameter_encoding_combo)->entry), 0, -1);
+	s = gtk_editable_get_chars (GTK_EDITABLE (GTK_BIN (e->parameter_encoding_combo)->child), 0, -1);
 	gb_smart_site_set_encoding (GB_SMART_SITE (p->bm), s);
+	gb_bookmark_set_time_modified_now (p->bm);
 	g_free (s);
 }
 
