@@ -23,10 +23,11 @@
 #include "bookmarks.h"
 #include "bookmarks-io.h"
 #include "bookmarks-toolbar-widgets.h"
-#include "bookmarks-default-icon-provider.h"
+#include "bookmarks-icon-provider.h"
 #include "bookmarks-util.h"
 #include "bookmarks-iterator.h"
 #include "galeon-marshal.h"
+#include "galeon-debug.h"
 #include "pixbuf-cache.h"
 #include "gul-general.h"
 #include "gul-string.h"
@@ -34,12 +35,10 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <string.h>
 #include "galeon-autocompletion-source.h"
+#include <time.h>
 
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
 
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x) 
-
 /**
  * Private functions, only availble from this file
  */
@@ -134,6 +133,7 @@ static void		gb_bookmark_set_fix_galeon1_mess_recursive 	(GbFolder *f);
 static void 		gb_bookmark_set_fix_galeon1_mess_item		(GbBookmark *b);
 static char *		gb_bookmark_set_fix_galeon1_mess_string		(const gchar *s);
 static gchar *		gb_smart_site_get_smarturl_only		(GbSmartSite *b);
+static void		gb_smart_site_set_smarturl_full		(GbSmartSite *b, const gchar *url);
 static gchar *		gb_smart_site_get_options		(GbSmartSite *b);
 static void		gb_smart_site_set_options		(GbSmartSite *b, gchar *options);
 static GSList *		gb_folder_list_children_reversed	(GbFolder *f);
@@ -147,7 +147,7 @@ static GbBookmark *	gb_folder_get_child_or_alias		(GbFolder *f, GbBookmark *c);
 
 static GbIconProvider *icon_provider = NULL;
 
-//#define DEBUG_REF
+/* #define DEBUG_REF */
 #ifdef DEBUG_REF
 static int live_bookmarks = 0;
 #endif
@@ -157,6 +157,7 @@ static int live_bookmarks = 0;
  */
 enum GbBookmarkSetSignalsEnum {
 	GB_BOOKMARK_SET_TOOLBAR,
+	GB_BOOKMARK_SET_CONTEXT_MENU,
 	GB_BOOKMARK_SET_LAST_SIGNAL
 };
 static gint GbBookmarkSetSignals[GB_BOOKMARK_SET_LAST_SIGNAL];
@@ -249,7 +250,7 @@ gb_bookmark_init_icons (void)
 {
 	if (icon_provider == NULL)
 	{
-		GbDefaultIconProvider *ip = gb_default_icon_provider_new ();
+		GbIconProvider *ip = gb_icon_provider_new ();
 		gb_system_set_icon_provider (GB_ICON_PROVIDER (ip));
 		g_object_unref (ip);
 	}
@@ -414,7 +415,7 @@ gb_bookmark_alias_create_impl_do (GbBookmark *b, GbAliasPlaceholder *ap,
 		char id[32];
 		do {
 			/* ID's have to begin with a Letter to be valid XML */
-			snprintf (id, sizeof(id), "id%lu", last_generated_id++);
+			g_snprintf (id, sizeof(id), "id%lu", last_generated_id++);
 			if (!b->set 
 			    || g_hash_table_lookup (b->set->id_to_bookmark, 
 						    id) == NULL)
@@ -458,9 +459,13 @@ gb_bookmark_alias_create_impl_do (GbBookmark *b, GbAliasPlaceholder *ap,
 	alias->notes = b->notes;
 
 	if (alias_data)
+	{
 		alias->add_to_context_menu = alias_data->add_to_context_menu;
+	}
 	else
+	{
 		alias->add_to_context_menu = b->add_to_context_menu;
+	}
 
 	alias->time_modified = b->time_modified;
 
@@ -520,8 +525,22 @@ gb_bookmark_set_set_impl (GbBookmark *b, GbBookmarkSet *set)
 
 	gb_bookmark_set_xbel_node (b, NULL);
 
+	if (oldset && b->add_to_context_menu)
+	{
+		g_slist_free (oldset->context_bookmarks);
+		oldset->context_bookmarks = NULL;
+		g_signal_emit (oldset, GbBookmarkSetSignals[GB_BOOKMARK_SET_CONTEXT_MENU], 0);
+	}
+
 	b->set = set;
 
+	if (set && b->add_to_context_menu)
+	{
+		g_slist_free (set->context_bookmarks);
+		set->context_bookmarks = NULL;
+		g_signal_emit (set, GbBookmarkSetSignals[GB_BOOKMARK_SET_CONTEXT_MENU], 0);
+	}
+
 	/* add to hashtables... */
 	if (b->set && b->id && !gb_bookmark_is_alias (b))
 	{
@@ -738,12 +757,21 @@ gb_bookmark_set_add_to_context_menu (GbBookmark *b, gboolean val)
 {
 	g_return_if_fail (GB_IS_BOOKMARK (b));
 
+	val = !!val;
+
 	if (b->add_to_context_menu != val)
 	{
 		b->add_to_context_menu = val;
 		
 		g_signal_emit (b, GbBookmarkSignals[GB_BOOKMARK_MODIFIED], 0);
 		gb_folder_emit_child_modified (b->parent, b);
+
+		if (b->set)
+		{
+			g_slist_free (b->set->context_bookmarks);
+			b->set->context_bookmarks = NULL;
+			g_signal_emit (b->set, GbBookmarkSetSignals[GB_BOOKMARK_SET_CONTEXT_MENU], 0);
+		}
 		
 		gb_bookmark_set_needs_saving (b);
 	}
@@ -762,6 +790,18 @@ gb_bookmark_set_time_added (GbBookmark *b, GTime val)
 	gb_bookmark_set_needs_saving (b);
 }
 
+static GTime 
+gb_system_get_current_time (void)
+{
+	return time (NULL);
+}
+
+void
+gb_bookmark_set_time_added_now (GbBookmark *b)
+{
+	gb_bookmark_set_time_added (b, gb_system_get_current_time ());
+}
+
 void
 gb_bookmark_set_time_modified (GbBookmark *b, GTime val)
 {
@@ -779,6 +819,12 @@ gb_bookmark_set_time_modified (GbBookmark *b, GTime val)
 
 }
 
+void
+gb_bookmark_set_time_modified_now (GbBookmark *b)
+{
+	gb_bookmark_set_time_modified (b, gb_system_get_current_time ());
+}
+
 void
 gb_bookmark_unparent (GbBookmark *b)
 {
@@ -826,7 +872,7 @@ gb_folder_save_descendants (GbFolder *f, GbFolder *unsafe)
 	GbBookmark *child;
 	g_return_if_fail (!gb_bookmark_is_alias (f));
 
-	DEBUG_MSG (("Trying to save descendants of %s not under %s\n", GB_BOOKMARK (f)->name, GB_BOOKMARK (unsafe)->name));
+	LOG ("Trying to save descendants of %s not under %s", GB_BOOKMARK (f)->name, GB_BOOKMARK (unsafe)->name);
 	
 	for (child = f->child; child; child = child->next)
 	{
@@ -835,13 +881,13 @@ gb_folder_save_descendants (GbFolder *f, GbFolder *unsafe)
 			GbBookmark *safe_child = gb_bookmark_get_parented_alias_not_under (child, unsafe);
 			if (safe_child)
 			{
-				DEBUG_MSG (("Found a safe calias of %s under %s\n", 
-					    child->name, GB_BOOKMARK (safe_child->parent)->name));
+				LOG ("Found a safe calias of %s under %s", 
+				     child->name, GB_BOOKMARK (safe_child->parent)->name);
 				gb_bookmark_alias_make_real (safe_child);
 			}
 			else 
 			{
-				DEBUG_MSG (("No safe parent found for %s\n", child->name));
+				LOG ("No safe parent found for %s", child->name);
 				if (GB_IS_FOLDER (child))
 				{
 					gb_folder_save_descendants (GB_FOLDER (child), unsafe);
@@ -873,12 +919,14 @@ gb_bookmark_unparent_safe (GbBookmark *b)
 
 		if (parented_alias)
 		{
-			DEBUG_MSG (("safe parent for unparented bm: %s\n", GB_BOOKMARK (parented_alias->parent)->name));
+			LOG ("safe parent for unparented bm: %s", 
+			     GB_BOOKMARK (parented_alias->parent)->name);
+
 			gb_bookmark_alias_make_real (parented_alias);
 		}
 		else
 		{
-			DEBUG_MSG (("no safe parent found\n"));
+			LOG ("no safe parent found");
 			if (GB_IS_FOLDER (b))
 			{
 				gb_folder_save_descendants (GB_FOLDER (b), GB_FOLDER (b));
@@ -1172,8 +1220,6 @@ static void
 gb_site_init (GbSite *b)
 {
 	b->url = g_strdup ("");
-
-	/* TODO */
 }
 
 static void
@@ -1332,7 +1378,7 @@ gb_site_set_url (GbSite *s, const char *val)
 		g_signal_emit (s, GbSiteSignals[GB_SITE_URL_MODIFIED], 0, s->url);
 		g_signal_emit (s, GbBookmarkSignals[GB_BOOKMARK_MODIFIED], 0);
 		gb_folder_emit_child_modified (b->parent, b);
-	} while ((s = (GbSite *) b = b->alias) != NULL);
+	} while ((s = (GbSite *) (b = b->alias)) != NULL);
 }
 
 void
@@ -1351,6 +1397,12 @@ gb_site_set_time_visited (GbSite *b, GTime val)
 	} while ((b = (GbSite *) ((GbBookmark *) b)->alias) != NULL);
 }
 
+void
+gb_site_set_time_visited_now (GbSite *b)
+{
+	gb_site_set_time_visited (b, gb_system_get_current_time ());
+}
+
 void
 gb_site_set_accel (GbSite *b, guint accel_key, guint accel_mods)
 {
@@ -1490,6 +1542,7 @@ gb_smart_site_finalize_impl (GObject *o)
 		g_free (s->smarturl);
 		g_slist_foreach (s->history, (GFunc) g_free, NULL);
 		g_slist_free (s->history);
+		g_free (s->entries_sizes);
 	}
 	gb_site_finalize_impl (o);
 }
@@ -1503,7 +1556,7 @@ gb_smart_site_new (GbBookmarkSet *set, const char *name,
 	b = g_object_new (GB_TYPE_SMART_SITE, NULL);
 	gb_bookmark_set_name ((GbBookmark *) b, name);
 	gb_site_set_url ((GbSite *) b, url);
-	gb_smart_site_set_smarturl (b, smarturl);
+	gb_smart_site_set_smarturl_full (b, smarturl);
 	gb_bookmark_set_set ((GbBookmark *) b, set);
 	return b;
 }
@@ -1515,8 +1568,8 @@ gb_smart_site_new_from_site (GbSite *site, const char *smarturl)
 	return NULL;
 }
 
-void
-gb_smart_site_set_smarturl (GbSmartSite *s, const gchar *val)
+static void
+gb_smart_site_set_smarturl_full (GbSmartSite *s, const gchar *val)
 {
 	gchar *newval;
 	GbBookmark *b = (GbBookmark *) s;
@@ -1535,7 +1588,7 @@ gb_smart_site_set_smarturl (GbSmartSite *s, const gchar *val)
 		s->smarturl = newval;
 		g_signal_emit (s, GbBookmarkSignals[GB_BOOKMARK_MODIFIED], 0);
 		gb_folder_emit_child_modified (b->parent, b);
-	} while ((s = (GbSmartSite *) b = b->alias) != NULL);
+	} while ((s = (GbSmartSite *) (b = b->alias)) != NULL);
 }
 
 static GbTbWidget *
@@ -1753,13 +1806,13 @@ gb_smart_site_set_options (GbSmartSite *b, gchar *options)
 	if (!closebrace) goto no_previous_options;
 
 	new_smarturl = g_strconcat ("{", options, "}", closebrace + 1, NULL);
-	gb_smart_site_set_smarturl (b, new_smarturl);
+	gb_smart_site_set_smarturl_full (b, new_smarturl);
 	g_free (new_smarturl);
 	return;
 
  no_previous_options:
 	new_smarturl = g_strconcat ("{", options, "}", b->smarturl, NULL);
-	gb_smart_site_set_smarturl (b, new_smarturl);
+	gb_smart_site_set_smarturl_full (b, new_smarturl);
 	g_free (new_smarturl);
 }
 
@@ -1799,6 +1852,27 @@ gb_smart_site_set_encoding (GbSmartSite *b, const gchar *encoding)
 	gb_smart_site_set_options (b, newoptions);
 }
 
+gchar *
+gb_smart_site_get_smarturl (GbSmartSite *b)
+{
+	return gb_smart_site_get_smarturl_only (b);
+}
+
+void
+gb_smart_site_set_smarturl (GbSmartSite *b, const gchar *url)
+{
+	gchar *options;
+	gchar *new_smarturl;
+
+	g_return_if_fail (GB_IS_SMART_SITE (b));
+	options = gb_smart_site_get_options (b);
+
+	new_smarturl = g_strconcat ("{", options, "}", url, NULL);
+	gb_smart_site_set_smarturl_full (b, new_smarturl);
+	g_free (new_smarturl);
+	g_free (options);
+}
+
 GSList *
 gb_smart_site_get_history (GbSmartSite *b, int entry_index)
 {
@@ -1842,7 +1916,7 @@ gb_smart_site_set_history (GbSmartSite *s, int entry_index, const GSList *histor
 		g_signal_emit (s, GbBookmarkSignals[GB_BOOKMARK_MODIFIED], 0);
 		g_signal_emit (s, GbSmartSiteSignals[GB_SMART_SITE_HISTORY_CHANGED], 0);
 		gb_folder_emit_child_modified (b->parent, b);
-	} while ((s = (GbSmartSite *) b = b->alias) != NULL);
+	} while ((s = (GbSmartSite *) (b = b->alias)) != NULL);
 }
 
 void
@@ -1854,7 +1928,7 @@ gb_smart_site_prepend_history (GbSmartSite *b, int entry_index, const gchar *ite
 	int items = 0;
 
 	g_return_if_fail (GB_IS_SMART_SITE (b));
-	DEBUG_MSG (("in gb_smart_site_prepend_history\n"));
+	LOG ("in gb_smart_site_prepend_history");
 
 	max_smart_site_history_items = (GB_BOOKMARK (b)->set) 
 		? GB_BOOKMARK (b)->set->max_smart_site_history_items
@@ -1878,7 +1952,7 @@ gb_smart_site_prepend_history (GbSmartSite *b, int entry_index, const gchar *ite
 	g_slist_foreach (newhistory, (GFunc) g_free, NULL);
 	g_slist_free (newhistory);
 
-	DEBUG_MSG (("%d history items\n", g_slist_length (b->history)));
+	LOG ("%d history items", g_slist_length (b->history));
 }
 
 /**
@@ -1979,10 +2053,12 @@ gb_folder_set_set_impl (GbBookmark *b, GbBookmarkSet *set)
 
 	/* set the set of the children too */
 	if (!gb_bookmark_is_alias (b))
+	{
 		for (bi = f->child; bi; bi = bi->next)
 		{
 			gb_bookmark_set_set (bi, set);
 		}
+	}
 }
 
 static GbBookmark *
@@ -2007,7 +2083,7 @@ gb_folder_dispose_impl (GObject *o)
 	GbBookmark *b = (GbBookmark *) o;
 	GbFolder *f = (GbFolder *) b;
 
-	DEBUG_MSG (("in gb_folder_dispose_impl\n"));
+	LOG ("in gb_folder_dispose_impl");
 
 	if (!gb_bookmark_is_alias (b))
 	{
@@ -2026,7 +2102,7 @@ gb_folder_dispose_impl (GObject *o)
 static void
 gb_folder_finalize_impl (GObject *o)
 {
-	DEBUG_MSG (("in gb_folder_finalize_impl\n"));
+	LOG ("in gb_folder_finalize_impl");
 
 	g_assert ((((GbFolder *) o)->child == NULL) || gb_bookmark_is_alias (o));
 	
@@ -2105,6 +2181,8 @@ gb_folder_set_create_toolbar (GbFolder *b, gboolean val)
 {
 	g_return_if_fail (GB_IS_FOLDER (b));
 
+	val = !!val;
+
 	if (b->create_toolbar != val)
 	{
 		GbBookmarkSet *set = GB_BOOKMARK (b)->set;
@@ -2132,7 +2210,7 @@ gb_folder_set_expanded (GbFolder *b, gboolean val)
 {
 	g_return_if_fail (GB_IS_FOLDER (b));
 
-	b->expanded = val;
+	b->expanded = !!val;
 
 	g_signal_emit (b, GbBookmarkSignals[GB_BOOKMARK_MODIFIED], 0);
 	gb_folder_emit_child_modified (((GbBookmark *) b)->parent, (GbBookmark *) b);
@@ -2785,7 +2863,7 @@ gb_alias_placeholder_resolve (GbAliasPlaceholder *ap)
 void
 gb_alias_placeholder_set_create_toolbar (GbAliasPlaceholder *ap, gboolean val)
 {
-	ap->create_toolbar = val;
+	ap->create_toolbar = !!val;
 
 	g_signal_emit (ap, GbBookmarkSignals[GB_BOOKMARK_MODIFIED], 0);
 	gb_folder_emit_child_modified (((GbBookmark *) ap)->parent, (GbBookmark *) ap);
@@ -2814,6 +2892,14 @@ gb_bookmark_set_class_init (GbBookmarkSetClass *klass)
 		galeon_marshal_VOID__OBJECT,
 		G_TYPE_NONE, 1, GB_TYPE_FOLDER);
 
+	GbBookmarkSetSignals[GB_BOOKMARK_SET_CONTEXT_MENU] = g_signal_new (
+		"context-menu", G_OBJECT_CLASS_TYPE (klass),  
+		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP,
+                G_STRUCT_OFFSET (GbBookmarkSetClass, gb_bookmark_set_context_menu), 
+		NULL, NULL, 
+		galeon_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
+
 	gb_bookmark_set_class = klass;
 }
 
@@ -2858,7 +2944,7 @@ gb_bookmark_set_dispose_impl (GObject *o)
 	g_print ("disposing the bookmarkset\n");
 #endif
 
-	DEBUG_MSG (("in gb_bookmark_set_dispose_impl\n"));
+	LOG ("in gb_bookmark_set_dispose_impl");
 
 	gb_bookmark_set_set (GB_BOOKMARK (set->root), NULL);	
 
@@ -2874,7 +2960,7 @@ gb_bookmark_set_finalize_impl (GObject *o)
 	g_print ("finalizing the bookmarkset\n");
 #endif
 
-	DEBUG_MSG (("in gb_bookmark_set_finalize_impl\n"));
+	LOG ("in gb_bookmark_set_finalize_impl");
 
 	gb_bookmark_set_set_xbel_doc (set, NULL);
 	
@@ -2896,10 +2982,8 @@ gb_bookmark_set_finalize_impl (GObject *o)
 	g_object_unref (G_OBJECT (set->root));
 
 	/* if everything is 0, then things work quite well */
-	DEBUG_MSG (("\t%d entries in url_to_bookmark\n"
-		    "\t%d entries in id_to_bookmark\n", 
-		    g_hash_table_size (set->url_to_bookmark), 
-		    g_hash_table_size (set->id_to_bookmark)));
+	LOG ("%d entries in url_to_bookmark", g_hash_table_size (set->url_to_bookmark));
+	LOG ("%d entries in id_to_bookmark", g_hash_table_size (set->id_to_bookmark));
 
 	g_hash_table_destroy (set->url_to_bookmark);
 #ifdef NICK_HASHTABLE
@@ -2910,8 +2994,8 @@ gb_bookmark_set_finalize_impl (GObject *o)
 	g_free (set->filename);
 
 	/* if everything is 0, then things work quite well */
-	DEBUG_MSG (("\t%d unresolved_aliases\n\t%d toolbars\n", 
-		    g_slist_length (set->unresolved_aliases), g_slist_length (set->toolbars)));
+	LOG ("%d unresolved_aliases", g_slist_length (set->unresolved_aliases));
+	LOG ("%d toolbars\n", g_slist_length (set->toolbars));
 
 	g_slist_free (set->unresolved_aliases);
 	g_slist_free (set->toolbars);
@@ -3085,11 +3169,12 @@ gb_bookmark_set_check_save (GbBookmarkSet *set)
 	{
 		if (set->filename && set->io)
 		{
-			gchar *tmpfilename = g_strdup_printf ("%s.tmp", set->filename);
+			gchar *tmpfilename = g_strconcat (set->filename, ".tmp", NULL);
 			gboolean ok = gb_io_save_to_file (set->io, set, tmpfilename);
 			if (ok)
 			{
 				gul_backup_file (set->filename, BOOKMARKS_NUM_BACKUPS, tmpfilename);
+				set->needs_saving = FALSE;
 			}
 			g_free (tmpfilename);
 			return ok;
@@ -3419,6 +3504,29 @@ gb_bookmark_set_get_all_noalias (GbBookmarkSet *set)
 	return set->all_bookmarks_noalias;
 }
 
+const GSList *
+gb_bookmark_set_get_context_bookmarks (GbBookmarkSet *set)
+{
+	if (!set->context_bookmarks)
+	{
+		const GSList *all = gb_bookmark_set_get_all (set);
+		const GSList *li;
+		GSList *t = NULL;
+		
+		for (li = all; li; li = li->next)
+		{
+			GbBookmark *bi = li->data;
+			if (bi->add_to_context_menu)
+			{
+				t = g_slist_prepend (t, bi);
+			}
+		}
+		
+		set->context_bookmarks = g_slist_reverse (t);
+	}
+	return set->context_bookmarks;
+}
+
 static void
 gb_bookmark_set_tree_changed (GbBookmarkSet *set)
 {
@@ -3432,4 +3540,9 @@ gb_bookmark_set_tree_changed (GbBookmarkSet *set)
 		g_slist_free (set->all_bookmarks);
 		set->all_bookmarks = NULL;
 	}
+	if (set->context_bookmarks)
+	{
+		g_slist_free (set->context_bookmarks);
+		set->context_bookmarks = NULL;
+	}
 }
