@@ -123,7 +123,7 @@ struct _GbBookmark
 	gchar *pixmap_file;
 
 	/* the in-memory binary data of the pixmap */
-	//GdkPixbuf *pixbuf_data;
+	/* GdkPixbuf *pixbuf_data; */
 
 	/* xbel node of the bookmark, may be NULL */
 	xmlNodePtr xbel_node;
@@ -132,7 +132,7 @@ struct _GbBookmark
 	gchar *notes; 
 
 	/* if TRUE, we add this bookmark to the document context menu */
-	gboolean add_to_context_menu : 1;
+	gboolean add_to_context_menu;
 	
 	/* time info (if they are 0, then they should be ignored) */
 	GTime time_added;
@@ -148,7 +148,9 @@ void 		gb_bookmark_set_notes	 		(GbBookmark *b, const gchar *val);
 void 		gb_bookmark_set_pixmap		 	(GbBookmark *b, const gchar *val);
 void 		gb_bookmark_set_add_to_context_menu	(GbBookmark *b, gboolean val);
 void 		gb_bookmark_set_time_added	 	(GbBookmark *b, GTime val);
+void 		gb_bookmark_set_time_added_now	 	(GbBookmark *b);
 void 		gb_bookmark_set_time_modified 		(GbBookmark *b, GTime val);
+void 		gb_bookmark_set_time_modified_now	(GbBookmark *b);
 void 		gb_bookmark_unparent	 		(GbBookmark *b);
 void		gb_bookmark_unparent_safe		(GbBookmark *b);
 GbBookmark *	gb_bookmark_copy			(GbBookmark *b);
@@ -205,6 +207,7 @@ GbSite *	gb_site_new			(GbBookmarkSet *set, const char *name,
 						 const char *url);
 void 		gb_site_set_url			(GbSite *b, const char *val);
 void		gb_site_set_time_visited 	(GbSite *b, GTime val);
+void		gb_site_set_time_visited_now	(GbSite *b);
 void		gb_site_set_accel		(GbSite *b, guint accel_key, guint accel_mods);
 
 /**
@@ -235,10 +238,10 @@ struct _GbSmartSite
 {
 	GbSite parent_object;
 	/* smart url for smart bookmark */
-	gchar *smarturl;
 	gboolean folded;
 
 	/* private */
+	gchar *smarturl;
 	gint *entries_sizes;
 	GSList *history;
 };
@@ -248,6 +251,7 @@ GbSmartSite *	gb_smart_site_new		(GbBookmarkSet *set, const char *name,
 						 const char *url, const char *smarturl);
 GbSmartSite *	gb_smart_site_new_from_site	(GbSite *site, const char *smarturl);
 void 		gb_smart_site_set_smarturl	(GbSmartSite *b, const gchar *val);
+gchar *		gb_smart_site_get_smarturl	(GbSmartSite *b);
 gint 		gb_smart_site_get_num_fields	(GbSmartSite *b);
 gint		gb_smart_site_get_entry_size	(GbSmartSite *b, int index);
 void		gb_smart_site_set_entry_size	(GbSmartSite *b, gint index, gint width);
@@ -297,20 +301,20 @@ struct _GbFolder
 	GbBookmark *child;
 
 	/* if TRUE create a toolbar from it */
-	gboolean create_toolbar : 1; 
+	gboolean create_toolbar; 
 
 	/* if TRUE then it was shown open in the editor last time */
-	gboolean expanded : 1; 
+	gboolean expanded; 
 
 	/* toolbar style (FIXME: does this need to be here?) */ 
-	//GtkToolbarStyle toolbar_style;
+	/* GtkToolbarStyle toolbar_style; */
 };
 
 GType 		gb_folder_get_type			(void);
 GbFolder *	gb_folder_new				(GbBookmarkSet *set, const gchar *name);
 void 		gb_folder_set_create_toolbar		(GbFolder *b, gboolean val);
 void		gb_folder_set_expanded			(GbFolder *b, gboolean val);
-//void		gb_folder_set_toolbar_style	 	(BookmarkItem *b, GtkToolbarStyle val);
+/* void		gb_folder_set_toolbar_style	 	(BookmarkItem *b, GtkToolbarStyle val); */
 void		gb_folder_add_child			(GbFolder *p, GbBookmark *c, gint position);
 void		gb_folder_move_child	 			(GbFolder *p, GbBookmark *c, gint position);
 gint		gb_folder_get_child_index		(GbFolder *p, GbBookmark *b);
@@ -409,7 +413,7 @@ struct _GbAliasPlaceholder
 	/* data that may be different betwen a bookmark and it's alias and it's 
 	   not in GbBookmark */
 	
-	gboolean create_toolbar : 1; 
+	gboolean create_toolbar; 
 
 	
 };
@@ -438,7 +442,8 @@ struct _GbBookmarkSetClass
 	GObjectClass parent_class;
 
 	/* Signals */
-	void		(*gb_bookmark_set_toolbar)		(GbFolder *f);
+	void		(*gb_bookmark_set_toolbar)		(GbBookmarkSet *set, GbFolder *f);
+	void		(*gb_bookmark_set_context_menu)		(GbBookmarkSet *set);
 };
 
 typedef enum
@@ -470,6 +475,7 @@ struct _GbBookmarkSet
 	/* all bookmarks in preorder, or NULL */
 	GSList *all_bookmarks;
 	GSList *all_bookmarks_noalias;
+	GSList *context_bookmarks;
 
 	GbIO *io;
 	gchar *filename;
@@ -491,6 +497,7 @@ void		gb_bookmark_set_set_root		(GbBookmarkSet *set, GbFolder *root);
 void		gb_bookmark_set_set_default_folder	(GbBookmarkSet *set, GbFolder *default_folder);
 void		gb_bookmark_set_resolve_aliases		(GbBookmarkSet *set);
 GSList *	gb_bookmark_set_get_toolbars	 	(GbBookmarkSet *set);
+const GSList *	gb_bookmark_set_get_context_bookmarks 	(GbBookmarkSet *set);
 void		gb_bookmark_set_set_needs_saving	(GbBookmarkSet *set, gboolean needs_saving);
 void		gb_bookmark_set_set_auto_save		(GbBookmarkSet *set, guint interval);
 void		gb_bookmark_set_add_default		(GbBookmarkSet *set, GbBookmark *b);
