@@ -20,7 +20,6 @@
 #endif
 
 #include "downloader-view.h"
-#include "disclosure-widget.h"
 #include "eel-gconf-extensions.h"
 #include "gul-gui.h"
 #include "prefs-strings.h"
@@ -37,11 +36,16 @@
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkprogressbar.h>
+#include <gtk/gtkexpander.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-dialog-util.h>
 #include <libgnomeui/gnome-dialog.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+
+#define KILOBYTE_FACTOR (1024.0)
+#define MEGABYTE_FACTOR (1024.0 * 1024.0)
 
 /* WHACK WHACK WHACK FIXME please
  * (nevermind the type mismatch, we'd typecast it away anyway)
@@ -50,26 +54,17 @@
  */
 extern GObject *galeon_shell;
 
-enum
-{
-	DOWNLOAD_REMOVE,
-	DOWNLOAD_PAUSE,
-	DOWNLOAD_RESUME,
-        LAST_SIGNAL
-};
-
 enum
 {
 	COL_PERCENT,
 	COL_FILENAME,
 	COL_SIZE,
 	COL_REMAINING,
-	COL_PERSIST_OBJECT
+	COL_DETAILS
 };
 
 struct DownloaderViewPrivate
 {
-	GHashTable *details_hash;
 	GtkTreeModel *model;
 	gboolean show_details;
 
@@ -89,22 +84,129 @@ struct DownloaderViewPrivate
 	GtkWidget *open_button;
 };
 
-typedef struct
+#define DOWNLOAD_TYPE	(download_get_type ())
+
+typedef enum
 {
-	glong elapsed;
-	glong remaining;
-	gfloat speed;
-	gint size_total;
-	gint size_done;
-	gfloat progress;
+	DOWNLOAD_STATUS_DOWNLOADING,
+	DOWNLOAD_STATUS_PAUSED,
+	DOWNLOAD_STATUS_RESUMING,
+	DOWNLOAD_STATUS_COMPLETED,
+	DOWNLOAD_STATUS_CANCELLED
+} DownloadStatus;
+
+#define STATUS_IS_UNFINISHED(status) \
+	((status) != DOWNLOAD_STATUS_COMPLETED && \
+	 (status) != DOWNLOAD_STATUS_CANCELLED)
+
+typedef struct _Download DownloadDetails;
+struct _Download
+{
+	GObject parent;
+
+	DownloaderView *view;
+	GtkTreeRowReference *ref;
+
+	guint  elapsed;		/* seconds */
+	guint  remaining;	/* seconds (G_MAXUINT if indeterminate) */
+	guint  speed;		/* bytes per second */
+	gulong size_total;	/* bytes (G_MAXULONG if indeterminate) */
+	gulong size_done;	/* bytes */
+	gfloat progress;	/* 0.0 .. 1.0 (or < 0 if indeterminate) */
+
 	gboolean can_pause;
 	gchar *filename;
 	gchar *source;
 	gchar *dest;
 	DownloadStatus status;
-	
-	GtkTreeRowReference *ref;
-} DownloadDetails;
+};
+
+enum {
+	CANCEL,
+	PAUSE,
+	RESUME,
+        LAST_SIGNAL
+};
+
+static GObjectClass *download_parent_class = NULL;
+static guint download_signals[LAST_SIGNAL] = { 0 };
+
+static void
+download_init (Download *self)
+{
+}
+
+static void
+download_finalize (GObject *object)
+{
+	Download *self = (Download *) object;
+
+	g_free (self->filename);
+	g_free (self->source);
+	g_free (self->dest);
+}
+
+static void
+download_class_init (DownloadClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS(klass);
+
+	download_parent_class = g_type_class_peek_parent (klass);
+
+	object_class->finalize = download_finalize;
+
+        /* signals */
+        download_signals[CANCEL] =
+                g_signal_new ("cancel",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (DownloadClass, cancel),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE, 0);
+	download_signals[PAUSE] =
+                g_signal_new ("pause",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (DownloadClass, pause),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE, 0);
+	download_signals[RESUME] =
+                g_signal_new ("resume",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (DownloadClass, resume),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE, 0);
+}
+
+static GType
+download_get_type (void)
+{
+	static GType type = 0;
+
+	if (type == 0)
+	{
+		static const GTypeInfo info =
+		{
+			sizeof (DownloadClass),
+			NULL, /* base_init */
+			NULL, /* base_finalize */
+			(GClassInitFunc) download_class_init,
+			NULL, /* class_finalize */
+			NULL, /* class_data */
+			sizeof (Download),
+			0,    /* n_preallocs */
+			(GInstanceInitFunc) download_init
+		};
+
+		type = g_type_register_static (G_TYPE_OBJECT, "Download", &info, 0);
+	}
+
+	return type;
+}
 
 typedef struct
 {
@@ -149,7 +251,7 @@ GaleonDialogProperty properties [] =
 	{ PROP_RESUME_BUTTON, "resume_button", NULL, PT_NORMAL, NULL },
 	{ PROP_ABORT_BUTTON, "abort_button", NULL, PT_NORMAL, NULL },
 	{ PROP_OPEN_BUTTON, "open_button", NULL, PT_NORMAL, NULL },
-	{ PROP_DETAILS_BUTTON, "details_togglebutton", CONF_DOWNLOADING_SHOW_DETAILS, PT_NORMAL, NULL },
+	{ PROP_DETAILS_BUTTON, "details_expander", CONF_DOWNLOADING_SHOW_DETAILS, PT_NORMAL, NULL },
 	
         { -1, NULL, NULL }
 };
@@ -177,14 +279,14 @@ gboolean
 download_dialog_delete_cb (GtkWidget *window, GdkEventAny *event,
 			   DownloaderView *dv);
 void
-download_dialog_details_cb (GtkToggleButton *button, 
-			    DownloaderView *dv);
+download_dialog_details_cb(GtkExpander *expander, 
+			   GParamSpec *pspec,
+			   DownloaderView *dv);
 void
 download_dialog_open_cb (GtkWidget *button, 
 			 DownloaderView *dv);
 
 static GObjectClass *parent_class = NULL;
-static guint downloader_view_signals[LAST_SIGNAL] = { 0 };
 
 GType
 downloader_view_get_type (void)
@@ -245,55 +347,22 @@ downloader_view_class_init (DownloaderViewClass *klass)
         galeon_dialog_class = GALEON_DIALOG_CLASS (klass);
 
         object_class->finalize = downloader_view_finalize;
-
-        /* init signals */
-        downloader_view_signals[DOWNLOAD_REMOVE] =
-                g_signal_new ("download_remove",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (DownloaderViewClass, download_remove),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
-	downloader_view_signals[DOWNLOAD_PAUSE] =
-                g_signal_new ("download_pause",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (DownloaderViewClass, download_pause),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
-	downloader_view_signals[DOWNLOAD_RESUME] =
-                g_signal_new ("download_resume",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (DownloaderViewClass, download_resume),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
-
 }
 
-static void 
-destroy_details_cb (DownloadDetails *details)
+static gboolean
+destroy_details_foreach (GtkTreeModel *model,
+		         GtkTreePath  *path,
+			 GtkTreeIter  *iter,
+			 gpointer      user_data)
 {
-	g_free (details->filename);
-	g_free (details->source);
-	g_free (details->dest);
-	g_free (details);
+	gtk_list_store_set (GTK_LIST_STORE(model), iter, COL_DETAILS, NULL, -1);
+	return FALSE;
 }
 
 static void
 downloader_view_init (DownloaderView *dv)
 {
         dv->priv = g_new0 (DownloaderViewPrivate, 1);
-	dv->priv->details_hash = g_hash_table_new_full (g_direct_hash, 
-						   	g_direct_equal,
-						   	NULL,
-							(GDestroyNotify)destroy_details_cb);
 	
 	downloader_view_build_ui (dv);
 
@@ -314,7 +383,7 @@ downloader_view_finalize (GObject *object)
 
         g_return_if_fail (dv->priv != NULL);
 
-	g_hash_table_destroy (dv->priv->details_hash);
+	gtk_tree_model_foreach (dv->priv->model, destroy_details_foreach, NULL);
 
 	g_free (dv->priv);
 
@@ -338,19 +407,15 @@ controls_info_foreach (GtkTreeModel *model,
 		       ControlsInfo *info)
 {
 	DownloadDetails *details;
-	GValue val = {0, };
-	gpointer persist_object;
 	
-	gtk_tree_model_get_value (model, iter, COL_PERSIST_OBJECT, &val);
-	persist_object = g_value_get_pointer (&val);
+	gtk_tree_model_get (model, iter, COL_DETAILS, &details, -1);
 
-	details = g_hash_table_lookup (info->priv->details_hash,
-				       persist_object);
-
-	info->can_pause |= details->can_pause;
+	info->can_abort  |= STATUS_IS_UNFINISHED(details->status);
+	info->can_pause  |= (details->status == DOWNLOAD_STATUS_DOWNLOADING && details->can_pause);
 	info->can_resume |= (details->status == DOWNLOAD_STATUS_PAUSED);
-	info->can_abort |= (details->status != DOWNLOAD_STATUS_COMPLETED);
-	info->can_open |= (details->status == DOWNLOAD_STATUS_COMPLETED);
+	info->can_open   |= (details->status == DOWNLOAD_STATUS_COMPLETED);
+
+	g_object_unref (details);
 }
 
 static void
@@ -385,11 +450,74 @@ downloader_view_update_controls (DownloaderViewPrivate *priv)
 	g_free (info);
 }
 
+static char *
+format_progress (gulong done, gulong total)
+{
+	if (total < KILOBYTE_FACTOR)
+		return g_strdup_printf (_("%u of %u bytes"),
+				        (guint)done, (guint)total);
+	else if (total < MEGABYTE_FACTOR)
+		return g_strdup_printf (_("%.1f of %.1f KB"),
+				        done/KILOBYTE_FACTOR,
+					total/KILOBYTE_FACTOR);
+	else
+		return g_strdup_printf (_("%.1f of %.1f MB"),
+				        done/MEGABYTE_FACTOR,
+					total/MEGABYTE_FACTOR);
+}
+
+static void
+format_speed (char *buf, gulong len, guint speed)
+{
+	if (speed < MEGABYTE_FACTOR)
+	{
+		g_snprintf (buf, len, _("%.1f KB/s"), speed/KILOBYTE_FACTOR);
+	}
+	else
+	{
+		g_snprintf (buf, len, _("%.1f MB/s"), speed/MEGABYTE_FACTOR);
+	}
+}
+
+static char *
+format_status (guint speed, gulong size_done, gulong size_total)
+{
+	char *status;
+
+	if (size_total < G_MAXULONG)
+	{
+		status = format_progress (size_done, size_total);
+	}
+	else
+	{
+		status = gnome_vfs_format_file_size_for_display (size_done);
+	}
+
+	if (speed > 0)
+	{
+		char spd[16];
+		char *tmp;
+
+		format_speed (spd, sizeof(spd), speed);
+
+		/* To translators: the following is for download progress, for
+		 * example: "1.0 of 4.2 MB at 42.0 KB/s" or
+		 * "1.2 MB at 14.6 KB/s"
+		 */
+		tmp = g_strdup_printf (_("%s at %s"), status, spd);
+		g_free (status);
+		status = tmp;
+	}
+
+	return status;
+}
+
 static void
 downloader_view_update_details (DownloaderViewPrivate *priv, 
 				DownloadDetails *details)
 {
 	gchar buffer[50];
+	char *status;
 
 	gul_ellipsizing_label_set_text 
 		(GUL_ELLIPSIZING_LABEL (priv->details_location),
@@ -398,38 +526,22 @@ downloader_view_update_details (DownloaderViewPrivate *priv,
 		(GUL_ELLIPSIZING_LABEL (priv->details_file),
 		 details->filename);
 
-	if (details->size_total >= 10000)
+	status = format_status (details->speed, details->size_done, details->size_total);
+	gtk_label_set_text (GTK_LABEL (priv->details_status), status);
+	g_free (status);
+
+	format_time (buffer, details->elapsed);
+	gtk_label_set_text (GTK_LABEL (priv->details_elapsed), buffer);
+
+	if (details->remaining < G_MAXUINT)
 	{
-		sprintf (buffer, _("%.1f of %.1f MB"),
-			 details->size_done / 1024.0,
-			 details->size_total / 1024.0);
-	}
-	else if (details->size_total > 0)
-	{
-		sprintf (buffer, _("%d of %d KB"),
-			 details->size_done,
-			 details->size_total);
+		format_time (buffer, details->remaining);
 	}
 	else
 	{
-		sprintf (buffer, _("%d KB"),
-			 details->size_done);
+		sprintf (buffer, _("Unknown"));
 	}
-
-	if (details->speed > 0)
-	{
-		sprintf (buffer, "%s at %.1f KB/s", buffer, details->speed);
-	}
-	gtk_label_set_text (GTK_LABEL (priv->details_status),
-			    buffer);
-
-	format_time (buffer, details->elapsed/1000000);
-	gtk_label_set_text (GTK_LABEL (priv->details_elapsed),
-			    buffer);
-
-	format_time (buffer, details->remaining);
-	gtk_label_set_text (GTK_LABEL (priv->details_remaining),
-			    buffer);
+	gtk_label_set_text (GTK_LABEL (priv->details_remaining), buffer);
 
 	if (details->progress >= 0)
 	{
@@ -471,6 +583,7 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 				   GtkTreeIter *iter)
 {
 	gchar buffer[50];
+	char *size;
 	GtkTreePath *path;
 	GtkTreePath *selected_path = NULL;
 	GtkTreeIter selected_iter;
@@ -493,38 +606,36 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 			    -1);
 
 	/* Progress */
-	if (details->status == DOWNLOAD_STATUS_COMPLETED)
-		details->progress = 1;
-	sprintf (buffer, "%.1f%%", 
-		 details->progress > 0 ? 
-		 details->progress * 100.0 :
-		 0);
+	if (details->progress >= 0)
+	{
+		sprintf (buffer, "%.1f%%", details->progress * 100.0);
+	}
+	else
+	{
+		sprintf (buffer, "---");
+	}
 	gtk_list_store_set (GTK_LIST_STORE (priv->model),
 			    iter,
 			    COL_PERCENT, buffer,
 			    -1);
 	
 	/* Total */
-	if (details->size_total >= 10000)
+	if (details->size_total < G_MAXULONG)
 	{
-		sprintf (buffer, "%.2f MB", details->size_total / 1024.0);
-	}
-	else if (details->size_total > 0)
-	{
-		sprintf (buffer, "%d KB", details->size_total);
+		size = gnome_vfs_format_file_size_for_display (details->size_total);
 	}
 	else
 	{
-		sprintf (buffer, _("Unknown"));
+		size = g_strdup (_("Unknown")); 
 	}
-
 	gtk_list_store_set (GTK_LIST_STORE (priv->model),
 			    iter,
-			    COL_SIZE, buffer,
+			    COL_SIZE, size,
 			    -1);
+	g_free (size);
 
 	/* Remaining */
-	if (details->remaining >= 0)
+	if (details->remaining < G_MAXUINT)
 	{
 		format_time (buffer, details->remaining);
 	}
@@ -567,34 +678,29 @@ ensure_selected_row (DownloaderView *dv)
 	}
 }
 
-void
+Download *
 downloader_view_add_download (DownloaderView *dv,
-			      gchar *filename,
-			      gchar *source,
-			      gchar *dest,
-			      gpointer persist_object)
+			      const char *source,
+			      const char *dest)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
 	DownloadDetails *details;
 	GtkTreeSelection *selection;
 	
-	details = g_new0 (DownloadDetails, 1);
-	details->filename = g_strdup (filename);
+	details = g_object_new (DOWNLOAD_TYPE, NULL);
+	details->view = dv;
+	details->filename = g_path_get_basename (dest);
 	details->source = g_strdup (source);
 	details->dest = g_strdup (dest);
-	details->elapsed = -1;
-	details->remaining = -1;
-	details->speed = -1;
-	details->size_total = -1;
-	details->size_done = -1;
+	details->elapsed = 0;
+	details->remaining = G_MAXUINT;
+	details->speed = 0;
+	details->size_total = G_MAXULONG;
+	details->size_done = 0;
 	details->progress = -1;
 	dv->priv->show_details = FALSE;
 
-	g_hash_table_insert (dv->priv->details_hash,
-			     persist_object,
-			     details);
-
 	gtk_list_store_append (GTK_LIST_STORE (dv->priv->model), 
 			       &iter);
 
@@ -605,7 +711,7 @@ downloader_view_add_download (DownloaderView *dv,
 
 	gtk_list_store_set (GTK_LIST_STORE (dv->priv->model),
 			    &iter,
-			    COL_PERSIST_OBJECT, persist_object,
+			    COL_DETAILS, details,
 			    -1);
 
 	selection = gtk_tree_view_get_selection 
@@ -616,19 +722,19 @@ downloader_view_add_download (DownloaderView *dv,
 	downloader_view_set_download_info (dv->priv, details, &iter);
 
 	galeon_dialog_show (GALEON_DIALOG(dv));
+
+	g_object_unref (details); /* the model holds our reference(??) */
+	return details;
 }
 
-void
+static void
 downloader_view_remove_download (DownloaderView *dv,
-				 gpointer persist_object)
+				 DownloadDetails *details)
 {
-	DownloadDetails *details;
 	GtkTreeIter iter;
 	GValue keep_open = {0, };
 	GtkTreePath *path;
 
-	details = g_hash_table_lookup (dv->priv->details_hash,
-				       persist_object);
 	g_return_if_fail (details);
 
 	path = gtk_tree_row_reference_get_path (details->ref);
@@ -638,13 +744,10 @@ downloader_view_remove_download (DownloaderView *dv,
 
 	gtk_list_store_remove (GTK_LIST_STORE (dv->priv->model), &iter);
 
-	g_hash_table_remove (dv->priv->details_hash,
-			     persist_object);
-
         galeon_dialog_get_value (GALEON_DIALOG(dv), PROP_KEEP_OPEN, &keep_open);
 	
 	if (!g_value_get_boolean (&keep_open) &&
-	    g_hash_table_size (dv->priv->details_hash) == 0)
+	    !gtk_tree_model_get_iter_first (dv->priv->model, &iter))
 	{
 		g_object_unref (dv);
 	}
@@ -654,32 +757,41 @@ downloader_view_remove_download (DownloaderView *dv,
 	}
 }
 
-void 
-downloader_view_set_download_progress (DownloaderView *dv,
-				       glong elapsed,
-				       glong remaining,
-				       gfloat speed,
-				       gint size_total,
-				       gint size_done,
-				       gfloat progress,
-				       gboolean can_pause,
-				       gpointer persist_object)
+void
+download_set_progress (Download	*details,
+		       guint	 elapsed,
+		       guint	 remaining,
+		       guint	 speed,
+		       gulong	 size_done,
+		       gulong	 size_total)
 {
-	DownloadDetails *details;
-	GtkTreePath *path;
-	GtkTreeIter iter;
-
-	details = g_hash_table_lookup (dv->priv->details_hash,
-				       persist_object);
-	g_return_if_fail (details);
-
 	details->elapsed = elapsed;
 	details->remaining = remaining;
 	details->speed = speed;
-	details->size_total = size_total;
 	details->size_done = size_done;
-	details->can_pause = can_pause;
-	details->progress = progress;
+
+	if (details->status == DOWNLOAD_STATUS_RESUMING)
+	{
+		details->status = DOWNLOAD_STATUS_DOWNLOADING;
+	}
+
+	download_set_size_total (details, size_total);
+}
+
+void
+download_set_size_total (DownloadDetails *details, gulong size_total)
+{
+	DownloaderView *dv;
+	GtkTreePath *path;
+	GtkTreeIter  iter;
+
+	details->size_total = size_total;
+	if (size_total > 0 && size_total < G_MAXULONG)
+	{
+		details->progress = (gfloat)details->size_done / size_total;
+	}
+
+	dv = details->view;
 
 	path = gtk_tree_row_reference_get_path (details->ref);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model), 
@@ -690,18 +802,22 @@ downloader_view_set_download_progress (DownloaderView *dv,
 }
 
 void
-downloader_view_set_download_status (DownloaderView *dv,
-				     DownloadStatus status,
-				     gpointer persist_object)
+download_set_can_pause (DownloadDetails *details, gboolean can_pause)
 {
-	DownloadDetails *details;
+	details->can_pause = can_pause;
+	downloader_view_update_controls (details->view->priv);
+}
+
+static void
+download_set_status (DownloadDetails *details, DownloadStatus status)
+{
+	DownloaderView *dv;
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	GValue keep_open = {0, };
 
-	details = g_hash_table_lookup (dv->priv->details_hash,
-				       persist_object);
 	g_return_if_fail (details);
+	dv = details->view;
 
 	details->status = status;
 
@@ -715,13 +831,49 @@ downloader_view_set_download_status (DownloaderView *dv,
 		
         galeon_dialog_get_value (GALEON_DIALOG(dv), PROP_KEEP_OPEN, &keep_open);
 	
-	if (status == DOWNLOAD_STATUS_COMPLETED &&
+	if (!STATUS_IS_UNFINISHED(status) &&
 	    !g_value_get_boolean (&keep_open))
 	{
-		downloader_view_remove_download (dv, persist_object);
+		downloader_view_remove_download (dv, details);
 	}
 } 
 
+void
+download_paused (Download *self)
+{
+	download_set_status (self, DOWNLOAD_STATUS_PAUSED);
+}
+
+void
+download_resumed (Download *self)
+{
+	download_set_status (self, DOWNLOAD_STATUS_RESUMING);
+}
+
+void
+download_completed (Download *self)
+{
+	if (self->size_total < G_MAXULONG)
+	{
+		self->size_done = self->size_total;
+	}
+	else
+	{
+		self->size_total = self->size_done;
+	}
+	self->remaining = 0;
+	self->progress = 1;
+
+	download_set_status (self, DOWNLOAD_STATUS_COMPLETED);
+}
+
+void
+download_cancelled (Download *self)
+{
+	self->remaining = 0;
+	download_set_status (self, DOWNLOAD_STATUS_CANCELLED);
+}
+
 static void
 downloader_view_build_ui (DownloaderView *dv)
 {
@@ -753,13 +905,7 @@ downloader_view_build_ui (DownloaderView *dv)
 	details_table = galeon_dialog_get_control (d, PROP_DETAILS_TABLE);
 
 	/* Set inital visibility state of details */
-	download_dialog_details_cb(GTK_TOGGLE_BUTTON(priv->details_button), dv);
-
-	/* create file and location details labels */
-	cddb_disclosure_set_labels (CDDB_DISCLOSURE (priv->details_button),
-				    _("Download Details"),
-				    _("Download Details"));
-
+	download_dialog_details_cb (GTK_EXPANDER (priv->details_button), NULL, dv);
 
 	priv->details_location = gul_ellipsizing_label_new ("");
 	priv->details_file = gul_ellipsizing_label_new ("");
@@ -779,7 +925,7 @@ downloader_view_build_ui (DownloaderView *dv)
 					G_TYPE_STRING,
 					G_TYPE_STRING,
 					G_TYPE_STRING,
-					G_TYPE_POINTER);
+					DOWNLOAD_TYPE);
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW(priv->treeview), 
 				 GTK_TREE_MODEL (liststore));
@@ -844,21 +990,15 @@ resume_selection_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *i
 			  DownloaderView *dv)
 {
 	DownloadDetails *details;
-	GValue val = {0, };
-	gpointer *persist_object;
 	
-	gtk_tree_model_get_value (model, iter, COL_PERSIST_OBJECT, &val);
-	persist_object = g_value_get_pointer (&val);
-
-	details = g_hash_table_lookup (dv->priv->details_hash,
-				       persist_object);
+	gtk_tree_model_get (model, iter, COL_DETAILS, &details, -1);
 	g_return_if_fail (details);
 
-	if (details->status == DOWNLOAD_STATUS_COMPLETED) return;
-
-	downloader_view_set_download_status (dv, DOWNLOAD_STATUS_RESUMING, persist_object);
-
-	g_signal_emit (G_OBJECT (dv), downloader_view_signals[DOWNLOAD_RESUME], 0);
+	if (details->status == DOWNLOAD_STATUS_PAUSED)
+	{
+		g_signal_emit (G_OBJECT (details), download_signals[RESUME], 0);
+	}
+	g_object_unref (details);
 }
 
 void 
@@ -879,21 +1019,15 @@ pause_selection_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *it
 			 DownloaderView *dv)
 {
 	DownloadDetails *details;
-	GValue val = {0, };
-	gpointer *persist_object;
 	
-	gtk_tree_model_get_value (model, iter, COL_PERSIST_OBJECT, &val);
-	persist_object = g_value_get_pointer (&val);
-
-	details = g_hash_table_lookup (dv->priv->details_hash,
-				       persist_object);
+	gtk_tree_model_get (model, iter, COL_DETAILS, &details, -1);
 	g_return_if_fail (details);
 	
-	if (details->status == DOWNLOAD_STATUS_COMPLETED) return;
-
-	downloader_view_set_download_status (dv, DOWNLOAD_STATUS_PAUSED, persist_object);
-
-	g_signal_emit (G_OBJECT (dv), downloader_view_signals[DOWNLOAD_PAUSE], 0);
+	if (details->status == DOWNLOAD_STATUS_DOWNLOADING && details->can_pause)
+	{
+		g_signal_emit (G_OBJECT (details), download_signals[PAUSE], 0);
+	}
+	g_object_unref (details);
 }
 
 void 
@@ -919,8 +1053,6 @@ download_dialog_abort_cb (GtkButton *button, DownloaderView *dv)
         for (l = refs; l != NULL; l = l->next)
         {
                 GtkTreeRowReference *node = l->data;
-		GValue val = {0, };
-		gpointer *persist_object;
 		GtkTreeIter iter;
 		GtkTreePath *path;
 		DownloadDetails *details;
@@ -930,17 +1062,12 @@ download_dialog_abort_cb (GtkButton *button, DownloaderView *dv)
 					 path);
 		gtk_tree_path_free (path);
 	
-		gtk_tree_model_get_value (dv->priv->model, &iter, 
-					  COL_PERSIST_OBJECT, &val);
-		persist_object = g_value_get_pointer (&val);
-
-		details = g_hash_table_lookup (dv->priv->details_hash,
-					       persist_object);
+		gtk_tree_model_get (dv->priv->model, &iter, COL_DETAILS, &details, -1);
 		g_return_if_fail (details);
 				
-		g_signal_emit (G_OBJECT (dv), downloader_view_signals[DOWNLOAD_REMOVE], 0);
+		g_signal_emit (G_OBJECT (details), download_signals[CANCEL], 0);
 		
-		downloader_view_remove_download (dv, persist_object);
+		g_object_unref (details);
         }
 
 	gul_gui_treeview_free_selection_refs (refs);
@@ -951,8 +1078,6 @@ downloader_treeview_selection_changed_cb (GtkTreeSelection *selection,
 					  DownloaderView *dv)
 {
 	GtkTreeIter iter;
-	GValue val = {0, };
-	gpointer *persist_object;
 	DownloadDetails *details = NULL;
 	GtkWidget *details_frame;
 	DownloaderViewPrivate *priv= dv->priv;
@@ -962,17 +1087,14 @@ downloader_treeview_selection_changed_cb (GtkTreeSelection *selection,
 	
 	if (get_selected_row (priv, &iter))
 	{
-		gtk_tree_model_get_value (priv->model, &iter, COL_PERSIST_OBJECT, &val);
-		persist_object = g_value_get_pointer (&val);
-
-		details = g_hash_table_lookup (priv->details_hash,
-					       persist_object);
+		gtk_tree_model_get (priv->model, &iter, COL_DETAILS, &details, -1);
 		g_return_if_fail (details);
 
 		gtk_widget_set_sensitive (details_frame, TRUE);
 
 		downloader_view_update_details (priv, details);
 		downloader_view_update_controls (priv);
+		g_object_unref (details);
 	}
 	else
 	{
@@ -989,26 +1111,44 @@ downloader_treeview_selection_changed_cb (GtkTreeSelection *selection,
 	}
 }
 
-static void
-alive_download_foreach (gpointer persist_object,
-			DownloadDetails *details,
-			gboolean *alive)
+static gboolean
+alive_download_foreach (GtkTreeModel *model,
+		        GtkTreePath  *path,
+			GtkTreeIter  *iter,
+			gpointer      user_data)
 {
-	if (details->status != DOWNLOAD_STATUS_COMPLETED)
+	gboolean        *alive = user_data;
+	DownloadDetails *details;
+	
+	gtk_tree_model_get (model, iter, COL_DETAILS, &details, -1);
+	g_return_val_if_fail (details != NULL, TRUE);
+
+	if (STATUS_IS_UNFINISHED(details->status))
 	{
 		*alive = TRUE;
 	}
+
+	g_object_unref (details);
+
+	return *alive;
 }
 
-static gboolean 
-delete_pending_foreach  (gpointer persist_object,
-			 DownloadDetails *details,
-			 DownloaderView *dv)
-{	
-	g_signal_emit (G_OBJECT (dv), downloader_view_signals[DOWNLOAD_REMOVE], 
-		       0, persist_object);
+static gboolean
+delete_pending_foreach (GtkTreeModel *model,
+		        GtkTreePath  *path,
+			GtkTreeIter  *iter,
+			gpointer      user_data)
+{
+	DownloadDetails *details;
 	
-	return TRUE;
+	gtk_tree_model_get (model, iter, COL_DETAILS, &details, -1);
+	g_return_val_if_fail (details != NULL, TRUE);
+
+	g_signal_emit (G_OBJECT (details), download_signals[CANCEL], 0);
+
+	g_object_unref (details);
+
+	return FALSE;
 }
 
 gboolean
@@ -1019,9 +1159,7 @@ download_dialog_delete_cb (GtkWidget *window, GdkEventAny *event,
 	gboolean choice;
 	gboolean alive_download = FALSE;
 
-	g_hash_table_foreach (dv->priv->details_hash,
-			      (GHFunc)alive_download_foreach,
-			      &alive_download);
+	gtk_tree_model_foreach (dv->priv->model, alive_download_foreach, &alive_download);
 
 	if (!alive_download) return FALSE;
 		
@@ -1040,9 +1178,7 @@ download_dialog_delete_cb (GtkWidget *window, GdkEventAny *event,
 	/* do the appropriate thing */
 	if (choice == GTK_RESPONSE_YES)
 	{
-		g_hash_table_foreach_remove (dv->priv->details_hash,
-				             (GHRFunc)delete_pending_foreach,
-				             dv);
+		gtk_tree_model_foreach (dv->priv->model, delete_pending_foreach, NULL);
 		return FALSE;
 	}
 
@@ -1050,7 +1186,8 @@ download_dialog_delete_cb (GtkWidget *window, GdkEventAny *event,
 }
 
 void
-download_dialog_details_cb(GtkToggleButton *button, 
+download_dialog_details_cb(GtkExpander *expander, 
+			   GParamSpec *pspec,
 			   DownloaderView *dv)
 {
 	GtkWidget *details_frame;
@@ -1062,10 +1199,9 @@ download_dialog_details_cb(GtkToggleButton *button,
 	details_separator = galeon_dialog_get_control(GALEON_DIALOG(dv), 
 						      PROP_DETAILS_SEPARATOR);
 
-	dialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
+	dialog = gtk_widget_get_toplevel (GTK_WIDGET(expander));
 
-
-	if (gtk_toggle_button_get_active(button))
+	if (gtk_expander_get_expanded (expander))
 	{
 		gtk_widget_show(GTK_WIDGET(details_frame));
 		gtk_widget_show(GTK_WIDGET(details_separator));
@@ -1089,19 +1225,13 @@ open_selection_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *ite
 			DownloaderView *dv)
 {
 	DownloadDetails *details;
-	GValue val = {0, };
-	gpointer *persist_object;
 	GnomeVFSMimeApplication *app;
 	char *mime;
 	
-	gtk_tree_model_get_value (model, iter, COL_PERSIST_OBJECT, &val);
-	persist_object = g_value_get_pointer (&val);
-
-	details = g_hash_table_lookup (dv->priv->details_hash,
-				       persist_object);
+	gtk_tree_model_get (model, iter, COL_DETAILS, &details, -1);
 	g_return_if_fail (details);
 
-	if (details->status != DOWNLOAD_STATUS_COMPLETED) return;
+	if (details->status != DOWNLOAD_STATUS_COMPLETED) goto out;
 	
 	mime = gnome_vfs_get_mime_type (details->dest);
 	g_return_if_fail (mime != NULL);
@@ -1120,7 +1250,11 @@ open_selection_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *ite
 		galeon_embed_utils_nohandler_dialog_run (parent);
 	}
 	
+	gnome_vfs_mime_application_free (app);
 	g_free(mime);
+
+out:
+	g_object_unref (details);
 }
 
 void
