@@ -16,9 +16,13 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "galeon-embed-event.h"
 
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <glib/ghash.h>
 #include <gtk/gtktypeutils.h>
 
@@ -140,11 +144,11 @@ galeon_embed_event_get_mouse_button (GaleonEmbedEvent *event,
 }
 
 gresult
-galeon_embed_event_get_mouse_coords	(GaleonEmbedEvent *event,
-					 guint *x, guint *y)
+galeon_embed_event_get_coords	(GaleonEmbedEvent *event,
+				 guint *x, guint *y)
 {
-	*x = event->mouse_x;
-	*y = event->mouse_y;
+	*x = event->x;
+	*y = event->y;
 	return G_OK;
 }
 
