@@ -24,13 +24,12 @@
 #include "eel-gconf-extensions.h"
 #include "galeon-config.h"
 #include "gul-bonobo-extensions.h"
-#include "gul-download.h"
 #include "gul-string.h"
 #include "galeon-embed-helper-list.h"
 #include "galeon-marshal.h"
 
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <bonobo/bonobo-ui-component.h>
 #include <gtk/gtkclipboard.h>
 #include <libgnome/gnome-exec.h>
@@ -269,8 +268,8 @@ setup_image_openwith (GaleonEmbedPopup *p)
 		/* it must be an image, so let's try with png */
 		galeon_embed_helper_list_add_mime_type (hl, "image/png");
 	}
-	galeon_embed_helper_list_add_to_menu (hl, p->priv->ui_component,
-					      GALEON_POPUP_IMAGE_OPENWITH_PATH);
+	galeon_embed_helper_list_add_to_bonoboui_menu (hl, p->priv->ui_component,
+						       GALEON_POPUP_IMAGE_OPENWITH_PATH);
 }
 
 static void 
@@ -332,7 +331,7 @@ setup_element_menu (GaleonEmbedPopup *p)
 
 		if (file)
 		{
-			esc_file = gul_string_escape_uline_accel(file);
+			esc_file = gul_string_double_underscores (file);
 			g_free(file);
 		}
 
@@ -405,8 +404,8 @@ setup_doc_openwith (GaleonEmbedPopup *popup)
 	
 	galeon_embed_helper_list_add_mime_type (hl, "text/plain");
 	
-	galeon_embed_helper_list_add_to_menu (hl, popup->priv->ui_component,
-					      GALEON_POPUP_DOC_OPENWITH_PATH);
+	galeon_embed_helper_list_add_to_bonoboui_menu (hl, popup->priv->ui_component,
+						       GALEON_POPUP_DOC_OPENWITH_PATH);
 }
 
 static void
@@ -567,55 +566,6 @@ embed_popup_copy_link_location_cmd (BonoboUIComponent *uic,
 	embed_popup_copy_to_clipboard (popup, location);
 }
 
-static void
-save_property_url (GaleonEmbedPopup *popup,
-		   gboolean ask_dest,
-		   gboolean show_progress,
-		   const char *property)
-{
-	GaleonEmbedEvent *info;
-	const char *location;
-	const GValue *value;
-	GtkWidget *widget;
-	GtkWidget *window;
-	GaleonEmbedPersist *persist;
-	
-	info = galeon_embed_popup_get_event (popup);
-	galeon_embed_event_get_property (info, property, &value);
-	location = g_value_get_string (value);
-
-	widget = GTK_WIDGET (popup->priv->embed);
-	window = gtk_widget_get_toplevel (widget);
-	
-	persist = galeon_embed_persist_new (popup->priv->embed);
-	
-	galeon_embed_persist_set_source (persist, location);
-
-	if (show_progress)
-	{
-		galeon_embed_persist_set_flags (persist, 
-						EMBED_PERSIST_SHOW_PROGRESS);
-	}
-	
-	if (!ask_dest)
-	{
-		/* if we're not asking for download directory, make sure the
-		 * file is downloaded in the specified download directory and
-		 * not somewhere else */
-		char *dir = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
-		eel_gconf_set_string (CONF_STATE_LAST_DOWNLOAD_DIR, dir);
-		g_free (dir);
-	}
-
-	galeon_embed_utils_save (window,
-				 CONF_STATE_LAST_DOWNLOAD_DIR,
-				 ask_dest,
-                                 FALSE,
-                                 persist);
-
-	g_object_unref (persist);
-}
-
 const char *
 galeon_embed_popup_get_popup_path (GaleonEmbedPopup *p)
 {
@@ -647,28 +597,10 @@ embed_popup_download_link_cmd (BonoboUIComponent *uic,
                       	       GaleonEmbedPopup *popup, 
                                const char* verbname)
 {
-        if (eel_gconf_get_boolean(CONF_DOWNLOADING_EXTERNAL_DOWNLOADER) ==
-            DOWNLOADER_EXTERNAL)
-        {
+	GaleonEmbedEvent *event = galeon_embed_popup_get_event (popup);
+	GaleonEmbed *embed = galeon_embed_popup_get_embed (popup);
 
-		GaleonEmbedEvent *info;
-		const char *location;
-		const GValue *value;
-
-		info = galeon_embed_popup_get_event (popup);
-		galeon_embed_event_get_property (info, "link", &value);
-		location = g_value_get_string (value);
-
-		return gul_download_external_save_url (location);
-        }
-	else
-	{
-		save_property_url (popup,
-				   eel_gconf_get_boolean 
-				   (CONF_DOWNLOADING_ASK_DIR),
-				   TRUE,
-				   "link");
-	}
+	galeon_embed_utils_download_event_property (embed, event, FALSE, NULL, "link");
 }
 
 static void 
@@ -676,7 +608,11 @@ embed_popup_save_image_as_cmd (BonoboUIComponent *uic,
                       	       GaleonEmbedPopup *popup, 
                                const char* verbname)
 {
-	save_property_url (popup, TRUE, FALSE, "image");
+	GaleonEmbedEvent *event = galeon_embed_popup_get_event (popup);
+	GaleonEmbed *embed = galeon_embed_popup_get_embed (popup);
+	
+	galeon_embed_utils_save_event_property (embed, event, TRUE, FALSE, 
+						_("Save Image As..."), "image" );
 }
 
 #define CONF_DESKTOP_BG_PICTURE "/desktop/gnome/background/picture_filename"
@@ -693,15 +629,13 @@ background_download_completed (GaleonEmbedPersist *persist,
 	eel_gconf_set_string (CONF_DESKTOP_BG_PICTURE, bg);
 
 	type = eel_gconf_get_string (CONF_DESKTOP_BG_TYPE);
-	if (type || strcmp (type, "none") == 0)
+	if (type && strcmp (type, "none") == 0)
 	{
 		eel_gconf_set_string (CONF_DESKTOP_BG_TYPE,
 				      "wallpaper");
 	}
 
 	g_free (type);
-
-	g_object_unref (persist);
 }
 
 static void 
@@ -729,12 +663,13 @@ embed_popup_set_image_as_background_cmd (BonoboUIComponent *uic,
 	galeon_embed_persist_set_source (persist, location);
 	galeon_embed_persist_set_dest (persist, dest);
 
-	galeon_embed_persist_save (persist);
-
 	g_signal_connect (persist, "completed",
 			  G_CALLBACK (background_download_completed),
 			  NULL);
 	
+	galeon_embed_persist_save (persist);
+	g_object_unref (persist);
+
 	g_free (dest);
 	g_free (base);
 }
@@ -754,56 +689,39 @@ embed_popup_copy_image_location_cmd (BonoboUIComponent *uic,
 	embed_popup_copy_to_clipboard (popup, location);
 }
 
-static void
-save_url (GaleonEmbedPopup *popup,
-	  gboolean ask_dest,
-	  gboolean show_progress,
-	  const char *url)
-{
-	GtkWidget *widget;
-	GtkWidget *window;
-	GaleonEmbedPersist *persist;
-	
-	widget = GTK_WIDGET (popup->priv->embed);
-	window = gtk_widget_get_toplevel (widget);
-	
-	persist = galeon_embed_persist_new (popup->priv->embed);
-	galeon_embed_persist_set_source (persist, url);
-
-	if (show_progress)
-	{
-		galeon_embed_persist_set_flags (persist, 
-						EMBED_PERSIST_SHOW_PROGRESS);
-	}
-	
-	galeon_embed_utils_save (window,
-				 CONF_DOWNLOADING_DIR,
-				 ask_dest,
-                                 FALSE,
-                                 persist);
-
-	g_object_unref (persist);
-}
-
 static void 
 embed_popup_save_page_as_cmd (BonoboUIComponent *uic, 
                       	      GaleonEmbedPopup *popup, 
                               const char* verbname)
 {
-	char *location;
+	GtkWidget *window;
+	GaleonEmbedPersist *persist;
+
+	window = gtk_widget_get_toplevel (GTK_WIDGET (popup->priv->embed));
 	
-	galeon_embed_get_location (popup->priv->embed,
-				   FALSE, FALSE, &location);
-	save_url (popup, TRUE, FALSE, location);
-	g_free (location);
+	persist = galeon_embed_persist_new (popup->priv->embed);
+	galeon_embed_persist_set_flags (persist, EMBED_PERSIST_MAINDOC
+					| EMBED_PERSIST_SAVE_CONTENT
+					| EMBED_PERSIST_ASK_DESTINATION);
+
+	galeon_embed_persist_set_fc_parent (persist, window);
+	galeon_embed_persist_set_fc_title (persist, _("Save Page As..."));
+	galeon_embed_persist_save (persist);
+
+	g_object_unref (persist);
 }
 
 static void 
 embed_popup_save_background_as_cmd (BonoboUIComponent *uic, 
                       	       	    GaleonEmbedPopup *popup, 
                                     const char* verbname)
-{
-	save_property_url (popup, TRUE, FALSE, "background_image");
+{	
+	GaleonEmbedEvent *event = galeon_embed_popup_get_event (popup);
+	GaleonEmbed *embed = galeon_embed_popup_get_embed (popup);
+	
+	galeon_embed_utils_save_event_property (embed, event, TRUE, FALSE,
+						_("Save Background As..."),
+						"background_image");
 }
 
 static void 
@@ -825,7 +743,7 @@ embed_popup_reload_frame_cmd (BonoboUIComponent *uic,
                               const char* verbname)
 {
 	galeon_embed_reload(popup->priv->embed,
-			   EMBED_RELOAD_FRAME);
+			    EMBED_RELOAD_FRAME);
 }
 
 static void 
