@@ -44,7 +44,6 @@ struct GaleonEmbedShellPrivate
 {
 	GlobalHistory *global_history;
 	DownloaderView *downloader_view;
-	GList *embeds;
 };
 
 static void
@@ -129,7 +128,6 @@ galeon_embed_shell_init (GaleonEmbedShell *ges)
 
 	ges->priv->global_history = NULL;
 	ges->priv->downloader_view = NULL;
-	ges->priv->embeds = NULL;
 }
 
 static void
@@ -199,34 +197,11 @@ galeon_embed_shell_get_supported (void)
         return supported_embeds;
 }
 
-void              
-galeon_embed_shell_add_embed (GaleonEmbedShell *ges,
-                              GaleonEmbed *embed)
+gboolean
+galeon_embed_shell_initialize (GaleonEmbedShell *shell)
 {
-	ges->priv->embeds = g_list_append (ges->priv->embeds, embed);
-}
-
-void 
-galeon_embed_shell_remove_embed (GaleonEmbedShell *ges,
-                                 GaleonEmbed *embed)
-{
-	ges->priv->embeds = g_list_remove (ges->priv->embeds, embed);
-}
-
-GaleonEmbed *
-galeon_embed_shell_get_active_embed (GaleonEmbedShell *ges)
-{
-	GList *list = ges->priv->embeds;
-
-        g_return_val_if_fail (ges->priv->embeds != NULL, NULL);
-        
-        return GALEON_EMBED (list->data);
-}
-
-GList *
-galeon_embed_shell_get_embeds (GaleonEmbedShell *ges)
-{
-	return ges->priv->embeds;
+	GaleonEmbedShellClass *klass = GALEON_EMBED_SHELL_GET_CLASS (shell);
+        return klass->initialize (shell);
 }
 
 void              
@@ -290,20 +265,19 @@ galeon_embed_shell_show_js_console (GaleonEmbedShell *shell)
 }
 
 gresult           
-galeon_embed_shell_get_charset_titles (GaleonEmbedShell *shell,
-				       const char *group,
-				       GList **charsets)
+galeon_embed_shell_get_encodings (GaleonEmbedShell *shell,
+				  GaleonEncodings **encodings)
 {
 	GaleonEmbedShellClass *klass = GALEON_EMBED_SHELL_GET_CLASS (shell);
-        return klass->get_charset_titles (shell, group, charsets);
+        return klass->get_encodings (shell, encodings);
 }
 
 gresult           
-galeon_embed_shell_get_charset_groups (GaleonEmbedShell *shell,
-			               GList **groups)
+galeon_embed_shell_get_font_langs (GaleonEmbedShell *shell,
+				   GList **langs)
 {
 	GaleonEmbedShellClass *klass = GALEON_EMBED_SHELL_GET_CLASS (shell);
-        return klass->get_charset_groups (shell, groups);
+        return klass->get_font_langs (shell, langs);
 }
 
 gresult
@@ -366,10 +340,11 @@ galeon_embed_shell_list_cookies (GaleonEmbedShell *shell,
 
 gresult           
 galeon_embed_shell_remove_cookies (GaleonEmbedShell *shell,
-				   GList *cookies)
+				   GList *cookies,
+				   gboolean block)
 {
 	GaleonEmbedShellClass *klass = GALEON_EMBED_SHELL_GET_CLASS (shell);
-        return klass->remove_cookies (shell, cookies);
+        return klass->remove_cookies (shell, cookies, block);
 }
 
 gresult           
@@ -390,43 +365,6 @@ galeon_embed_shell_remove_passwords (GaleonEmbedShell *shell,
         return klass->remove_passwords (shell, passwords, type);
 }
 
-/**
- * show_file_picker: Shows a file picker. Can be configured to select a
- * file or a directory.
- * @parentWidget: Parent Widget for file picker.
- * @title: Title for file picker.
- * @directory: Initial directory to start in. 
- * @file: Initial filename to show in filepicker.
- * @mode: Mode to run filepicker in (modeOpen, modeSave, modeGetFolder)
- * @ret_fullpath: On a successful return, will hold the full path to selected
- *		file or directory.
- * @file_formats: an array of FileFormat structures to fill the format chooser
- *              optionmenu. NULL if not needed. The last item must have 
- *              description == NULL.
- * @ret_file_format: where to store the index of the format selected (can be
- *              NULL)
- * returns: TRUE for success, FALSE for failure.
- */
-
-gresult 
-galeon_embed_shell_show_file_picker (GaleonEmbedShell *shell,
-				     GtkWidget *parentWidget, 
-		       		     const char *title,
-		       		     const char *directory,
-		                     const char *file, 
-		       		     FilePickerMode mode,
-                       		     char **ret_fullpath, 
-		       		     gboolean *ret_save_content, 
-                       		     FileFormat *file_formats, 
-		       		     int *ret_file_format)
-{
-	GaleonEmbedShellClass *klass = GALEON_EMBED_SHELL_GET_CLASS (shell);
-        return klass->show_file_picker (shell, parentWidget, title, 
-					directory, file, mode,
-					ret_fullpath, ret_save_content,
-					file_formats, ret_file_format);	
-}
-
 static GlobalHistory *
 impl_get_global_history (GaleonEmbedShell *shell)
 {
