@@ -58,17 +58,14 @@ enum
 	FAVICON,
 	POPUP_BLOCKED,
 	CONTEXTMENU,
+	NET_STOP,
+	NET_START,
 	LAST_SIGNAL
 };
 
 static void
 galeon_embed_base_init (gpointer base_class);
 
-struct GaleonEmbedPrivate
-{
-	gpointer dummy;
-};
-
 static guint galeon_embed_signals[LAST_SIGNAL] = { 0 };
 
 GType 
@@ -283,6 +280,24 @@ galeon_embed_base_init (gpointer g_class)
                               G_TYPE_NONE,
                               1,
 			      GALEON_EMBED_EVENT_TYPE);
+	galeon_embed_signals[NET_STOP] =
+                g_signal_new ("ge_net_stop",
+                              GALEON_EMBED_TYPE,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GaleonEmbedClass, net_stop),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+	galeon_embed_signals[NET_START] =
+                g_signal_new ("ge_net_start",
+                              GALEON_EMBED_TYPE,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GaleonEmbedClass, net_start),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
 	initialized = TRUE;
 	}
 }
@@ -615,19 +630,53 @@ galeon_embed_get_security_level (GaleonEmbed *embed,
 }
 
 gresult
-galeon_embed_find (GaleonEmbed *embed,
-                   EmbedFindInfo *info)
+galeon_embed_find_set_properties (GaleonEmbed *embed,
+				  char *search_string,
+				  gboolean case_sensitive,
+				  gboolean wrap_around)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->find (embed, info);
+        return klass->find_set_properties (embed, search_string, 
+					   case_sensitive, wrap_around);
 }
 
 gresult
-galeon_embed_set_charset (GaleonEmbed *embed,
-                          const char *charset)
+galeon_embed_find_next (GaleonEmbed *embed,
+			gboolean backwards)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->set_charset (embed, charset);
+        return klass->find_next (embed, backwards);
+}
+
+
+gresult
+galeon_embed_set_encoding (GaleonEmbed *embed,
+			   const char *encoding)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+        return klass->set_encoding (embed, encoding);
+}
+
+gresult
+galeon_embed_get_encoding_info (GaleonEmbed *embed,
+				GaleonEncodingPageInfo **info)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+        return klass->get_encoding_info (embed, info);
+}
+
+gresult
+galeon_embed_has_modified_forms (GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+	return klass->has_modified_forms (embed);
+}
+
+gresult
+galeon_embed_can_view_source (GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
+	return klass->can_view_source (embed);
 }
 
 gresult
@@ -661,6 +710,21 @@ galeon_embed_print_preview_navigate (GaleonEmbed *embed,
 	return klass->print_preview_navigate (embed, navType, pageNum);
 }
 
+gresult
+galeon_embed_get_page_properties(GaleonEmbed *embed,
+				 EmbedPageProperties **retProperties)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->get_page_properties(embed, retProperties);
+}
+
+gresult
+galeon_embed_show_page_certificate (GaleonEmbed *embed)
+{
+	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS(embed);
+	return klass->show_page_certificate (embed);
+}
+
 gresult
 galeon_embed_get_stylesheets(GaleonEmbed *embed, GList **retSheets)
 {
