@@ -24,7 +24,7 @@
 #include "galeon-embed.h"
 
 #include <string.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <gtk/gtktextview.h>
 #include <gtk/gtktextbuffer.h>
 
@@ -70,7 +70,6 @@ typedef struct
         gchar *type;     /* error type string, for example Error  */
         gchar *text;     /* the error message itself              */
         guint column;    /* the column that should be highlighted */
-        GdkColor *color; /* the color used for the type string    */
 } Msg;
 
 static GObjectClass *parent_class = NULL;
@@ -291,7 +290,7 @@ javascript_console_append (JavascriptConsole *jc,
 	}
 }
 
-void static
+static void
 update_filtering_info (JavascriptConsole *dialog)
 {
 	GValue warnings = {0, };
@@ -307,7 +306,7 @@ update_filtering_info (JavascriptConsole *dialog)
 	dialog->priv->show_messages = g_value_get_boolean (&messages);
 }
 
-void static
+static void
 clear_textview (JavascriptConsole *dialog)
 {
 	GtkTextIter iter1, iter2;
@@ -325,7 +324,7 @@ fill_textview (JavascriptConsole *dialog)
 			dialog);
 }
 
-void static
+static void
 init_textview (JavascriptConsole *dialog)
 {
 	GtkWidget *textview;
@@ -376,7 +375,7 @@ init_textview (JavascriptConsole *dialog)
 	
 }
 
-void static
+static void
 ensure_constructed (JavascriptConsole *dialog)
 {
 
@@ -432,6 +431,15 @@ impl_show (GaleonDialog *dialog)
 	GALEON_DIALOG_CLASS (parent_class)->show (dialog);
 }
 
+
+static void
+free_js_console_msg (Msg *msg)
+{
+	g_free (msg->type);
+	g_free (msg->text);
+	g_free (msg);
+}
+
 static void
 javascript_console_finalize (GObject *object)
 {
@@ -444,6 +452,9 @@ javascript_console_finalize (GObject *object)
 
         g_return_if_fail (dialog->priv != NULL);
 
+	g_list_foreach (dialog->priv->errors, (GFunc)free_js_console_msg, NULL);
+	g_list_free (dialog->priv->errors);
+
         g_free (dialog->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -460,7 +471,12 @@ void
 js_console_clear_button_clicked_cb (GtkWidget *button, 
 			            GaleonDialog *dialog) 
 {
-	clear_textview (JAVASCRIPT_CONSOLE(dialog));
+	JavascriptConsole *js_console = JAVASCRIPT_CONSOLE(dialog);
+
+	clear_textview (js_console);
+	g_list_foreach (js_console->priv->errors, (GFunc)free_js_console_msg, NULL);
+	g_list_free (js_console->priv->errors);
+	js_console->priv->errors = NULL;
 }
 
 void
