@@ -16,9 +16,14 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "print-dialog.h"
 #include "prefs-strings.h"
 #include <gtk/gtkdialog.h>
+#include <gtk/gtktogglebutton.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #define CONF_PRINT_BOTTOM_MARGIN "/apps/galeon/Print/bottom_margin"
@@ -41,6 +46,7 @@
 static void print_dialog_class_init (PrintDialogClass *klass);
 static void print_dialog_init (PrintDialog *dialog);
 static void print_dialog_finalize (GObject *object);
+static void impl_show (GaleonDialog *dialog);
 
 /* Glade callbacks */
 void
@@ -78,7 +84,9 @@ enum
 	TO_PROP,
 	FROM_PROP,
 	COLOR_PROP,
-	ORIENTATION_PROP
+	ORIENTATION_PROP,
+	PREVIEW_PROP,
+	SELECTION_PROP
 };
 
 enum
@@ -107,6 +115,8 @@ GaleonDialogProperty properties [] =
 	{ FROM_PROP, "from_spinbutton", NULL, PT_NORMAL, NULL },
 	{ COLOR_PROP, "print_color_radiobutton", CONF_PRINT_COLOR, PT_NORMAL, NULL },
 	{ ORIENTATION_PROP, "orient_p_radiobutton", CONF_PRINT_ORIENTATION, PT_NORMAL, NULL },
+	{ PREVIEW_PROP, "preview_button", NULL, PT_NORMAL, NULL },
+	{ SELECTION_PROP, "selection_radiobutton", NULL, PT_NORMAL, NULL},
 
 	{ -1, NULL, NULL }
 };
@@ -146,10 +156,13 @@ static void
 print_dialog_class_init (PrintDialogClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GaleonDialogClass *galeon_dialog_class;
 
+	galeon_dialog_class = GALEON_DIALOG_CLASS (klass);
         parent_class = g_type_class_peek_parent (klass);
 
         object_class->finalize = print_dialog_finalize;
+	galeon_dialog_class->show = impl_show;
 
 	print_dialog_signals[PREVIEW] =
                 g_signal_new ("preview",
@@ -193,6 +206,43 @@ print_dialog_finalize (GObject *object)
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+impl_show (GaleonDialog *dialog)
+{
+	PrintDialog *print_dialog = PRINT_DIALOG (dialog);
+	GaleonEmbed *embed = galeon_embed_dialog_get_embed (GALEON_EMBED_DIALOG (dialog));
+
+	GALEON_DIALOG_CLASS (parent_class)->show (dialog);
+
+	if (print_dialog->only_collect_info)
+	{
+		GtkWidget *button;
+		/* disappear preview button  */
+		button = galeon_dialog_get_control (GALEON_DIALOG (dialog),
+						    PREVIEW_PROP);
+		gtk_widget_hide (button);
+	}
+
+	if (galeon_embed_selection_can_copy (embed) != G_OK)
+	{
+		GtkWidget *widget;
+		/* Make selection button disabled */
+		widget = galeon_dialog_get_control (GALEON_DIALOG (dialog),
+						    SELECTION_PROP);
+
+		gtk_widget_set_sensitive (widget, FALSE);
+
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
+		{
+			GtkWidget *all_pages;
+			all_pages = galeon_dialog_get_control (GALEON_DIALOG (dialog),
+							    ALL_PAGES_PROP);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (all_pages), TRUE);
+		}
+	}
+}
+
+
 GaleonDialog *
 print_dialog_new (GaleonEmbed *embed,
 		  EmbedPrintInfo **ret_info)
@@ -203,8 +253,11 @@ print_dialog_new (GaleonEmbed *embed,
 				     "GaleonEmbed", embed,
 				     NULL));
 
-	if (!embed) dialog->only_collect_info = TRUE;
-	dialog->ret_info = ret_info;
+	if (ret_info != NULL)
+	{
+		dialog->only_collect_info = TRUE;
+		dialog->ret_info = ret_info;
+	}
 
 	return GALEON_DIALOG(dialog);
 }
@@ -221,8 +274,11 @@ print_dialog_new_with_parent (GtkWidget *window,
 				     "ParentWindow", window,
 				     NULL));
 
-	if (!embed) dialog->only_collect_info = TRUE;
-	dialog->ret_info = ret_info;
+	if (ret_info != NULL)
+	{
+		dialog->only_collect_info = TRUE;
+		dialog->ret_info = ret_info;
+	}
 
 	return GALEON_DIALOG(dialog);
 }
@@ -356,6 +412,8 @@ print_dialog_print (GaleonDialog *dialog)
 	if(PRINT_DIALOG(dialog)->only_collect_info && PRINT_DIALOG(dialog)->ret_info)
 	{
 		*(PRINT_DIALOG(dialog)->ret_info) = info;
+		/* When in collect_info mode the caller owns the reference */
+		return;
 	}
 	else
 	{		
@@ -375,23 +433,27 @@ print_dialog_preview (GaleonDialog *dialog)
 {
 	EmbedPrintInfo *info;
 	GaleonEmbed *embed;
+	gresult result;
 
 	info = print_get_info (dialog);
 
+	/* Should not be called in collect_info mode */
 	if(PRINT_DIALOG(dialog)->only_collect_info && PRINT_DIALOG(dialog)->ret_info)
 	{
-		*(PRINT_DIALOG(dialog)->ret_info) = info;
+		g_warning( "Should not be called" );
+		return;
 	}
-	else
-	{	
-		embed = galeon_embed_dialog_get_embed 
-			(GALEON_EMBED_DIALOG(dialog));
+
+	embed = galeon_embed_dialog_get_embed (GALEON_EMBED_DIALOG(dialog));
 	
-		info->preview = TRUE;
-		galeon_embed_print (embed, info);
-		print_free_info (info);
+	info->preview = TRUE;
+	result = galeon_embed_print (embed, info);
+	print_free_info (info);
+
+	if (result == G_OK)
+	{
+		g_signal_emit (G_OBJECT (dialog), print_dialog_signals[PREVIEW], 0);
 	}
-	g_signal_emit (G_OBJECT (dialog), print_dialog_signals[PREVIEW], 0);
 	
 	g_object_unref (G_OBJECT(dialog));
 }
@@ -400,6 +462,12 @@ void
 print_cancel_button_cb (GtkWidget *widget,
 			GaleonDialog *dialog)
 {
+	if (PRINT_DIALOG (dialog)->only_collect_info)
+	{
+		/* When in collect_info mode the caller owns the reference */
+		return;
+	}
+
 	g_object_unref (G_OBJECT(dialog));
 }
 
@@ -414,9 +482,7 @@ void
 print_preview_button_cb (GtkWidget *widget,
 	 	         GaleonDialog *dialog)
 {
-	//FIXME: Don't show preview button at all.
-	if(!(PRINT_DIALOG(dialog)->only_collect_info))
-		print_dialog_preview (dialog);
+	print_dialog_preview (dialog);
 }
 
 
