@@ -138,6 +138,9 @@ impl_shistory_go_nth (GaleonEmbed *embed,
 static gboolean
 impl_shistory_copy (GaleonEmbed *source,
                     GaleonEmbed *dest);
+static gresult
+impl_shistory_clear (GaleonEmbed *source);
+
 static gresult
 impl_scroll (GaleonEmbed *embed, 
              EmbedScrollDirection direction);
@@ -271,6 +274,7 @@ galeon_embed_init (GaleonEmbedClass *embed_class)
 	embed_class->shistory_get_pos = impl_shistory_get_pos;
 	embed_class->shistory_go_nth = impl_shistory_go_nth;
 	embed_class->shistory_copy = impl_shistory_copy;
+	embed_class->shistory_clear = impl_shistory_clear;
 	embed_class->scroll = impl_scroll;
 	embed_class->get_security_level = impl_get_security_level;
 	embed_class->find = impl_find;
@@ -598,6 +602,12 @@ impl_shistory_copy (GaleonEmbed *source,
 	return G_NOT_IMPLEMENTED;
 }
 
+static gresult
+impl_shistory_clear (GaleonEmbed *embed)
+{
+	return G_NOT_IMPLEMENTED;
+}
+
 static gresult
 impl_scroll (GaleonEmbed *embed, 
              EmbedScrollDirection direction)
