@@ -77,8 +77,12 @@ NS_IMETHODIMP GBaseProtocolContentHandler::NewChannel(nsIURI *aURI,
 
 NS_IMETHODIMP GBaseProtocolContentHandler::HandleContent (
 					const char * aContentType,
+#if MOZILLA_SNAPSHOT > 16
+					nsIInterfaceRequestor * aWindowContext,
+#else
 					const char * aCommand,
 					nsISupports * aWindowContext,
+#endif
 					nsIRequest *aRequest)
 {
 	nsresult rv = NS_OK;
