@@ -27,11 +27,12 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIDOMHTMLImageElement.h"
 #include "nsIDOMElement.h"
+#include "nsIDOMWindow.h"
 #include "nsIDOMXULDocument.h"
 #include "nsIURI.h"
+#include "nsNetUtil.h"
 #include "nsIDOMNSDocument.h"
 #include "nsReadableUtils.h"
-#include "nsUnicharUtils.h"
 #include "nsGUIEvent.h"
 #include "nsIDOMNSEvent.h"
 #include "nsIDOMCharacterData.h"
@@ -131,8 +132,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 		rv = element->GetTagName(tag);
 		if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-		if (tag.Equals(NS_LITERAL_STRING("img"),
-			       nsCaseInsensitiveStringComparator()))
+		if (tag.EqualsIgnoreCase("img"))
 		{
 			info->context |= EMBED_CONTEXT_IMAGE;
 
@@ -174,8 +174,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 						   img);
 			}
 		}
-		else if (tag.Equals(NS_LITERAL_STRING("input"),
-				    nsCaseInsensitiveStringComparator()))
+		else if (tag.EqualsIgnoreCase("input"))
 		{
 			nsCOMPtr<nsIDOMElement> element;
 			element = do_QueryInterface (node);
@@ -185,8 +184,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			nsAutoString value;
 			element->GetAttribute (attr, value);
 
-			if (value.Equals(NS_LITERAL_STRING("image"),
-					 nsCaseInsensitiveStringComparator()))
+			if (value.EqualsIgnoreCase("image"))
 			{
 				info->context |= EMBED_CONTEXT_IMAGE;
 				nsCOMPtr<nsIDOMHTMLInputElement> input;
@@ -205,29 +203,20 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 
 				if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 			}
-			else if (!value.Equals(NS_LITERAL_STRING("radio"),
-					       nsCaseInsensitiveStringComparator()) &&
-				 !value.Equals(NS_LITERAL_STRING("submit"),
-					       nsCaseInsensitiveStringComparator()) &&
-				 !value.Equals(NS_LITERAL_STRING("reset"),
-					       nsCaseInsensitiveStringComparator()) &&
-				 !value.Equals(NS_LITERAL_STRING("hidden"),
-					       nsCaseInsensitiveStringComparator()) &&
-				 !value.Equals(NS_LITERAL_STRING("button"),
-					       nsCaseInsensitiveStringComparator()) &&
-				 !value.Equals(NS_LITERAL_STRING("checkbox"),
-					       nsCaseInsensitiveStringComparator()))
+			else if (!value.EqualsIgnoreCase("radio") &&
+				 !value.EqualsIgnoreCase("submit") &&
+				 !value.EqualsIgnoreCase("hidden") &&
+				 !value.EqualsIgnoreCase("button") &&
+				 !value.EqualsIgnoreCase("checkbox"))
 			{
 				info->context |= EMBED_CONTEXT_INPUT;
 			}
 		}
-		else if (tag.Equals(NS_LITERAL_STRING("textarea"),
-				    nsCaseInsensitiveStringComparator()))
+		else if (tag.EqualsIgnoreCase("textarea"))
 		{
 			info->context |= EMBED_CONTEXT_INPUT;
 		}
-		else if (tag.Equals(NS_LITERAL_STRING("object"),
-				    nsCaseInsensitiveStringComparator()))
+		else if (tag.EqualsIgnoreCase("object"))
 		{
 			nsCOMPtr<nsIDOMHTMLObjectElement> object;
 			object = do_QueryInterface (node);
@@ -236,11 +225,8 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			nsAutoString value;
 			object->GetType(value);
 
-			//Forming a substring and confirming it's contents
-			//is quicker than doing a Find on the full string
-			//and then checking that "image/" is at the beginning
-			if (Substring(value, 0, 6).Equals(NS_LITERAL_STRING("image/"),
-			    nsCaseInsensitiveStringComparator()))
+			// MIME types are always lower case
+			if (Substring(value, 0, 6).Equals(NS_LITERAL_STRING("image/")))
 			{
 				info->context |= EMBED_CONTEXT_IMAGE;
 				
@@ -280,14 +266,14 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			nsAutoString value;
 			dom_elem->GetAttributeNS (nspace, localname_type, value);
 
-			if (value.Equals(NS_LITERAL_STRING("simple"),
-					 nsCaseInsensitiveStringComparator()))
+			if (value.EqualsIgnoreCase("simple"))
 			{
 				info->context |= EMBED_CONTEXT_LINK;
 				NS_NAMED_LITERAL_STRING (localname_href, "href");
 				dom_elem->GetAttributeNS (nspace, localname_href, value);
 				
 				SetStringProperty ("link", value);
+				CheckLinkScheme (value);
 			}
 		}
 
@@ -303,18 +289,19 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
 			/* Link */
-			if (tag.Equals(NS_LITERAL_STRING("a"),
-				       nsCaseInsensitiveStringComparator()))
+			if (tag.EqualsIgnoreCase("a"))
 			{
+				nsAutoString tmp;
+
+				rv = GatherTextUnder (node, tmp);
+				if (NS_SUCCEEDED(rv))
+                                	SetStringProperty ("link_text", tmp);
+
 				nsCOMPtr <nsIDOMHTMLAnchorElement> anchor =
 					do_QueryInterface(node);
-				nsAutoString tmp;
-				rv = anchor->GetHref (tmp);
-				if (NS_FAILED(rv))
-					return NS_ERROR_FAILURE;
+				anchor->GetHref (tmp);
 
-				if (Substring(tmp, 0, 7).Equals(NS_LITERAL_STRING("mailto:"),
-			    	    nsCaseInsensitiveStringComparator()))
+				if (StringBeginsWith (tmp, NS_LITERAL_STRING ("mailto:")))
 				{
 					info->context |= EMBED_CONTEXT_EMAIL_LINK;
 					const nsAString &address = Substring(tmp, 7, tmp.Length()-7);
@@ -326,6 +313,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					info->context |= EMBED_CONTEXT_LINK;
 
 					SetStringProperty ("link", tmp);
+					CheckLinkScheme (tmp);
 					rv = anchor->GetHreflang (tmp);
 					if (NS_SUCCEEDED(rv))
 						SetStringProperty ("link_lang", tmp);
@@ -345,8 +333,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					if (NS_SUCCEEDED(rv))
 						SetStringProperty ("link_type", tmp);
 
-					if (tmp.Equals(NS_LITERAL_STRING("text/smartbookmark"),
-						       nsCaseInsensitiveStringComparator()))
+					if (tmp.EqualsIgnoreCase("text/smartbookmark"))
 					{
 						SetIntProperty ("link_is_smart", TRUE);
 						
@@ -368,21 +355,19 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 							}
 						}
 					}
-#ifdef NOT_PORTED
-					/* Get the text of the link */
-					info->linktext = mozilla_get_link_text (node);
-#endif
+					else
+					{
+						SetIntProperty ("link_is_smart", FALSE);
+					}
 				}
 			
 			}
-			else if (tag.Equals(NS_LITERAL_STRING("option"),
-					    nsCaseInsensitiveStringComparator()))
+			else if (tag.EqualsIgnoreCase("option"))
 			{
 				info->context = EMBED_CONTEXT_NONE;
 				return NS_OK;
 			}
-			if (tag.Equals(NS_LITERAL_STRING("area"),
-				       nsCaseInsensitiveStringComparator()))
+			if (tag.EqualsIgnoreCase("area"))
 			{
 				info->context |= EMBED_CONTEXT_LINK;
 				nsCOMPtr <nsIDOMHTMLAreaElement> area =
@@ -395,12 +380,11 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 						return NS_ERROR_FAILURE;
 					
 					SetStringProperty ("link", href);
+					CheckLinkScheme (href);
 				}
 			}
-			else if (tag.Equals(NS_LITERAL_STRING("textarea"),
-					    nsCaseInsensitiveStringComparator()) ||
-				 tag.Equals(NS_LITERAL_STRING("input"),
-					    nsCaseInsensitiveStringComparator()))
+			else if (tag.EqualsIgnoreCase("textarea") ||
+				 tag.EqualsIgnoreCase("input"))
 			{
 				info->context |= EMBED_CONTEXT_INPUT;
 			}
@@ -547,16 +531,11 @@ nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, GaleonE
 	nsAutoString nodename;
 	OriginalNode->GetNodeName(nodename);
 
-	if (nodename.Equals(NS_LITERAL_STRING("xul:scrollbarbutton"),
-			    nsCaseInsensitiveStringComparator()) ||
-	    nodename.Equals(NS_LITERAL_STRING("xul:thumb"),
-			    nsCaseInsensitiveStringComparator()) ||
-	    nodename.Equals(NS_LITERAL_STRING("xul:vbox"),
-			    nsCaseInsensitiveStringComparator()) ||
-	    nodename.Equals(NS_LITERAL_STRING("xul:spacer"),
-			    nsCaseInsensitiveStringComparator()) ||
-	    nodename.Equals(NS_LITERAL_STRING("xul:slider"),
-			    nsCaseInsensitiveStringComparator()))
+	if (nodename.EqualsIgnoreCase("xul:scrollbarbutton") ||
+	    nodename.EqualsIgnoreCase("xul:thumb") ||
+	    nodename.EqualsIgnoreCase("xul:vbox") ||
+	    nodename.EqualsIgnoreCase("xul:spacer") ||
+	    nodename.EqualsIgnoreCase("xul:slider"))
 		return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDOMEventTarget> EventTarget;
@@ -579,7 +558,7 @@ nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, GaleonE
 	if (mod_key) info->modifier |= GDK_SHIFT_MASK;
 
 	aMouseEvent->GetMetaKey(&mod_key);
-	if (mod_key) info->modifier |= GDK_Meta_L;
+	if (mod_key) info->modifier |= GDK_MOD2_MASK;
 	
 	aMouseEvent->GetCtrlKey(&mod_key);
 	if (mod_key) info->modifier |= GDK_CONTROL_MASK;
@@ -639,7 +618,7 @@ nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, GaleonEmbedEv
 	if (mod_key) info->modifier |= GDK_SHIFT_MASK;
 
 	aKeyEvent->GetMetaKey(&mod_key);
-	if (mod_key) info->modifier |= GDK_Meta_L;
+	if (mod_key) info->modifier |= GDK_MOD2_MASK;
 	
 	aKeyEvent->GetCtrlKey(&mod_key);
 	if (mod_key) info->modifier |= GDK_CONTROL_MASK;
@@ -675,6 +654,110 @@ nsresult EventContext::GetTargetDocument (nsIDOMDocument **domDoc)
 	return NS_OK;
 }
 
+nsresult EventContext::CheckLinkScheme (const nsAString &link)
+{
+	nsCOMPtr<nsIURI> uri;
+	NS_NewURI (getter_AddRefs (uri), link);
+	if (!uri) return NS_ERROR_FAILURE;
+
+	nsresult rv;
+	nsCAutoString scheme;
+	rv = uri->GetScheme (scheme);
+	if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
+
+	if (scheme.EqualsIgnoreCase ("http")  ||
+	    scheme.EqualsIgnoreCase ("https") ||
+	    scheme.EqualsIgnoreCase ("ftp")   ||
+	    scheme.EqualsIgnoreCase ("file")  ||
+	    scheme.EqualsIgnoreCase ("about") ||
+	    scheme.EqualsIgnoreCase ("data")  ||
+	    scheme.EqualsIgnoreCase ("resource")  ||
+	    scheme.EqualsIgnoreCase ("gopher"))
+	{
+		SetIntProperty ("link-can-open-in-new-tab", TRUE);
+	}
+	return NS_OK;
+}
+
+
+nsresult EventContext::GatherTextUnder (nsIDOMNode* aNode, nsString& aResult)
+{
+	nsAutoString text;
+	nsCOMPtr<nsIDOMNode> node;
+	aNode->GetFirstChild(getter_AddRefs(node));
+	PRUint32 depth = 1;
+
+	while (node && depth)
+	{
+		nsCOMPtr<nsIDOMCharacterData> charData(do_QueryInterface(node));
+		PRUint16 nodeType;
+
+		node->GetNodeType(&nodeType);
+		if (charData && nodeType == nsIDOMNode::TEXT_NODE)
+		{
+			/* Add this text to our collection. */
+			text += NS_LITERAL_STRING(" ");
+			nsAutoString data;
+			charData->GetData(data);
+			text += data;
+		}
+		else
+		{
+			nsCOMPtr<nsIDOMHTMLImageElement> img(do_QueryInterface(node));
+			if (img)
+			{
+				nsAutoString altText;
+				img->GetAlt(altText);
+				if (!altText.IsEmpty())
+				{
+					text = altText;
+					break;
+				}
+			}
+		}
+
+		/* Find the next node to test. */
+		PRBool hasChildNodes;
+		node->HasChildNodes(&hasChildNodes);
+		if (hasChildNodes)
+		{
+			nsCOMPtr<nsIDOMNode> temp = node;
+			temp->GetFirstChild(getter_AddRefs(node));
+			depth++;
+		}
+		else
+		{
+			nsCOMPtr<nsIDOMNode> nextSibling;
+			node->GetNextSibling(getter_AddRefs(nextSibling));
+			if (nextSibling)
+			{
+				node = nextSibling;
+			}
+			else
+			{
+				nsCOMPtr<nsIDOMNode> parentNode;
+				node->GetParentNode(getter_AddRefs(parentNode));
+				if (!parentNode)
+				{
+					node = nsnull;
+				}
+				else
+				{
+					nsCOMPtr<nsIDOMNode> nextSibling;
+					parentNode->GetNextSibling(getter_AddRefs(nextSibling));
+					node = nextSibling;
+					depth--;
+				}
+			}
+		}
+	}
+
+	text.CompressWhitespace();
+	aResult = text;
+
+	return NS_OK;
+}
+	
 nsresult EventContext::SetIntProperty (const char *name, int value)
 {
 
@@ -709,24 +792,41 @@ nsresult EventContext::SetStringProperty (const char *name, const nsAString &val
 
 nsresult ResolveBaseURL (nsIDocument *doc, const nsAString &relurl, nsACString &url)
 {
+#if MOZILLA_SNAPSHOT > 13
+	nsIURI *base = doc->GetBaseURI ();
+	if (!base) return NS_ERROR_FAILURE;
+#elif MOZILLA_SNAPSHOT > 11
+	nsIURI *base = doc->GetBaseURL ();
+	if (!base) return NS_ERROR_FAILURE;
+#elif MOZILLA_SNAPSHOT > 9
 	nsresult rv;
 	nsCOMPtr<nsIURI> base;
-#if MOZILLA_SNAPSHOT > 9
 	rv = doc->GetBaseURL (getter_AddRefs(base));
-#else
-	rv = doc->GetBaseURL (*getter_AddRefs(base));
-#endif
 	if (NS_FAILED(rv)) return rv;
+#else
+	nsresult rv;
+	nsCOMPtr<nsIURI> base;
+	rv = doc->GetBaseURL (*getter_AddRefs(base));
+	if (NS_FAILED(rv)) return rv;
+#endif
 
 	return base->Resolve (NS_ConvertUCS2toUTF8(relurl), url);
 }
 
 nsresult ResolveDocumentURL (nsIDocument *doc, const nsAString &relurl, nsACString &url)
 {
+#if MOZILLA_SNAPSHOT > 13
+	nsIURI *uri = doc->GetDocumentURI ();
+	if (!uri) return NS_ERROR_FAILURE;
+#elif MOZILLA_SNAPSHOT > 11
+	nsIURI *uri = doc->GetDocumentURL ();
+	if (!uri) return NS_ERROR_FAILURE;
+#else
 	nsresult rv;
 	nsCOMPtr<nsIURI> uri;
 	rv = doc->GetDocumentURL(getter_AddRefs(uri));
 	if (NS_FAILED(rv)) return rv;
+#endif
 
 	return uri->Resolve (NS_ConvertUCS2toUTF8(relurl), url);
 }
