@@ -59,6 +59,9 @@ private:
 	nsresult SetIntProperty (const char *name, int value);
 	nsresult SetStringProperty (const char *name, const char *value);
 	nsresult SetStringProperty (const char *name, const nsAString &value);
+	nsresult CheckLinkScheme (const nsAString &link);
+	nsresult GatherTextUnder (nsIDOMNode* aNode, nsString& aResult);
+
 	GaleonEmbedEvent *mEmbedEvent;
 };
 
