@@ -20,7 +20,6 @@
 #define GALEON_WRAPPER_H
 
 #include "nsIDocShell.h"
-#include "ProgressListener.h"
 #include "nsIWebNavigation.h"
 #include "nsIWebPageDescriptor.h"
 #include "nsISHistory.h"
@@ -31,10 +30,18 @@
 #include "nsIDOMDocument.h"
 #include "nsPIDOMWindow.h"
 #include "nsIDOMEventListener.h"
+#include "nsIChannel.h"
+#include "nsICacheEntryDescriptor.h"
+#include "nsIStyleSheet.h"
+#include "nsString.h"
 #include <gtkmozembed.h>
 
 #include "nsIPrintSettings.h"
 
+#ifdef HAVE_MOZILLA_PSM
+class nsISSLStatus;
+#endif
+
 class GDOMEventListener : public nsIDOMEventListener
 {
   public:
@@ -77,15 +84,14 @@ class GaleonWrapper
 	nsresult PrintPreviewNumPages (int *numPages);
 	nsresult PrintPreviewNavigate(PRInt16 navType, PRInt32 pageNum);
 
-	nsresult Find (const PRUnichar *search_string,
-		       PRBool interactive, PRBool matchcase,
-		       PRBool search_backwards, PRBool search_wrap_around,
-		       PRBool search_for_entire_word, PRBool search_in_frames,
-		       PRBool *didFind);
+	nsresult FindSetProperties (const PRUnichar *search_string,
+				    PRBool case_sensitive,
+				    PRBool wrap_around);
+
+	nsresult Find (PRBool backwards, PRBool *didFind);
 
 	nsresult GetMainDocumentUrl (nsCString &url);
 	nsresult GetDocumentUrl (nsCString &url);
-	nsresult GetDocumentTitle (char **title);
 
 	nsresult ReloadDocument ();
 	nsresult LoadDocument(nsISupports *aPageDescriptor, PRUint32 aDisplayType);
@@ -95,13 +101,18 @@ class GaleonWrapper
 	nsresult GetSHTitleAtIndex (PRInt32 index, PRUnichar **title);
 	nsresult GetSHUrlAtIndex (PRInt32 index, nsCString &url);
 
-	nsresult CopyHistoryTo (GaleonWrapper *embed, PRBool back_history, PRBool forward_history, PRBool set_current);
+	nsresult CopyHistoryTo (GaleonWrapper *embed, PRBool back_history,
+				PRBool forward_history, PRBool set_current);
 
 	nsresult GoToHistoryIndex (PRInt16 index);
 
 	nsresult ClearHistory();
 
-	nsresult ForceCharacterSet (const char *charset);
+	nsresult ForceEncoding (const char *encoding);
+	
+	nsresult GetEncodingInfo (GaleonEncodingPageInfo **info);
+	
+	nsresult GetHasModifiedForms (PRBool *modified);
 
 	nsresult CanCutSelection(PRBool *result);
 
@@ -115,9 +126,6 @@ class GaleonWrapper
 
 	nsresult Paste(void);
 
-	nsresult Activate ();
-	nsresult Deactivate ();
-
 	nsresult GetStyleSheets (nsIDOMStyleSheetList **list);
 
 	nsresult GetMainDOMDocument (nsIDOMDocument **aDOMDocument);
@@ -139,10 +147,11 @@ class GaleonWrapper
 
 	nsresult FineScroll (int horiz, int vert);
 
-	nsresult GetLastModified (gchar **ret);
-	nsresult GetImages (GList **ret);
-	nsresult GetForms (GList **ret);
-	nsresult GetLinks (GList **ret);
+	nsresult GetPageProperties (EmbedPageProperties *props);
+	nsresult ShowPageCertificate ();
+
+	nsresult GetSecurityTooltip (char **tooltip);
+
 	nsresult EvaluateJS (char *script);
 
 	nsresult PushTargetDocument (nsIDOMDocument *domDoc);
@@ -151,13 +160,13 @@ class GaleonWrapper
 	nsresult GetDOMDocument (nsIDOMDocument **aDOMDocument);
 	nsresult GetDOMWindow (nsIDOMWindow **aDOMWindow);
 
+	nsresult SetChannel (nsIChannel *channel);
+
 	nsCOMPtr<nsIWebBrowser>           mWebBrowser;
-
-	nsCOMPtr<nsIWebNavigation>        mChromeNav;
-
   private:
+
 	nsCOMPtr<nsIDOMDocument> mTargetDocument;
-	nsCOMPtr<nsIWebProgressListener> mProgress;
+	nsCOMPtr<nsIChannel> mChannel;
 
 	nsCOMPtr<nsIDOMEventTarget> mEventTarget;
 	nsCOMPtr<GDOMEventListener> mFaviconEventListener;
@@ -165,11 +174,24 @@ class GaleonWrapper
 
 	nsresult SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell);
 	nsresult GetDocShell (nsIDocShell **aDocShell);
-	nsresult GetCSSBackground (nsIDOMNode *node, nsAutoString& url);
 	nsresult GetFocusedDOMWindow (nsIDOMWindow **aDOMWindow);
 	nsresult GetSHistory (nsISHistory **aSHistory);
-	nsresult GetPIDOMWindow(nsPIDOMWindow **aPIWin);
 	nsresult GetWebNavigation(nsIWebNavigation **aWebNavigation);
+
+#ifdef HAVE_MOZILLA_PSM
+	nsresult GetSSLStatus (nsISSLStatus **aSSLStatus);
+#endif
+
+	nsresult GetCacheEntryDescriptor(const nsAString &aKey,
+					 nsICacheEntryDescriptor **aCacheEntryDescriptor);
+	nsresult GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, 
+					      PRUint32 *aNumTextFields, PRBool *aIsModified);
+
+	nsresult GetPageSecurityInfo (EmbedPageProperties *props);
+	nsresult GetMetaTags(GList **ret);
+	nsresult GetImages (GList **ret);
+	nsresult GetForms (GList **ret);
+	nsresult GetLinks (GList **ret);
 };
 
 #endif
