@@ -21,6 +21,9 @@
 
 #include <nsError.h>
 #include <nsIGlobalHistory.h>
+#if MOZILLA_SNAPSHOT > 13
+#include <nsIGlobalHistory2.h>
+#endif
 #include <nsIBrowserHistory.h>
 
 #include "mozilla-embed-shell.h"
@@ -31,7 +34,11 @@
  { 0xbe0c42c1, 0x39d4, 0x4271, { 0xb7, 0x9e, 0xf7, 0xaa, 0x49, 0xeb, 0x6a, 0x15}}
 
 class MozGlobalHistory: public nsIGlobalHistory,
+#if MOZILLA_SNAPSHOT > 13
+			public nsIGlobalHistory2
+#else
 			public nsIBrowserHistory
+#endif
 {
 	public:
 		MozGlobalHistory ();
@@ -39,6 +46,9 @@ class MozGlobalHistory: public nsIGlobalHistory,
 
 		NS_DECL_ISUPPORTS
 		NS_DECL_NSIGLOBALHISTORY
+#if MOZILLA_SNAPSHOT > 13
+		NS_DECL_NSIGLOBALHISTORY2
+#endif
 		NS_DECL_NSIBROWSERHISTORY
 
 	private:
