@@ -5,9 +5,10 @@
 #include <nsIWindowWatcher.h>
 #include <nsIEmbeddingSiteWindow.h>
 #include <nsIWebBrowserChrome.h>
+#include <nsString.h>
 #include <gtkmozembed.h>
 
-GtkWidget *MozillaFindGtkParent (nsIDOMWindow *aDOMWindow)
+GtkWidget *MozillaFindEmbed (nsIDOMWindow *aDOMWindow)
 {
         nsresult result;
 
@@ -15,12 +16,17 @@ GtkWidget *MozillaFindGtkParent (nsIDOMWindow *aDOMWindow)
                 (do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
         if (!wwatch) return nsnull;
 
-	nsCOMPtr<nsIDOMWindow> domWindow(aDOMWindow);
-	if (!domWindow)
+	nsCOMPtr<nsIDOMWindow> domWindow;
+	if (aDOMWindow)
 	{
-		result = wwatch->GetActiveWindow(getter_AddRefs(domWindow));
-		if (NS_FAILED(result) || !domWindow) return nsnull;
+		/* Get the top-level Window - bug 130425 */
+		aDOMWindow->GetTop (getter_AddRefs (domWindow));
 	}
+	else
+	{
+		wwatch->GetActiveWindow(getter_AddRefs(domWindow));
+	}
+	NS_ENSURE_TRUE (domWindow, nsnull);
 
         nsCOMPtr<nsIWebBrowserChrome> windowChrome;
         result = wwatch->GetChromeForWindow(domWindow,
@@ -35,9 +41,16 @@ GtkWidget *MozillaFindGtkParent (nsIDOMWindow *aDOMWindow)
         result = window->GetSiteWindow((void **)&mozembed);
         if (NS_FAILED(result)) return nsnull;
 
-	return gtk_widget_get_toplevel(GTK_WIDGET(mozembed));
+	return GTK_WIDGET(mozembed);
 }
 
+GtkWidget *MozillaFindGtkParent (nsIDOMWindow *aDOMWindow)
+{
+        GtkWidget *embed = MozillaFindEmbed (aDOMWindow);
+        if (!embed) return nsnull;
+
+        return gtk_widget_get_toplevel (GTK_WIDGET (embed));
+}
 
 NS_METHOD MozillaCollatePrintSettings (const EmbedPrintInfo *info,
 				       nsIPrintSettings *options)
