@@ -30,7 +30,8 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "nsCOMPtr.h"
 #include "nsIIOService.h"
@@ -382,7 +383,7 @@ render_from_item (OutStream *stream, GbBookmark *b, const gchar *prefix,
 		}
 		else
 		{
-			escapedname = gul_string_escape_path (strippedname);
+			escapedname = gnome_vfs_escape_string (strippedname);
 			newprefix = g_strconcat (prefix, escapedname, "/", NULL);
 			g_free(escapedname);
 		}
