@@ -42,15 +42,19 @@ GPrintingPromptService::~GPrintingPromptService()
   /* destructor code */
 }
 
-/* void showPrintDialog (in nsIDOMWindow parent, in nsIWebBrowserPrint webBrowserPrint, in nsIPrintSettings printSettings); */
-NS_IMETHODIMP GPrintingPromptService::ShowPrintDialog(nsIDOMWindow *parent, nsIWebBrowserPrint *webBrowserPrint, nsIPrintSettings *printSettings)
+/* void showPrintDialog (in nsIDOMWindow parent, in nsIWebBrowserPrint webBrowserPrint, 
+ *                       in nsIPrintSettings printSettings); */
+NS_IMETHODIMP GPrintingPromptService::ShowPrintDialog(nsIDOMWindow *parent, 
+		       nsIWebBrowserPrint *webBrowserPrint, nsIPrintSettings *printSettings)
 {
 	GaleonDialog *dialog;
 	EmbedPrintInfo *info;
+	nsresult rv;
 
 	GtkWidget *gtkParent = MozillaFindGtkParent(parent);
+	GaleonEmbed *embed = GALEON_EMBED (MozillaFindEmbed (parent));
 
-	dialog = print_dialog_new_with_parent (gtkParent, NULL, &info);
+	dialog = print_dialog_new_with_parent (gtkParent, embed, &info);
 	galeon_dialog_set_modal (dialog, TRUE);
 
 	gint ret = galeon_dialog_run (dialog);
@@ -59,10 +63,15 @@ NS_IMETHODIMP GPrintingPromptService::ShowPrintDialog(nsIDOMWindow *parent, nsIW
 		MozillaCollatePrintSettings(info, printSettings);
 		print_free_info(info);
 
-		return NS_OK;
+		rv = NS_OK;
 	}
 	else
-		return NS_ERROR_FAILURE;
+	{
+		rv = NS_ERROR_ABORT;
+	}
+	g_object_unref (dialog);
+
+	return rv;
 }
 
 /* void showProgress (in nsIDOMWindow parent, in nsIWebBrowserPrint webBrowserPrint, in nsIPrintSettings printSettings, in nsIObserver openDialogObserver, in boolean isForPrinting, out nsIWebProgressListener webProgressListener, out nsIPrintProgressParams printProgressParams, out boolean notifyOnOpen); */
