@@ -87,6 +87,9 @@ static gint embed_mouse_click_cb  (GaleonEmbed *embed,
 				   GaleonEmbedEvent *event,
 				   gpointer data);
 
+static void embed_new_window_cb (GaleonEmbed *embed, GaleonEmbed **new_embed,
+				 EmbedChromeMask chromemask, gpointer data);
+
 GSidebar *GlobalSidebar(nsnull);
 
 /* Implementation file */
@@ -211,6 +214,10 @@ void GSidebar::ShowPage(GaleonSidebar *aSidebar, const nsACString &aUrl)
 			  "ge_dom_mouse_click",
 			  G_CALLBACK(embed_mouse_click_cb),
 			  NULL);
+	g_signal_connect (G_OBJECT (sidebarEmbed),
+			  "ge_new_window",
+			  G_CALLBACK(embed_new_window_cb),
+			  NULL);
 
 	galeon_sidebar_set_content(aSidebar, G_OBJECT(sidebarEmbed));
 	galeon_embed_load_url(sidebarEmbed, PromiseFlatCString(aUrl).get());
@@ -268,13 +275,14 @@ void GSidebar::LoadSidebarPages(void)
 
 	/* load the file */
 	xmlDocPtr doc = xmlParseFile (filename);
-	g_free (filename);
 	if (doc == NULL) 
 	{
 		g_warning ("Unable to parse `%s', no sidebars loaded.",
 			   filename);
+		g_free (filename);
 		return;
 	}
+	g_free (filename);
 
 	/* iterate over sidebar pages in document */
         for (xmlNodePtr page = doc->children->children;
@@ -332,8 +340,7 @@ void GSidebar::SaveSidebarPages(void)
 	}
 
 	/* save it all out to disk */
-
-        xmlSaveFile (filename, doc);
+	gul_general_safe_xml_save (filename, doc);
 	xmlFreeDoc (doc);
 	g_free (filename);
 }
@@ -347,9 +354,8 @@ embed_mouse_down_cb  (GaleonEmbed *embed,
 		      GaleonEmbedEvent *event,
 		      gpointer data)
 {
-	GaleonEmbedPopup *popup;
+	GaleonPopup *popup;
 	GaleonWindow *window;
-	BonoboWindow *bwin;
 	guint button;
 	EmbedEventContext context;
 
@@ -364,14 +370,11 @@ embed_mouse_down_cb  (GaleonEmbed *embed,
 	window = GALEON_WINDOW(gtk_widget_get_toplevel (GTK_WIDGET(embed)));
 	g_return_val_if_fail (window != NULL, FALSE);
 
-	bwin = BONOBO_WINDOW(window);
-	g_assert (bwin != NULL);
-
 	if (button == 2)
 	{
-		popup = GALEON_EMBED_POPUP (galeon_window_get_popup_factory (window));
-		galeon_embed_popup_set_event (popup, event);
-		galeon_embed_popup_show (popup, embed);
+		popup = GALEON_POPUP (galeon_window_get_popup_factory (window));
+		galeon_popup_set_event (popup, event);
+		galeon_popup_show (popup, embed);
 	}
 	else if (button == 1 && 
 		 (context & EMBED_CONTEXT_LINK))
@@ -422,6 +425,16 @@ embed_mouse_click_cb  (GaleonEmbed *embed,
 	return FALSE;
 }
 
+static void 
+embed_new_window_cb (GaleonEmbed *embed, GaleonEmbed **new_embed,
+		     EmbedChromeMask chromemask, gpointer data)
+{
+	GaleonEmbedShell *shell = galeon_shell_get_embed_shell (galeon_shell);
+
+	/* Use the shell to generate the new window */
+	g_signal_emit_by_name (shell, "new_window_orphan", new_embed, chromemask);
+}
+
 //------------------------------------------------------------------------------
 // GSidebar callbacks
 //------------------------------------------------------------------------------
