@@ -24,9 +24,10 @@
  * basis.
  */
 
-#include <nsIProgrammingLanguage.h>
 #include <nsCOMPtr.h>
+#include <nsCRT.h>
 #include <nsIDOMWindowInternal.h>
+#include <nsIProgrammingLanguage.h>
 
 #include "SideBarProxy.h"
 
@@ -53,33 +54,67 @@ GSidebarProxy::~GSidebarProxy()
 //------------------------------------------------------------------------------
 
 /* void getInterfaces (out PRUint32 count, [array, size_is (count), retval] out nsIIDPtr array); */
-NS_IMETHODIMP GSidebarProxy::GetInterfaces(PRUint32 *count, nsIID * **aArray)
+NS_IMETHODIMP GSidebarProxy::GetInterfaces(PRUint32 *aCount, nsIID * **aArray)
 {
-	return NS_ERROR_NOT_IMPLEMENTED;
+	PRUint32 count = 2;
+
+	*aCount = count;
+
+	*aArray = NS_STATIC_CAST(nsIID **, nsMemory::Alloc(count * sizeof(nsIID *)));
+	NS_ENSURE_TRUE(*aArray, NS_ERROR_OUT_OF_MEMORY);
+
+	  nsIID *iid = NS_STATIC_CAST(nsIID *,
+				      nsMemory::Clone(&(NS_GET_IID(nsISidebar)),
+				      sizeof(nsIID)));
+	if (!iid)
+	{
+		NS_FREE_XPCOM_ALLOCATED_POINTER_ARRAY(0, *aArray);
+
+		return NS_ERROR_OUT_OF_MEMORY;
+	}
+	(*aArray)[0] = iid;
+
+
+	iid = NS_STATIC_CAST(nsIID *,
+			     nsMemory::Clone(&(NS_GET_IID(nsIClassInfo)),
+			     sizeof(nsIID)));
+	if (!iid)
+	{
+		NS_FREE_XPCOM_ALLOCATED_POINTER_ARRAY(1, *aArray);
+
+		return NS_ERROR_OUT_OF_MEMORY;
+	}
+	(*aArray)[1] = iid;
+
+	return NS_OK;
 }
 
 /* nsISupports getHelperForLanguage (in PRUint32 language); */
 NS_IMETHODIMP GSidebarProxy::GetHelperForLanguage(PRUint32 language, nsISupports **_retval)
 {
-	return NS_ERROR_NOT_IMPLEMENTED;
+	*_retval = nsnull;
+	return NS_OK;
 }
 
 /* readonly attribute string contractID; */
 NS_IMETHODIMP GSidebarProxy::GetContractID(char * *aContractID)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+	*aContractID = nsCRT::strdup(NS_SIDEBAR_CONTRACTID);
+	return NS_OK;
 }
 
 /* readonly attribute string classDescription; */
 NS_IMETHODIMP GSidebarProxy::GetClassDescription(char * *aClassDescription)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+	*aClassDescription = nsCRT::strdup("Sidebar");
+	return NS_OK;
 }
 
 /* readonly attribute nsCIDPtr classID; */
 NS_IMETHODIMP GSidebarProxy::GetClassID(nsCID * *aClassID)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+	*aClassID = nsnull;
+	return NS_OK;
 }
 
 /* readonly attribute PRUint32 implementationLanguage; */
@@ -99,5 +134,5 @@ NS_IMETHODIMP GSidebarProxy::GetFlags(PRUint32 *aFlags)
 /* [notxpcom] readonly attribute nsCID classIDNoAlloc; */
 NS_IMETHODIMP GSidebarProxy::GetClassIDNoAlloc(nsCID *aClassIDNoAlloc)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+	return NS_ERROR_NOT_AVAILABLE;
 }
