@@ -15,7 +15,7 @@
 #include <nsString.h>
 
 #include <ctime>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 
 CookieInfo *
 mozilla_cookie_to_info (nsICookie *aCookie)
@@ -53,7 +53,7 @@ mozilla_cookie_to_info (nsICookie *aCookie)
 
 		strftime (buf, sizeof(buf), _("%a, %b %-d %Y at %-I:%M %p"), 
 			  localtime_r (&t, &tm));
-		info->expire = g_strdup (buf);
+		info->expire = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
 	}
 
 	return info;
