@@ -33,16 +33,16 @@
 #include "eel-gconf-extensions.h"
 #include "galeon-embed-prefs.h"
 #include "MozRegisterComponents.h"
-#include "FilePicker.h"
 #include "SideBar.h"
 #include "nsIConsoleService.h"
 #include "JSConsoleListener.h"
 #include "nsIJSConsoleService.h"
 #include "CookieObserver.h"
 #include "mozilla-cookie.h"
+#include "mozilla-encodings.h"
 
 #include <time.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <string.h>
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
@@ -68,13 +68,19 @@
 #include <nsIPasswordManager.h>
 #include <nsIPassword.h>
 #include <nsICookie.h>
+#if MOZILLA_SNAPSHOT <= 12
 #include <nsCCookieManager.h>
+#endif
 #include <nsCPasswordManager.h>
 #include <nsIObserverService.h>
 #include <nsIPrefService.h>
 
 //For setting the locale. hopefully gtkmozembed will do itself soon
+#if MOZILLA_SNAPSHOT > 13
+#include <nsIChromeRegistry.h>
+#else
 #include <chrome/nsIChromeRegistry.h>
+#endif
 #include <locale/nsILocaleService.h>
 
 #define MOZILLA_PROFILE_DIR  GALEON_DIR"/mozilla"
@@ -98,6 +104,9 @@ mozilla_embed_shell_init (MozillaEmbedShell *ges);
 static void
 mozilla_embed_shell_finalize (GObject *object);
 
+static gboolean
+impl_initialize (GaleonEmbedShell *mes);
+
 static void              
 impl_get_capabilities (GaleonEmbedShell *shell,
 		       EmbedShellCapabilities *caps);
@@ -115,12 +124,11 @@ impl_show_java_console (GaleonEmbedShell *shell);
 static gresult           
 impl_shell_show_js_console (GaleonEmbedShell *shell);
 static gresult           
-impl_get_charset_titles (GaleonEmbedShell *shell,
-			 const char *group,
-			 GList **charsets);
+impl_get_encodings (GaleonEmbedShell *shell,
+		    GaleonEncodings **encodings);
 static gresult           
-impl_get_charset_groups (GaleonEmbedShell *shell,
-		         GList **groups);
+impl_get_font_langs (GaleonEmbedShell *shell,
+		     GList **langs);
 static gresult
 impl_get_font_list (GaleonEmbedShell *shell,
 		    const char *langGroup,
@@ -150,7 +158,8 @@ impl_list_cookies (GaleonEmbedShell *shell,
 		   GList **cookies);
 static gresult           
 impl_remove_cookies (GaleonEmbedShell *shell,
-		     GList *cookies);
+		     GList *cookies,
+		     gboolean block);
 static gresult           
 impl_list_passwords (GaleonEmbedShell *shell,
 		     PasswordType type, 
@@ -159,17 +168,6 @@ static gresult
 impl_remove_passwords (GaleonEmbedShell *shell,
 		       GList *passwords,
 		       PasswordType type);
-static gresult 
-impl_show_file_picker (GaleonEmbedShell *shell,
-		       GtkWidget *parentWidget, 
-		       const char *title,
-		       const char *directory,
-		       const char *file, 
-		       FilePickerMode mode,
-                       char **ret_fullpath, 
-		       gboolean *ret_save_content, 
-                       FileFormat *file_formats, 
-		       int *ret_file_format);
 
 static void mozilla_embed_shell_new_window_orphan_cb (GtkMozEmbedSingle *embed,
             	           		              GtkMozEmbed **retval, 
@@ -178,9 +176,8 @@ static void mozilla_embed_shell_new_window_orphan_cb (GtkMozEmbedSingle *embed,
 
 struct MozillaEmbedShellPrivate
 {
-	GHashTable *charsets_hash;
-	GList *sorted_charsets_titles;
 	GaleonDialog *js_console;
+	MozillaEncodings *encodings;
 };
 
 static NS_DEFINE_CID(kJVMManagerCID, NS_JVMMANAGER_CID);
@@ -228,14 +225,15 @@ mozilla_embed_shell_class_init (MozillaEmbedShellClass *klass)
 	
         object_class->finalize = mozilla_embed_shell_finalize;
 
+	shell_class->initialize = impl_initialize;
 	shell_class->get_capabilities = impl_get_capabilities;
 	shell_class->clear_cache = impl_clear_cache;
 	shell_class->set_offline_mode = impl_set_offline_mode;
 	shell_class->load_proxy_autoconf = impl_load_proxy_autoconf;
 	shell_class->show_java_console = impl_show_java_console;
 	shell_class->show_js_console = impl_shell_show_js_console;
-	shell_class->get_charset_titles = impl_get_charset_titles;
-	shell_class->get_charset_groups = impl_get_charset_groups;
+	shell_class->get_encodings   = impl_get_encodings;
+	shell_class->get_font_langs = impl_get_font_langs;
 	shell_class->get_font_list = impl_get_font_list;
 	shell_class->set_permission = impl_set_permission;
 	shell_class->list_permissions = impl_list_permissions;
@@ -245,94 +243,14 @@ mozilla_embed_shell_class_init (MozillaEmbedShellClass *klass)
 	shell_class->remove_cookies = impl_remove_cookies;
 	shell_class->list_passwords = impl_list_passwords;
 	shell_class->remove_passwords = impl_remove_passwords;
-	shell_class->show_file_picker = impl_show_file_picker;
 }
 
-static void
-mozilla_load_proxy_prefs (MozillaEmbedShell *shell)
-{
-	char *tmp;
-	int i, mozilla_mode = 0;
-	
-	/* Proxy mode */
-	tmp = eel_gconf_get_string(CONF_NETWORK_PROXY_MODE);
-	
-	if (!tmp)
-	{
-		mozilla_mode = 0;
-	}
-	else if (strcmp(tmp, "manual") == 0)
-	{
-		mozilla_mode = 1;
-	}
-	else if (strcmp(tmp, "auto") == 0)
-	{
-		mozilla_mode = 2;
-	}
-
-	mozilla_prefs_set_int("network.proxy.type", mozilla_mode);
-	g_free(tmp);
-
-	/* Http proxy */
-	tmp = eel_gconf_get_string(CONF_NETWORK_HTTP_PROXY);
-	if (tmp)
-	{
-		mozilla_prefs_set_string("network.proxy.http", tmp);
-		g_free (tmp);
-	}
-
-	i = eel_gconf_get_integer(CONF_NETWORK_HTTP_PROXY_PORT);
-	mozilla_prefs_set_int("network.proxy.http_port", i);
-	
-	/* Ftp proxy */
-	tmp = eel_gconf_get_string(CONF_NETWORK_FTP_PROXY);
-	if (tmp)
-	{
-		mozilla_prefs_set_string("network.proxy.ftp", tmp);
-		g_free(tmp);
-	}
-
-	i = eel_gconf_get_integer(CONF_NETWORK_FTP_PROXY_PORT);
-	mozilla_prefs_set_int("network.proxy.ftp_port", i);
-
-	/* Secure proxy */
-	tmp = eel_gconf_get_string(CONF_NETWORK_SSL_PROXY);
-	if (tmp)
-	{
-		mozilla_prefs_set_string("network.proxy.ssl", tmp);
-		g_free(tmp);
-	}
-
-	i = eel_gconf_get_integer(CONF_NETWORK_SSL_PROXY_PORT);
-	mozilla_prefs_set_int("network.proxy.ssl_port", i);
-
-	/* Socks proxy */
-	tmp = eel_gconf_get_string(CONF_NETWORK_SOCKS_PROXY);
-	if (tmp)
-	{
-		mozilla_prefs_set_string("network.proxy.socks", tmp);
-		g_free(tmp);
-	}
-
-	i = eel_gconf_get_integer(CONF_NETWORK_SOCKS_PROXY_PORT);
-	mozilla_prefs_set_int("network.proxy.socks_port", i);
-
-	/* Autoconfiguration */
-	tmp = eel_gconf_get_string(CONF_NETWORK_PROXY_AUTO_URL);
-	if (tmp)
-	{
-		galeon_embed_shell_load_proxy_autoconf 
-			(GALEON_EMBED_SHELL (shell), tmp);
-		g_free (tmp);
-	}
-}
-
-static void
+static gboolean
 mozilla_set_default_prefs (void)
 {
 	nsCOMPtr<nsIPrefService> prefs
 		= do_GetService (NS_PREFSERVICE_CONTRACTID);
-	g_return_if_fail (prefs);
+	if (!prefs) return FALSE;
 
 	// read our predefined default prefs
 	nsresult rv;
@@ -340,14 +258,14 @@ mozilla_set_default_prefs (void)
 	rv = NS_NewNativeLocalFile(
 		NS_LITERAL_CSTRING(DEFAULT_PROFILE_FILE),
 		PR_TRUE, getter_AddRefs(file));
-	if (NS_SUCCEEDED(rv))
-	{
-		rv = prefs->ReadUserPrefs (file);
-	}
+	if (NS_FAILED(rv)) return FALSE;
+
+	rv = prefs->ReadUserPrefs (file);
 
 	if (NS_FAILED(rv))
 	{
 		g_warning ("failed to read default preferences, error: %x", rv);
+		return FALSE;
 	}
 
 	// Load the default user preferences as well.  This also makes the
@@ -361,16 +279,18 @@ mozilla_set_default_prefs (void)
 
 	nsCOMPtr<nsIPrefBranch> pref;
 	prefs->GetDefaultBranch ("", getter_AddRefs(pref));
-	g_return_if_fail (pref);
+	if (!pref) return FALSE;
 
 	// Search engine for keyword search, although disabled by default, set
 	// here to facilitate l10n.
-	pref->SetCharPref ("keyword.URL", _("http://www.google.com/search?q="));
+	pref->SetCharPref ("keyword.URL", _("http://www.google.com/search?ie=UTF-8&q="));
 
 	// Also set vendor and vendorSub prefs so that mozilla can construct
 	// the user agent string for us.
 	pref->SetCharPref ("general.useragent.vendor", "Galeon");
 	pref->SetCharPref ("general.useragent.vendorSub", VERSION);
+
+	return TRUE;
 }
 
 static void
@@ -411,34 +331,6 @@ mozilla_init_profile (void)
         g_free (profile_path);
 }
 
-static gboolean
-is_new_build (void)
-{
-	gboolean new_build = FALSE;
-	char *mozprefs, *build_test;
-	
-	mozprefs = g_build_filename (g_get_home_dir (), 
-				     MOZILLA_PROFILE_DIR,
-				     MOZILLA_PROFILE_NAME,
-				     MOZILLA_PROFILE_FILE,
-				     NULL);
-
-	/* no mozilla prefs ? or new galeon build */
-        build_test = eel_gconf_get_string ("/apps/galeon/gconf_test");
-        if (!g_file_test(mozprefs, G_FILE_TEST_EXISTS) || 
-            build_test == NULL ||
-            strncmp (build_test, __TIME__, 8) != 0)
-        {
-                new_build = TRUE;
-                eel_gconf_set_string ("/apps/galeon/gconf_test", __TIME__);
-        }
-	
-        g_free (mozprefs);
-        g_free (build_test);
-
-	return new_build;
-}
-
 static void
 mozilla_init_js_console (MozillaEmbedShell *mes)
 {
@@ -454,6 +346,7 @@ mozilla_init_js_console (MozillaEmbedShell *mes)
 	mozilla_js_console = G_OBJECT(mes->priv->js_console);
 }
 
+#if 0
 static gboolean
 have_gnome_url_handler (const gchar *protocol)
 {
@@ -477,6 +370,7 @@ have_gnome_url_handler (const gchar *protocol)
 
 	return rv;
 }
+#endif
 
 static void
 mozilla_register_external_protocols (void)
@@ -496,7 +390,7 @@ mozilla_register_external_protocols (void)
 	mozilla_register_MailtoProtocolHandler ();
 }
 
-static void
+static gboolean
 mozilla_init_cookie_observer(void)
 {
 	//We new this observer but never delete it
@@ -508,13 +402,16 @@ mozilla_init_cookie_observer(void)
 	nsresult rv;
 
 	nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1", &rv));
-	if(NS_FAILED(rv) || ! os) return;
+	if(NS_FAILED(rv) || ! os) return FALSE;
 
 	GCookieObserver* cookieObserver = new GCookieObserver();
 
 	rv = os->AddObserver(cookieObserver,
 			     PERM_CHANGE_NOTIFICATION,
 			     PR_FALSE);
+	if (NS_FAILED(rv)) return FALSE;
+
+	return TRUE;
 }
 
 static nsresult
@@ -523,66 +420,97 @@ getUILang(nsAString& aUILang)
 	nsresult       result;
 	// get a locale service
 	nsCOMPtr<nsILocaleService> localeService = do_GetService(NS_LOCALESERVICE_CONTRACTID, &result);
-	NS_ASSERTION(NS_SUCCEEDED(result),"getUILang: get locale service failed");
 
+	if (NS_FAILED (result) || !localeService)
+	{
+		g_warning ("Could not get locale service!\n");
+		return NS_ERROR_FAILURE;
+	}
+        
+#if MOZILLA_SNAPSHOT > 12
+	result = localeService->GetLocaleComponentForUserAgent (aUILang);
+#else
 	nsXPIDLString uiLang;
 	result = localeService->GetLocaleComponentForUserAgent(getter_Copies(uiLang));
-	aUILang = uiLang;
-	return result;
+	aUILang.Assign(uiLang);
+#endif
+	if (NS_FAILED (result))
+	{
+		g_warning ("Could not determine locale!\n");
+		return result;
+	}
+
+	return NS_OK;
 }
  
 // update global locale if possible (in case when user-*.rdf can be updated)
 // so that any apps after this can be invoked in the UILocale and contentLocale
-static nsresult mozilla_init_chrome(void)
+static gboolean
+mozilla_init_chrome (void)
 {
 	nsresult rv = NS_OK;
 	nsAutoString uiLang;
 	// compute lang code
 	rv = getUILang(uiLang);
+	if (NS_FAILED (rv)) return FALSE;
 
 	nsCOMPtr<nsIXULChromeRegistry> chromeRegistry = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
-	if (chromeRegistry) {
-		chromeRegistry->SetRuntimeProvider(PR_TRUE);
-		rv = chromeRegistry->SelectLocale(NS_ConvertUCS2toUTF8(uiLang), PR_FALSE);
-
-		// Make sure skin is classic so we get native scrollbars.
-		rv = chromeRegistry->SelectSkin(NS_LITERAL_CSTRING("classic/1.0"), PR_FALSE);
+	if (NS_FAILED (rv) || !chromeRegistry)
+	{
+		g_warning ("Could not get the chrome registry!\n");
+		return FALSE;
 	}
+	
+	// set locale
+	chromeRegistry->SetRuntimeProvider(PR_TRUE);
 
-	return NS_OK;
+	// Dont worry if this fails, it is probably because the lang pack
+	// isn't installed.
+	chromeRegistry->SelectLocale (NS_ConvertUCS2toUTF8(uiLang), PR_FALSE);
+
+	// Make sure skin is classic so we get native scrollbars
+	rv = chromeRegistry->SelectSkin (NS_LITERAL_CSTRING("classic/1.0"), PR_FALSE);
+	if (NS_FAILED (rv)) return FALSE;
+
+	return TRUE;
 }
 
 static void
 mozilla_embed_shell_init (MozillaEmbedShell *mes)
 {
-	gboolean new_build;
-
         mes->priv = g_new0 (MozillaEmbedShellPrivate, 1);
 
-	mes->priv->charsets_hash = NULL;
-	mes->priv->sorted_charsets_titles = NULL;
+	mes->priv->encodings = NULL;
 
-	new_build = is_new_build ();
-	
 	/* Pre initialization */
 	mozilla_init_home ();
 	mozilla_init_profile ();
 	
 	/* Fire up the best */
 	gtk_moz_embed_push_startup ();
+}
+
+
+static gboolean
+impl_initialize (GaleonEmbedShell *ges)
+{
+	gboolean ret;
+	MozillaEmbedShell *mes = MOZILLA_EMBED_SHELL(ges);
 
 	/* Until gtkmozembed does this itself */
-	mozilla_init_chrome();
+	ret = mozilla_init_chrome();
+	if (ret == FALSE) return FALSE;
 
 	/* Post initialization */
-	mozilla_set_default_prefs ();
+	ret = mozilla_set_default_prefs ();
+	if (ret == FALSE) return FALSE;
+
 	mozilla_notifiers_init (mes);
 
-	mozilla_load_proxy_prefs (mes);
-	
 	mozilla_init_single (mes);
 	
-	mozilla_register_components ();
+	ret = mozilla_register_components ();
+	if (ret == FALSE) return FALSE;
 
 	mozilla_init_js_console (mes);
 
@@ -592,7 +520,7 @@ mozilla_embed_shell_init (MozillaEmbedShell *mes)
 
 	mozilla_init_cookie_observer();
 		
-	/* FIXME alert if fails */
+	return TRUE;
 }
 
 static void 
@@ -637,6 +565,8 @@ mozilla_embed_shell_new_window_orphan_cb (GtkMozEmbedSingle *embed,
 
 	g_assert (new_embed != NULL);
 	
+	gtk_moz_embed_set_chrome_mask (GTK_MOZ_EMBED(new_embed), chrome_mask);
+
 	*retval = GTK_MOZ_EMBED(GALEON_EMBED(new_embed));
 }
 
@@ -658,16 +588,10 @@ mozilla_embed_shell_finalize (GObject *object)
 	
 	gtk_moz_embed_pop_startup ();
 	
-	if (mes->priv->charsets_hash)
+	if (mes->priv->encodings)
 	{
-		g_hash_table_destroy (mes->priv->charsets_hash);
+		g_object_unref (mes->priv->encodings);
 	}
-
-	if (mes->priv->sorted_charsets_titles)
-	{
-		g_list_free (mes->priv->sorted_charsets_titles);
-	}
-
         g_free (mes->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -770,200 +694,39 @@ impl_shell_show_js_console (GaleonEmbedShell *shell)
 	return G_OK;
 }
 
-#if MOZILLA_SNAPSHOT > 9
-static gresult
-fill_charsets_lists (MozillaEmbedShellPrivate *priv)
-{
-	nsresult rv;
-
-	nsCOMPtr<nsICharsetConverterManager> ccm =
-		do_GetService (NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-	if (NS_FAILED(rv)) return G_FAILED;
-
-	nsCOMPtr<nsIUTF8StringEnumerator> e;
-	rv = ccm->GetDecoderList (getter_AddRefs(e));
-	if (NS_FAILED(rv)) return G_FAILED;
-
-        PRUint32 translated_cscount = get_translated_cscount ();
-        priv->charsets_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-
-	PRBool more;
-	for (rv = e->HasMore(&more); more; rv = e->HasMore(&more))
-	{
-		if (NS_FAILED(rv)) return G_FAILED;
-
-		nsCAutoString charset;
-		rv = e->GetNext(charset);
-		if (NS_FAILED(rv)) return G_FAILED;
-
-		nsAutoString title;
-		rv = ccm->GetCharsetTitle (charset.get(), title);
-		if (NS_FAILED(rv)) return G_FAILED;
-
-		char *charset_title_str = g_strdup(NS_ConvertUCS2toUTF8(title).get());
-		for (PRUint32 j = 0; j < translated_cscount; j++)
-                {
-                        if (g_ascii_strcasecmp (
-                                charset.get(), 
-                                charset_trans_array[j].charset_name) == 0)
-                        {
-                                g_free (charset_title_str);
-                                charset_title_str = g_strdup (_(charset_trans_array[j].charset_title));
-                                        _(charset_trans_array[j].charset_title);
-                                break;
-                        }
-                }
-
-		/* fill the hash and the sorted list */
-		g_hash_table_insert (priv->charsets_hash, charset_title_str, 
-				     g_strdup (charset.get()));
-                priv->sorted_charsets_titles = 
-			g_list_insert_sorted (priv->sorted_charsets_titles,
-                                              (gpointer)charset_title_str,
-                                              (GCompareFunc)g_ascii_strcasecmp); 
-	}
-
-	return G_OK;
-}
-#else
-static gresult
-fill_charsets_lists (MozillaEmbedShellPrivate *priv)
-{
-	nsresult rv;
-	char *tmp;
-        PRUint32 cscount;
-        PRUint32 translated_cscount = get_translated_cscount ();
-        char *charset_str, *charset_title_str;
-
-        nsCOMPtr<nsIAtom> docCharsetAtom;
-        nsCOMPtr<nsICharsetConverterManager2> ccm2 =
-                do_GetService (NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-        if (!NS_SUCCEEDED(rv)) return G_FAILED;
-
-        nsCOMPtr <nsISupportsArray> cs_list;
-        rv = ccm2->GetDecoderList (getter_AddRefs(cs_list));
-        if (!NS_SUCCEEDED(rv)) return G_FAILED;
-
-        rv = cs_list->Count(&cscount);
-        priv->charsets_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-	for (PRUint32 i = 0; i < cscount; i++)
-        {
-                nsCOMPtr<nsISupports> cssupports =
-                                        (dont_AddRef)(cs_list->ElementAt(i));
-                nsCOMPtr<nsIAtom> csatom ( do_QueryInterface(cssupports) );
-                nsAutoString charset_ns, charset_title_ns;
-
-                /* charset name */
-		rv = csatom->ToString(charset_ns);
-                tmp = ToNewCString (charset_ns);
-                if (tmp == NULL || strlen (tmp) == 0)
-                {
-                        continue;
-                }
-		charset_str = g_strdup (tmp);
-		nsMemory::Free (tmp);
-		tmp = nsnull;
-
-                /* charset readable title */
-                rv = ccm2->GetCharsetTitle2(csatom, &charset_title_ns);
-                tmp = ToNewCString (charset_title_ns);
-                if (tmp == NULL || 
-                    strlen (tmp) == 0)
-                {
-                        if (tmp) nsMemory::Free (tmp);
-                        charset_title_str = g_strdup (charset_str);
-                }
-		else
-		{
-			charset_title_str = g_strdup (tmp);
-			nsMemory::Free (tmp);
-			tmp = nsnull;
-		}
-
-		for (PRUint32 j = 0; j < translated_cscount; j++)
-                {
-                        if (g_ascii_strcasecmp (
-                                charset_str, 
-                                charset_trans_array[j].charset_name) == 0)
-                        {
-                                g_free (charset_title_str);
-                                charset_title_str = g_strdup (_(charset_trans_array[j].charset_title));
-                                break;
-                        }
-                }
-
-		/* fill the hash and the sorted list */
-		g_hash_table_insert (priv->charsets_hash, charset_title_str, charset_str);
-                priv->sorted_charsets_titles = 
-			g_list_insert_sorted (priv->sorted_charsets_titles,
-                                              (gpointer)charset_title_str,
-                                              (GCompareFunc)g_ascii_strcasecmp); 
-        }
-
-	return G_OK;
-}
-#endif
-
-static void
-ensure_charsets_tables (MozillaEmbedShell *shell)
-{
-	if (!shell->priv->charsets_hash)
-	{
-		fill_charsets_lists (shell->priv);
-	}
-}
-
 static gresult           
-impl_get_charset_titles (GaleonEmbedShell *shell,
-		         const char *group,
-			 GList **charsets)
+impl_get_encodings (GaleonEmbedShell *shell,
+		    GaleonEncodings **encodings)
 {
 	MozillaEmbedShell *mshell = MOZILLA_EMBED_SHELL(shell);
-	int count = get_translated_cscount ();
-	GList *l = NULL;
-	int j;
-	
-	ensure_charsets_tables (mshell);
-	g_return_val_if_fail (mshell->priv->charsets_hash != NULL, G_FAILED);
 
-	for (j = 0; j < count; j++)
-	{	
-		if (group == NULL ||
-		    strcmp (group, _(lgroups[charset_trans_array[j].lgroup])) == 0) 
-		{
-			CharsetInfo *info;
-			info = g_new0 (CharsetInfo, 1);
-			info->name = charset_trans_array[j].charset_name;
-			info->title = _(charset_trans_array[j].charset_title);
-			l = g_list_append (l, info);
-
-			/* FIXME check that the encoding exists in mozilla before
-			 * adding it */
-		}
+	if (! mshell->priv->encodings)
+	{
+		mshell->priv->encodings = mozilla_encodings_new ();
 	}
-
-	*charsets = l;
-
+	*encodings = GALEON_ENCODINGS (mshell->priv->encodings);
 	return G_OK;
 }
 
-static gresult           
-impl_get_charset_groups (GaleonEmbedShell *shell,
-		         GList **groups)
+
+static gresult
+impl_get_font_langs (GaleonEmbedShell *shell,
+		     GList **langs)
 {
 	GList *l = NULL;
 	int i;
-	
-	for (i = 0; lgroups[i] != NULL; i++)
+	for (i = 0; i < num_font_languages; i++)
 	{
-		l = g_list_append (l, _(lgroups[i]));
+		FontLangInfo *info = g_new0 (FontLangInfo, 1);
+		info->key   = font_languages[i].key;
+		info->title = _(font_languages[i].title);
+		l = g_list_append (l, info);
 	}
-	
-	*groups = l;
-	
+	*langs = l;
 	return G_OK;
 }
 
+
 static gresult
 impl_get_font_list (GaleonEmbedShell *shell,
 		    const char *langGroup,
@@ -1131,6 +894,8 @@ impl_test_permission (GaleonEmbedShell *shell,
 	nsCOMPtr<nsIPermissionManager> permissionManager =
                         do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
 
+	if (!url) return G_FAILED;
+
 	nsCOMPtr<nsIURI> uri;
 	rv = NS_NewURI(getter_AddRefs(uri), url);
 	if (NS_FAILED(rv) || !uri) return G_FAILED;
@@ -1191,7 +956,8 @@ impl_list_cookies (GaleonEmbedShell *shell,
 
 static gresult           
 impl_remove_cookies (GaleonEmbedShell *shell,
-		     GList *cookies)
+		     GList *cookies,
+		     gboolean block)
 {
 	nsresult result;
         nsCOMPtr<nsICookieManager> cookieManager =
@@ -1204,7 +970,7 @@ impl_remove_cookies (GaleonEmbedShell *shell,
                 result = cookieManager->Remove (nsDependentCString(c->domain),
                                                 nsDependentCString(c->name),
                                                 nsDependentCString(c->path),
-                                                PR_FALSE);
+                                                block);
                 if (NS_FAILED(result)) return G_FAILED;
         };
 
@@ -1289,78 +1055,3 @@ impl_remove_passwords (GaleonEmbedShell *shell,
         return G_OK;
 }
 
-static gresult 
-impl_show_file_picker (GaleonEmbedShell *shell,
-		       GtkWidget *parentWidget, 
-		       const char *title,
-		       const char *directory,
-		       const char *file, 
-		       FilePickerMode mode,
-                       char **ret_fullpath, 
-		       gboolean *ret_save_content, 
-                       FileFormat *file_formats, 
-		       int *ret_file_format)
-{
-	PRBool showContentCheck;
-        gchar *expanded_directory;
-
-        if (ret_save_content == NULL)
-                showContentCheck = PR_FALSE;
-        else
-                showContentCheck = PR_TRUE;
-
-        GFilePicker *filePicker = new GFilePicker (showContentCheck, 
-                                                   file_formats);
-
-        /* expand the tilde, if there is one */
-        expanded_directory = gul_string_expand_home_dir (directory);
-
-        /* make sure the directory exists, and use the home directory
-         * otherwise */
-        if (!expanded_directory ||
-            !g_file_test (expanded_directory, G_FILE_TEST_IS_DIR))
-        {
-                if (expanded_directory) g_free (expanded_directory);
-                expanded_directory = g_strdup (g_get_home_dir());
-        }
-
-        nsCOMPtr<nsILocalFile> dir = 
-                                do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-        dir->InitWithNativePath (nsDependentCString(expanded_directory));
-        g_free (expanded_directory);
-
-        filePicker->InitWithGtkWidget (parentWidget, title, mode);
-        filePicker->SetDefaultString (NS_ConvertUTF8toUCS2(file).get());
-        filePicker->SetDisplayDirectory (dir);
-        
-        PRInt16 retval;
-        filePicker->Show (&retval);
-
-        if (ret_save_content != NULL)
-        {
-                if (retval == GFilePicker::returnOKSaveContent)
-                        *ret_save_content = TRUE;
-                else
-                        *ret_save_content = FALSE;
-        }
-        if (ret_file_format != NULL)
-        {
-                *ret_file_format = filePicker->mSelectedFileFormat;
-        }
-
-        if (retval == nsIFilePicker::returnCancel)
-        {
-                delete filePicker;
-                return G_FAILED;
-        }
-        else
-        {
-                nsCOMPtr<nsILocalFile> file;
-                filePicker->GetFile (getter_AddRefs(file));
-		nsCAutoString tempFullPathStr;
-                file->GetNativePath (tempFullPathStr);
-                *ret_fullpath = g_strdup (tempFullPathStr.get());
-                delete filePicker;
-                return G_OK;
-        }
-}
