@@ -22,16 +22,17 @@
 
 #include "galeon-config.h"
 #include "gtkmozembed.h"
-#include "gtkmozembed_internal.h"
 #include "gul-string.h"
 #include "gul-gobject-misc.h"
+#include "galeon-embed-utils.h"
 #include "galeon-embed.h"
 #include "mozilla-embed.h"
 #include "MozillaPrivate.h"
 #include "GaleonWrapper.h"
 #include "EventContext.h"
+#include "galeon-debug.h"
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include <nsIURI.h>
 #include <nsIURL.h>
@@ -40,9 +41,6 @@
 #include <nsIRequest.h>
 #include <nsIWebProgressListener.h>
 #include <nsIWindowWatcher.h>
-#include <nsITransportSecurityInfo.h>
-#include <nsIPrintOptions.h>
-#include <nsGfxCIID.h>
 #include <nsIDOMStyleSheetList.h>
 #include <nsIDOMStyleSheet.h>
 #include <nsIDOMHTMLLinkElement.h>
@@ -51,9 +49,6 @@
 
 #define WINDOWWATCHER_CONTRACTID "@mozilla.org/embedcomp/window-watcher;1"
 
-#define DEBUG_MSG(x)
-//#define DEBUG_MSG(x) g_print x
-
 static void
 mozilla_embed_class_init (MozillaEmbedClass *klass);
 static void
@@ -61,8 +56,6 @@ mozilla_embed_init (MozillaEmbed *gs);
 static void
 mozilla_embed_finalize (GObject *object);
 static void
-mozilla_embed_destroy (GtkObject *object);
-static void
 galeon_embed_init (GaleonEmbedClass *embed_class);
 
 static void
@@ -180,13 +173,29 @@ static gresult
 impl_get_security_level (GaleonEmbed *embed, 
                          EmbedSecurityLevel *level,
                          char **description);
-static gresult
-impl_find (GaleonEmbed *embed,
-           EmbedFindInfo *info);
+static gresult 
+impl_find_next (GaleonEmbed *embed, 
+		gboolean backwards);
 
 static gresult
-impl_set_charset (GaleonEmbed *embed,
-                  const char *charset);
+impl_find_set_properties (GaleonEmbed *embed, 
+                          char *search_string,
+	                  gboolean case_sensitive,
+			  gboolean wrap_around);
+
+static gresult
+impl_set_encoding (GaleonEmbed *embed,
+		   const char *charset);
+
+static gresult
+impl_get_encoding_info (GaleonEmbed *embed,
+			GaleonEncodingPageInfo **info);
+
+static gresult
+impl_has_modified_forms (GaleonEmbed *embed);
+
+static gresult
+impl_can_view_source (GaleonEmbed *embed);
 
 static gresult
 impl_print (GaleonEmbed *embed, 
@@ -203,6 +212,11 @@ impl_print_preview_navigate (GaleonEmbed *embed,
 			     EmbedPrintPreviewNavType navType,
 			     gint pageNum);
 
+static gresult
+impl_get_page_properties(GaleonEmbed *embed,
+			 EmbedPageProperties **retProperties);
+static gresult
+impl_show_page_certificate (GaleonEmbed *embed);
 static gresult
 impl_get_stylesheets(GaleonEmbed *embed,
 		     GList **retSheets);
@@ -234,6 +248,10 @@ static void
 mozilla_embed_net_state_all_cb (GtkMozEmbed *embed, const char *aURI,
                                 gint state, guint status, MozillaEmbed *membed);
 static void
+mozilla_embed_net_stop_cb (GtkMozEmbed *embed, MozillaEmbed *membed);
+static void
+mozilla_embed_net_start_cb (GtkMozEmbed *embed, MozillaEmbed *membed);
+static void
 mozilla_embed_progress_cb (GtkMozEmbed *embed, const char *aURI,
                            gint curprogress, gint maxprogress, MozillaEmbed *membed);
 static void
@@ -290,6 +308,8 @@ signal_connections[] =
         { "new_window",      (void *) mozilla_embed_new_window_cb        },
         { "security_change", (void *) mozilla_embed_security_change_cb   },
 	{ "dom_key_down",    (void *) mozilla_embed_dom_key_down_cb      },
+	{ "net_stop",        (void *) mozilla_embed_net_stop_cb          },
+	{ "net_start",       (void *) mozilla_embed_net_start_cb         },
 
         /* terminator -- must be last in the list! */
         { NULL, NULL } 
@@ -297,19 +317,26 @@ signal_connections[] =
 
 struct MozillaEmbedPrivate
 {
-	MozillaEmbedPrivate() : wrapper(NULL), security_state(-1), no_page(1)
+	MozillaEmbedPrivate() : wrapper(NULL), security_state(-1), 
+				loaded_url (FALSE), loading_url(0)
 	{ /* nothing */ }
 
 	GaleonWrapper *wrapper;
-	nsCOMPtr<nsIRequest> request;
 	gint security_state;
 
-	/* HACK 1: No page loaded, 0: Loading an empty page, -1: Page loaded */ 
-	gint no_page;
+	/* LOAD HACK: When loading a site, either initially, or when
+	 * loaded from "about:blank" remember the url that was
+	 * attempted to load, and always return that for the
+	 * get_location, and also fake it so that refresh loads that
+	 * url
+	 *
+	 * Note that we only do this hack for the "about:blank" case
+	 * when the first page was "about:blank"
+	 */
+	gboolean loaded_url; //!< Has a web page been loaded
+	gchar *loading_url;  //!< The URL we are trying to load
 };
 
-static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
-
 static GObjectClass *parent_class = NULL;
 
 MAKE_GET_TYPE_IFACE (mozilla_embed, "MozillaEmbed", MozillaEmbed,
@@ -356,13 +383,19 @@ galeon_embed_init (GaleonEmbedClass *embed_class)
 	embed_class->scroll = impl_scroll;
 	embed_class->fine_scroll = impl_fine_scroll;
 	embed_class->get_security_level = impl_get_security_level;
-	embed_class->find = impl_find;
-	embed_class->set_charset = impl_set_charset;
+	embed_class->find_set_properties = impl_find_set_properties;
+	embed_class->find_next = impl_find_next;
+	embed_class->set_encoding = impl_set_encoding;
+	embed_class->get_encoding_info = impl_get_encoding_info;
+	embed_class->has_modified_forms = impl_has_modified_forms;
+	embed_class->can_view_source = impl_can_view_source;
 	embed_class->select_all = impl_select_all;
 	embed_class->print = impl_print;
 	embed_class->print_preview_close = impl_print_preview_close;
 	embed_class->print_preview_num_pages = impl_print_preview_num_pages;
 	embed_class->print_preview_navigate = impl_print_preview_navigate;
+	embed_class->get_page_properties = impl_get_page_properties;
+	embed_class->show_page_certificate = impl_show_page_certificate;
 	embed_class->get_stylesheets = impl_get_stylesheets;
 	embed_class->get_selected_stylesheet = impl_get_selected_stylesheet;
 	embed_class->set_stylesheet = impl_set_stylesheet;
@@ -464,25 +497,44 @@ mozilla_embed_remove (GtkContainer *container, GtkWidget *widget)
  * End dirty mouse wheel zoom hack
  *************************************************************/
 
+static void
+mozilla_embed_realize (GtkWidget *widget)
+{
+	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (widget)->priv;
+
+	GTK_WIDGET_CLASS(parent_class)->realize (widget);
+
+	nsresult result;
+	result = mpriv->wrapper->Init (GTK_MOZ_EMBED (widget));
+
+	if (NS_FAILED(result))
+	{
+               	g_warning ("Browser initialization failed");
+	}
+}
+
 static void
 mozilla_embed_class_init (MozillaEmbedClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
-     	GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass); 
 	GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
+     	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass); 
 
 	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
 	
 	container_class->add    = mozilla_embed_add;
 	container_class->remove = mozilla_embed_remove;
-	gtk_object_class->destroy = mozilla_embed_destroy;
         object_class->finalize = mozilla_embed_finalize;
+	widget_class->realize = mozilla_embed_realize;
 }
 
 static void
 mozilla_embed_init (MozillaEmbed *embed)
 {
-        embed->priv = new MozillaEmbedPrivate;
+        embed->priv          = new MozillaEmbedPrivate;
+	embed->priv->wrapper = new GaleonWrapper ();
+
+	LOG ("MozillaEmbed ctor (%p)", embed);
 
 	mozilla_embed_connect_signals (embed);
 }
@@ -498,29 +550,13 @@ mozilla_embed_connect_signals (MozillaEmbed *embed)
         /* connect signals */
         for (i = 0; signal_connections[i].event != NULL; i++)
         {
-                g_signal_connect (G_OBJECT(embed),
-                                  signal_connections[i].event,
-                                  G_CALLBACK(signal_connections[i].func), 
-                                  embed);
+                g_signal_connect_object (G_OBJECT(embed),
+					 signal_connections[i].event,
+					 G_CALLBACK(signal_connections[i].func), 
+					 embed, (GConnectFlags)0);
         }
 }
 
-static void
-mozilla_embed_destroy (GtkObject *object)
-{
-	int i;
-	
-	for (i = 0; signal_connections[i].event != NULL; i++)
-        {
-                g_signal_handlers_disconnect_by_func
-			(G_OBJECT(object),
-                         (gpointer)signal_connections[i].func, 
-                         (void *)object);
-        }
-	
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
-}
-
 static void
 mozilla_embed_finalize (GObject *object)
 {
@@ -540,13 +576,13 @@ mozilla_embed_finalize (GObject *object)
         	embed->priv->wrapper = NULL;
 	}
 
+	g_free (embed->priv->loading_url);
+
 	delete embed->priv;
 	
         G_OBJECT_CLASS (parent_class)->finalize (object);
 
-#ifdef DEBUG_MARCO
-	g_print ("MozillaEmbed finalized %p\n", embed);
-#endif
+	LOG ("MozillaEmbed dtor (%p)", embed);
 }
 
 gpointer
@@ -588,6 +624,20 @@ impl_load_url (GaleonEmbed *embed,
         gtk_moz_embed_load_url (GTK_MOZ_EMBED(embed),
                                 clean_url);
 
+	/* LOAD HACK: If we haven't loaded a url yet, remember this one, unless it
+	 * "about:blank", and flag that we are trying to load this url */
+	MozillaEmbed *membed = MOZILLA_EMBED (embed);
+	if (!membed->priv->loaded_url && strcmp ("about:blank", clean_url))
+	{
+		g_free (membed->priv->loading_url);
+		membed->priv->loading_url = clean_url;
+
+		/* Fake a "location" signal, so that the layer above picks up
+		 * this url, and doesn't think there is no current url */
+		g_signal_emit_by_name (membed, "ge_location");
+		return G_OK;
+	}
+
 	g_free (clean_url);
 
 	return G_OK;
@@ -721,6 +771,15 @@ mozilla_embed_get_uri_parent (const char *aUri)
         rv = NS_NewURI (getter_AddRefs(uri), aUri);
         if (NS_FAILED(rv) || !uri) return NULL;
 
+	/* Don't support going 'up' with chrome url's, mozilla handily
+	 * fixes them up for us, so it doesn't work properly, see
+	 *    rdf/chrome/src/nsChromeProtocolHandler.cpp::NewURI()
+	 * (the Canonify() call) */
+	nsCAutoString scheme;
+	rv = uri->GetScheme (scheme);
+	if (NS_FAILED(rv) || !scheme.Length()) return NULL;
+	if (scheme == NS_LITERAL_CSTRING ("chrome")) return NULL;
+
         nsCOMPtr<nsIURL> url = do_QueryInterface(uri, &rv);
         if (NS_FAILED(rv) || !url) return NULL;
 
@@ -808,16 +867,26 @@ static gresult
 impl_get_title (GaleonEmbed *embed,
                 char **title)
 {
-	nsXPIDLString uTitle;
-
-	*getter_Copies(uTitle) =
-		gtk_moz_embed_get_title_unichar (GTK_MOZ_EMBED(embed));
-
-	*title = g_strdup (NS_ConvertUCS2toUTF8(uTitle).get());
+	*title = gtk_moz_embed_get_title (GTK_MOZ_EMBED(embed));
 	
 	return G_OK;
 }
 
+/* This can be used to get the real location of the currently
+ * loaded page. It bypasses gtkmozembed, and the loading_page
+ * hack, to get the actual location mozilla thinks it has loaded
+ */
+char *
+mozilla_embed_get_real_location (MozillaEmbed *embed)
+{
+	nsCAutoString url;
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	nsresult rv = wrapper->GetRealURL (url);
+	if (NS_FAILED (rv)) return 0;
+
+	return (!url.IsEmpty()) ? g_strdup (url.get()) : NULL;
+}
+
 static gresult 
 impl_get_location (GaleonEmbed *embed, 
                    gboolean toplevel,
@@ -831,14 +900,6 @@ impl_get_location (GaleonEmbed *embed,
 
 	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 
-	/* if the wrapper is NULL than we have no location,
-	 * in fact the wrapper is initialized on net start */
-	if (!wrapper) 
-	{
-		*location = NULL;
-		return G_FAILED;
-	}
-	
 	/* FIXME !toplevel requested not implemented */
 	
 	if (toplevel)
@@ -854,24 +915,35 @@ impl_get_location (GaleonEmbed *embed,
 	}
 	else if (requested)
 	{
-		rv = wrapper->GetRealURL (url);
-		l = (NS_SUCCEEDED (rv) && !url.IsEmpty()) ?
-		     g_strdup (url.get()) : NULL;	   	
+		l = mozilla_embed_get_real_location (MOZILLA_EMBED (embed));
+   	
+	}
+
+	/* LOAD HACK: If we are trying to load a url, and we didn't
+	 * manage it (the location is still "about:blank"), return the
+	 * attempted url to the caller, so that they know the right thing */
+	MozillaEmbed *membed = MOZILLA_EMBED (embed);
+	if (!membed->priv->loaded_url && membed->priv->loading_url && l &&
+	    !strcmp ("about:blank", l))
+	{
+		g_free (l);
+		l = g_strdup (membed->priv->loading_url);
 	}
 
 	*location = l;
 	
-	return G_OK;
+	return ( l == NULL ) ? G_FAILED : G_OK;
 }
 
 static gresult 
 impl_reload (GaleonEmbed *embed, 
              EmbedReloadFlags flags)
 {
+	char *location;
+
 	if ((flags & EMBED_RELOAD_FRAME))
 	{
 		GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-		g_return_val_if_fail(wrapper != NULL, G_FAILED);
 
 		nsresult rv = wrapper->ReloadDocument();
 		return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
@@ -879,25 +951,38 @@ impl_reload (GaleonEmbed *embed,
 
 	guint32 mflags = GTK_MOZ_EMBED_FLAG_RELOADNORMAL;
 	
-	if ((flags & EMBED_RELOAD_BYPASSCACHE) &&
-	    (flags & EMBED_RELOAD_BYPASSPROXY))
+	MozillaEmbed *membed = MOZILLA_EMBED (embed);
+	location = mozilla_embed_get_real_location (membed);
+	if (!location) return G_FAILED;
+
+	/* LOAD HACK: If we are loading a url, but it didn't load
+	 * properly, then load the url that was originally requested */
+	if (!membed->priv->loaded_url && membed->priv->loading_url &&
+	    !strcmp (location,"about:blank"))
+	{
+		galeon_embed_load_url (embed, membed->priv->loading_url);
+		g_free (location);
+		return G_OK;
+	} 
+
+	/* NOTE, bypassing the proxy just tells the proxy not to read from
+	 * its cache, it doesn't actually talk to the server directly */
+
+	/* If the page is a view-source: url, we need to force the reload */
+	if (location && g_str_has_prefix (location, "view-source:"))
 	{
 		mflags = GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXYANDCACHE;
-		DEBUG_MSG (("Should bypass cache and proxy\n"));
+		LOG ("Forcing bypass cache and proxy for view-source:");
 	}
-	else if (flags & EMBED_RELOAD_BYPASSCACHE)
+	else if (flags & EMBED_RELOAD_FORCE)
 	{
-		mflags = GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE;
-		DEBUG_MSG (("Should bypass cache\n"));
-	}
-	else if (flags & EMBED_RELOAD_BYPASSPROXY)
-	{
-		mflags = GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXY;
-		DEBUG_MSG (("Should bypass proxy\n"));
+		mflags = GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXYANDCACHE;
+		LOG ("Should bypass cache and proxy");
 	}
 	
-	gtk_moz_embed_reload (GTK_MOZ_EMBED(embed),
-			      mflags);
+	g_free (location);
+
+	gtk_moz_embed_reload (GTK_MOZ_EMBED(embed), mflags);
 	
 	return G_OK;
 }
@@ -907,21 +992,8 @@ impl_copy_page (GaleonEmbed *dest,
 		GaleonEmbed *source,
 		EmbedDisplayType display_type)
 {
-	GaleonWrapper *dWrapper;
-	dWrapper = MOZILLA_EMBED(dest)->priv->wrapper;
-	if (dWrapper == NULL)
-	{
-		// We need to load some page to get the wrapper initialized.
-		// That's an implementation specific issue the caller doesn't
-		// need to know.
-		gtk_moz_embed_load_url (GTK_MOZ_EMBED(dest), "about:blank");
-		dWrapper = MOZILLA_EMBED(dest)->priv->wrapper;
-	}
-	g_return_val_if_fail (dWrapper != NULL, G_FAILED);
-
-	GaleonWrapper *sWrapper;
-	sWrapper = MOZILLA_EMBED(source)->priv->wrapper;
-	g_return_val_if_fail (sWrapper != NULL, G_FAILED);
+	GaleonWrapper *dWrapper = MOZILLA_EMBED(dest)->priv->wrapper;;
+	GaleonWrapper *sWrapper = MOZILLA_EMBED(source)->priv->wrapper;;
 
         nsresult rv;
 
@@ -932,6 +1004,21 @@ impl_copy_page (GaleonEmbed *dest,
         rv = dWrapper->LoadDocument(pageDescriptor, static_cast<PRUint32>(display_type));
         if (NS_FAILED(rv)) return G_FAILED;
 
+	/* LOAD HACK: Ensure that we remember the url that we are loading 
+	 * (if the source is in the middle of loading the url */
+	if (!MOZILLA_EMBED (dest)->priv->loaded_url)
+	{
+		char *url;
+		if (galeon_embed_get_location (source, FALSE, TRUE, &url) == G_OK)
+		{
+			MOZILLA_EMBED (dest)->priv->loading_url = url;
+
+			/* Fake a "location" signal, so that the layer above picks up
+			 * this url, and doesn't think there is no current url */
+			g_signal_emit_by_name (dest, "ge_location");
+		}
+	}
+
         return G_OK;
 }
 
@@ -956,12 +1043,9 @@ impl_zoom_set (GaleonEmbed *embed,
                int zoom, 
                gboolean reflow)
 {
-	GaleonWrapper *wrapper;
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	nsresult result;
 
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
-
 	result = wrapper->SetZoom ((float)(zoom) / 100, reflow);
 
 	if (NS_SUCCEEDED (result))
@@ -976,38 +1060,22 @@ static gresult
 impl_zoom_get (GaleonEmbed *embed,
                int *zoom)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	float f;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	if (!wrapper)
-	{
-		*zoom = 100;
-		return G_OK;
-	}
-	
-	nsresult result = wrapper->GetZoom (&f);
-	
-	if (NS_SUCCEEDED (result))
-	{
-		*zoom = (int) rint (f * 100);
 
-		return G_OK;
-	}
-	else
-	{
-		return G_FAILED;
-	}
+	nsresult result = wrapper->GetZoom (&f);
+	if (NS_FAILED (result)) return G_FAILED;
+
+	*zoom = (int) rint (f * 100);
+
+	return G_OK;
 }
 
 static gresult 
 impl_selection_can_cut (GaleonEmbed *embed)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	gboolean result;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
 	wrapper->CanCutSelection (&result);
 	
@@ -1017,11 +1085,8 @@ impl_selection_can_cut (GaleonEmbed *embed)
 static gresult 
 impl_selection_can_copy (GaleonEmbed *embed)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	gboolean result;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
 	wrapper->CanCopySelection (&result);
 	
@@ -1031,11 +1096,8 @@ impl_selection_can_copy (GaleonEmbed *embed)
 static gresult 
 impl_can_paste (GaleonEmbed *embed)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	gboolean result;
-	GaleonWrapper *wrapper;
-
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
 	wrapper->CanPaste (&result);
 	
@@ -1045,11 +1107,8 @@ impl_can_paste (GaleonEmbed *embed)
 static gresult 
 impl_select_all (GaleonEmbed *embed)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	nsresult result;
-	GaleonWrapper *wrapper;
-
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
 	result = wrapper->SelectAll ();
 	
@@ -1059,11 +1118,8 @@ impl_select_all (GaleonEmbed *embed)
 static gresult
 impl_selection_cut (GaleonEmbed *embed)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	nsresult rv;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
 	rv = wrapper->CutSelection ();
 	
@@ -1073,11 +1129,8 @@ impl_selection_cut (GaleonEmbed *embed)
 static gresult
 impl_selection_copy (GaleonEmbed *embed)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	nsresult rv;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
 	rv = wrapper->CopySelection ();
 	
@@ -1087,11 +1140,8 @@ impl_selection_copy (GaleonEmbed *embed)
 static gresult
 impl_paste (GaleonEmbed *embed)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	nsresult rv;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
 	rv = wrapper->Paste ();
 	
@@ -1102,13 +1152,10 @@ static gresult
 impl_shistory_count  (GaleonEmbed *embed,
                       int *count)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	nsresult rv;
-	GaleonWrapper *wrapper;
 	int c, index;
 	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
-	
 	rv = wrapper->GetSHInfo (&c, &index);
 
 	*count = c;
@@ -1123,11 +1170,8 @@ impl_shistory_get_nth (GaleonEmbed *embed,
                        char **aUrl,
                        char **aTitle)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	nsresult rv;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 
 	if (is_relative)
 	{
@@ -1136,7 +1180,7 @@ impl_shistory_get_nth (GaleonEmbed *embed,
 		if (galeon_embed_shistory_get_pos 
 		    (GALEON_EMBED(embed), &pos) == G_OK)
 		{
-			pos += nth;
+			nth += pos;
 		}
 		else
 		{
@@ -1160,13 +1204,10 @@ static gresult
 impl_shistory_get_pos (GaleonEmbed *embed,
                        int *pos)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	nsresult rv;
-	GaleonWrapper *wrapper;
 	int count, index;
 	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
-	
 	rv = wrapper->GetSHInfo (&count, &index);
 
 	*pos = index;
@@ -1178,11 +1219,8 @@ static gresult
 impl_shistory_go_nth (GaleonEmbed *embed, 
                       int nth)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	nsresult rv;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
 	rv = wrapper->GoToHistoryIndex (nth);
 
@@ -1197,15 +1235,9 @@ impl_shistory_copy (GaleonEmbed *source,
 		    gboolean set_current)
 {
 	nsresult rv;
-	GaleonWrapper *s_wrapper;
-	GaleonWrapper *d_wrapper;
+	GaleonWrapper *s_wrapper = MOZILLA_EMBED(source)->priv->wrapper;
+	GaleonWrapper *d_wrapper = MOZILLA_EMBED(dest)->priv->wrapper;
 	
-	s_wrapper = MOZILLA_EMBED(source)->priv->wrapper;
-	g_return_val_if_fail (s_wrapper != NULL, G_FAILED);
-
-	d_wrapper = MOZILLA_EMBED(dest)->priv->wrapper;
-	g_return_val_if_fail (d_wrapper != NULL, G_FAILED);
-
 	rv = s_wrapper->CopyHistoryTo (d_wrapper, back_history, forward_history, set_current);
 
 	return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
@@ -1214,11 +1246,8 @@ impl_shistory_copy (GaleonEmbed *source,
 static gresult
 impl_shistory_clear (GaleonEmbed *embed)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	nsresult rv;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
 	rv = wrapper->ClearHistory();
 
@@ -1230,10 +1259,7 @@ static gresult
 impl_scroll (GaleonEmbed *embed, 
              EmbedScrollDirection direction)
 {
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 
 	switch (direction)
 	{
@@ -1257,10 +1283,7 @@ impl_scroll (GaleonEmbed *embed,
 static gresult
 impl_fine_scroll (GaleonEmbed *embed, int horiz, int vert)
 {
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 
 	wrapper->FineScroll (horiz, vert);
 
@@ -1274,31 +1297,10 @@ impl_get_security_level (GaleonEmbed *embed,
 {
 	nsresult result;
 
-        nsCOMPtr<nsIChannel> channel;
-	channel = do_QueryInterface (MOZILLA_EMBED(embed)->priv->request, 
-				     &result);
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	result = wrapper->GetSecurityTooltip (description);
         if (NS_FAILED (result)) return G_FAILED;
 
-        nsCOMPtr<nsISupports> info;
-        result = channel->GetSecurityInfo(getter_AddRefs(info));
-        if (NS_FAILED (result)) return G_FAILED;
-
-	*description = NULL;
-	if (info)
-	{
-		nsCOMPtr<nsITransportSecurityInfo> secInfo(do_QueryInterface(info));
-		if (!secInfo) return G_FAILED;
-
-		nsXPIDLString tooltip;
-		result = secInfo->GetShortSecurityDescription(getter_Copies(tooltip));
-		if (NS_FAILED (result)) return G_FAILED;
-
-		if (tooltip)
-		{
-			*description = g_strdup (NS_ConvertUCS2toUTF8(tooltip).get());
-		}
-	}
-	
 	*level = mozilla_embed_security_level (MOZILLA_EMBED (embed));
 	return G_OK;
 }
@@ -1307,15 +1309,12 @@ static gresult
 impl_print (GaleonEmbed *embed,
             EmbedPrintInfo *info)
 {
-	nsresult result = NS_OK;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	nsresult result;
 
         nsCOMPtr<nsIPrintSettings> options;
         result = wrapper->GetPrintSettings(getter_AddRefs(options));
-        g_assert (NS_SUCCEEDED (result));
+	if (NS_FAILED (result)) return G_FAILED;
 
 	MozillaCollatePrintSettings(info, options);
 
@@ -1323,17 +1322,16 @@ impl_print (GaleonEmbed *embed,
 
 	result = wrapper->Print(options, info->preview);
 
+	options->SetPrintSilent (PR_FALSE);
+
 	return NS_SUCCEEDED (result) ? G_OK : G_FAILED;
 }
 
 static gresult
 impl_print_preview_close (GaleonEmbed *embed)
 {
-	nsresult result = NS_OK;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	nsresult result;
 
 	result = wrapper->PrintPreviewClose();
 	return NS_SUCCEEDED(result) ? G_OK : G_FAILED;
@@ -1343,11 +1341,8 @@ static gresult
 impl_print_preview_num_pages (GaleonEmbed *embed,
 			      gint *retNum)
 {
-	nsresult result = NS_OK;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	nsresult result;
 
 	result = wrapper->PrintPreviewNumPages(retNum);
 	return NS_SUCCEEDED(result) ? G_OK : G_FAILED;
@@ -1358,71 +1353,118 @@ impl_print_preview_navigate (GaleonEmbed *embed,
 			     EmbedPrintPreviewNavType navType,
 			     gint pageNum)
 {
-	nsresult result = NS_OK;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	nsresult result;
 
 	result = wrapper->PrintPreviewNavigate(navType, pageNum);
 	return NS_SUCCEEDED(result) ? G_OK : G_FAILED;
 }
 
 static gresult
-impl_find (GaleonEmbed *embed, 
-            EmbedFindInfo *info)
+impl_find_next (GaleonEmbed *embed, 
+                gboolean backwards)
 {
-	nsresult result = NS_OK;
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
-	
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	nsresult result;
         PRBool didFind;
 
-        result = wrapper->Find ((NS_ConvertUTF8toUCS2(info->search_string)).get(),
-        			info->interactive,
-				info->match_case, 
-                 	        info->backwards, info->wrap,
-                       	        info->entire_word, info->search_frames,
-                                &didFind);
+        result = wrapper->Find (backwards, &didFind);
 	
 	return didFind ? G_OK : G_FAILED;
 }
 
+
 static gresult
-impl_set_charset (GaleonEmbed *embed,
-                  const char *charset)
+impl_find_set_properties (GaleonEmbed *embed, 
+                          char *search_string,
+	                  gboolean case_sensitive,
+			  gboolean wrap_around)
 {
-	nsresult rv;
-	GaleonWrapper *wrapper;
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	nsresult result;
 
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
-
-	rv = wrapper->ForceCharacterSet (charset);
-	if (NS_SUCCEEDED (rv))
-	{
-		gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
-				      GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
-	}
+        result = wrapper->FindSetProperties
+		((NS_ConvertUTF8toUCS2(search_string)).get(),
+		 case_sensitive, wrap_around);
 	
-	return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
+	return NS_SUCCEEDED (result) ? G_OK : G_FAILED;
 }
 
+static gresult
+impl_set_encoding (GaleonEmbed *embed,
+		   const char *charset)
+{
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	nsresult rv;
+
+	rv = wrapper->ForceEncoding (charset);
+	if (NS_FAILED (rv)) return G_FAILED;
+
+	gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
+			      GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
+
+	return G_OK;
+}
+
+static gresult
+impl_get_encoding_info (GaleonEmbed *embed,
+			GaleonEncodingPageInfo **info)
+{
+	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
+	nsresult result;
+
+	*info = NULL;
+	result = mpriv->wrapper->GetEncodingInfo (info);
+
+	if (NS_FAILED (result))
+	{
+		galeon_encoding_page_info_free (*info);
+		return G_FAILED;
+	}
+
+	return G_OK;
+}
+
+
+static gresult
+impl_has_modified_forms (GaleonEmbed *embed)
+{
+	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
+	nsresult result;
+
+	PRBool modified;
+	result = mpriv->wrapper->GetHasModifiedForms (&modified);
+
+	return modified == PR_TRUE ? G_OK : G_FAILED;
+}
+
+static gresult
+impl_can_view_source (GaleonEmbed *embed)
+{
+	char *location;
+	gresult ret = G_OK;
+
+	if (galeon_embed_get_location (embed, TRUE, FALSE, &location) != G_OK)
+		return G_FAILED;
+	g_return_val_if_fail (location != NULL, G_FAILED);
+
+	if (g_str_has_prefix (location, "view-source:") ||
+	    g_str_has_prefix (location, "wyciwyg:"))
+	{
+		ret = G_FAILED;
+	}
+
+	g_free (location);
+
+	return ret;
+}
+
+
 static void
 mozilla_embed_location_changed_cb (GtkMozEmbed *embed, 
 				   MozillaEmbed *membed)
 {
-	/* Do not emit signal if we are loading the
-	 * fallback about:blank. We dont want the user
-	 * to know about it. */
-	if (membed->priv->no_page != 0)
-	{
-		g_signal_emit_by_name (membed, "ge_location");
-	}
-
-	membed->priv->no_page = -1;
+	g_signal_emit_by_name (membed, "ge_location");
 }
 
 static void
@@ -1460,27 +1502,6 @@ mozilla_embed_net_state_all_cb (GtkMozEmbed *embed, const char *aURI,
 		{ 0, EMBED_STATE_UNKNOWN }
 	};
 
-	/* No page loaded, default to about:blank */
-	if (membed->priv->no_page > 0 && 
-	    (state &  GTK_MOZ_EMBED_FLAG_STOP) &&
-	    (state &  GTK_MOZ_EMBED_FLAG_IS_DOCUMENT))
-	{
-		galeon_embed_load_url (GALEON_EMBED(membed), "about:blank");
-		membed->priv->no_page = 0;
-	}
-	
-	if (!membed->priv->wrapper)
-	{
-		membed->priv->wrapper = new GaleonWrapper ();
-
-		nsresult result;
-		result = membed->priv->wrapper->Init (GTK_MOZ_EMBED(embed));
-		if (NS_FAILED(result))
-		{
-                	g_warning ("Wrapper initialization failed");
-		}
-	}
-
 	for (i = 0; conversion_map[i].state != 0; i++)
 	{
 		if (state & conversion_map[i].state)
@@ -1492,6 +1513,35 @@ mozilla_embed_net_state_all_cb (GtkMozEmbed *embed, const char *aURI,
 	g_signal_emit_by_name (membed, "ge_net_state", aURI, estate);
 }
 
+static void
+mozilla_embed_net_stop_cb (GtkMozEmbed *embed, MozillaEmbed *membed)
+{
+	/* LOAD HACK: Check to see if a real url is currently loaded.
+	 * If it is, then forget the "loading_url", and mark the embed
+	 * as having had a url loaded */
+	if (!membed->priv->loaded_url)
+	{
+		char *l = mozilla_embed_get_real_location (MOZILLA_EMBED(embed));
+		if (strcmp ("about:blank", l))
+		{
+			/* We have loaded a real page */
+			g_free (membed->priv->loading_url);
+			membed->priv->loading_url = 0;
+			membed->priv->loaded_url = TRUE;
+		}
+		g_free (l);
+	}
+	g_signal_emit_by_name (membed, "ge_net_stop" );
+
+}
+
+static void
+mozilla_embed_net_start_cb (GtkMozEmbed *embed, MozillaEmbed *membed)
+{
+	g_signal_emit_by_name (membed, "ge_net_start" );
+
+}
+
 static void
 mozilla_embed_progress_cb (GtkMozEmbed *embed, const char *aURI,
                            gint curprogress, gint maxprogress, 
@@ -1505,24 +1555,22 @@ static void
 mozilla_embed_link_message_cb (GtkMozEmbed *embed, 
 			       MozillaEmbed *membed)
 {
-	nsXPIDLString message;
+	char *msg = gtk_moz_embed_get_link_message (embed);
 
-	*getter_Copies(message) = gtk_moz_embed_get_link_message_unichar (embed);
-	
-	g_signal_emit_by_name (membed, "ge_link_message",
-			       NS_ConvertUCS2toUTF8(message).get());
+	g_signal_emit_by_name (membed, "ge_link_message", msg );
+
+	g_free (msg);
 }
 
 static void
 mozilla_embed_js_status_cb (GtkMozEmbed *embed, 
 			    MozillaEmbed *membed)
 {
-	nsXPIDLString status;
+	char *msg = gtk_moz_embed_get_js_status (embed);
 
-	*getter_Copies(status) = gtk_moz_embed_get_js_status_unichar (embed);
-	
-	g_signal_emit_by_name (membed, "ge_js_status",
-			       NS_ConvertUCS2toUTF8(status).get());
+	g_signal_emit_by_name (membed, "ge_js_status", msg);
+
+	g_free (msg);
 }
 
 static void
@@ -1532,7 +1580,6 @@ mozilla_embed_visibility_cb (GtkMozEmbed *embed, gboolean visibility,
 	g_signal_emit_by_name (membed, "ge_visibility", visibility); 
 
 	GaleonWrapper *wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
-	if (wrapper == NULL) return;
 
 	nsresult rv;
 	nsCOMPtr<nsIWindowWatcher> wwatch
@@ -1557,14 +1604,11 @@ static gint
 mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event, 
 				  MozillaEmbed *membed)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	GaleonEmbedEvent *info;
 	gint return_value = 0;
-	GaleonWrapper *wrapper;
 	nsresult result;
 
-	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
-	g_return_val_if_fail(wrapper != NULL, G_FAILED);
-	
 	info = galeon_embed_event_new();
 	
 	EventContext ctx;
@@ -1596,14 +1640,11 @@ static gint
 mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event, 
 				 MozillaEmbed *membed)
 {
+	GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	GaleonEmbedEvent *info;
 	gint return_value = 0;
-	GaleonWrapper *wrapper;
 	nsresult result;
 	
-	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
-	
 	info = galeon_embed_event_new ();
 	
 	EventContext ctx;
@@ -1642,14 +1683,14 @@ mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	}
 
 	GaleonWrapper *wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 
 	GaleonEmbedEvent *info;
 	info = galeon_embed_event_new ();
 
 	gboolean ret = FALSE;
 
-	// Just check for Shift-F10 so that we know to popup the context menu.
+	// Just check for Shift-F10 and the Contect menu key
+	// so that we know to popup the context menu.
 	//
 	// The DOM_VK_* keycodes are not compatible with the keycodes defined
 	// in GDK, so making a generic dom_key_down signal is probably not
@@ -1660,8 +1701,13 @@ mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	ctx.Init (wrapper);
 	rv = ctx.GetKeyEventInfo (ev, info);
 	if (NS_SUCCEEDED(rv) &&
-	    (info->keycode == nsIDOMKeyEvent::DOM_VK_F10 &&
-	     info->modifier == GDK_SHIFT_MASK))
+	    ((info->keycode == nsIDOMKeyEvent::DOM_VK_F10 &&
+	      info->modifier == GDK_SHIFT_MASK) 
+#if MOZILLA_SNAPSHOT > 14
+	     || (info->keycode == nsIDOMKeyEvent::DOM_VK_CONTEXT_MENU &&
+		 !info->modifier)
+#endif
+		    ))
 	{
 		// Translate relative coordinates to absolute values, and try
 		// to avoid covering links by adding a little offset.
@@ -1718,7 +1764,10 @@ mozilla_embed_new_window_cb (GtkMozEmbed *embed,
 		{ GTK_MOZ_EMBED_FLAG_DEFAULTCHROME, EMBED_CHROME_DEFAULT },
 		{ GTK_MOZ_EMBED_FLAG_MENUBARON, EMBED_CHROME_MENUBARON },
 		{ GTK_MOZ_EMBED_FLAG_TOOLBARON, EMBED_CHROME_TOOLBARON },
-		{ GTK_MOZ_EMBED_FLAG_PERSONALTOOLBARON, EMBED_CHROME_PERSONALTOOLBARON },
+		/* Map the bookmarks bar to the default, as otherwise _blank
+		 * pages don't honour the gconf settings, and always have the
+		 * bookmarks bar, which just confuses people (bug 122410) */
+		{ GTK_MOZ_EMBED_FLAG_PERSONALTOOLBARON, EMBED_CHROME_DEFAULT },
 		{ GTK_MOZ_EMBED_FLAG_STATUSBARON, EMBED_CHROME_STATUSBARON },
 		{ GTK_MOZ_EMBED_FLAG_WINDOWRAISED, EMBED_CHROME_WINDOWRAISED },
 		{ GTK_MOZ_EMBED_FLAG_WINDOWLOWERED, EMBED_CHROME_WINDOWLOWERED },
@@ -1740,6 +1789,8 @@ mozilla_embed_new_window_cb (GtkMozEmbed *embed,
 
 	g_assert (new_embed != NULL);
 	
+	gtk_moz_embed_set_chrome_mask (GTK_MOZ_EMBED(new_embed), chromemask);
+
 	*newEmbed = GTK_MOZ_EMBED(new_embed);
 }
 
@@ -1750,8 +1801,15 @@ mozilla_embed_security_change_cb (GtkMozEmbed *embed,
 				  MozillaEmbed *membed)
 {
 	EmbedSecurityLevel level;
+	nsresult result;
+
+        nsCOMPtr<nsIChannel> channel;
+	channel = do_QueryInterface ((nsIRequest*)request, 
+				     &result);
+
+	GaleonWrapper *wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
+	wrapper->SetChannel (channel);	
 
-	membed->priv->request = (nsIRequest *) request;
 	membed->priv->security_state = state;
 	level = mozilla_embed_security_level (membed);
 
@@ -1799,6 +1857,31 @@ stylesheet_find_func (gconstpointer a, gconstpointer b)
 	return strcmp (sheet->name, name);;
 }
 
+static gresult
+impl_get_page_properties(GaleonEmbed *embed,
+			 EmbedPageProperties **retProperties)
+{
+	nsresult result;
+
+        GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+
+	EmbedPageProperties *props = g_new0(EmbedPageProperties, 1);
+
+	result = wrapper->GetPageProperties (props);
+
+	(*retProperties) = props;
+
+	return NS_FAILED (result) ? G_FAILED : G_OK;
+}
+
+
+static gresult
+impl_show_page_certificate (GaleonEmbed *embed)
+{
+        GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
+	wrapper->ShowPageCertificate();
+	return G_OK;
+}
 
 static gresult
 impl_get_stylesheets(GaleonEmbed *embed,
@@ -1807,7 +1890,6 @@ impl_get_stylesheets(GaleonEmbed *embed,
 	nsresult result;
 
         GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-        g_return_val_if_fail(wrapper != NULL, G_FAILED);
 
 	nsCOMPtr<nsIDOMStyleSheetList> list;
 	result = wrapper->GetStyleSheets(getter_AddRefs(list));
@@ -1892,7 +1974,6 @@ impl_get_selected_stylesheet(GaleonEmbed *embed,
 	nsresult result;
 
         GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-        g_return_val_if_fail(wrapper != NULL, G_FAILED);
 	
 	nsCOMPtr<nsIDOMStyleSheetList> list;
 	result = wrapper->GetStyleSheets(getter_AddRefs(list));
@@ -1960,7 +2041,6 @@ impl_set_stylesheet(GaleonEmbed *embed,
 	nsresult result;
 
         GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-        g_return_val_if_fail(wrapper != NULL, G_FAILED);
 	
 	nsCOMPtr<nsIDOMStyleSheetList> list;
 	result = wrapper->GetStyleSheets(getter_AddRefs(list));
@@ -2014,11 +2094,14 @@ impl_apply_user_stylesheet(GaleonEmbed *embed,
 			   EmbedStyleSheet **retSheet)
 {
         GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-        g_return_val_if_fail(wrapper != NULL, G_FAILED);
 
 	nsIStyleSheet *item = nsnull;
 
-	gchar *tmp = g_strconcat("file://", sheetfile, NULL);
+	gchar *path = galeon_embed_utils_get_user_sheet_path(sheetfile);
+
+	gchar *tmp = g_strconcat("file://", path, NULL);
+	g_free(path);
+
 	wrapper->LoadOverrideStyleSheet(tmp, &item);
 	g_free (tmp);
 
@@ -2035,16 +2118,16 @@ static gresult
 impl_remove_user_stylesheet(GaleonEmbed *embed,
 			    EmbedStyleSheet *sheet)
 {
+	if (!sheet) return G_OK;
+
         GaleonWrapper *wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-        g_return_val_if_fail(wrapper != NULL, G_FAILED);
 
 	nsIStyleSheet *item = NS_STATIC_CAST(nsIStyleSheet*, sheet->sheet);
-	wrapper->RemoveOverrideStyleSheet(item);
 
-	g_free(sheet->name);
-	g_free(sheet);
+	nsresult rv = NS_OK;
+	rv = wrapper->RemoveOverrideStyleSheet(item);
 
-	return G_OK;
+	return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
 }
 
 static gboolean
