@@ -29,7 +29,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <locale.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <stdlib.h>
 #include <sys/utsname.h>
 #include "nsBuildID.h"
@@ -214,21 +214,6 @@ custom_notifiers [] =
 	{NULL, NULL}
 };
 
-static void
-mozilla_font_size_notifier (GConfClient *client,
-		            guint cnxn_id,
-		            GConfEntry *entry,
-		            char *pref)
-{
-	char key[255];
-	
-	if (entry->value == NULL) return;
-
-	sprintf (key, "font.%s", pref);
-
-	mozilla_prefs_set_int (key, eel_gconf_get_integer (entry->key));
-}
-
 static void
 mozilla_proxy_mode_notifier (GConfClient *client,
 		             guint cnxn_id,
@@ -321,13 +306,14 @@ mozilla_proxy_autoconfig_notifier (GConfClient *client,
 
 	url = eel_gconf_get_string (entry->key);
 
-	if (url)
+	if (url && url[0] != '\0')
 	{
 		galeon_embed_shell_load_proxy_autoconf (embed_shell, url);
-		g_free (url);
 	}
+	g_free (url);
 }
 
+
 static void
 add_notification_and_notify (GConfClient		*client,
 			     const char 		*key,
@@ -362,7 +348,7 @@ add_notification_and_notify (GConfClient		*client,
 	{
 		func (client, cnxn_id, entry, user_data);
 	}
-	gconf_entry_free (entry);
+	gconf_entry_unref (entry);
 }
 
 void 
@@ -408,49 +394,49 @@ mozilla_notifiers_init(MozillaEmbedShell *shell)
 	}
 
 	/* fonts notifiers */
-	for (i = 0; i < LANG_ENC_NUM; i++)
+	for (i = 0; i < num_font_languages; i++)
 	{
 		int k;
 		char *types [] = { "serif", "sans-serif", "cursive", "fantasy", "monospace" };
 		char key[255];
 		char *info;
-		
+		const char* language = font_languages[i].key;
+
 		for (k = 0; k < 5; k++)
 		{
-			info = g_strconcat (types[k], ".", lang_encode_item[i], NULL);
+			info = g_strconcat (types[k], ".", language, NULL);
 			
 			sprintf (key, "%s_%s_%s", CONF_RENDERING_FONT, 
-			 	 types[k], 
-                 	 	 lang_encode_item[i]);
-			galeon_notification_add (key,
-						 (GConfClientNotifyFunc)mozilla_font_notifier,
-						  info,
-						  &mozilla_notifiers);
+			 	 types[k], language);
+			add_notification_and_notify (client,
+						     key,
+						     (GConfClientNotifyFunc)mozilla_font_notifier,
+						     info);
 			font_infos = g_list_append (font_infos, info);
 		}
 
-		sprintf (key, "%s_%s", CONF_RENDERING_FONT_MIN_SIZE, lang_encode_item[i]);
-		info = g_strconcat ("minimum-size", ".", lang_encode_item[i], NULL);
-		galeon_notification_add (key,
-					 (GConfClientNotifyFunc)mozilla_font_size_notifier,
-					 info,
-				         &mozilla_notifiers);
+		sprintf (key, "%s_%s", CONF_RENDERING_FONT_MIN_SIZE, language);
+		info = g_strconcat ("font.minimum-size", ".", language, NULL);
+		add_notification_and_notify (client,
+					     key,
+					     (GConfClientNotifyFunc)generic_mozilla_int_notifier,
+					     info);
 		font_infos = g_list_append (font_infos, info);
 
-		sprintf (key, "%s_%s", CONF_RENDERING_FONT_FIXED_SIZE, lang_encode_item[i]);
-		info = g_strconcat ("size.fixed", ".", lang_encode_item[i], NULL);
-		galeon_notification_add (key,
-					 (GConfClientNotifyFunc)mozilla_font_size_notifier,
-					 info,
-				         &mozilla_notifiers);
+		sprintf (key, "%s_%s", CONF_RENDERING_FONT_FIXED_SIZE, language);
+		info = g_strconcat ("font.size.fixed", ".", language, NULL);
+		add_notification_and_notify (client,
+					     key,
+					     (GConfClientNotifyFunc)generic_mozilla_int_notifier,
+					     info);
 		font_infos = g_list_append (font_infos, info);
 
-		sprintf (key, "%s_%s", CONF_RENDERING_FONT_VAR_SIZE, lang_encode_item[i]);
-		info = g_strconcat ("size.variable", ".", lang_encode_item[i], NULL);
-		galeon_notification_add (key,
-					 (GConfClientNotifyFunc)mozilla_font_size_notifier,
-					 info,
-				         &mozilla_notifiers);
+		sprintf (key, "%s_%s", CONF_RENDERING_FONT_VAR_SIZE, language);
+		info = g_strconcat ("font.size.variable", ".", language, NULL);
+		add_notification_and_notify (client,
+					     key,
+					     (GConfClientNotifyFunc)generic_mozilla_int_notifier,
+					     info);
 		font_infos = g_list_append (font_infos, info);		
 	}
 }
