@@ -18,13 +18,14 @@
 
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libxml/tree.h>
+#include <libxml/xmlwriter.h>
 #include <string.h>
-#include <gtk/gtktoolbar.h>
 #include <sys/stat.h>
 
 #include "galeon-embed-persist.h"
 #include "galeon-config.h"
 #include "pixbuf-cache.h"
+#include "gul-general.h"
 
 #include "galeon-favicon-cache.h"
 
@@ -48,11 +49,10 @@ struct GaleonFaviconCachePrivate
 	GHashTable *favicon_to_pixbuf;
 
 	guint timeout;
+	gboolean dirty;
 
 	char *filename;
 	char *directory;
-
-	GdkPixbuf *default_pixbuf;
 };
 
 enum
@@ -117,8 +117,6 @@ galeon_favicon_cache_class_init (GaleonFaviconCacheClass *klass)
 static void
 galeon_favicon_cache_init (GaleonFaviconCache *cache)
 {
-	GtkWidget *dummy;
-
 	cache->priv = g_new0 (GaleonFaviconCachePrivate, 1);
 
 	cache->priv->url_to_pixbuf     = g_hash_table_new_full (g_str_hash,
@@ -138,6 +136,8 @@ galeon_favicon_cache_init (GaleonFaviconCache *cache)
 						   GALEON_DIR,
 						   "favicon_cache/",
 						   NULL);
+
+	cache->priv->dirty = FALSE;
 	
 	if (g_file_test (cache->priv->directory, G_FILE_TEST_IS_DIR) == FALSE)
 	{
@@ -152,12 +152,6 @@ galeon_favicon_cache_init (GaleonFaviconCache *cache)
 		}
 	}
 
-	dummy = gtk_toolbar_new ();
-	cache->priv->default_pixbuf = gtk_widget_render_icon (dummy,
-							      GALEON_STOCK_BOOKMARK,
-							      GTK_ICON_SIZE_MENU, NULL);
-	gtk_widget_destroy (dummy);
-
 	galeon_favicon_cache_sync_from_disc (cache);
 
 	/* sync to disc every 10 minutes */
@@ -185,8 +179,6 @@ galeon_favicon_cache_finalize (GObject *object)
 	g_hash_table_destroy (cache->priv->url_to_pixbuf);
 	g_hash_table_destroy (cache->priv->favicon_to_pixbuf);
 
-	g_object_unref (G_OBJECT (cache->priv->default_pixbuf));
-
 	g_free (cache->priv->filename);
 	g_free (cache->priv->directory);
 
@@ -217,7 +209,9 @@ galeon_favicon_cache_lookup (GaleonFaviconCache *cache,
 	g_return_val_if_fail (GALEON_IS_FAVICON_CACHE (cache), NULL);
 
 	if (url == NULL)
-		return cache->priv->default_pixbuf;
+	{
+		return NULL;
+	}
 
 	cache_url = galeon_favicon_cache_url (url);
 	ret = galeon_favicon_cache_lookup_direct (cache, cache_url);
@@ -233,13 +227,12 @@ galeon_favicon_cache_lookup_direct (GaleonFaviconCache *cache,
 	GdkPixbuf *ret;
 
 	if (cache_url == NULL)
-		return cache->priv->default_pixbuf;
+	{
+		return NULL;
+	}
 	
 	ret = g_hash_table_lookup (cache->priv->url_to_pixbuf,
 				   cache_url);
-	if (ret == NULL)
-		ret = cache->priv->default_pixbuf;
-
 	return ret;
 }
 
@@ -295,33 +288,75 @@ galeon_favicon_cache_insert (GaleonFaviconCache *cache,
 			      cache_url,
 			      pixbuf);
 
+	cache->priv->dirty = TRUE;
 	g_signal_emit (G_OBJECT (cache), galeon_favicon_cache_signals[CHANGED], 0, cache_url);
 }
 
 static void
-save_item (const char *key, GObject *value, xmlNodePtr parent)
+save_item (const char *key, GObject *value, xmlTextWriterPtr writer)
 {
-	xmlNodePtr node;
+	xmlTextWriterStartElement (writer, "entry");
 
-	node = xmlNewChild (parent, NULL, "entry", NULL);
+	xmlTextWriterWriteAttribute (writer, "url", key);
+	xmlTextWriterWriteAttribute (writer, "favicon", 
+				     g_object_get_data (value, "location"));
 
-	xmlSetProp (node, "url", key);
-	xmlSetProp (node, "favicon", g_object_get_data (value, "location"));
+	xmlTextWriterEndElement (writer);
+}
+
+static int
+galeon_favicon_cache_save_to_file (GaleonFaviconCache *cache,
+				   xmlTextWriterPtr writer)
+{
+	int ret;
+
+	ret = xmlTextWriterStartDocument (writer, "1.0", NULL, NULL);
+	if (ret < 0) return ret;
+	ret = xmlTextWriterStartElement (writer, "GaleonFaviconCache");
+	if (ret < 0) return ret;
+	
+	g_hash_table_foreach (cache->priv->url_to_pixbuf, (GHFunc) save_item, writer);
+
+	ret = xmlTextWriterEndElement (writer); /* root */
+	if (ret < 0) return ret;
+
+	ret = xmlTextWriterEndDocument (writer);
+	if (ret < 0) return ret;
+	
+	return 0;
 }
 
 static void
 galeon_favicon_cache_sync_to_disc (GaleonFaviconCache *cache)
 {
-	xmlDocPtr doc;
+	xmlTextWriterPtr writer;
+	gchar *tmpfile;
+	int ret;
 
-	xmlIndentTreeOutput = TRUE;
-	doc = xmlNewDoc ("1.0");
-	doc->children = xmlNewDocNode (doc, NULL, "GaleonFaviconCache", NULL);
+	if (!cache->priv->dirty)
+		return;
 
-	g_hash_table_foreach (cache->priv->url_to_pixbuf, (GHFunc) save_item, doc->children);
+	tmpfile = g_strconcat (cache->priv->filename, ".tmp", NULL);
 
-	xmlSaveFormatFile (cache->priv->filename, doc, 1);
-	xmlFreeDoc (doc);
+	writer = xmlNewTextWriterFilename (tmpfile, 0);
+	xmlTextWriterSetIndent (writer, 1);
+	xmlTextWriterSetIndentString (writer, "  ");
+
+	ret = galeon_favicon_cache_save_to_file (cache, writer);
+	
+	if (ret < 0)
+	{
+		g_warning ("Failed writing to %s", cache->priv->filename);
+	}
+	else
+	{
+		gul_general_switch_temp_file (cache->priv->filename, tmpfile);
+	}
+
+	g_free (tmpfile);
+	xmlFreeTextWriter (writer);
+
+	cache->priv->dirty = FALSE;
 }
 
 static void
@@ -370,11 +405,15 @@ galeon_favicon_cache_url (const char *url)
 	int clean_url_len;
 
 	if (url == NULL)
+	{
 		return NULL;
+	}
 
 	uri = gnome_vfs_uri_new (url);
 	if (uri == NULL)
+	{
 		return NULL;
+	}
 
 	clean_url = gnome_vfs_uri_to_string (uri,
 					     GNOME_VFS_URI_HIDE_USER_NAME |
@@ -420,10 +459,14 @@ galeon_favicon_cache_dest (GaleonFaviconCache *cache, const char *url)
 	char *slashpos, *dest;
 
 	if (clean_url == NULL)
+	{
 		return NULL;
+	}
 
 	while ((slashpos = strstr (clean_url, "/")) != NULL)
+	{
 		*slashpos = '_';
+	}
 	
 	dest = g_build_filename (cache->priv->directory, clean_url, NULL);
 	
@@ -464,6 +507,8 @@ galeon_favicon_cache_insert_from_url (GaleonFaviconCache *cache,
 			  cache);
 
 	galeon_embed_persist_save (persist);
+
+	g_object_unref (persist);
 }
 
 static void
@@ -473,6 +518,4 @@ favicon_download_completed_cb (GaleonEmbedPersist *persist,
 	galeon_favicon_cache_insert (cache,
 				     g_object_get_data (G_OBJECT (persist), "url"),
 				     g_object_get_data (G_OBJECT (persist), "favicon"));
-
-	g_object_unref (G_OBJECT (persist));
 }
