@@ -24,12 +24,15 @@
 #include "galeon-automation.h"
 #include "galeon-window.h"
 #include "gul-state.h"
+#include "galeon-debug.h"
+#include "hig-alert.h"
 #include "GaleonAutomation.h"
 
 #include <libbonoboui.h>
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-window-icon.h>
+#include <libgnomeui/gnome-app-helper.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <glade/glade-init.h>
 #ifdef ENABLE_NLS
@@ -38,15 +41,12 @@
 
 #define GALEON_FACTORY_OAFIID "OAFIID:GNOME_Galeon_Automation_Factory"
 
-static gboolean
-galeon_main_automation_init (void);
 static gint
 galeon_main_translate_url_arguments (poptContext context, gchar ***urls);
 static void
 galeon_main_start (void);
 
 /* FIXME */
-GnomeProgram *program;
 CORBA_Environment corba_env;			/* Global for downloader	*/
 static gboolean open_in_existing      = FALSE;  /* load in existing window?     */
 static gboolean open_in_new_tab       = FALSE;  /* force open in a new tab?     */
@@ -61,7 +61,6 @@ static gboolean quit_option           = FALSE;  /* --quit
 static gboolean galeon_server_mode    = FALSE;
 static gboolean open_as_nautilus_view = FALSE;
 
-static BonoboObject *automation_object;
 static gint n_urls; 
 static gchar **url;
 
@@ -126,6 +125,7 @@ main (int argc, char *argv[])
 	poptContext context;
         GValue context_as_value = { 0 };
 	GnomeProgram *program;
+	CORBA_Object factory;
 	
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
@@ -156,13 +156,28 @@ main (int argc, char *argv[])
 	/* load arguments that aren't regular options (urls to load) */
         n_urls = galeon_main_translate_url_arguments (context, &url);
 
-	if (galeon_main_automation_init ())
+	g_value_unset (&context_as_value);
+
+	factory = bonobo_activation_activate_from_id 
+		(GALEON_FACTORY_OAFIID,
+		 Bonobo_ACTIVATION_FLAG_EXISTING_ONLY, 
+		 NULL, NULL);
+
+	if (factory != NULL)
 	{
+		/* Connect to existing instance */
+		galeon_main_start ();
+		gdk_notify_startup_complete ();
+	}
+	else
+	{
+		galeon_debug_init ();
+
 		gul_state_init ();
 		
 		gnome_vfs_init ();
 		
-		glade_gnome_init ();
+		glade_init ();
 
 		gnome_window_icon_set_default_from_file (DATADIR "/pixmaps/galeon.png");
 		
@@ -181,12 +196,12 @@ main (int argc, char *argv[])
 		
 		bonobo_main ();
 		
+		gul_state_shutdown ();
+
+		gnome_accelerators_sync ();
+
 		gnome_vfs_shutdown ();
 	} 
-	else
-	{
-		gdk_notify_startup_complete ();
-	}
 
 	return 0;
 }
@@ -201,6 +216,12 @@ galeon_main_start (void)
 	
 	gaserver = bonobo_activation_activate_from_id ("OAFIID:GNOME_Galeon_Automation",
 						       0, NULL, &corba_env);
+	
+	if (corba_env._major != CORBA_NO_EXCEPTION)
+	{
+		g_warning ("Error: unable to initialize corba: %s\n", 
+			   CORBA_exception_id (&corba_env));
+	} 
 
 	/* if none of the open in tab/window/existing options is given, fall
 	 * back to open in a new window, otherwise the url can get opened in a
@@ -212,28 +233,34 @@ galeon_main_start (void)
 
 	if (gaserver == NULL)
 	{
+		/* At this point the galeon_shell will have initialized the
+		 * AutomationFactory, that obviously worked, so the .server file
+		 * exists, therefore this was probably because a previous instance
+		 * locked up */
+
 		GtkWidget *dialog;
-		dialog = gtk_message_dialog_new 
-			(NULL,
-                         GTK_DIALOG_MODAL,
-                         GTK_MESSAGE_ERROR,
-                         GTK_BUTTONS_CLOSE,
-                         "Galeon can't be used now. "
-                         "Running the command \"bonobo-slay\" "
-                         "from the console may fix the problem. If not, "
-                         "you can try rebooting the computer or "
-                         "installing Galeon again.\n\n"
-                         "Bonobo couldn't locate the GNOME_Galeon_Automation.server. ");
+		dialog = hig_alert_new (NULL,
+					GTK_DIALOG_MODAL,
+					HIG_ALERT_ERROR,
+					_("Galeon failed to start."),
+					_("The existing Galeon instance is not responding. "
+					  "Terminating the <tt>\"galeon-bin\"</tt> processes "
+					  "may fix the problem."),
+					GTK_STOCK_OK,
+					GTK_RESPONSE_OK,
+					NULL);
 		gtk_dialog_run (GTK_DIALOG (dialog));
-		
+		gtk_widget_destroy (dialog);
+		exit(1);
 	}
 	/* Server mode */
-	else if (galeon_server_mode)
+	if (galeon_server_mode && !close_option)
 	{
 		GNOME_GaleonAutomation_setServerMode (gaserver, TRUE, &corba_env);
 	}
+
 	/* load the session if requested */
-	else if (session_filename)
+	if (session_filename)
 	{
 		GNOME_GaleonAutomation_loadSession 
 			(gaserver, session_filename, &corba_env);
@@ -251,7 +278,8 @@ galeon_main_start (void)
 	}
 	/* provided with urls? */
 	else if (n_urls == 0 && 
-		 !open_as_nautilus_view)
+		 !open_as_nautilus_view &&
+		 !galeon_server_mode)
 	{
 		/* no, open a default window */
 		GNOME_GaleonAutomation_loadurl 
@@ -291,28 +319,6 @@ galeon_main_start (void)
 	CORBA_exception_free (&corba_env);
 }
 
-static gboolean
-galeon_main_automation_init (void)
-{
-	CORBA_Object factory;
-
-	factory = bonobo_activation_activate_from_id 
-		(GALEON_FACTORY_OAFIID,
-		 Bonobo_ACTIVATION_FLAG_EXISTING_ONLY, 
-		 NULL, NULL);
-
-	if (!factory)
-	{
-		automation_object = galeon_automation_new ();
-		return TRUE;
-	}
-	else
-	{
-		galeon_main_start ();
-		return FALSE;
-	}
-}
-
 /**
  * translate_url_arguments: gather URL arguments and expand them fully
  * with realpath if they're filenames
