@@ -30,14 +30,11 @@
 #include "print-dialog.h"
 #include "galeon-embed-autoscroller.h"
 #include "galeon-embed-manual-scroller.h"
+#include "galeon-debug.h"
 
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
 
-#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
-
 static void		gnv_embed_location_cb 			(GaleonEmbed *embed, 
 								 GaleonNautilusView *view);
 static void		gnv_embed_title_cb 			(GaleonEmbed *embed, 
@@ -79,9 +76,6 @@ static void		gnv_zoomable_zoom_to_fit_cb		(BonoboZoomable *zoomable,
 static void		gnv_zoomable_zoom_to_default_cb		(BonoboZoomable *zoomable,
 								 GaleonNautilusView *view);
 /* commands */
-static void		gnv_cmd_set_charset			(BonoboUIComponent *uic, 
-								 EncodingMenuData *data, 
-								 const char* verbname);
 static void 		gnv_cmd_file_print			(BonoboUIComponent *uic, 
 								 GaleonNautilusView *view, 
 								 const char* verbname);
@@ -103,6 +97,12 @@ static void 		gnv_popup_cmd_frame_in_new_window	(BonoboUIComponent *uic,
 								 GaleonEmbedPopup *popup, 
 								 const char* verbname);
 
+/* Fix up functions for libnautilus 2.5+ */
+#ifdef HAVE_NAUTILUS_VIEW_OPEN_LOCATION
+#  define nautilus_view_open_location_force_new_window(a,b,c) \
+              nautilus_view_open_location \
+                  (a,b,Nautilus_ViewFrame_OPEN_ACCORDING_TO_MODE,0,c)
+#endif
 
 static float preferred_zoom_levels[] = {
 	0.2, 0.4, 0.6, 0.8,
@@ -164,9 +164,6 @@ BonoboUIVerb galeon_verbs [] = {
         BONOBO_UI_VERB_END
 };
 
-#define CHARSET_MENU_PATH "/menu/View/Encoding"
-
-
 BONOBO_CLASS_BOILERPLATE (GaleonNautilusView, galeon_nautilus_view,
 			  NautilusView, NAUTILUS_TYPE_VIEW)
 
@@ -510,7 +507,8 @@ gnv_embed_new_window_cb (GaleonEmbed *embed, GaleonEmbed **new_embed,
 					  EMBED_CHROME_OPENASPOPUP);
 	}
 	new_tab = galeon_tab_new ();
-	galeon_window_add_tab (window, new_tab, FALSE);
+	galeon_window_add_tab (window, new_tab,
+			       GALEON_WINDOW_ADD_TAB_LAST, FALSE);
 	
 	*new_embed = galeon_tab_get_embed (new_tab);
 }
@@ -527,11 +525,6 @@ gnv_bonobo_control_activate_cb (BonoboControl *control, gboolean state, GaleonNa
 						 "nautilus-galeon-view-ui.xml", "GaleonNutilusView");
 		g_return_if_fail (BONOBO_IS_UI_COMPONENT (p->ui));
 		
-		galeon_embed_utils_build_charsets_submenu (p->ui,
-							   CHARSET_MENU_PATH,
-							   (BonoboUIVerbFn) gnv_cmd_set_charset,
-							   view);
-
 		bonobo_ui_component_add_verb_list_with_data (p->ui, galeon_verbs, view);
 	}
 }
@@ -600,22 +593,6 @@ gnv_popup_cmd_frame_in_new_window (BonoboUIComponent *uic,
 						      location, NULL);
 }
 
-void 
-gnv_cmd_set_charset (BonoboUIComponent *uic, 
-		     EncodingMenuData *data, 
-		     const char* verbname)
-{
-	GaleonNautilusView *view = data->data;
-	GaleonNautilusViewPrivate *p;
-
-	g_return_if_fail (GALEON_IS_NAUTILUS_VIEW (view));
-
-	p = view->priv;
-	
-	DEBUG_MSG ((data->encoding));
-	galeon_embed_set_charset (p->embed, data->encoding);
-}
-
 static void
 gnv_cmd_file_print (BonoboUIComponent *uic, 
 		    GaleonNautilusView *view, 
@@ -626,10 +603,12 @@ gnv_cmd_file_print (BonoboUIComponent *uic,
 	
 	dialog = print_dialog_new (p->embed, NULL);
 
-	//g_signal_connect (G_OBJECT(dialog),
-	//		  "preview",
-	//		  G_CALLBACK (print_dialog_preview_cb),
-	//		  window);
+	/*
+	g_signal_connect (G_OBJECT(dialog),
+			  "preview",
+			  G_CALLBACK (print_dialog_preview_cb),
+			  window);
+	*/
 
 	galeon_dialog_set_modal (dialog, TRUE);
 	galeon_dialog_show (dialog);
@@ -829,13 +808,13 @@ vfs_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data)
 {
 	GaleonNautilusView *view = data;
 
-	DEBUG_MSG (("+%s GnomeVFSResult: %u\n", G_GNUC_FUNCTION, (unsigned)result));
+	LOG ("+%s GnomeVFSResult: %u", G_GNUC_FUNCTION, (unsigned)result);
 
 	if (result != GNOME_VFS_OK)
 	{
 		gtk_moz_embed_close_stream (GTK_MOZ_EMBED (view->priv->embed->mozembed));
 		/* NOTE: the view may go away after a call to report_load_failed */
-		DEBUG_MSG ((">nautilus_view_report_load_failed\n"));
+		LOG (">nautilus_view_report_load_failed");
 		nautilus_view_report_load_failed (view->priv->nautilus_view);
 	} else {
 		if (view->priv->vfs_read_buffer == NULL) {
@@ -844,7 +823,7 @@ vfs_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data)
 		gtk_moz_embed_open_stream (GTK_MOZ_EMBED (view->priv->embed->mozembed), "file:///", "text/html");
 		gnome_vfs_async_read (handle, view->priv->vfs_read_buffer, VFS_READ_BUFFER_SIZE, vfs_read_cb, view);
 	}
-	DEBUG_MSG (("-%s\n", G_GNUC_FUNCTION));
+	LOG ("-%s", G_GNUC_FUNCTION);
 }
 
 /**
@@ -861,7 +840,7 @@ vfs_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer buffer
 {
 	GaleonNautilusView *view = data;
 
-	DEBUG_MSG (("+%s %ld/%ld bytes\n", G_GNUC_FUNCTION, (long)bytes_requested, (long) bytes_read));
+	LOG ("+%s %ld/%ld bytes", G_GNUC_FUNCTION, (long)bytes_requested, (long) bytes_read);
 
 	if (bytes_read != 0) {
 		gtk_moz_embed_append_data (GTK_MOZ_EMBED (view->priv->embed->mozembed), buffer, bytes_read);
@@ -875,15 +854,15 @@ vfs_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer buffer
 		
 		gnome_vfs_async_close (handle, (GnomeVFSAsyncCloseCallback) gtk_true, NULL);
 
-		DEBUG_MSG ((">nautilus_view_report_load_complete\n"));
+		LOG (">nautilus_view_report_load_complete");
 		nautilus_view_report_load_complete (view->priv->nautilus_view);
 
-		DEBUG_MSG (("=%s load complete\n", G_GNUC_FUNCTION));
+		LOG ("=%s load complete", G_GNUC_FUNCTION);
     	} else {
 		gnome_vfs_async_read (handle, view->priv->vfs_read_buffer, VFS_READ_BUFFER_SIZE, vfs_read_cb, view);
 	}
 
-	DEBUG_MSG (("-%s\n", G_GNUC_FUNCTION));
+	LOG ("-%s\n", G_GNUC_FUNCTION);
 }
 
 /***********************************************************************************/
@@ -921,7 +900,7 @@ navigate_mozilla_to_nautilus_uri (GaleonNautilusView *view,
 		 * premature realization may cause other issues)
 		 */
 		
-		DEBUG_MSG (("=%s: Postponing navigation request to widget realization\n", G_GNUC_FUNCTION));
+		LOG ("=%s: Postponing navigation request to widget realization", G_GNUC_FUNCTION);
 		/* Note that view->priv->uri is still set below */
 	} else {
 		if (should_mozilla_load_uri_directly (uri)) {
@@ -937,7 +916,7 @@ navigate_mozilla_to_nautilus_uri (GaleonNautilusView *view,
 			old_uri = view->priv->embed->location;
 
 			if (old_uri != NULL && uris_identical (uri, old_uri)) {
-				DEBUG_MSG (("=%s uri's identical, telling galeon to reload\n", G_GNUC_FUNCTION));
+				LOG ("=%s uri's identical, telling galeon to reload", G_GNUC_FUNCTION);
 				embed_reload (view->priv->embed,
 					      GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE);
 			} else {
@@ -945,7 +924,7 @@ navigate_mozilla_to_nautilus_uri (GaleonNautilusView *view,
 			}
 
 		} else {
-			DEBUG_MSG (("=%s loading URI via gnome-vfs\n", G_GNUC_FUNCTION));
+			LOG ("=%s loading URI via gnome-vfs", G_GNUC_FUNCTION);
 			gnome_vfs_async_open (&(view->priv->vfs_handle), uri,
 					      GNOME_VFS_OPEN_READ, GNOME_VFS_PRIORITY_DEFAULT, 
 					      vfs_open_cb, view);
@@ -955,7 +934,7 @@ navigate_mozilla_to_nautilus_uri (GaleonNautilusView *view,
 	g_free (view->priv->uri);
 	view->priv->uri = g_strdup (uri);
 
-	DEBUG_MSG (("=%s current URI is now '%s'\n", G_GNUC_FUNCTION, view->priv->uri));
+	LOG ("=%s current URI is now '%s'", G_GNUC_FUNCTION, view->priv->uri);
 }
 
 /*
