@@ -31,13 +31,17 @@
 #include "gul-glade-gconf.h"
 #include "gul-gobject-misc.h"
 #include "gul-gui-option.h"
+#include "gul-gui.h"
 #include "pixbuf-cache.h"
 #include "prefs-strings.h"
+#include "hig-alert.h"
 
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
+#include <gtk/gtkcelllayout.h>
 #include <gtk/gtkcellrendererpixbuf.h>
 #include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcombo.h>
+#include <gtk/gtkcombobox.h>
+#include <gtk/gtkcomboboxentry.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkiconfactory.h>
 #include <gtk/gtkliststore.h>
@@ -45,10 +49,10 @@
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtkstock.h>
+#include <gtk/gtktreemodelsort.h>
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtktogglebutton.h>
-#include <gtk/gtkmessagedialog.h>
 #include <gtk/gtklabel.h>
 #include <libgnome/gnome-config.h>
 
@@ -85,7 +89,7 @@ struct _GaleonPrefsDialogPrivate {
 	GtkWidget  *language_editor;
 
 	gboolean    fonts_ignore_updates;
-	const char *fonts_encoding;
+	char       *fonts_encoding;
 	GtkWidget  *fonts_serif;
 	GtkWidget  *fonts_sans;
 	GtkWidget  *fonts_cursive;
@@ -124,56 +128,51 @@ end_busy (GaleonPrefsDialog *self)
 }
 
 static void
-fill_option_menu_from_array (GtkWidget          *optionmenu,
-		             const GulGuiOption *options,
-			     guint               n_options)
+fill_combo_box_from_array (GtkWidget          *combobox,
+			   const GulGuiOption *options,
+			   guint               n_options)
 {
-	GSList *list;
-
-	list = gul_gui_option_list_new (options, n_options);
-
-	gul_gui_option_menu_set_options (GTK_OPTION_MENU(optionmenu), list);
-	g_object_weak_ref (G_OBJECT(optionmenu),
-			   (GWeakNotify)gul_gui_option_list_free, list);
+	gul_gui_option_combobox_populate(GTK_COMBO_BOX(combobox),
+					 options, n_options, FALSE);
 }
 
 static void
-on_optionmenu_changed (GtkOptionMenu *optionmenu, const char *gconf_key)
+on_combobox_changed (GtkComboBox *combobox, const char *gconf_key)
 {
-	const char *value;
+	gchar *value = gul_gui_option_combobox_get_value(combobox);
 
-	value = gul_gui_option_menu_get_value (optionmenu);
-	g_return_if_fail (value != NULL);
-
-	eel_gconf_set_integer (gconf_key, atoi (value));
+	if (value && value[0]) {
+		eel_gconf_set_integer(gconf_key, atoi(value));
+	}
+	g_free(value);
 }
 
-/** connect_optionmenu:
+/** connect_combobox:
  * 
- * Connect option menu options to GConf key.  Specialized version for handling
+ * Connect combo box options to GConf key.  Specialized version for handling
  * enums-as-ints as inherited from Galeon 1.2.*
  */
 static void
-connect_optionmenu (GtkWidget          *optionmenu,
-		    const char         *gconf_key,
-		    const GulGuiOption *options,
-		    guint               n_options)
+connect_combobox(GtkWidget          *combobox,
+		 const char         *gconf_key,
+		 const GulGuiOption *options,
+		 guint               n_options)
 {
 	int  intval;
 	char strval[8];
 
-	g_return_if_fail (GTK_IS_OPTION_MENU(optionmenu));
-	g_return_if_fail (gconf_key != NULL);
+	g_return_if_fail(GTK_IS_COMBO_BOX(combobox));
+	g_return_if_fail(gconf_key != NULL);
 
-	fill_option_menu_from_array (optionmenu, options, n_options);
+	fill_combo_box_from_array(combobox, options, n_options);
 
-	intval = eel_gconf_get_integer (gconf_key);
-	g_snprintf (strval, sizeof(strval), "%u", intval);
-	gul_gui_option_menu_set_value (GTK_OPTION_MENU(optionmenu), strval);
+	intval = eel_gconf_get_integer(gconf_key);
+	g_snprintf(strval, sizeof(strval), "%u", intval);
+	gul_gui_option_combobox_set_value (GTK_COMBO_BOX(combobox), strval);
 
-	g_signal_connect_data (G_OBJECT(optionmenu), "changed",
-			       G_CALLBACK(on_optionmenu_changed),
-			       g_strdup (gconf_key), (GClosureNotify)g_free, 0);
+	g_signal_connect_data(G_OBJECT(combobox), "changed",
+			      G_CALLBACK(on_combobox_changed),
+			      g_strdup(gconf_key), (GClosureNotify)g_free, 0);
 }
 
 /************************************************************************
@@ -248,6 +247,7 @@ static const GulGuiOption language_languages[] = {
 	{ N_("Danish"), "da" },
 	{ N_("Dutch"), "nl" },
 	{ N_("English"), "en" },
+	{ N_("English (British)"), "en-GB" },
 	{ N_("Esperanto"), "eo" },
 	{ N_("Estonian"), "et" },
 	{ N_("Faeroese"), "fo" },
@@ -271,6 +271,7 @@ static const GulGuiOption language_languages[] = {
 	{ N_("Norwegian/Nynorsk"), "nn" },
 	{ N_("Norwegian/Bokmaal"), "nb" },
 	{ N_("Norwegian"), "no" },
+	{ N_("Occitan"), "oc" },
 	{ N_("Polish"), "pl" },
 	{ N_("Portuguese"), "pt" },
 	{ N_("Portuguese of Brazil"), "pt-BR" },
@@ -443,36 +444,48 @@ static const GulGuiOption language_autodetects[] = {
 };
 
 static void
-on_language_default_charset_changed (GtkOptionMenu *optionmenu, GaleonPrefsDialog *self)
+on_language_default_charset_changed (GtkComboBox *combobox, GaleonPrefsDialog *self)
 {
-	const char *value;
+	char *value;
 
-	value = gul_gui_option_menu_get_value (optionmenu);
+	value = gul_gui_option_combobox_get_value (combobox);
 	g_return_if_fail (value != NULL);
 
 	eel_gconf_set_string (CONF_LANGUAGE_DEFAULT_CHARSET, value);
+
+	g_free(value);
 }
 
 static void
-language_fill_charsets (GtkWidget *optionmenu, GList *charsets)
+language_fill_charsets (GtkWidget *combobox, GList *charsets)
 {
-	GSList    *options;
 	GList     *l;
+	GtkListStore *store;
 
-	g_return_if_fail (GTK_IS_OPTION_MENU(optionmenu));
+	g_return_if_fail (GTK_IS_COMBO_BOX(combobox));
+
+	/* 
+	 * Adding rows produces a warning if this is not done. *sigh*
+	 */
+	gtk_widget_realize(combobox);
+
+	gul_gui_option_combobox_populate(GTK_COMBO_BOX(combobox),
+					 NULL, 0, TRUE);
+
+	store = GTK_LIST_STORE(gtk_tree_model_sort_get_model(GTK_TREE_MODEL_SORT(
+			gtk_combo_box_get_model(GTK_COMBO_BOX(combobox)))));
 
-	options = NULL;
 	for (l = charsets; l != NULL; l = l->next)
 	{
-		const CharsetInfo *info = l->data;
-		GulGuiOption *item;
+		GtkTreeIter iter;
+		const GaleonEncodingInfo *info = l->data;
 
-		item = gul_gui_option_new (info->title, info->name);
-		options = g_slist_prepend (options, item);
+		gtk_list_store_append(store, &iter);
+		gtk_list_store_set(store, &iter,
+				   OPTION_COL_TITLE, _(info->title),
+				   OPTION_COL_VALUE, info->encoding,
+				   -1);
 	}
-
-	options = gul_gui_option_list_sort (options);
-	gul_gui_option_menu_set_options (GTK_OPTION_MENU(optionmenu), options);
 }
 
 static void
@@ -484,6 +497,7 @@ galeon_prefs_language_construct (GaleonPrefsDialog *self, GladeXML *gxml)
 	GtkWidget *defcharset;
 	GList     *charsets;
 	char      *strval;
+	GaleonEncodings *encodings;
 
 	p->language_editor = galeon_language_editor_new ();
 	galeon_language_editor_set_available
@@ -499,8 +513,8 @@ galeon_prefs_language_construct (GaleonPrefsDialog *self, GladeXML *gxml)
 		NULL);
 
 	g_return_if_fail (GTK_IS_ENTRY(p->language_entry));
-	g_return_if_fail (GTK_IS_OPTION_MENU(autodetect));
-	g_return_if_fail (GTK_IS_OPTION_MENU(defcharset));
+	g_return_if_fail (GTK_IS_COMBO_BOX(autodetect));
+	g_return_if_fail (GTK_IS_COMBO_BOX(defcharset));
 
 	/* languages */
 
@@ -511,20 +525,19 @@ galeon_prefs_language_construct (GaleonPrefsDialog *self, GladeXML *gxml)
 
 	/* autodetect charsets */
 
-	connect_optionmenu (autodetect, 
-			    CONF_LANGUAGE_AUTODETECT_CHARSET,
-			    language_autodetects,
-			    G_N_ELEMENTS(language_autodetects));
+	connect_combobox (autodetect, 
+			  CONF_LANGUAGE_AUTODETECT_CHARSET,
+			  language_autodetects,
+			  G_N_ELEMENTS(language_autodetects));
 
 	/* default charsets */
-
-	galeon_embed_shell_get_charset_titles (embed_shell, NULL, &charsets);
+	galeon_embed_shell_get_encodings (embed_shell, &encodings);
+	charsets = galeon_encodings_get_encodings (encodings, LG_ALL);
 	language_fill_charsets (defcharset, charsets);
-	g_list_foreach (charsets, (GFunc)g_free, NULL);
 	g_list_free (charsets);
 
 	strval = eel_gconf_get_string (CONF_LANGUAGE_DEFAULT_CHARSET);
-	gul_gui_option_menu_set_value (GTK_OPTION_MENU(defcharset), strval);
+	gul_gui_option_combobox_set_value (GTK_COMBO_BOX(defcharset), strval);
 	g_free (strval);
 
 	g_signal_connect (G_OBJECT(defcharset), "changed",
@@ -665,9 +678,10 @@ fonts_load_font (GaleonPrefsDialog *self, const char *type, GtkWidget *combo)
 	char     gconf_key[256];
 	char    *font;
 	GList   *fonts = NULL, *rest = NULL, *l;
+	gint index = -1;
 	gresult  rv;
 
-	g_return_if_fail (GTK_IS_COMBO(combo));
+	g_return_if_fail (GTK_IS_COMBO_BOX(combo));
 
 	/* list all available fonts, show first those that are "good" for the
 	 * given encoding, after those show the remaining fonts */
@@ -703,19 +717,37 @@ fonts_load_font (GaleonPrefsDialog *self, const char *type, GtkWidget *combo)
 		font = g_strdup (fonts != NULL ? fonts->data : "");
 	}
 
+	/* Clear out the old list before populating the new one */
+	gtk_list_store_clear(GTK_LIST_STORE(
+			gtk_combo_box_get_model(GTK_COMBO_BOX(combo))));
+
 	if (fonts == NULL)
 	{
 		gtk_widget_set_sensitive (combo, FALSE);
 	}
 	else
 	{
+		gint i = 0;
+
 		gtk_widget_set_sensitive (combo, TRUE);
-		gtk_combo_set_popdown_strings (GTK_COMBO(combo), fonts);
+		for (l = fonts; l; l = l->next, i++)
+		{
+			gtk_combo_box_append_text(GTK_COMBO_BOX(combo),
+						  (char *)l->data);
+			if (strcmp(font, (char *)l->data) == 0)
+			{
+				index = i;
+			}
+		}
 	}
 	g_list_foreach (fonts, (GFunc)g_free, NULL);
 	g_list_free (fonts);
 
-	gtk_entry_set_text (GTK_ENTRY(GTK_COMBO(combo)->entry), font);
+	/* 
+	 * If nothing was active, index defaults to -1 which does
+	 * the right now and selects nothing.
+	 */
+	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), index);
 	g_free (font);
 }
 
@@ -758,9 +790,9 @@ fonts_update_widgets (GaleonPrefsDialog *self)
 	fonts_load_font (self, "fantasy",    p->fonts_fantasy);
 	fonts_load_font (self, "monospace",  p->fonts_monospace);
 
-	fonts_load_font_size (self, "var_size",   p->fonts_proportional_size, 12);
-	fonts_load_font_size (self, "fixed_size", p->fonts_monospace_size,    12);
-	fonts_load_font_size (self, "min_size",   p->fonts_minimum_size,      -1);
+	fonts_load_font_size (self, "variable_size",   p->fonts_proportional_size, 11);
+	fonts_load_font_size (self, "monospace_size",  p->fonts_monospace_size,    10);
+	fonts_load_font_size (self, "minimum_size",    p->fonts_minimum_size,      -1);
 
 	p->fonts_ignore_updates = FALSE;
 
@@ -768,29 +800,93 @@ fonts_update_widgets (GaleonPrefsDialog *self)
 }
 
 static void
-on_fonts_encoding_changed (GtkOptionMenu *optionmenu, GaleonPrefsDialog *self)
+on_fonts_encoding_changed (GtkComboBox *combobox, GaleonPrefsDialog *self)
 {
 	GaleonPrefsDialogPrivate *p = self->priv;
 
-	p->fonts_encoding = gul_gui_option_menu_get_value (optionmenu);
+	p->fonts_encoding = gul_gui_option_combobox_get_value (combobox);
 	fonts_update_widgets (self);
 
 	gnome_config_set_string (STATE_FONTS_ENCODING, p->fonts_encoding);
-	gnome_config_sync ();
 }
 
 static void
-fonts_set_type_in_entry (GtkWidget *entry, const char *value)
+fonts_set_type_in_combo(GtkWidget *combo, const char *value)
 {
-	g_return_if_fail (GTK_IS_ENTRY(entry));
-	g_object_set_data (G_OBJECT(entry), "fonts-entry::type", (gpointer)value);
+	g_return_if_fail(GTK_IS_COMBO_BOX(combo));
+	g_object_set_data(G_OBJECT(combo), "fonts-combo::type",
+	                  (gpointer)value);
 }
 
 static const char *
-fonts_get_type_from_entry (GtkWidget *entry)
+fonts_get_type_from_combo(GtkWidget *combo)
 {
-	g_return_val_if_fail (GTK_IS_ENTRY(entry), NULL);
-	return g_object_get_data (G_OBJECT(entry), "fonts-entry::type");
+	g_return_val_if_fail(GTK_IS_COMBO_BOX(combo), NULL);
+	return g_object_get_data(G_OBJECT(combo), "fonts-combo::type");
+}
+
+static void
+on_fonts_combo_changed(GtkWidget *combo, GaleonPrefsDialog *self)
+{
+	GaleonPrefsDialogPrivate *p = self->priv;
+	const char *type, *font;
+	char        gconf_key[256];
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+
+	g_return_if_fail(GTK_IS_COMBO_BOX(combo));
+
+	if (p->fonts_ignore_updates) return;
+
+	type = fonts_get_type_from_combo(combo);
+	g_return_if_fail (type != NULL);
+
+	model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo));
+	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo), &iter))
+	{
+		gtk_tree_model_get(model, &iter,
+		                   0, &font,
+		                   -1);
+	        g_return_if_fail(font || font[0]);
+	}
+
+	fonts_make_gconf_key(gconf_key, sizeof(gconf_key),
+			     type, p->fonts_encoding);
+	eel_gconf_set_string(gconf_key, font);
+}
+
+static void
+fonts_connect_combo (GaleonPrefsDialog *self, GladeXML *gxml, 
+		     const char *type,
+		     const char *widget_name, GtkWidget **widget)
+{
+	*widget = glade_xml_get_widget(gxml, widget_name);
+	g_return_if_fail(GTK_IS_COMBO_BOX(*widget));
+
+	/*
+	 * HACK: Glade 2.5.x doesn't yet allow setting useful
+	 * properties on a GtkComboBoxEntry or the entry child
+	 * so we have to do it in code.
+	 */
+	fonts_set_type_in_combo(*widget, type);
+
+	g_signal_connect(G_OBJECT(*widget), "changed",
+			 G_CALLBACK(on_fonts_combo_changed), self);
+}
+
+static void
+fonts_set_type_in_spinbutton(GtkWidget *spin, const char *value)
+{
+	g_return_if_fail(GTK_IS_SPIN_BUTTON(spin));
+	g_object_set_data(G_OBJECT(spin), "fonts-spinbutton::type",
+			  (gpointer)value);
+}
+
+static const char *
+fonts_get_type_from_spinbutton(GtkWidget *spin)
+{
+	g_return_val_if_fail(GTK_IS_SPIN_BUTTON(spin), NULL);
+	return g_object_get_data(G_OBJECT(spin), "fonts-spinbutton::type");
 }
 
 static gboolean
@@ -805,7 +901,7 @@ fonts_timeout_func (gpointer data)
 
 	g_return_val_if_fail (GTK_IS_SPIN_BUTTON(widget), FALSE);
 
-	type = fonts_get_type_from_entry (widget);
+	type = fonts_get_type_from_spinbutton(widget);
 	g_return_val_if_fail (type != NULL, FALSE);
 
 	size = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(widget));
@@ -849,60 +945,24 @@ fonts_timeout_stop (GaleonPrefsDialog *self, gboolean update)
 }
 
 static void
-on_fonts_entry_changed (GtkWidget *entry, GaleonPrefsDialog *self)
+on_fonts_spinbutton_changed (GtkWidget *spin, GaleonPrefsDialog *self)
 {
 	GaleonPrefsDialogPrivate *p = self->priv;
 	const char *type;
-	char        gconf_key[256];
 
-	g_return_if_fail (GTK_IS_ENTRY(entry));
+	g_return_if_fail(GTK_IS_SPIN_BUTTON(spin));
 
 	if (p->fonts_ignore_updates) return;
 
-	type = fonts_get_type_from_entry (entry);
-	g_return_if_fail (type != NULL);
+	type = fonts_get_type_from_spinbutton(spin);
+	g_return_if_fail(type != NULL);
 
-	if (GTK_IS_SPIN_BUTTON(entry))
-	{
-		/* we have to wait for a while before updating the gconf or the
-		 * spinbuttons will be ridiculously unresponsive thanks to
-		 * mozilla taking its time to reflow the page
-		 */
-		fonts_timeout_stop (self, FALSE);
-		fonts_timeout_start (self, entry);
-	}
-	else
-	{
-		const char *font;
-
-		font = gtk_entry_get_text (GTK_ENTRY(entry));
-		g_return_if_fail (font != NULL);
-
-		/* hmm...? */
-		if (!*font) return;
-
-		fonts_make_gconf_key (gconf_key, sizeof(gconf_key),
-				      type, p->fonts_encoding);
-		eel_gconf_set_string (gconf_key, font);
-	}
-}
-
-static void
-fonts_connect_combo (GaleonPrefsDialog *self, GladeXML *gxml, 
-		     const char *type,
-		     const char *widget_name, GtkWidget **widget)
-{
-	GtkWidget *entry;
-
-	*widget = glade_xml_get_widget (gxml, widget_name);
-	g_return_if_fail (GTK_IS_COMBO(*widget));
-
-	entry = GTK_COMBO(*widget)->entry;
-
-	fonts_set_type_in_entry (entry, type);
-
-	g_signal_connect (G_OBJECT(entry), "changed",
-			  G_CALLBACK(on_fonts_entry_changed), self);
+	/* we have to wait for a while before updating the gconf or the
+	 * spinbuttons will be ridiculously unresponsive thanks to
+	 * mozilla taking its time to reflow the page
+	 */
+	fonts_timeout_stop(self, FALSE);
+	fonts_timeout_start(self, spin);
 }
 
 static void
@@ -910,13 +970,13 @@ fonts_connect_spin_button (GaleonPrefsDialog *self, GladeXML *gxml,
 		           const char *type,
 			   const char *widget_name, GtkWidget **widget)
 {
-	*widget = glade_xml_get_widget (gxml, widget_name);
-	g_return_if_fail (GTK_IS_SPIN_BUTTON(*widget));
+	*widget = glade_xml_get_widget(gxml, widget_name);
+	g_return_if_fail(GTK_IS_SPIN_BUTTON(*widget));
 
-	fonts_set_type_in_entry (*widget, type);
+	fonts_set_type_in_spinbutton(*widget, type);
 
-	g_signal_connect (G_OBJECT(*widget), "value-changed",
-			  G_CALLBACK(on_fonts_entry_changed), self);
+	g_signal_connect(G_OBJECT(*widget), "value-changed",
+			 G_CALLBACK(on_fonts_spinbutton_changed), self);
 }
 
 static void
@@ -947,28 +1007,28 @@ galeon_prefs_fonts_construct (GaleonPrefsDialog *self, GladeXML *gxml)
 	fonts_connect_combo (self, gxml, "monospace",  "fonts_monospace",
 			     &p->fonts_monospace);
 
-	fonts_connect_spin_button (self, gxml, "var_size",   
+	fonts_connect_spin_button (self, gxml, "variable_size",   
 			           "fonts_proportional_size",
 				   &p->fonts_proportional_size);
-	fonts_connect_spin_button (self, gxml, "fixed_size",
+	fonts_connect_spin_button (self, gxml, "monospace_size",
 			           "fonts_monospace_size",
 				   &p->fonts_monospace_size);
-	fonts_connect_spin_button (self, gxml, "min_size",
+	fonts_connect_spin_button (self, gxml, "minimum_size",
 			           "fonts_minimum_size",
 				   &p->fonts_minimum_size);
 
 	encoding = glade_xml_get_widget (gxml, "fonts_encoding");
-	fill_option_menu_from_array (encoding,
-			             fonts_encodings,
-				     G_N_ELEMENTS(fonts_encodings));
+	fill_combo_box_from_array (encoding,
+			           fonts_encodings,
+				   G_N_ELEMENTS(fonts_encodings));
 
 	tmp = gnome_config_get_string (STATE_FONTS_ENCODING "=x-western");
-	gul_gui_option_menu_set_value (GTK_OPTION_MENU(encoding), tmp);
+	gul_gui_option_combobox_set_value (GTK_COMBO_BOX(encoding), tmp);
 	g_free (tmp);
 
 	g_signal_connect (G_OBJECT(encoding), "changed",
 			  G_CALLBACK(on_fonts_encoding_changed), self);
-	on_fonts_encoding_changed(GTK_OPTION_MENU(encoding), self);
+	on_fonts_encoding_changed(GTK_COMBO_BOX(encoding), self);
 
 	gul_glade_gconf_connect_simple (gxml,
 		"fonts_always", CONF_RENDERING_USE_OWN_FONTS,
@@ -985,9 +1045,9 @@ on_toggle_button_toggled_invert (GtkToggleButton *button, const char *gconf_key)
 }
 
 static const GulGuiOption mouse_middle_actions[] = {
-	{ N_("Bookmarks menu"),           NULL /* "0" */ },
+//	{ N_("Bookmarks menu"),           NULL /* "0" */ },
 	{ N_("Paste URL in new tab"),     "1" },
-	{ N_("Go Back"),                  NULL /* "2" */ },
+//	{ N_("Go Back"),                  NULL /* "2" */ },
 	{ N_("Gestures"),                 "3" },
 	{ N_("Automatic scroll"),         "4" },
 	{ N_("Manual scroll"),            "5" },
@@ -1002,10 +1062,10 @@ galeon_prefs_mouse_construct (GaleonPrefsDialog *self, GladeXML *gxml)
 	/* middle button */
 
 	widget = glade_xml_get_widget (gxml, "mouse_middle");
-	connect_optionmenu (widget,
-			    CONF_MOUSE_MIDDLE_BUTTON_ACTION,
-			    mouse_middle_actions,
-			    G_N_ELEMENTS(mouse_middle_actions));
+	connect_combobox (widget,
+			  CONF_MOUSE_MIDDLE_BUTTON_ACTION,
+			  mouse_middle_actions,
+			  G_N_ELEMENTS(mouse_middle_actions));
 
 	/* right button */
 
@@ -1057,32 +1117,22 @@ on_prefs_network_proxy_button_clicked (GtkWidget *button, GaleonPrefsDialog *sel
 {
 	GtkWidget *parent;
 	GtkWidget *dialog;
-	GtkWidget *label;
-	char 	  *message ;
 	GError 	  *err = NULL;
 
 	if (g_spawn_command_line_async (GNOME_NETWORK_PREFERENCES, &err))
 		return;
 
-	message = g_strdup_printf("<b>%s</b>\n\n%s",
-			          _("Cannot start network proxy "
-				    "configuration program; the "
-				    "following error occured:"),
-				  err->message);
-	g_clear_error (&err);
-
 	parent = gtk_widget_get_toplevel (button);
-	dialog = gtk_message_dialog_new (GTK_WINDOW(parent),
-			                 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_OK,
-					 message);
-	g_free (message);
-
-	/* XXX The label is private, but until GtkMessageDialog has constructor
-	 * that supports markup, this appears to be the way to go. */
-	label = GTK_MESSAGE_DIALOG(dialog)->label;
-	gtk_label_set_use_markup (GTK_LABEL(label), TRUE);
+	dialog = hig_alert_new (GTK_WINDOW(parent),
+				GTK_DIALOG_MODAL,
+				HIG_ALERT_ERROR,
+				_("Cannot start network proxy "
+				  "configuration program."),
+				err->message,
+				GTK_STOCK_OK,
+				GTK_RESPONSE_OK,
+				NULL);
+	g_clear_error (&err);
 
 	gtk_dialog_run (GTK_DIALOG(dialog));
 	gtk_widget_destroy (dialog);
@@ -1335,7 +1385,7 @@ galeon_prefs_dialog_response (GtkDialog *dialog, gint response_id)
 {
 	if (response_id == GTK_RESPONSE_HELP)
 	{
-		/* FIXME need docs */
+		gul_gui_help (GTK_WINDOW (dialog), "galeon.xml", "preferences");
 		return;
 	}
 
@@ -1360,9 +1410,7 @@ galeon_prefs_dialog_new (void)
 	gtk_container_set_border_width (GTK_CONTAINER(GTK_DIALOG(self)->action_area), 6);
 
 	gtk_dialog_add_buttons (GTK_DIALOG(self), 
-			        /* FIXME need docs
-				 * GTK_STOCK_HELP,  GTK_RESPONSE_HELP,
-				 */
+				GTK_STOCK_HELP,  GTK_RESPONSE_HELP,
 			        GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 				NULL);
 
@@ -1379,6 +1427,8 @@ galeon_prefs_dialog_init (GaleonPrefsDialog *self)
 
 	self->priv = g_new0(GaleonPrefsDialogPrivate, 1);
 
+	self->priv->fonts_encoding = NULL;
+
 	gxml = gul_glade_widget_new ("prefs-dialog.glade", "preferences_dialog", NULL, self);
 	g_return_if_fail (gxml != NULL);
 
@@ -1412,9 +1462,9 @@ galeon_prefs_dialog_init (GaleonPrefsDialog *self)
 		NULL);
 
 	galeon_prefs_general_construct (self, gxml);
+	galeon_prefs_language_construct (self, gxml);
 	galeon_prefs_fonts_construct (self, gxml);
 	galeon_prefs_tabs_construct (self, gxml);
-	galeon_prefs_language_construct (self, gxml);
 	galeon_prefs_mouse_construct (self, gxml);
 	galeon_prefs_web_content_construct (self, gxml);
 	galeon_prefs_privacy_construct (self, gxml);
@@ -1438,6 +1488,7 @@ galeon_prefs_dialog_finalize (GObject *object)
 {
 	GaleonPrefsDialog *self = GALEON_PREFS_DIALOG(object);
 
+	g_free (self->priv->fonts_encoding);
 	gtk_widget_destroy (self->priv->language_editor);
 	g_free (self->priv);
 
