@@ -30,13 +30,18 @@
 #include "galeon-bookmarks-icon-provider.h"
 #include "gul-filesystem-autocompletion.h"
 #include "galeon-window.h"
+#include "galeon-automation.h"
+#include "hig-alert.h"
+#include "migrate.h"
+#include "galeon-debug.h"
 
 #include <libgnomeui/gnome-client.h>
 #include <bonobo/bonobo-main.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmessagedialog.h>
+#include <gconf/gconf-client.h>
 
 #ifdef ENABLE_NAUTILUS_VIEW
 
@@ -47,9 +52,6 @@
 
 #endif
 
-/* #define DEBUG_MSG(x) g_print x */
-#define DEBUG_MSG(x)
-
 struct GaleonShellPrivate
 {
 	GbBookmarkSet *bookmark_set;
@@ -57,7 +59,10 @@ struct GaleonShellPrivate
 	Session *session;
 	GaleonFaviconCache *favicon_cache;
 	GaleonAutocompletion *autocompletion;
+	GaleonWindow *active_window;
+	EggToolbarsModel *toolbars_model;
 	gboolean server_mode;
+	BonoboGenericFactory *automation_factory;
 };
 
 enum
@@ -159,16 +164,20 @@ galeon_shell_new_window_cb (GaleonEmbedShell *shell,
 	}
 	
 	new_tab = galeon_tab_new ();
-	galeon_window_add_tab (window, new_tab, FALSE);
+	galeon_window_add_tab (window, new_tab,
+			       GALEON_WINDOW_ADD_TAB_LAST, FALSE);
 	*new_embed = galeon_tab_get_embed (new_tab);
 }
 
 static void
 galeon_shell_init (GaleonShell *gs)
 {
+	gboolean ret;
+	GConfSchema *schema;
+
 	galeon_shell = gs;
 	g_object_add_weak_pointer (G_OBJECT(galeon_shell),
-				   (gpointer *)&galeon_shell);
+				   (gpointer)&galeon_shell);
 	
 	gul_pixbuf_cache_register_stocks ();
 	
@@ -176,6 +185,55 @@ galeon_shell_init (GaleonShell *gs)
 	gs->priv->session     = NULL;
 	gs->priv->server_mode = FALSE;
 
+	gs->priv->automation_factory = galeon_automation_factory_new ();
+	if (gs->priv->automation_factory == NULL)
+	{
+		GtkWidget *dialog;
+		char * text;
+
+		dialog = hig_alert_new (NULL,
+					GTK_DIALOG_MODAL,
+					HIG_ALERT_ERROR,
+					_("Galeon failed to start."),
+					NULL,
+					GTK_STOCK_OK,
+					GTK_RESPONSE_OK,
+					NULL);
+		text = g_strdup_printf (_("Bonobo couldn't locate the file %s."),
+			                "<tt>\"GNOME_Galeon_Automation.server\"</tt>");
+		hig_alert_set_secondary_printf (HIG_ALERT (dialog), "%s\n\n%s", text,
+			      _("Usually this is an indication of an incomplete installation. "
+				"See the INSTALL document for more details."));
+		g_free (text);
+
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		exit (1);
+	}
+
+	/* Check the gconf installation */
+	schema = gconf_client_get_schema (eel_gconf_client_get_global (),
+					  "/schemas" CONF_TABS_TABBED_EDGE, NULL);
+	if (!schema)
+	{
+		GtkWidget *dialog;
+		dialog = hig_alert_new (NULL,
+					GTK_DIALOG_MODAL,
+					HIG_ALERT_ERROR,
+					_("Galeon failed to start."),
+					_("The gconf schema couldn't be found.\n\n"
+					  "This shouldn't generally happen, but can usually "
+					  "be fixed by running <tt>\"galeon-config-tool "
+					  "--install-schemas\"</tt>."),
+					GTK_STOCK_OK,
+					GTK_RESPONSE_OK,
+					NULL);
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		exit(1);
+	}
+	gconf_schema_free (schema);
+
+
 #ifdef ENABLE_GTKHTML_EMBED
 	gs->priv->embed_shell = galeon_embed_shell_new ("gtkhtml");	
 #elif ENABLE_MOZILLA_EMBED
@@ -183,6 +241,34 @@ galeon_shell_init (GaleonShell *gs)
 #endif
 	g_assert (gs->priv->embed_shell != NULL);
 
+	/* Now that we have all the important stuff created, migrate any
+	 * preferences, this MUST be done before the shell_initialize, as the
+	 * font migrations need the embed_shell, but need to happen before the
+	 * shell is initialized, when they will get used */
+	migrate_gconf_settings ();
+
+	ret = galeon_embed_shell_initialize (gs->priv->embed_shell);
+	if (ret == FALSE)
+	{
+		GtkWidget *dialog;
+
+		dialog = hig_alert_new (NULL, GTK_DIALOG_MODAL,
+					HIG_ALERT_ERROR,
+					_("Galeon failed to start."),
+					NULL,
+					GTK_STOCK_OK, GTK_RESPONSE_OK,
+					NULL);
+		
+		hig_alert_set_secondary_printf (HIG_ALERT (dialog),
+				_("Ensure the %s environment variable is set "
+				  "to the correct Mozilla installation "
+				  "directory."),
+				"<tt>\"MOZILLA_FIVE_HOME\"</tt>");
+
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		exit (1);
+	}
+
 	g_signal_connect (G_OBJECT(embed_shell),
 			  "new_window_orphan",
 			  G_CALLBACK(galeon_shell_new_window_cb),
@@ -224,6 +310,14 @@ galeon_shell_finalize (GObject *object)
 		g_object_unref (gs->priv->autocompletion);
 	}
 
+	if (gs->priv->toolbars_model)
+	{
+		gchar *tb_file = g_build_filename (g_get_home_dir (), GALEON_DIR, "toolbars.xml", NULL);
+		egg_toolbars_model_save (gs->priv->toolbars_model, tb_file, "0");
+		g_free (tb_file);
+		g_object_unref (gs->priv->toolbars_model);
+	}
+
 	g_return_if_fail (GALEON_IS_FAVICON_CACHE(gs->priv->favicon_cache));
 	g_object_unref (G_OBJECT (gs->priv->favicon_cache));
 
@@ -231,19 +325,20 @@ galeon_shell_finalize (GObject *object)
 	gb_bookmark_set_check_save (gs->priv->bookmark_set);
 	g_object_unref (G_OBJECT (gs->priv->bookmark_set));
 	
-        g_free (gs->priv);
-
         G_OBJECT_CLASS (parent_class)->finalize (object);
 
-#ifdef DEBUG_MARCO
-	g_print ("Galeon shell finalized\n");
-#endif
+	LOG ("Galeon shell finalized");
+
+	if (gs->priv->automation_factory)
+	{
+		bonobo_object_unref (gs->priv->automation_factory);
+	}
 	
+	g_free (gs->priv);
+
 	bonobo_main_quit ();
 
-#ifdef DEBUG_MARCO
-	g_print ("Bonobo quit done\n");
-#endif
+	LOG ("Bonobo quit done");
 }
 
 GaleonShell *
@@ -269,68 +364,6 @@ galeon_shell_get_embed_shell (GaleonShell *gs)
 }
 
 
-/* Old keys to be able to migrate old proxy prefs to the new location */
-#define OLD_NETWORK_PROXY_MODE "/apps/galeon/Advanced/Network/proxy_mode"
-#define OLD_NETWORK_HTTP_PROXY "/apps/galeon/Advanced/Network/http_proxy"
-#define OLD_NETWORK_SSL_PROXY "/apps/galeon/Advanced/Network/ssl_proxy"
-#define OLD_NETWORK_FTP_PROXY "/apps/galeon/Advanced/Network/ftp_proxy"
-#define OLD_NETWORK_SOCKS_PROXY "/apps/galeon/Advanced/Network/socks_proxy"
-#define OLD_NETWORK_HTTP_PROXY_PORT "/apps/galeon/Advanced/Network/http_proxy_port"
-#define OLD_NETWORK_SSL_PROXY_PORT "/apps/galeon/Advanced/Network/ssl_proxy_port"
-#define OLD_NETWORK_FTP_PROXY_PORT "/apps/galeon/Advanced/Network/ftp_proxy_port"
-#define OLD_NETWORK_SOCKS_PROXY_PORT "/apps/galeon/Advanced/Network/socks_proxy_port"
-#define OLD_NETWORK_PROXY_AUTO_URL "/apps/galeon/Advanced/Network/proxy_auto_url"
-#define CONF_NETWORK_PROXY_MIGRATED "/apps/galeon/Advanced/Network/proxy_prefs_converted"
-
-static void
-migrate_proxy_prefs ()
-{
-	const gchar *keys[][2] = 
-	{{OLD_NETWORK_HTTP_PROXY,       CONF_NETWORK_HTTP_PROXY},
-	 {OLD_NETWORK_SSL_PROXY,        CONF_NETWORK_SSL_PROXY},
-	 {OLD_NETWORK_FTP_PROXY,        CONF_NETWORK_FTP_PROXY},
-	 {OLD_NETWORK_SOCKS_PROXY,      CONF_NETWORK_SOCKS_PROXY}, 
-	 {OLD_NETWORK_HTTP_PROXY_PORT,  CONF_NETWORK_HTTP_PROXY_PORT},
-	 {OLD_NETWORK_SSL_PROXY_PORT,   CONF_NETWORK_SSL_PROXY_PORT},
-	 {OLD_NETWORK_FTP_PROXY_PORT,   CONF_NETWORK_FTP_PROXY_PORT},
-	 {OLD_NETWORK_SOCKS_PROXY_PORT, CONF_NETWORK_SOCKS_PROXY_PORT},
-	 {OLD_NETWORK_PROXY_AUTO_URL,   CONF_NETWORK_PROXY_AUTO_URL},
-	 {NULL, NULL}};
-	int proxy_mode;
-	const char *proxy_modes[3] = {"none", "manual", "auto"};
-	/* 
-	 * If the proxy prefs were never migrated, and if the user
-	 * has no prefs set for the gnome-wide proxy settings, migrate
-	 * its prefs
-	 */
-	if (!eel_gconf_key_exists (CONF_NETWORK_PROXY_AUTO_URL) &&
-	    !eel_gconf_get_boolean (CONF_NETWORK_PROXY_MIGRATED)) 
-	{
-		GConfValue *value;	
-		int i;
-
-		for (i=0; keys[i][0] != NULL; i++)
-		{		  
-			value = eel_gconf_get_value (keys[i][0]);
-			if (value)
-			{
-				eel_gconf_set_value (keys[i][1], value);
-				eel_gconf_value_free (value);
-			}
-		}
-
-		proxy_mode = eel_gconf_get_integer (OLD_NETWORK_PROXY_MODE);
-		if (proxy_mode < 0 || proxy_mode > 2) 
-		{
-			proxy_mode = 0;
-		}
-		eel_gconf_set_string (CONF_NETWORK_PROXY_MODE, 
-				      proxy_modes[proxy_mode]);
-	}
-	
-	eel_gconf_set_boolean (CONF_NETWORK_PROXY_MIGRATED, TRUE);	
-}
-
 static void
 galeon_init_services (GaleonShell *gs)
 {
@@ -340,8 +373,6 @@ galeon_init_services (GaleonShell *gs)
 	eel_gconf_monitor_add ("/apps/nautilus/preferences");
 	eel_gconf_monitor_add ("/system/proxy");
 
-	migrate_proxy_prefs();
-
 	bookmarks_init (gs);
 
 #ifdef ENABLE_NAUTILUS_VIEW
@@ -410,6 +441,22 @@ load_homepage (GaleonShell *gs,
 	g_free (result);
 }
 
+/**
+ * galeon_shell_set_active_window:
+ * @gs: a #GaleonShell
+ * @window: the active #GaleonWindow
+ *
+ * Set the active window. Not really useful standalone, but called
+ * from a focus handler in GaleonWindow so the active window is
+ * always tracked
+ **/
+
+void
+galeon_shell_set_active_window (GaleonShell *gs, GaleonWindow *window)
+{
+	gs->priv->active_window = window;
+}
+
 /**
  * galeon_shell_get_active_window:
  * @gs: a #GaleonShell
@@ -424,15 +471,21 @@ load_homepage (GaleonShell *gs,
 GaleonWindow *
 galeon_shell_get_active_window (GaleonShell *gs)
 {
-	Session *session;
-	const GList *windows;
-	
-	session = galeon_shell_get_session (gs);
-	windows = session_get_windows (session);
+	GList *windows = session_get_windows(gs->priv->session);
+	if (windows &&
+	    !g_list_find(windows, gs->priv->active_window))
+	{
+		gs->priv->active_window = GALEON_WINDOW(windows->data);
+	}
+	else if (!windows)
+	{
+		/* The case where we're in server mode with
+		 * no windows open
+		 */
+		gs->priv->active_window = NULL;
+	}
 
-	if (windows == NULL) return NULL;
-	
-	return GALEON_WINDOW(windows->data);
+	return gs->priv->active_window;
 }
 
 /**
@@ -490,6 +543,7 @@ galeon_shell_new_tab_from_embed (GaleonShell *shell,
 	GaleonEmbed *embed;
 	gboolean in_new_window;
 	gboolean jump_to;
+	gint position;
 	
 	in_new_window = !eel_gconf_get_boolean (CONF_TABS_TABBED);
 
@@ -510,13 +564,19 @@ galeon_shell_new_tab_from_embed (GaleonShell *shell,
         	window = galeon_window_new ();
 	}
 
+	position = (flags & GALEON_NEW_TAB_APPEND) ?
+		   GALEON_WINDOW_ADD_TAB_LAST :
+		   GALEON_WINDOW_ADD_TAB_GROUPED;
+
 	tab = galeon_tab_new ();
-	embed = galeon_tab_get_embed (tab);
-	gtk_widget_show (GTK_WIDGET(embed));
-	galeon_window_add_tab (window, tab, 
+	gtk_widget_show (GTK_WIDGET (tab));
+	
+	galeon_window_add_tab (window, tab,
+			       position,
 			       jump_to);
 	gtk_widget_show (GTK_WIDGET(window));
 
+	embed = galeon_tab_get_embed (tab);
 	if (flags & GALEON_NEW_TAB_HOMEPAGE)
 	{
 		load_homepage (shell, embed, previous_embed);
@@ -585,13 +645,10 @@ bookmarks_init (GaleonShell *gs)
 				       GALEON_DIR "/bookmarks.xbel", NULL);
 	
 	/* init the bookmark icon provider */
-	GbIconProvider *def = gb_system_get_icon_provider ();
 	GbGaleonIconProvider *gip = gb_galeon_icon_provider_new ();
-
-	gb_galeon_icon_provider_set_default_provider (gip, def);
 	gb_system_set_icon_provider (GB_ICON_PROVIDER (gip));
 
-	DEBUG_MSG (("trying to load bookmarks from %s\n", filename));
+	LOG ("trying to load bookmarks from %s", filename);
 	gs->priv->bookmark_set = gb_io_load_from_file (GB_IO (io), filename);
 
 	if (gs->priv->bookmark_set == NULL)
@@ -617,12 +674,12 @@ bookmarks_init (GaleonShell *gs)
 		gb_bookmark_set_set_filename (gs->priv->bookmark_set, filename);
 	}
 
-	DEBUG_MSG (("Detected version of bookmarks file: %s\n", 
+	LOG ("Detected version of bookmarks file: %s", 
 		 gs->priv->bookmark_set->file_format_version == GB_FILE_FORMAT_VERSION_GALEON_1 
 		 ? "galeon1" 
 		 : (gs->priv->bookmark_set->file_format_version == GB_FILE_FORMAT_VERSION_GALEON_2 
 		    ? "galeon2" 
-		    : "unknown (galeon1)")));
+		    : "unknown (galeon1)"));
 
 	if (gs->priv->bookmark_set->file_format_version == GB_FILE_FORMAT_VERSION_UNKNOWN)
 	{
@@ -649,6 +706,7 @@ bookmarks_init (GaleonShell *gs)
 
 	gb_galeon_icon_provider_add_bookmark_set (gip, gs->priv->bookmark_set);
 
+	g_object_unref (G_OBJECT (gip));
 	g_object_unref (G_OBJECT (io));
 	g_free (filename);
 		
@@ -788,3 +846,29 @@ galeon_shell_modifier_flags (GdkModifierType modifier)
 
 	return flags;
 }
+
+EggToolbarsModel *
+galeon_shell_get_toolbars_model  (GaleonShell *gs)
+{
+	if (!gs->priv->toolbars_model)
+	{
+		gboolean loaded = FALSE;
+		EggToolbarsModel *tm = egg_toolbars_model_new ();
+		gchar *user_file = gul_general_user_file ("toolbars.xml", FALSE);
+		if (user_file)
+		{
+			loaded = egg_toolbars_model_load (tm, user_file);
+			g_free (user_file);
+		}
+
+		if (loaded == FALSE)
+		{
+			gchar *default_file = gul_general_user_file ("galeon-egg-ui-toolbar.xml", TRUE);
+			egg_toolbars_model_load (tm, default_file);
+			g_free (default_file);
+		}
+		gs->priv->toolbars_model = tm;
+	}
+	return gs->priv->toolbars_model;
+}
+
