@@ -26,6 +26,7 @@
 #include "bookmarks.h"
 #include "session.h"
 #include "galeon-favicon-cache.h"
+#include "egg-toolbars-model.h"
 
 #include <glib-object.h>
 #include <glib.h>
@@ -85,6 +86,8 @@ GaleonShell        *galeon_shell_new                 (void);
 
 GaleonEmbedShell   *galeon_shell_get_embed_shell     (GaleonShell *gs);
 
+void		    galeon_shell_set_active_window   (GaleonShell *gs,
+						      GaleonWindow *window);
 GaleonWindow 	   *galeon_shell_get_active_window   (GaleonShell *gs);
 
 GaleonTab          *galeon_shell_new_tab	     (GaleonShell *shell,
@@ -112,6 +115,8 @@ GaleonNewTabFlags   galeon_shell_modifier_flags	     (GdkModifierType modifier);
 gboolean            galeon_shell_get_server_mode     (GaleonShell *gs);
 void                galeon_shell_set_server_mode     (GaleonShell *gs, gboolean mode);
 
+EggToolbarsModel *  galeon_shell_get_toolbars_model  (GaleonShell *gs);
+
 #ifdef __cplusplus
 }
 #endif
