@@ -39,8 +39,8 @@
 #include <gtk/gtksignal.h>
 #include <libgnome/gnome-macros.h>
 #include <libgnome/gnome-util.h>
+#include <libgnomeui/gnome-icon-theme.h>
 #include <math.h>
-#include <libgnomevfs/gnome-vfs.h>
 
 #define spinner_DEFAULT_TIMEOUT 100	/* Milliseconds Per Frame */
 
@@ -48,19 +48,18 @@ struct GaleonSpinnerDetails {
 	GList	*image_list;
 
 	GdkPixbuf *quiescent_pixbuf;
+
+	GnomeIconTheme *icon_theme;
 	
 	int	max_frame;
 	int	delay;
 	int	current_frame;	
 	guint	timer_task;
 	
-	gboolean ready;
 	gboolean small_mode;
 
 	gboolean button_in;
 	gboolean button_down;
-
-	gint theme_notif;
 };
 
 static void galeon_spinner_class_init (GaleonSpinnerClass *class);
@@ -70,18 +69,6 @@ static void galeon_spinner_unload_images          (GaleonSpinner *spinner);
 static void galeon_spinner_remove_update_callback (GaleonSpinner *spinner);
 
 
-static GList *spinner_directories = NULL;
-
-static void
-galeon_spinner_init_directory_list (void);
-static void 		  
-galeon_spinner_search_directory (const gchar *base, GList **spinner_list);
-static GaleonSpinnerInfo *
-galeon_spinner_get_theme_info (const gchar *base, const gchar *theme_name);
-static gchar *
-galeon_spinner_get_theme_path (const gchar *theme_name);
-
-
 static GObjectClass *parent_class = NULL;
 
 GType 
@@ -107,8 +94,6 @@ galeon_spinner_get_type (void)
                 galeon_spinner_type = g_type_register_static (GTK_TYPE_EVENT_BOX,
                                                               "GaleonSpinner",
                                                               &our_info, 0);
-
-		galeon_spinner_init_directory_list ();
         }
 
         return galeon_spinner_type;
@@ -149,9 +134,14 @@ get_spinner_dimensions (GaleonSpinner *spinner, int *spinner_width, int* spinner
 	GList *current_entry;
 	GdkPixbuf *pixbuf;
 	
+	current_width = 0;
+	current_height = 0;
+
 	/* start with the quiescent image */
-	current_width = gdk_pixbuf_get_width (spinner->details->quiescent_pixbuf);
-	current_height = gdk_pixbuf_get_height (spinner->details->quiescent_pixbuf);
+	if (spinner->details->quiescent_pixbuf != NULL) {
+		current_width = gdk_pixbuf_get_width (spinner->details->quiescent_pixbuf);
+		current_height = gdk_pixbuf_get_height (spinner->details->quiescent_pixbuf);
+	}
 
 	/* loop through all the installed images, taking the union */
 	current_entry = spinner->details->image_list;
@@ -178,14 +168,9 @@ get_spinner_dimensions (GaleonSpinner *spinner, int *spinner_width, int* spinner
 
 /* handler for handling theme changes */
 static void
-galeon_spinner_theme_changed (GConfClient *client,
-                              guint cnxn_id,
-                              GConfEntry *entry,
-			      gpointer user_data)
+galeon_spinner_theme_changed (GnomeIconTheme *icon_theme,
+		              GaleonSpinner  *spinner)
 {
-	GaleonSpinner *spinner;
-
-	spinner = GALEON_SPINNER (user_data);
 	gtk_widget_hide (GTK_WIDGET (spinner));
 	galeon_spinner_load_images (spinner);
 	gtk_widget_show (GTK_WIDGET (spinner));	
@@ -208,15 +193,12 @@ galeon_spinner_init (GaleonSpinner *spinner)
 	spinner->details = g_new0 (GaleonSpinnerDetails, 1);
 	
 	spinner->details->delay = spinner_DEFAULT_TIMEOUT;		
+	spinner->details->icon_theme = gnome_icon_theme_new ();
+	g_signal_connect (spinner->details->icon_theme, "changed",
+			  G_CALLBACK (galeon_spinner_theme_changed), spinner);
 	
 	galeon_spinner_load_images (spinner);
 	gtk_widget_show (widget);
-
-	spinner->details->theme_notif = 
-		eel_gconf_notification_add (CONF_TOOLBAR_SPINNER_THEME,
-			 		    (GConfClientNotifyFunc)
-					     galeon_spinner_theme_changed,
-				    	     spinner);
 }
 
 /* here's the routine that selects the image to draw, based on the spinner's state */
@@ -519,97 +501,105 @@ galeon_spinner_unload_images (GaleonSpinner *spinner)
 	spinner->details->image_list = NULL;
 }
 
-static GdkPixbuf*
-load_themed_image (const char *path, const char *file_name, 
-		   gboolean small_mode)
+static GdkPixbuf *
+scale_to_real_size (GdkPixbuf *pixbuf, int wanted_size)
 {
-	GdkPixbuf *pixbuf, *temp_pixbuf;
-	char *image_path;
-	
-	image_path = g_build_filename (path, file_name, NULL);
-	
-	if (!g_file_test(image_path, G_FILE_TEST_EXISTS))
-        {
-		g_free (image_path);
-		return NULL;
-	}
-	
-	if (image_path) {
-		pixbuf = gdk_pixbuf_new_from_file (image_path, NULL);
-		
-		if (small_mode && pixbuf) {
-			temp_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
-							       gdk_pixbuf_get_width (pixbuf) * 2 / 3,
-							       gdk_pixbuf_get_height (pixbuf) * 2 / 3,
-							       GDK_INTERP_BILINEAR);
-			g_object_unref (pixbuf);
-			pixbuf = temp_pixbuf;
-		}
+	GdkPixbuf *result;
+	int        size;
 
-		g_free (image_path);
+	size = gdk_pixbuf_get_width (pixbuf);
 
-		return pixbuf;
+	if (wanted_size != size) {
+		result = gdk_pixbuf_scale_simple (pixbuf,
+						  wanted_size, wanted_size,
+						  GDK_INTERP_BILINEAR);
+	} else {
+		result = g_object_ref (pixbuf);
 	}
-	return NULL;
+
+	return result;
 }
 
-/* utility to make the spinner frame name from the index */
-
-static char *
-make_spinner_frame_name (int index)
+static GdkPixbuf *
+extract_frame (GdkPixbuf *grid_pixbuf, int x, int y, int size, int wanted_size)
 {
-	return g_strdup_printf ("%03d.png", index);
+	GdkPixbuf *pixbuf, *frame;
+
+	pixbuf = gdk_pixbuf_new_subpixbuf (grid_pixbuf, x, y, size, size);
+	g_return_val_if_fail (pixbuf != NULL, NULL);
+
+	frame = scale_to_real_size (pixbuf, wanted_size);
+	g_object_unref (pixbuf);
+
+	return frame;
 }
 
 /* load all of the images of the spinner sequentially */
 static void
 galeon_spinner_load_images (GaleonSpinner *spinner)
 {
-	int index;
-	char *spinner_frame_name;
-	GdkPixbuf *pixbuf, *qpixbuf;
-	GList *image_list;
-	char *image_theme;
-	char *path;
+	char      *path;
+	GdkPixbuf *pixbuf;
+	int        grid_width, grid_height;
+	int        wanted_size, size;
+	int        x, y;
+	GList     *image_list;
 	
 	galeon_spinner_unload_images (spinner);
 	
-	image_theme = eel_gconf_get_string (CONF_TOOLBAR_SPINNER_THEME);
-	
-	path = galeon_spinner_get_theme_path (image_theme);
-	if (path == NULL)
-	{
-		g_warning ("spinner theme `%s' not found, using default `%s'",
-			   image_theme, "galeon");
-		path = galeon_spinner_get_theme_path ("galeon");
+	/* FIXME: should lookup the sizes for GTK_ICON_SIZE_SMALL_TOOLBAR and
+	 * GTK_ICON_SIZE_LARGE_TOOLBAR, but most of the time 24 and 32 should
+	 * be close enough...
+	 */
+	wanted_size = spinner->details->small_mode ? 24 : 32;
+
+	path = gnome_icon_theme_lookup_icon (spinner->details->icon_theme,
+			                     "gnome-spinner", 
+					     wanted_size, NULL, &size);
+	if (path == NULL) {
+		g_warning ("Spinner animation not found");
+		return;
 	}
-	g_free (image_theme);
-	g_return_if_fail (path != NULL);
 
-	qpixbuf = load_themed_image (path, "rest.png", 
-			spinner->details->small_mode);
+	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
+	g_free (path);
 
-	g_return_if_fail (qpixbuf != NULL);
-	spinner->details->quiescent_pixbuf = qpixbuf;
-	    
-	spinner->details->max_frame = 50;
+	grid_width  = gdk_pixbuf_get_width (pixbuf);
+	grid_height = gdk_pixbuf_get_height (pixbuf);
 
 	image_list = NULL;
-	for (index = 1; index <= spinner->details->max_frame; index++) {
-		spinner_frame_name = make_spinner_frame_name (index);
-		pixbuf = load_themed_image (path, spinner_frame_name, 
-					    spinner->details->small_mode);
-		g_free (spinner_frame_name);
-		if (pixbuf == NULL) {
-			spinner->details->max_frame = index - 1;
-			break;
-		}
-		image_list = g_list_prepend (image_list, pixbuf);
-	}
-	spinner->details->image_list = g_list_reverse (image_list);
-	spinner->details->current_frame = 0;
+	for (y = 0; y <= grid_height - size; y += size) {
+		for (x = 0; x <= grid_width - size; x += size) {
+			GdkPixbuf *frame;
 
+			frame = extract_frame (pixbuf, x, y, size, wanted_size);
+			if (frame != NULL) {
+				image_list = g_list_prepend (image_list, frame);
+			} else {
+				g_warning ("Couldn't extract frame from grid");
+			}
+		}
+	}
+	spinner->details->max_frame  = g_list_length (image_list);
+	spinner->details->image_list = g_list_reverse (image_list);
+
+	g_object_unref (pixbuf);
+
+	path = gnome_icon_theme_lookup_icon (spinner->details->icon_theme,
+			                     "gnome-spinner-rest", 
+					     wanted_size, NULL, &size);
+	if (path == NULL) {
+		g_warning ("Spinner rest icon not found");
+		return;
+	}
+
+	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
 	g_free (path);
+
+	spinner->details->quiescent_pixbuf = 
+		scale_to_real_size (pixbuf, wanted_size);
+
+	g_object_unref (pixbuf);
 }
 
 static gboolean
@@ -729,7 +719,7 @@ galeon_spinner_finalize (GObject *object)
 	galeon_spinner_remove_update_callback (spinner);
 	galeon_spinner_unload_images (spinner);
 	
-	eel_gconf_notification_remove (spinner->details->theme_notif);
+	g_object_unref (spinner->details->icon_theme);
 	
 	g_free (spinner->details);
 
@@ -754,137 +744,3 @@ galeon_spinner_class_init (GaleonSpinnerClass *class)
 	widget_class->size_request = galeon_spinner_size_request;	
 }
 
-static void
-galeon_spinner_search_directory  (const gchar *base, GList **spinner_list)
-{
-	GnomeVFSResult rc;
-	GList *list, *node;
-
-	rc = gnome_vfs_directory_list_load
-		(&list, base, (GNOME_VFS_FILE_INFO_GET_MIME_TYPE | 
-			       GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE | 
-			       GNOME_VFS_FILE_INFO_FOLLOW_LINKS));
-	if (rc != GNOME_VFS_OK) return;
-
-	for (node = list; node != NULL; node = node->next)
-	{
-		GnomeVFSFileInfo *file_info = node->data;
-		GaleonSpinnerInfo *info;
-
-		if (file_info->name[0] == '.')
-			continue;
-		if (file_info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
-			continue;
-
-		info = galeon_spinner_get_theme_info (base, file_info->name);
-		if (info != NULL)
-		{
-			*spinner_list = g_list_append (*spinner_list, info);
-		}
-	}
-
-	gnome_vfs_file_info_list_free (list);
-}
-
-static GaleonSpinnerInfo *
-galeon_spinner_get_theme_info (const gchar *base, const gchar *theme_name)
-{
-	GaleonSpinnerInfo *info;
-	gchar *path;
-	gchar *icon;
-
-	path = g_build_filename (base, theme_name, NULL);
-	icon = g_build_filename (path, "rest.png", NULL);
-
-	if (!g_file_test (icon, G_FILE_TEST_EXISTS))
-	{
-		g_free (path);
-		g_free (icon);
-
-		/* handle nautilus throbbers as well */
-
-		path = g_build_filename (base, theme_name, "throbber", NULL);
-		icon = g_build_filename (path, "rest.png", NULL);
-	}
-
-	if (!g_file_test (icon, G_FILE_TEST_EXISTS))
-	{
-		g_free (path);
-		g_free (icon);
-
-		return NULL;
-	}
-
-	info = g_new(GaleonSpinnerInfo, 1);
-	info->name 	= g_strdup (theme_name);
-	info->directory	= path;
-	info->filename	= icon;
-
-	return info;
-}
-
-static void
-galeon_spinner_init_directory_list (void)
-{
-	gchar *path;
-	
-	path = g_build_filename (g_get_home_dir (), GALEON_DIR, "spinners", NULL);
-	spinner_directories = g_list_append (spinner_directories, path);
-
-	path = g_build_filename (SHARE_DIR, "spinners", NULL);
-	spinner_directories = g_list_append (spinner_directories, path);
-
-	path = g_build_filename (SHARE_DIR, "..", "pixmaps", "nautilus", NULL);
-	spinner_directories = g_list_append (spinner_directories, path);
-
-#ifdef NAUTILUS_PREFIX
-	path = g_build_filename (NAUTILUS_PREFIX, "share", "pixmaps", "nautilus", NULL);
-	spinner_directories = g_list_append (spinner_directories, path);
-#endif
-}
-
-GList *
-galeon_spinner_list_spinners (void)
-{
-	GList *spinner_list = NULL;
-	GList *tmp;
-
-	for (tmp = spinner_directories; tmp != NULL; tmp = tmp->next)
-	{
-		gchar *path = tmp->data;
-		galeon_spinner_search_directory (path, &spinner_list);
-	}
-
-	return spinner_list;
-}
-
-static gchar *
-galeon_spinner_get_theme_path (const gchar *theme_name)
-{
-	GaleonSpinnerInfo *info;
-	GList *tmp;
-
-	for (tmp = spinner_directories; tmp != NULL; tmp = tmp->next)
-	{
-		gchar *path = tmp->data;
-
-		info = galeon_spinner_get_theme_info (path, theme_name);
-		if (info != NULL)
-		{
-			path = g_strdup (info->directory);
-			galeon_spinner_info_free (info);
-			return path;
-		}
-	}
-
-	return NULL;
-}
-
-void
-galeon_spinner_info_free (GaleonSpinnerInfo *info)
-{
-	g_free (info->name);
-	g_free (info->directory);
-	g_free (info->filename);
-	g_free (info);
-}
