@@ -19,13 +19,10 @@
 #ifndef GALEON_TAB_H
 #define GALEON_TAB_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #include "galeon-embed.h"
+#include <gtk/gtkbin.h>
 
-#include <glib-object.h>
+G_BEGIN_DECLS
 
 typedef struct GaleonTabClass GaleonTabClass;
 
@@ -38,11 +35,6 @@ typedef struct GaleonTabClass GaleonTabClass;
 typedef struct GaleonTab GaleonTab;
 typedef struct GaleonTabPrivate GaleonTabPrivate;
 
-typedef enum 
-{
-	TAB_CONTROL_TITLE
-} TabControlID;
-
 typedef enum
 {
 	TAB_LOAD_NONE,
@@ -52,13 +44,13 @@ typedef enum
 
 struct GaleonTab 
 {
-        GObject parent;
+        GtkBin parent;
         GaleonTabPrivate *priv;
 };
 
 struct GaleonTabClass
 {
-        GObjectClass parent_class;
+        GtkBinClass parent_class;
 };
 
 /* Include the header down here to resolve circular dependency */
@@ -68,8 +60,6 @@ GType         galeon_tab_get_type     		(void);
 
 GaleonTab    *galeon_tab_new          		(void);
 
-void 	      galeon_tab_destroy		(GaleonTab *tab);
-
 GaleonEmbed  *galeon_tab_get_embed    		(GaleonTab *tab);
 
 void          galeon_tab_set_window   		(GaleonTab *tab, 
@@ -97,6 +87,8 @@ const char   *galeon_tab_get_location           (GaleonTab *tab);
 void	      galeon_tab_set_location           (GaleonTab *tab,
 						 char *location);
 
+int           galeon_tab_get_zoom               (GaleonTab *tab);
+
 const char   *galeon_tab_get_blocked_popup_uri  (GaleonTab *tab);
 void	     galeon_tab_clear_blocked_popup_uri (GaleonTab *tab);
 
@@ -106,10 +98,6 @@ void	      galeon_tab_get_size		(GaleonTab *tab,
 
 GtkWidget    *galeon_tab_get_icon               (GaleonTab *tab);
 
-void	      galeon_tab_update_control		(GaleonTab *tab,
-						 TabControlID id);
+G_END_DECLS
 
-#ifdef __cplusplus
-}
-#endif
 #endif
