@@ -29,12 +29,13 @@
 #include "gul-string.h"
 #include "gul-state.h"
 #include "location-entry.h"
+#include "galeon-debug.h"
 
 #include <gtk/gtktreeview.h>
 #include <gtk/gtktreestore.h>
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtkcellrenderertext.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 
 #define CONF_HISTORY_SEARCH_TEXT "/apps/galeon/Browsing/History/search_text"
 #define CONF_HISTORY_SEARCH_TIME "/apps/galeon/Browsing/History/search_time"
@@ -46,19 +47,13 @@ static void history_dialog_finalize (GObject *object);
 static void history_dialog_set_embedded (HistoryDialog *d,
 			     	         gboolean embedded);
 
-//#define DEBUG_TIME
-
-#ifdef DEBUG_TIME
-#include <glib/gtimer.h>
-#endif
-
 /* Glade callbacks */
 void
 history_host_checkbutton_toggled_cb (GtkWidget *widget, 
 				     HistoryDialog *dialog);
 void
-history_time_optionmenu_changed_cb (GtkWidget *widget, 
-			            HistoryDialog *dialog);
+history_time_combobox_changed_cb (GtkWidget *widget, 
+				  HistoryDialog *dialog);
 void
 history_entry_changed_cb (GtkWidget *widget, 
 			  HistoryDialog *dialog);
@@ -113,7 +108,7 @@ GaleonDialogProperty properties [] =
 	{ PROP_DIALOG, "history_dialog", NULL, PT_NORMAL, NULL },
 	{ PROP_TREEVIEW, "history_treeview", NULL, PT_NORMAL, NULL },
 	{ PROP_WORD, "history_entry", CONF_HISTORY_SEARCH_TEXT, PT_NORMAL, NULL },
-	{ PROP_TIME, "history_time_optionmenu", CONF_HISTORY_SEARCH_TIME, PT_NORMAL, NULL },
+	{ PROP_TIME, "history_time_combobox", CONF_HISTORY_SEARCH_TIME, PT_NORMAL, NULL },
 	{ PROP_GROUP, "history_host_checkbutton", CONF_HISTORY_HOST_GROUP, PT_NORMAL, NULL },
 	{ -1, NULL, NULL }
 };
@@ -208,15 +203,14 @@ history_dialog_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path,
 	GaleonEmbed *embed;
         GtkTreeIter iter;
         GtkTreeModel *model = gtk_tree_view_get_model (tree_view);
-        GValue val = {0, };
-        const char *url;
+        char *url;
 
         gtk_tree_model_get_iter (model, &iter, path);
-        gtk_tree_model_get_value (model, &iter, 1, &val);
-        url = g_value_get_string (&val);
+        gtk_tree_model_get (model, &iter, 1, &url, -1);
 
 	embed = galeon_embed_dialog_get_embed (GALEON_EMBED_DIALOG(dialog));
 	galeon_embed_load_url (embed, url);
+	g_free (url);
 }
 
 static void
@@ -231,8 +225,7 @@ each_url_get_data_binder (GaleonDragEachSelectedItemDataGet iteratee,
 	
 	for (l = refs; l != NULL; l = l->next)
 	{
-		GValue val = {0, };
-        	const char *url, *title;
+        	char *url, *title;
 		GtkTreeIter iter;
                 GtkTreePath *path;
 
@@ -241,12 +234,15 @@ each_url_get_data_binder (GaleonDragEachSelectedItemDataGet iteratee,
                 gtk_tree_model_get_iter 
                         (model, &iter, path);
 
-		gtk_tree_model_get_value (model, &iter, 1, &val);
-	        url = g_value_get_string (&val);
+		gtk_tree_model_get (model, &iter, 
+				    0, &title,
+				    1, &url,
+				    -1);
 
-		gtk_tree_model_get_value (model, &iter, 0, &val);
-	        title = g_value_get_string (&val);
 		iteratee (url, title, -1, -1, -1, -1, data);
+
+		g_free (url);
+		g_free (title);
 	}
 
 	gul_gui_treeview_free_selection_refs (refs);
@@ -277,13 +273,15 @@ history_dialog_setup_view (HistoryDialog *dialog)
 	GtkTreeSelection *selection;
 
 	/* set tree model */
-        treestore = gtk_tree_store_new (6,
+        treestore = gtk_tree_store_new (8,
                                         G_TYPE_STRING,
                                         G_TYPE_STRING,
                                         G_TYPE_STRING,
                                         G_TYPE_STRING,
                                         G_TYPE_INT,
-					G_TYPE_POINTER);
+					G_TYPE_POINTER,
+					G_TYPE_INT,
+					G_TYPE_INT);
         dialog->priv->model = GTK_TREE_MODEL(treestore);
         gtk_tree_view_set_model (treeview, 
                                  GTK_TREE_MODEL (treestore));
@@ -339,7 +337,7 @@ history_dialog_setup_view (HistoryDialog *dialog)
         column = gtk_tree_view_get_column (treeview, 2);
         gtk_tree_view_column_set_resizable (column, TRUE);
         gtk_tree_view_column_set_reorderable (column, TRUE);
-        gtk_tree_view_column_set_sort_column_id (column, 2);
+        gtk_tree_view_column_set_sort_column_id (column, 6);
 
         gtk_tree_view_insert_column_with_attributes (treeview,
                                                      3, _("First visited"),
@@ -349,7 +347,7 @@ history_dialog_setup_view (HistoryDialog *dialog)
         column = gtk_tree_view_get_column (treeview, 3);
         gtk_tree_view_column_set_resizable (column, TRUE);
         gtk_tree_view_column_set_reorderable (column, TRUE);
-        gtk_tree_view_column_set_sort_column_id (column, 3);
+        gtk_tree_view_column_set_sort_column_id (column, 7);
 
         gtk_tree_view_insert_column_with_attributes (treeview,
                                                      4, _("Times visited"),
@@ -383,6 +381,8 @@ history_dialog_update_host_item (HistoryDialog *dialog,
                             3, text[3],
                             4, item->visits,
 			    5, item,
+			    6, item->last,
+			    7, item->first,
                             -1);
 }
 
@@ -437,6 +437,8 @@ history_dialog_update_url_item (HistoryDialog *dialog,
                             3, text[3],
                             4, item->visits,
 			    5, item,
+			    6, item->last,
+			    7, item->first,
                             -1);
 }
 
@@ -491,6 +493,11 @@ history_dialog_fill_view (HistoryDialog *dialog)
 							     item);
 			history_dialog_add_url_items (dialog, urls, iter);
 			g_list_free (urls);
+
+			if (iter)
+			{
+				gtk_tree_iter_free (iter);
+			}
 		}
 	}
 
@@ -511,6 +518,8 @@ history_dialog_setup_filter (HistoryDialog *dialog)
 	GValue time = {0, };
 
 	galeon_dialog_get_value (GALEON_DIALOG(dialog), PROP_WORD, &word);
+	/* The cast is ok as we put the string in there using g_value_get_string */
+	g_free ( (char*)dialog->priv->filter.word);
         dialog->priv->filter.word = g_value_get_string (&word);
 	
 	galeon_dialog_get_value (GALEON_DIALOG(dialog), PROP_TIME, &time);
@@ -596,11 +605,12 @@ history_item_add_cb (GlobalHistory *gh,
 						     host);
 	}
 
-	g_return_if_fail (iter != NULL);
-
 	history_dialog_add_url_item (dialog, item, iter);
-	
-	gtk_tree_iter_free (iter);
+
+	if (iter)
+	{
+		gtk_tree_iter_free (iter);
+	}
 }
 
 static void
@@ -714,7 +724,7 @@ history_dialog_set_embedded (HistoryDialog *dialog,
 				                   PROP_DIALOG),
 			"History", -1, -1);
 	}
-	
+
 	dialog->priv->treeview = GTK_TREE_VIEW (
 				 galeon_dialog_get_control (GALEON_DIALOG(dialog), 
 							    PROP_TREEVIEW));
@@ -755,6 +765,8 @@ history_dialog_finalize (GObject *object)
 		 G_CALLBACK (history_item_remove_cb),
 		 dialog);
 	
+	/* The cast is ok as we put the string in there using g_value_get_string */
+	g_free ((char*)p->filter.word);
         g_free (p);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -805,31 +817,25 @@ history_filter_entry_timeout_cb (gpointer data)
 {
 	HistoryDialog *dialog = data;
 	HistoryDialogPrivate *p = dialog->priv;
-#ifdef DEBUG_TIME
-	GTimer *timer = g_timer_new ();
-	g_timer_start (timer);
-#endif
 
 	p->filter_entry_timeout = 0;
 
+	START_PROFILER ("Clear liststore");
+
 	history_dialog_clear_view (dialog);
-#ifdef DEBUG_TIME
-	g_print ("HD: %f elapsed clearing liststore\n", g_timer_elapsed (timer, NULL));
-	g_timer_start (timer);
-#endif	
+
+	STOP_PROFILER ("Clear liststore");
+	START_PROFILER ("Setting filter");
+
 	history_dialog_setup_filter (dialog);
-#ifdef DEBUG_TIME
-	g_print ("HD: %f elapsed setting filter\n", g_timer_elapsed (timer, NULL));
-	g_timer_start (timer);
-#endif	
+
+	STOP_PROFILER ("Setting filter");
+	START_PROFILER ("Filling view");
+
 	history_dialog_fill_view (dialog);
-#ifdef DEBUG_TIME
-	g_print ("HD: %f elapsed filling view\n", g_timer_elapsed (timer, NULL));
-#endif	
+
+	STOP_PROFILER ("Filling view");
 
-#ifdef DEBUG_TIME
-	g_timer_destroy (timer);
-#endif
 	return FALSE;
 }
 
@@ -852,8 +858,8 @@ history_entry_changed_cb (GtkWidget *widget,
 }
 
 void
-history_time_optionmenu_changed_cb (GtkWidget *widget, 
-			            HistoryDialog *dialog)
+history_time_combobox_changed_cb (GtkWidget *widget, 
+				  HistoryDialog *dialog)
 {
 	if (dialog->priv->treeview == NULL) return;
 	history_dialog_clear_view (dialog);
