@@ -29,7 +29,7 @@
 #include <string.h>
 #include <time.h>
 
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkdialog.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
@@ -298,26 +298,6 @@ session_autoresume (Session *session)
 	return (session->priv->windows != NULL) ? SESSION_RESUMED : SESSION_OK;
 }
 
-static void
-create_session_directory (void)
-{
-	char *dir;
-
-	dir = g_build_filename (g_get_home_dir(),
-			        GALEON_DIR"/sessions", 
-			        NULL);
-	
-	if (!g_file_test (dir, G_FILE_TEST_EXISTS))
-	{
-		if (mkdir (dir, 488) != 0)
-                {
-                        g_error ("couldn't make %s' directory", dir);
-                }
-	}
-
-	g_free (dir);
-}
-
 static gboolean 
 save_yourself_cb  (GnomeClient *client,
                    gint phase,
@@ -327,21 +307,32 @@ save_yourself_cb  (GnomeClient *client,
                    gboolean fast,
 		   Session *session)
 {
-	char *argv[] = { "galeon", "--load-session", NULL };
+	char *argv[4] = { "galeon", NULL, NULL, NULL };
 	char *discard_argv[] = { "rm", "-r", NULL };
-	
-	argv[2] = get_session_filename (SESSION_GNOME);
-	gnome_client_set_restart_command 
-		(client, 3, argv);
-     
-	discard_argv[2] = argv[2];
-     	gnome_client_set_discard_command (client, 3, 
-					  discard_argv);
-			
-	session_save (session, argv[2]);
+	char *session_file = NULL;
+	int count = 1;
 
-	g_free (argv[2]);
-	
+	if (galeon_shell_get_server_mode (galeon_shell))
+	{
+		argv[count++] = "--server";
+	}
+
+	if (session_get_windows (session))
+	{
+		session_file = get_session_filename (SESSION_GNOME);
+		discard_argv[2] = session_file;
+		gnome_client_set_discard_command (client, 3, discard_argv);
+
+		argv[count++] = "--load-session";
+		argv[count++] = session_file;
+
+		session_save (session, session_file);
+	}
+
+	gnome_client_set_restart_command (client, count, argv);
+
+	g_free (session_file);
+     
 	return TRUE;
 }
 
@@ -385,8 +376,6 @@ session_init (Session *session)
 	session->priv->dont_remove_crashed = FALSE;
 	session->priv->session_saved = FALSE;
 	
-	create_session_directory ();
-
 	gnome_session_init (session);
 }
 
@@ -535,6 +524,13 @@ session_save (Session *session,
 	
 	g_return_if_fail (IS_SESSION (session));
 
+	/* Dont save empty sessions, just delete the session file */
+	if (!session->priv->windows)
+	{
+		session_delete (session, filename);
+		return;
+	}
+
 	save_to = get_session_filename (filename);
 		
         doc = xmlNewDoc ("1.0");
@@ -563,14 +559,14 @@ session_save (Session *session,
                 gtk_window_get_position (GTK_WINDOW(wmain), &x, &y);
 
                 /* set window properties */
-                snprintf(buffer, 32, "%d", x);
+                g_snprintf(buffer, 32, "%d", x);
                 xmlSetProp (window_node, "x", buffer);
-                snprintf(buffer, 32, "%d", y);
+                g_snprintf(buffer, 32, "%d", y);
 
 		xmlSetProp (window_node, "y", buffer);
-                snprintf(buffer, 32, "%d", width);
+                g_snprintf(buffer, 32, "%d", width);
                 xmlSetProp (window_node, "width", buffer);
-                snprintf(buffer, 32, "%d", height);
+                g_snprintf(buffer, 32, "%d", height);
                 xmlSetProp (window_node, "height", buffer);
 
 		for (l = tabs; l != NULL; l = l->next)
@@ -584,7 +580,7 @@ session_save (Session *session,
         }
 
         /* save it all out to disk */
-        xmlSaveFile (save_to, doc);
+	gul_general_safe_xml_save (save_to, doc);
         xmlFreeDoc (doc);
 
 	g_free (save_to);
@@ -631,8 +627,13 @@ parse_xml (Session *session,
 	save_to = get_session_filename (filename);
 
 	doc = xmlParseFile (save_to);
+	if (doc == NULL)
+	{
+		return;
+	}
 
 	child = xmlDocGetRootElement (doc); 
+	g_return_if_fail (child!= NULL);
 
 	/* skip the session node */
 	child = child->children;
@@ -668,9 +669,10 @@ create_new_tab (const char *url, const char *title, GaleonWindow *window)
 	tab = galeon_tab_new ();	
 	embed = galeon_tab_get_embed (tab);
 	
-	gtk_widget_show (GTK_WIDGET(embed));
+	gtk_widget_show (GTK_WIDGET(tab));
 	
-	galeon_window_add_tab (window, tab, FALSE);
+	galeon_window_add_tab (window, tab,
+			       GALEON_WINDOW_ADD_TAB_LAST, FALSE);
 	
 	galeon_embed_load_url (embed, url);
 }
@@ -710,6 +712,7 @@ bookmark_from_tab (const char *url, const char *title, GbFolder *folder)
 	set = galeon_shell_get_bookmark_set (galeon_shell);
 	new_site = gb_site_new (set, title, url);
 	gb_folder_add_child (folder, GB_BOOKMARK(new_site), -1);
+	gb_bookmark_set_time_added_now (GB_BOOKMARK (new_site));
 }
 
 static void
@@ -736,7 +739,7 @@ session_load (Session *session, const char *filename)
 	parse_xml (session, filename, (WindowCb)create_new_window, NULL);
 }
 
-const GList *
+GList *
 session_get_windows (Session *session)
 {
 	g_return_val_if_fail (IS_SESSION (session), NULL);
@@ -749,14 +752,13 @@ session_get_windows (Session *session)
  * in a subfolder of the root folder called "Sites opened before
  * crash on <date> at <time>"
  */
-#define DATE_LENGTH 30
 static
 void session_create_bookmarks (Session *session, const char *filename)
 {
 	GbFolder *root_folder;
 	GbFolder *new_folder;
 	GbBookmarkSet *set;
-	char date[DATE_LENGTH];
+	char date[64];
 	const char *date_hack="%c"; /* This is necessary to avoid a warning */
 	struct tm *tm;
 	time_t timep;
@@ -772,7 +774,7 @@ void session_create_bookmarks (Session *session, const char *filename)
 
 	time (&timep);
 	tm = localtime (&timep);
-	if (strftime (date, DATE_LENGTH, date_hack, tm) != 0) {
+	if (strftime (date, sizeof(date), date_hack, tm) != 0) {
 		gchar *date_utf8 = g_locale_to_utf8 (date, -1,
 						     NULL, NULL, NULL);
 		
@@ -783,8 +785,9 @@ void session_create_bookmarks (Session *session, const char *filename)
 	}
 	new_folder =  gb_folder_new (set, folder_name);
 	g_free (folder_name);
-	gb_folder_add_child (root_folder, GB_BOOKMARK(new_folder), -1);
-	
+	gb_folder_add_child (root_folder, GB_BOOKMARK (new_folder), -1);
+	gb_bookmark_set_time_added_now (GB_BOOKMARK (new_folder));
+
 	/* Add bookmarks corresponding to the previous session to this folder
 	 */
 	parse_xml (session, filename, (WindowCb)bookmarks_from_window, 
@@ -828,6 +831,8 @@ session_remove_window (Session *session,
 
 	session->priv->windows = g_list_remove (session->priv->windows, window);
 
+	session_save (session, SESSION_CRASHED);
+
 	/* autodestroy of the session, necessay to avoid
 	 * conflicts with the nautilus view */
 	if (session->priv->windows == NULL && 
