@@ -78,7 +78,7 @@ void	      session_save     		(Session *session,
 
 SessionResumeType session_autoresume   	(Session *session);
 
-const GList  *session_get_windows     	(Session *session);
+GList	     *session_get_windows     	(Session *session);
 
 void          session_add_window      	(Session *session,
 					 GaleonWindow *window);
@@ -86,8 +86,6 @@ void          session_add_window      	(Session *session,
 void          session_remove_window     (Session *session,
 					 GaleonWindow *window);
 
-GaleonWindow *session_get_active_window (Session *session);
-
 G_END_DECLS
 
 #endif
