@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen
+ *  Copyright (C) 2003 Philip Langdale (Egg port)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,13 +20,11 @@
 #ifndef STATUSBAR_H
 #define STATUSBAR_H
 
-#include "galeon-window.h"
+#include <gtk/gtkstatusbar.h>
+#include <gtk/gtktoggleaction.h>
 
 G_BEGIN_DECLS
 
-#include <glib-object.h>
-#include <glib.h>
-
 typedef struct Statusbar Statusbar;
 typedef struct StatusbarClass StatusbarClass;
 
@@ -37,20 +36,29 @@ typedef struct StatusbarClass StatusbarClass;
 
 typedef struct StatusbarPrivate StatusbarPrivate;
 
+typedef enum {
+	STATUSBAR_SHOW_SECURITY_INFO
+} StatusbarRequest;
+
 struct Statusbar
 {
-        GObject parent;
+        GtkStatusbar parent;
         StatusbarPrivate *priv;
 };
 
 struct StatusbarClass
 {
-        GObjectClass parent_class;
+        GtkStatusbarClass parent_class;
+
+	/* signals */
+
+	void (* request)   (Statusbar  *self,
+			    StatusbarRequest  request);
 };
 
 GType         statusbar_get_type             (void);
 
-Statusbar    *statusbar_new                  (GaleonWindow *window);
+GtkWidget    *statusbar_new                  (GtkToggleAction *offline_action);
 
 void          statusbar_set_visibility       (Statusbar *s, 
 					      gboolean visibility);
