@@ -21,61 +21,49 @@
 #include "galeon-shell.h"
 #include "galeon-config.h"
 #include "window-commands.h"
-#include "find-dialog.h"
 #include "print-dialog.h"
+#include "bookmarks-add-dialog.h"
 #include "bookmarks-editor.h"
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
 #include "galeon-embed-utils.h"
 #include "gul-general.h"
+#include "page-info-dialog.h"
 #include "pdm-dialog.h"
-#include "toolbar.h"
 #include "galeon-window-bookmark-activated-proxy.h"
-#include "gul-toolbar-editor.h"
-#include "gul-bonobo-extensions.h"
 #include "galeon-prefs-dialog.h"
+#include "egg-toolbar-editor.h"
+#include "gul-state.h"
+#include "gul-gui.h"
+#include "gul-file-chooser.h"
 
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-about.h>
-#include <libgnome/gnome-help.h>
-#include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkeditable.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkaction.h>
+#include <gtk/gtkstock.h>
 
-#define AVAILABLE_TOOLBAR_ITEMS \
-	"new=new_button;" \
-	"back=navigation_button(direction=back,arrow=FALSE);" \
-	"back_menu=navigation_button(direction=back,arrow=TRUE);" \
-	"forward=navigation_button(direction=forward,arrow=FALSE);" \
-	"forward_menu=navigation_button(direction=forward,arrow=TRUE);" \
-	"up=navigation_button(direction=up,arrow=FALSE);" \
-	"up_menu=navigation_button(direction=up,arrow=TRUE);" \
-	"stop=stop_button;" \
-	"reload=reload_button;" \
-	"home=home_button;" \
-	"favicon=favicon;" \
-	"location=location;" \
-	"go=go_button;" \
-	"zoom=zoom;" \
-	"spinner=spinner;" \
-	"fullscreen=fullscreen_button;" \
-	"separator;"
+#define VIEW_IMAGES_BLOCKSITE_ACTION	"ViewImagesBlockSiteAction"
+#define VIEW_IMAGES_ALLOWSITE_ACTION	"ViewImagesAllowSiteAction"
+#define WEB_COOKIES_BLOCKSITE_ACTION	"WebCookiesBlockSiteAction"
+#define WEB_COOKIES_ALLOWSITE_ACTION	"WebCookiesAllowSiteAction"
+#define WEB_POPUPS_ALLOWSITE_ACTION	"WebPopupsAllowSiteAction"
 
+enum
+{
+	RESPONSE_ADD_TOOLBAR
+};
 
 
 void
-window_cmd_edit_find (BonoboUIComponent *uic, 
-		      GaleonWindow *window, 
-		      const char* verbname)
+window_cmd_edit_find (GtkAction *action, 
+		      GaleonWindow *window) 
 {
-	GaleonDialog *dialog;
-	dialog = galeon_window_get_find_dialog (window);
-
-	g_object_ref (dialog);
-	
-	galeon_dialog_show (dialog);
+	galeon_window_show_find (window);
 }
 
 static void 
@@ -86,9 +74,8 @@ print_dialog_preview_cb (PrintDialog *dialog,
 }
 
 void 
-window_cmd_file_print (BonoboUIComponent *uic, 
-		       GaleonWindow *window, 
-		       const char* verbname)
+window_cmd_file_print (GtkAction *action, 
+		       GaleonWindow *window) 
 {
 	GaleonDialog *dialog;
 	GaleonEmbed *embed;
@@ -107,9 +94,8 @@ window_cmd_file_print (BonoboUIComponent *uic,
 }
 
 void
-window_cmd_go_back (BonoboUIComponent *uic, 
-		    GaleonWindow *window, 
-		    const char* verbname)
+window_cmd_go_back (GtkAction *action, 
+		    GaleonWindow *window) 
 {
 	GaleonEmbed *embed;
 
@@ -120,9 +106,8 @@ window_cmd_go_back (BonoboUIComponent *uic,
 }
 
 void
-window_cmd_go_up (BonoboUIComponent *uic, 
-		    GaleonWindow *window, 
-		    const char* verbname)
+window_cmd_go_up (GtkAction *action, 
+		    GaleonWindow *window) 
 {
 	GaleonEmbed *embed;
 
@@ -133,9 +118,8 @@ window_cmd_go_up (BonoboUIComponent *uic,
 }
 
 void 
-window_cmd_file_send_to	(BonoboUIComponent *uic, 
-		      	 GaleonWindow *window, 
-		      	 const char* verbname)
+window_cmd_file_send_to	(GtkAction *action, 
+		      	 GaleonWindow *window) 
 {
 	char *url;
 	GaleonTab *tab;
@@ -173,9 +157,8 @@ window_cmd_file_send_to	(BonoboUIComponent *uic,
 }
 
 void
-window_cmd_go_forward (BonoboUIComponent *uic, 
-		       GaleonWindow *window, 
-		       const char* verbname)
+window_cmd_go_forward (GtkAction *action, 
+		       GaleonWindow *window) 
 {
 	GaleonEmbed *embed;
 
@@ -186,28 +169,8 @@ window_cmd_go_forward (BonoboUIComponent *uic,
 }
 
 void
-window_cmd_go_go (BonoboUIComponent *uic, 
-		  GaleonWindow *window, 
-		  const char* verbname)
-{
-	Toolbar *tb;
-	
-	g_return_if_fail (IS_GALEON_WINDOW (window));
-	
-	tb = galeon_window_get_toolbar (window);
-
-	if (tb)
-	{
-		char *location = toolbar_get_location (tb);
-		galeon_window_load_url (window, location);
-		g_free (location);
-	}
-}
-
-void
-window_cmd_go_home (BonoboUIComponent *uic, 
-		    GaleonWindow *window, 
-		    const char* verbname)
+window_cmd_go_home (GtkAction *action, 
+		    GaleonWindow *window) 
 {
 	GaleonEmbed *embed;
 	char *location;
@@ -224,9 +187,8 @@ window_cmd_go_home (BonoboUIComponent *uic,
 }
 
 void
-window_cmd_go_myportal (BonoboUIComponent *uic, 
-		        GaleonWindow *window, 
-		        const char* verbname)
+window_cmd_go_myportal (GtkAction *action, 
+		        GaleonWindow *window) 
 {
 	GaleonEmbed *embed;
 	
@@ -237,17 +199,15 @@ window_cmd_go_myportal (BonoboUIComponent *uic,
 }
 
 void
-window_cmd_go_location (BonoboUIComponent *uic, 
-		        GaleonWindow *window, 
-		        const char* verbname)
+window_cmd_go_location (GtkAction *action, 
+		        GaleonWindow *window) 
 {
 	galeon_window_edit_location (window);
 }
 
 void
-window_cmd_go_stop (BonoboUIComponent *uic, 
-		    GaleonWindow *window, 
-		    const char* verbname)
+window_cmd_go_stop (GtkAction *action, 
+		    GaleonWindow *window) 
 {
 	GaleonEmbed *embed;
 
@@ -258,23 +218,34 @@ window_cmd_go_stop (BonoboUIComponent *uic,
 }
 
 void
-window_cmd_go_reload (BonoboUIComponent *uic, 
-		      GaleonWindow *window, 
-		      const char* verbname)
+window_cmd_go_reload (GtkAction *action, 
+		      GaleonWindow *window) 
 {
 	GaleonEmbed *embed;
+	GdkEvent *event;
+	EmbedReloadFlags reload_type = EMBED_RELOAD_NORMAL;
 
 	embed = galeon_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	galeon_embed_reload (embed, EMBED_RELOAD_NORMAL);
+	event = gtk_get_current_event ();
+	if (event)
+	{
+		if (event->type == GDK_BUTTON_RELEASE &&
+		    event->button.state & GDK_SHIFT_MASK)
+		{
+			reload_type = EMBED_RELOAD_FORCE;
+		}
+		gdk_event_free (event);
+	}
+
+	galeon_embed_reload (embed, reload_type);
 }
 
 static void
-open_new_tab	(GaleonWindow *window, GaleonNewTabFlags flags)
+open_new_tab (GaleonWindow *window, GaleonNewTabFlags flags)
 {
 	GaleonTab *tab;
-	Toolbar *tb;
 
 	flags |= GALEON_NEW_TAB_HOMEPAGE | GALEON_NEW_TAB_JUMP;
 
@@ -283,39 +254,35 @@ open_new_tab	(GaleonWindow *window, GaleonNewTabFlags flags)
 	tab = galeon_shell_new_tab (galeon_shell, window, tab, NULL, flags);
 
 	window = galeon_tab_get_window (tab);
-	tb = galeon_window_get_toolbar (window);
-	toolbar_set_location (tb, "");
-	toolbar_edit_location (tb);
+	galeon_window_set_location_entry_location (window, "");
+	galeon_window_edit_location (window);
 }
 
 void
-window_cmd_new (BonoboUIComponent *uic, 
-		GaleonWindow *window, 
-		const char* verbname)
+window_cmd_new (GtkAction *action, 
+		GaleonWindow *window) 
 {
-	open_new_tab (window, 0);
+	open_new_tab (window, GALEON_NEW_TAB_APPEND);
 }
 
 void
-window_cmd_new_window (BonoboUIComponent *uic, 
-		       GaleonWindow *window, 
-		       const char* verbname)
+window_cmd_new_window (GtkAction *action, 
+		       GaleonWindow *window) 
 {
 	open_new_tab (window, GALEON_NEW_TAB_IN_NEW_WINDOW);
 }
 
 void
-window_cmd_new_tab (BonoboUIComponent *uic, 
-		    GaleonWindow *window, 
-		    const char* verbname)
+window_cmd_new_tab (GtkAction *action, 
+		    GaleonWindow *window) 
 {
-	open_new_tab (window, GALEON_NEW_TAB_IN_EXISTING_WINDOW);
+	open_new_tab (window, GALEON_NEW_TAB_IN_EXISTING_WINDOW |
+			      GALEON_NEW_TAB_APPEND);
 }
 
 void
-window_cmd_bookmarks_edit (BonoboUIComponent *uic, 
-			   GaleonWindow *window, 
-			   const char* verbname)
+window_cmd_bookmarks_edit (GtkAction *action, 
+			   GaleonWindow *window) 
 {
 	GbBookmarkSet *set;
 	GbEditor *e;
@@ -332,134 +299,164 @@ window_cmd_bookmarks_edit (BonoboUIComponent *uic,
 	galeon_window_bookmark_activated_proxy_connect (wbap, G_OBJECT (e));
 }
 
-void
-window_cmd_bookmarks_add_default (BonoboUIComponent *uic, 
-				  GaleonWindow *window, 
-				  const char* verbname)
+void window_cmd_bookmarks_add_interactive(GtkAction *action, 
+			    		  GaleonWindow *window)
 {
 	GaleonTab *tab;
 	const char *title;
 	const char *location;
+	GaleonDialog *dialog;
 	GbBookmarkSet *set;
 	GbSite *b;
+	GSList *list = NULL;
 
 	tab = galeon_window_get_active_tab (window);
 	g_return_if_fail (tab);
 
-	title = galeon_tab_get_title (tab);
-	location = galeon_tab_get_location (tab);
-
 	set = galeon_shell_get_bookmark_set (galeon_shell);
 	g_return_if_fail (set);
 
-	b = gb_site_new (set, title, location);
-	gb_bookmark_set_add_default (set, GB_BOOKMARK (b));
+	title = galeon_tab_get_title (tab);
+	location = galeon_tab_get_location (tab);
+
+	b = gb_site_new(set, title, location);
+
+	list = g_slist_append(list, b);
+
+	dialog = gb_add_dialog_new(GTK_WIDGET(window), set, list);
+
+	galeon_dialog_show(dialog);
 }
 
-void
-window_cmd_file_open (BonoboUIComponent *uic, 
-		      GaleonWindow *window, 
-		      const char* verbname)
+void window_cmd_bookmarks_add_tab_folder(GtkAction *action, 
+					 GaleonWindow *window)
 {
-	gchar *dir, *retDir;
-        gchar *file;
-        GnomeVFSURI *uri;
-	GtkWidget *wmain;
-	GaleonEmbedShell *embed_shell;
-	gresult result;
+	GaleonDialog *dialog;
+	GbBookmarkSet *set;
+	GList *tabs, *i;
+	GSList *list = NULL;
 
-	embed_shell = galeon_shell_get_embed_shell (galeon_shell);
-	g_return_if_fail (embed_shell != NULL);
-	
-	wmain = GTK_WIDGET (window);
-	g_return_if_fail (wmain != NULL);
-	
-        dir = eel_gconf_get_string (CONF_STATE_OPEN_DIR);
+	tabs = galeon_window_get_tabs(window);
+	g_return_if_fail(tabs);
 
-	result = galeon_embed_shell_show_file_picker 
-		(embed_shell, wmain, 
-		 _("Select the file to open"),
-                 dir, NULL, modeOpen,
-                 &file, NULL, NULL, NULL);
+	set = galeon_shell_get_bookmark_set(galeon_shell);
+	g_return_if_fail(set);
 
-	if (result == G_OK)
+	for (i = tabs ; i ; i = i->next)
 	{
-                uri = gnome_vfs_uri_new (file);
-                if (uri)
-                {
-			
-                        galeon_window_load_url(window, file);
+		GaleonTab *tab = GALEON_TAB(i->data);
+		const gchar *title = galeon_tab_get_title(tab);
+		const gchar *location = galeon_tab_get_location(tab);
+		GbSite *b = gb_site_new(set, title, location);
 
-                        retDir = gnome_vfs_uri_extract_dirname (uri);
+		list = g_slist_append(list, b);
+	}
 
-                        /* set default open dir */
-                        eel_gconf_set_string (CONF_STATE_OPEN_DIR, 
-                                              retDir);
+	dialog = gb_add_dialog_new(GTK_WIDGET(window), set, list);
 
-                        g_free (retDir);
-                        gnome_vfs_uri_unref (uri);
-                }
+	galeon_dialog_show(dialog);
+}
 
-		g_free (file);
-        }
+static void
+open_response_cb (GtkDialog *dialog, int response, GaleonWindow *window)
+{
+	if (response == GTK_RESPONSE_ACCEPT)
+	{
+		char *uri, *converted;
 
-        g_free (dir);   
+		uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+		if (uri != NULL)
+		{
+			converted = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
+	
+			if (converted != NULL)
+			{
+				galeon_window_load_url (window, converted);
+			}
+	
+			g_free (converted);
+			g_free (uri);
+	        }
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 void
-window_cmd_file_save_as (BonoboUIComponent *uic, 
-		      	 GaleonWindow *window, 
-		         const char* verbname)
+window_cmd_file_open (GtkAction *action,
+		      GaleonWindow *window)
+{
+	GulFileChooser *dialog;
+
+	dialog = gul_file_chooser_new (_("Open"),
+				       GTK_WIDGET (window),
+				       GTK_FILE_CHOOSER_ACTION_OPEN,
+				       CONF_STATE_OPEN_DIR);
+
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (open_response_cb), window);
+
+	gtk_widget_show (GTK_WIDGET (dialog));
+}
+
+
+void
+window_cmd_file_save_as (GtkAction *action, 
+		      	 GaleonWindow *window) 
 {
 	GaleonEmbed *embed;
 	GaleonEmbedPersist *persist;
-	char *url;
 	
 	embed = galeon_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
 	persist = galeon_embed_persist_new (embed);
 	galeon_embed_persist_set_flags (persist,
-					EMBED_PERSIST_MAINDOC);
+					EMBED_PERSIST_MAINDOC | 
+					EMBED_PERSIST_ASK_DESTINATION |
+					EMBED_PERSIST_SAVE_CONTENT);
 
-	galeon_embed_get_location(embed, TRUE, TRUE, &url);
-	galeon_embed_persist_set_source(persist, url);
-
-	galeon_embed_utils_save (GTK_WIDGET(window), 
-			    	 CONF_STATE_SAVE_DIR,
-			    	 TRUE,
-			    	 TRUE,
-			    	 persist);
+	galeon_embed_persist_set_fc_parent (persist, GTK_WIDGET (window));
+	galeon_embed_persist_set_fc_title (persist, _("Save As..."));
+	galeon_embed_persist_save (persist);
 
 	g_object_unref (G_OBJECT(persist));
 }
 
-void window_cmd_file_quit(BonoboUIComponent *uic,
-			  GaleonWindow *window,
-			  const char* verbname)
+void window_cmd_file_quit(GtkAction *action,
+			  GaleonWindow *window)
 {
 	Session *session =
 		galeon_shell_get_session(galeon_shell);
 
+	GList *windows = session_get_windows (session);
+	GList *l;
+	for (l = windows ; l ; l = l->next)
+	{
+		if (!galeon_window_can_close (GALEON_WINDOW (l->data)))
+		{
+			return;
+		}
+	}
+	/* Don't free the windows list, it is directly from the Session */
+
 	session_close(session);
 }
 
 void
-window_cmd_file_close_tab (BonoboUIComponent *uic, 
-		           GaleonWindow *window, 
-		           const char* verbname)
+window_cmd_file_close_tab (GtkAction *action, 
+		           GaleonWindow *window) 
 {
 	GaleonTab *tab;
 	
 	tab = galeon_window_get_active_tab (window);
 	g_return_if_fail (tab != NULL);
-	galeon_tab_destroy (tab);
+	galeon_window_remove_tab (window, tab);
 }
 
 void 
-window_cmd_edit_cut (BonoboUIComponent *uic, 
-		     GaleonWindow *window, 
-		     const char* verbname)
+window_cmd_edit_cut (GtkAction *action, 
+		     GaleonWindow *window) 
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
 
@@ -478,9 +475,8 @@ window_cmd_edit_cut (BonoboUIComponent *uic,
 }
 
 void
-window_cmd_edit_copy (BonoboUIComponent *uic, 
-		      GaleonWindow *window, 
-		      const char* verbname)
+window_cmd_edit_copy (GtkAction *action, 
+		      GaleonWindow *window) 
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
 
@@ -500,9 +496,8 @@ window_cmd_edit_copy (BonoboUIComponent *uic,
 }
 
 void
-window_cmd_edit_paste (BonoboUIComponent *uic, 
-		       GaleonWindow *window, 
-		       const char* verbname)
+window_cmd_edit_paste (GtkAction *action, 
+		       GaleonWindow *window) 
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
 
@@ -522,9 +517,8 @@ window_cmd_edit_paste (BonoboUIComponent *uic,
 }
 
 void 
-window_cmd_edit_select_all (BonoboUIComponent *uic, 
-		      	    GaleonWindow *window, 
-		      	    const char* verbname)
+window_cmd_edit_select_all (GtkAction *action, 
+		      	    GaleonWindow *window) 
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (window));
 
@@ -544,41 +538,38 @@ window_cmd_edit_select_all (BonoboUIComponent *uic,
 }
 
 void
-window_cmd_edit_find_next (BonoboUIComponent *uic, 
-			   GaleonWindow *window, 
-			   const char* verbname)
+window_cmd_edit_find_next (GtkAction *action, 
+			   GaleonWindow *window) 
 {
-	GaleonDialog *dialog;
-	
-	dialog = galeon_window_get_find_dialog (window);
+	GaleonEmbed *embed;
 
-	find_dialog_go_next (FIND_DIALOG(dialog), FALSE);
+	embed = galeon_window_get_active_embed (window);
+	g_return_if_fail (embed != NULL);
 
-	galeon_window_update_control (window, FindControl);
+	galeon_embed_find_next (embed, FALSE);
 }
 
 void
-window_cmd_edit_find_prev (BonoboUIComponent *uic, 
-			   GaleonWindow *window, 
-			   const char* verbname)
+window_cmd_edit_find_prev (GtkAction *action, 
+			   GaleonWindow *window) 
 {
-	GaleonDialog *dialog;
-	
-	dialog = galeon_window_get_find_dialog (window);
+	GaleonEmbed *embed;
 
-	find_dialog_go_prev (FIND_DIALOG(dialog), FALSE);
+	embed = galeon_window_get_active_embed (window);
+	g_return_if_fail (embed != NULL);
 
-	galeon_window_update_control (window, FindControl);
+	galeon_embed_find_next (embed, TRUE);
 }
 
 void 
-window_cmd_set_permission(BonoboUIComponent *uic, 
-		      	  GaleonWindow *window, 
-			  const char* verbname)
+window_cmd_set_permission(GtkAction *action, 
+		      	  GaleonWindow *window) 
 {
+
 	gresult result;
 
 	gchar *url;
+	const char *verbname;
 	PermissionType type;
 	gboolean value;
 
@@ -592,67 +583,39 @@ window_cmd_set_permission(BonoboUIComponent *uic,
 	result = galeon_embed_get_location(embed, TRUE, FALSE, &url);
 	if(result != G_OK) return;
 
-	if (!strcmp(verbname, "AllowCookieSite"))
+	verbname = gtk_action_get_name (action);
+
+	if (!strcmp(verbname, WEB_COOKIES_ALLOWSITE_ACTION))
 	{
 		type = COOKIES_PERMISSION;
 		value = TRUE;
-#if MOZILLA_SNAPSHOT <= 6
-		gul_bonobo_set_sensitive(uic, "/commands/AllowCookieSite",
-					 FALSE);
-		gul_bonobo_set_sensitive(uic, "/commands/BlockCookieSite",
-					 TRUE);
-#endif
 	}
-	else if (!strcmp(verbname, "BlockCookieSite"))
+	else if (!strcmp(verbname, WEB_COOKIES_BLOCKSITE_ACTION))
 	{
 		type = COOKIES_PERMISSION;
 		value = FALSE;
-#if MOZILLA_SNAPSHOT <= 6
-		gul_bonobo_set_sensitive(uic, "/commands/AllowCookieSite",
-					 TRUE);
-		gul_bonobo_set_sensitive(uic, "/commands/BlockCookieSite",
-					 FALSE);
-#endif
 	}
-	else if (!strcmp(verbname, "AllowImageSite"))
+	else if (!strcmp(verbname, VIEW_IMAGES_ALLOWSITE_ACTION))
 	{
 		type = IMAGES_PERMISSION;
 		value = TRUE;
-#if MOZILLA_SNAPSHOT <= 6
-		gul_bonobo_set_sensitive(uic, "/commands/AllowImageSite",
-					 FALSE);
-		gul_bonobo_set_sensitive(uic, "/commands/BlockImageSite",
-					 TRUE);
-#endif
 	}
-	else if (!strcmp(verbname, "BlockImageSite"))
+	else if (!strcmp(verbname, VIEW_IMAGES_BLOCKSITE_ACTION))
 	{
 		type = IMAGES_PERMISSION;
 		value = FALSE;
-#if MOZILLA_SNAPSHOT <= 6
-		gul_bonobo_set_sensitive(uic, "/commands/AllowImageSite",
-					 TRUE);
-		gul_bonobo_set_sensitive(uic, "/commands/BlockImageSite",
-					 FALSE);
-#endif
 	}
-	else //if (!strcmp(verbname, "AllowPopupSite"))
+	else /* if (!strcmp(verbname, WEB_POPUPS_ALLOWSITE_ACTION)) */
 	{
 		type = POPUPS_PERMISSION;
 		value = TRUE;
-#if MOZILLA_SNAPSHOT <=6
-		gul_bonobo_set_sensitive(uic, "/commands/AllowPopupSite",
-					 FALSE);
-#endif
 	}
-
 	galeon_embed_shell_set_permission(embed_shell, url, type, value);
 }
 
 void
-window_cmd_show_java_console(BonoboUIComponent *uic,
-			     GaleonWindow *window,
-			     const char* verbname)
+window_cmd_show_java_console(GtkAction *action,
+			     GaleonWindow *window)
 {
 	GaleonEmbedShell *embed_shell;
 	gresult result;
@@ -663,9 +626,8 @@ window_cmd_show_java_console(BonoboUIComponent *uic,
 	result = galeon_embed_shell_show_java_console(embed_shell);
 }
 
-void window_cmd_show_js_console(BonoboUIComponent *uic,
-				GaleonWindow *window,
-				const char* verbname)
+void window_cmd_show_js_console(GtkAction *action,
+				GaleonWindow *window)
 {
 	GaleonEmbedShell *embed_shell;
 	gresult result;
@@ -677,9 +639,8 @@ void window_cmd_show_js_console(BonoboUIComponent *uic,
 }
 
 void 
-window_cmd_view_zoom_in	(BonoboUIComponent *uic, 
-		      	 GaleonWindow *window, 
-		      	 const char *verbname)
+window_cmd_view_zoom_in	(GtkAction *action, 
+		      	 GaleonWindow *window) 
 {
 	GaleonEmbed *embed;
 	int zoom;
@@ -692,9 +653,8 @@ window_cmd_view_zoom_in	(BonoboUIComponent *uic,
 }
 
 void 
-window_cmd_view_zoom_out (BonoboUIComponent *uic, 
-		      	  GaleonWindow *window, 
-		          const char* verbname)
+window_cmd_view_zoom_out (GtkAction *action, 
+		      	  GaleonWindow *window) 
 {
 	GaleonEmbed *embed;
 	int zoom;
@@ -710,17 +670,15 @@ window_cmd_view_zoom_out (BonoboUIComponent *uic,
 }
 
 void 
-window_cmd_view_zoom_normal (BonoboUIComponent *uic, 
-		      	     GaleonWindow *window, 
-		      	     const char* verbname)
+window_cmd_view_zoom_normal (GtkAction *action, 
+		      	     GaleonWindow *window) 
 {
 	galeon_window_set_zoom (window, 100);
 }
 
 void 
-window_cmd_view_page_source (BonoboUIComponent *uic, 
-		      	     GaleonWindow *window, 
-		      	     const char* verbname)
+window_cmd_view_page_source (GtkAction *action, 
+		      	     GaleonWindow *window) 
 {
 	GaleonTab *tab;
 	
@@ -728,13 +686,20 @@ window_cmd_view_page_source (BonoboUIComponent *uic,
 	g_return_if_fail (tab != NULL);
 
 	galeon_shell_new_tab (galeon_shell, window, tab, NULL, 
-			      GALEON_NEW_TAB_VIEW_SOURCE);
+			      GALEON_NEW_TAB_VIEW_SOURCE |
+			      GALEON_NEW_TAB_JUMP);
 }
 
 void 
-window_cmd_tools_history (BonoboUIComponent *uic, 
-		      	  GaleonWindow *window, 
-		      	  const char* verbname)
+window_cmd_view_page_info(GtkAction *action, 
+		      	  GaleonWindow *window)
+{
+	galeon_window_show_page_info (window, PAGE_INFO_GENERAL);
+}
+
+void 
+window_cmd_tools_history (GtkAction *action, 
+		      	  GaleonWindow *window) 
 {
 	galeon_window_show_history (window);
 }
@@ -766,9 +731,8 @@ open_pdm_dialog (GtkWidget *window, int page_num)
 }
 
 void 
-window_cmd_tools_pdm (BonoboUIComponent *uic, 
-		      GaleonWindow *window, 
-		      const char* verbname)
+window_cmd_tools_pdm (GtkAction *action, 
+		      GaleonWindow *window) 
 {
 	open_pdm_dialog (GTK_WIDGET(window), -1);
 }
@@ -796,9 +760,10 @@ on_prefs_dialog_request (GaleonPrefsDialog  *dialog,
 						 | EMBED_CHROME_OPENASCHROME
 						 | EMBED_CHROME_OPENASPOPUP);
 		tab = galeon_tab_new ();
-		galeon_window_add_tab (new_win, tab, TRUE);
+		galeon_window_add_tab (new_win, tab,
+				       GALEON_WINDOW_ADD_TAB_LAST, TRUE);
 		
-		gtk_widget_show_all (GTK_WIDGET(galeon_tab_get_embed (tab)));
+		gtk_widget_show_all (GTK_WIDGET (tab));
 
 		galeon_window_load_url (new_win, "chrome://pippki/content/certManager.xul");
 
@@ -835,9 +800,8 @@ on_prefs_dialog_location (GaleonPrefsDialog *dialog,
 }
 
 void 
-window_cmd_edit_prefs (BonoboUIComponent *uic, 
-		       GaleonWindow *window, 
-		       const char* verbname)
+window_cmd_edit_prefs (GtkAction *action, 
+		       GaleonWindow *window) 
 {
 	static GtkWidget *dialog = NULL;
 
@@ -873,99 +837,87 @@ window_cmd_edit_prefs (BonoboUIComponent *uic,
 }
 
 static void
-window_cmd_settings_toolbar_editor_revert_clicked_cb (GtkButton *b, GulTbEditor *tbe)
+toolbar_editor_destroy_cb (GtkWidget *tbe,
+			   EggEditableToolbar *t)
 {
-	gchar *def;
-
-	g_return_if_fail (GUL_IS_TB_EDITOR (tbe));
-	
-	eel_gconf_unset (CONF_TOOLBAR_SETUP);
-	def = eel_gconf_get_string (CONF_TOOLBAR_SETUP);
-	if (def)
-	{
-		GulToolbar *current;
-		GulToolbar *avail;
-		current = gul_tb_editor_get_toolbar (tbe);
-		gul_toolbar_parse (current, def);
-		g_free (def);
-
-		avail = gul_tb_editor_get_available (tbe);
-		g_object_ref (avail);
-		gul_toolbar_parse (avail, AVAILABLE_TOOLBAR_ITEMS);
-		gul_tb_editor_set_available (tbe, avail);
-		g_object_unref (avail);
-	}
-
+	egg_editable_toolbar_set_edit_mode (t, FALSE);
 }
 
 static void
-window_cmd_settings_toolbar_editor_current_changed_cb (GulToolbar *tb, gpointer data)
+toolbar_editor_response_cb (GtkDialog  *dialog, gint response_id, gpointer data)
 {
-	gchar *current_str;
+	EggToolbarsModel *model;
+	int n;
 
-	g_return_if_fail (GUL_IS_TOOLBAR (tb));
-
-	current_str = gul_toolbar_to_string (tb);
-	eel_gconf_set_string (CONF_TOOLBAR_SETUP, current_str);
-	g_free (current_str);
+	switch (response_id)
+	{
+	case GTK_RESPONSE_CLOSE:
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		break;
+	case RESPONSE_ADD_TOOLBAR:
+		model = galeon_shell_get_toolbars_model (galeon_shell);
+		n = egg_toolbars_model_n_toolbars (EGG_TOOLBARS_MODEL (model));
+		egg_toolbars_model_add_toolbar (model, n - 1, "UserCreated");
+		break;
+	case GTK_RESPONSE_HELP:
+		gul_gui_help (GTK_WINDOW (dialog), "galeon.xml", "toolbar-editor");
+		break;
+	}
 }
 
 void 
-window_cmd_settings_toolbar_editor (BonoboUIComponent *uic, 
-				    GaleonWindow *window, 
-				    const char* verbname)
+window_cmd_settings_toolbar_editor (GtkAction *action, 
+				    GaleonWindow *window) 
 {
-	static GulTbEditor *tbe = NULL;
-	GulToolbar *avail;
-	GulToolbar *current;
-	gchar *current_str;
-	GtkButton *revert_button;
+	GtkWidget *editor;
+	EggToolbarsModel *model;
+	GtkWidget *dialog;
+	gchar *actions_file = gul_general_user_file ("galeon-egg-ui-toolbar.xml", TRUE);
+	EggEditableToolbar *toolbar = galeon_window_get_toolbar (window);
 
-	if (tbe != NULL)
-	{
-		gul_tb_editor_set_parent (tbe, GTK_WIDGET(window));
-		gul_tb_editor_show (tbe);
-                return;
-	}
+	model = galeon_shell_get_toolbars_model (galeon_shell);
 
-	tbe = gul_tb_editor_new ();
-	g_object_add_weak_pointer (G_OBJECT (tbe),
-				   (gpointer)&tbe);
-	gul_tb_editor_set_parent (tbe, 
-				  GTK_WIDGET(window));
+	dialog = gtk_dialog_new ();
 
-	avail = gul_toolbar_new ();
-	gul_toolbar_parse (avail, AVAILABLE_TOOLBAR_ITEMS);
+	gul_state_monitor_window (dialog, "Toolbar Editor", 550, 350);
 
-	current_str = eel_gconf_get_string (CONF_TOOLBAR_SETUP);
-	current = gul_toolbar_new ();
-	if (current_str)
-	{
-		gul_toolbar_parse (current, current_str);
-		g_free (current_str);
-	}
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Toolbar Editor"));
+        gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
+
+	editor = egg_toolbar_editor_new (GTK_UI_MANAGER (window->merge), EGG_TOOLBARS_MODEL (model));
+	egg_toolbar_editor_load_actions (EGG_TOOLBAR_EDITOR (editor), actions_file);
+	g_free (actions_file);
+
+	gtk_container_set_border_width (GTK_CONTAINER (editor), 5);
+	gtk_box_set_spacing (GTK_BOX (editor), 5);
+	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), editor);
+	g_signal_connect (editor, "destroy",
+			  G_CALLBACK (toolbar_editor_destroy_cb), toolbar);
+	gtk_widget_show (editor);
 	
-	gul_tb_editor_set_toolbar (tbe, current);
-	gul_tb_editor_set_available (tbe, avail);
-	g_object_unref (avail);
-	g_object_unref (current);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
 
-	g_signal_connect (current, "changed", 
-			  G_CALLBACK (window_cmd_settings_toolbar_editor_current_changed_cb), NULL);
-
-	revert_button = gul_tb_editor_get_revert_button (tbe);
-	gtk_widget_show (GTK_WIDGET (revert_button));
-
-	g_signal_connect (revert_button, "clicked", 
-			  G_CALLBACK (window_cmd_settings_toolbar_editor_revert_clicked_cb), tbe);
+	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)), 5);
 	
-	gul_tb_editor_show (tbe);
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       _("_Add a New Toolbar"), RESPONSE_ADD_TOOLBAR);
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
+	
+	g_signal_connect (G_OBJECT (dialog), "response",
+			  G_CALLBACK (toolbar_editor_response_cb), NULL);
+
+	gtk_widget_show (dialog);
+
+	egg_editable_toolbar_set_edit_mode (toolbar, TRUE);
 }
 
 void
-window_cmd_help_about (BonoboUIComponent *uic, 
-		       GaleonWindow *window, 
-		       const char* verbname)
+window_cmd_help_about (GtkAction *action, 
+		       GaleonWindow *window) 
 {
 	static GtkWidget *about = NULL;
 	GdkPixbuf  	 *pixbuf;
@@ -1024,7 +976,7 @@ window_cmd_help_about (BonoboUIComponent *uic,
 	
 	about = gnome_about_new(
 		       _("Galeon"), VERSION,
-		       "Copyright \xc2\xa9 2003 The Galeon Developers\nCopyright \xc2\xa9 2000-2002 Marco Pesenti Gritti",
+		       "Copyright \xc2\xa9 2003-2004 The Galeon Developers\nCopyright \xc2\xa9 2000-2002 Marco Pesenti Gritti",
 		       _("A GNOME browser based on Mozilla"),
 		       (const char **)authors,
 		       (const char **)documenters,
@@ -1041,40 +993,22 @@ window_cmd_help_about (BonoboUIComponent *uic,
 }
 
 void 
-window_cmd_set_charset (BonoboUIComponent *uic, 
-			EncodingMenuData *data, 
-			const char* verbname)
-{
-	GaleonWindow *window = data->data;
-	GaleonEmbed *embed;
-	
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	g_print (data->encoding);
-	galeon_embed_set_charset (embed, data->encoding);
-}
-
-void 
-window_cmd_tabs_next (BonoboUIComponent *uic, 
-		      GaleonWindow *window, 
-		      const char* verbname)
+window_cmd_tabs_next (GtkAction *action, 
+		      GaleonWindow *window) 
 {
 	galeon_window_next_tab (window);
 }
 
 void 
-window_cmd_tabs_previous (BonoboUIComponent *uic, 
-			  GaleonWindow *window, 
-			  const char* verbname)
+window_cmd_tabs_previous (GtkAction *action, 
+			  GaleonWindow *window) 
 {
 	galeon_window_prev_tab (window);
 }
 
 void 
-window_cmd_tabs_move_left  (BonoboUIComponent *uic, 
-			    GaleonWindow *window, 
-			    const char* verbname)
+window_cmd_tabs_move_left  (GtkAction *action, 
+			    GaleonWindow *window) 
 {
 	GList *tabs;
 	GaleonTab *tab;
@@ -1095,9 +1029,8 @@ window_cmd_tabs_move_left  (BonoboUIComponent *uic,
 	}
 }
 
-void window_cmd_tabs_move_right (BonoboUIComponent *uic, 
-				 GaleonWindow *window, 
-				 const char* verbname)
+void window_cmd_tabs_move_right (GtkAction *action, 
+				 GaleonWindow *window) 
 {
 	GList *tabs;
 	GaleonTab *tab;
@@ -1119,9 +1052,8 @@ void window_cmd_tabs_move_right (BonoboUIComponent *uic,
 }
 
 void
-window_cmd_tabs_clone(BonoboUIComponent *uic,
-		      GaleonWindow *window,
-		      const char *verbname)
+window_cmd_tabs_clone(GtkAction *action,
+		      GaleonWindow *window)
 {
         GaleonTab *tab;
 
@@ -1135,9 +1067,8 @@ window_cmd_tabs_clone(BonoboUIComponent *uic,
 }
 
 void 
-window_cmd_tabs_detach  (BonoboUIComponent *uic, 
-			 GaleonWindow *window, 
-			 const char* verbname)
+window_cmd_tabs_detach (GtkAction *action, 
+			GaleonWindow *window) 
 {
 	GaleonWindow *new_win;
 	GaleonTab *tab;
@@ -1154,31 +1085,40 @@ window_cmd_tabs_detach  (BonoboUIComponent *uic,
 	gtk_widget_show (GTK_WIDGET (new_win));
 }
 
-void 
-window_cmd_help_manual (BonoboUIComponent *uic, 
-			char *filename, 
-			const char* verbname)
+void window_cmd_tabs_close_other_tabs(GtkAction *action,
+				      GaleonWindow *window)
 {
-	GError *error;
-	GtkWidget *dialog;
+	GaleonTab *active_tab;
+	GList *i, *tabs = galeon_window_get_tabs(window);
 
-	error = NULL;
-	gnome_help_display ("galeon.xml", NULL, &error);
-
-	if (error) 
+	if (g_list_length(tabs) <= 1)
 	{
-		dialog = gtk_message_dialog_new (NULL,
-						 GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("There was an error displaying help: \n%s"),
-						 error->message);
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
+		return;
+	}
 
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		gtk_widget_show (dialog);
-		g_error_free (error);
+	active_tab = galeon_window_get_active_tab(window);
+	g_return_if_fail(active_tab);
+
+	for (i = tabs ; i ; i = i->next)
+	{
+		GaleonTab *tab = GALEON_TAB(i->data);
+
+		if (tab != active_tab)
+			galeon_window_remove_tab(window, tab);
 	}
 }
+
+void 
+window_cmd_help_manual (GtkAction *action, 
+			GaleonWindow *window) 
+{
+	gul_gui_help (GTK_WINDOW (window), "galeon.xml", NULL);
+}
+
+void
+window_cmd_help_plugins (GtkAction *action, 
+			GaleonWindow *window) 
+{
+	galeon_shell_new_tab(galeon_shell, window, NULL, "about:plugins",
+			     GALEON_NEW_TAB_JUMP);
+}
