@@ -16,13 +16,19 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <libgnome/gnome-i18n.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <glib/gi18n.h>
 #include "galeon-autocompletion-window.h"
 #include "gul-gobject-misc.h"
 #include "gul-string.h"
 #include "galeon-marshal.h"
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
+#include "galeon-debug.h"
+
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkscrolledwindow.h>
@@ -49,15 +55,6 @@
 #define ADD_MATCHES_DELAY 400
 #undef DISABLE_PROGRESSIVE_FILLING
 
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
-
-//#define DEBUG_TIME
-
-#ifdef DEBUG_TIME
-#include <glib/gtimer.h>
-#endif
-
 /**
  * Private data
  */
@@ -463,15 +460,10 @@ galeon_autocompletion_window_fill_store_chunk (GaleonAutocompletionWindow *aw)
 	guint i;
 	guint nmatches;
 	guint last = p->last_added_match;
-#ifdef DEBUG_TIME
-	GTimer *timer;
-#endif
-	DEBUG_MSG (("ACW: Filling the list from %d\n", last));
 
-#ifdef DEBUG_TIME
-	timer = g_timer_new ();
-	g_timer_start (timer);
-#endif
+	LOG ("ACW: Filling the list from %d", last);
+	
+	START_PROFILER ("filling liststore");
 
 	nmatches = galeon_autocompletion_get_num_matches (p->autocompletion);
 	matches = galeon_autocompletion_get_matches_sorted_by_score (p->autocompletion);
@@ -486,14 +478,11 @@ galeon_autocompletion_window_fill_store_chunk (GaleonAutocompletionWindow *aw)
 
 	p->last_added_match = last;
 
-#ifdef DEBUG_TIME
-	DEBUG_MSG (("ACW: %f elapsed filling the gtkliststore\n", g_timer_elapsed (timer, NULL)));
-	g_timer_destroy (timer);
-#endif
+	STOP_PROFILER ("filling liststore");
 
 	if (i == MAX_CHUNK_ALTERNATIVES && p->last_added_match < MAX_ALTERNATIVES)
 	{
-		DEBUG_MSG (("ACW: will continue...\n"));
+		LOG ("ACW: will continue...");
 		return TRUE;
 	}
 	else
@@ -509,11 +498,7 @@ galeon_autocompletion_window_show (GaleonAutocompletionWindow *aw)
 	gint x, y, height, width;
 	gboolean show_titles = eel_gconf_get_boolean (CONF_COMPLETION_SHOW_TITLES);
 	guint nmatches;
-#ifdef DEBUG_TIME
-	GTimer *timer1;
-	GTimer *timer2;
-#endif
-	
+
 	g_return_if_fail (p->window);
 	g_return_if_fail (p->autocompletion);
 	
@@ -523,17 +508,13 @@ galeon_autocompletion_window_show (GaleonAutocompletionWindow *aw)
 		galeon_autocompletion_window_hide (aw);
 		return;
 	}
+
+	LOG ("ACW: showing window.");
+
+	START_PROFILER ("showing window");
 	
-#ifdef DEBUG_TIME
-	DEBUG_MSG (("ACW: showing window.\n"));
-	timer1 = g_timer_new ();
-	g_timer_start (timer1);
-#endif
-	
-#ifdef DEBUG_TIME
-	timer2 = g_timer_new ();
-	g_timer_start (timer2);
-#endif
+	START_PROFILER ("creating liststore");
+
 	if (p->list_store) g_object_unref (p->list_store);
 	p->list_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 	
@@ -554,22 +535,15 @@ galeon_autocompletion_window_show (GaleonAutocompletionWindow *aw)
 			 (GSourceFunc) galeon_autocompletion_window_add_matches_to, aw);
 	}
 
-#ifdef DEBUG_TIME
-	DEBUG_MSG (("ACW: %f elapsed creating liststore\n", g_timer_elapsed (timer2, NULL)));
-#endif
+	STOP_PROFILER ("creating liststore");
 	
 	gtk_tree_view_set_model (p->tree_view, GTK_TREE_MODEL (p->list_store));
 	
-#ifdef DEBUG_TIME
-	g_timer_start (timer2);
-#endif
+	START_PROFILER ("calculating dimensions");
 	
 	galeon_autocompletion_window_get_dimensions (aw, &x, &y, &width, &height);
 	
-#ifdef DEBUG_TIME
-	DEBUG_MSG (("ACW: %f elapsed calculating dimensions\n", g_timer_elapsed (timer2, NULL)));
-	g_timer_destroy (timer2);
-#endif
+	STOP_PROFILER ("calculating dimensions");
 	
 	if (show_titles)
 	{
@@ -598,10 +572,8 @@ galeon_autocompletion_window_show (GaleonAutocompletionWindow *aw)
 	gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (p->tree_view), 0, 0);
 
 	gtk_widget_grab_focus (GTK_WIDGET (p->tree_view));
-#ifdef DEBUG_TIME
-	DEBUG_MSG (("ACW: %f elapsed showing window\n", g_timer_elapsed (timer1, NULL)));
-	g_timer_destroy (timer1);
-#endif
+
+	STOP_PROFILER ("showing window");
 }
 
 static gboolean
@@ -641,7 +613,8 @@ galeon_autocompletion_window_key_press_cb (GtkWidget *widget,
 	/* allow keyboard navigation in the alternatives clist */
 	if (event->keyval == GDK_Up || event->keyval == GDK_Down 
 	    || event->keyval == GDK_Page_Up ||  event->keyval == GDK_Page_Down
-	    || ((event->keyval == GDK_space || event->keyval == GDK_Return)
+	    || ((event->keyval == GDK_space || event->keyval == GDK_Return 
+		 || event->keyval == GDK_KP_Enter)
 		&& p->selected))
 	{
 		return galeon_autocompletion_window_key_press_hack (aw, event);
@@ -653,7 +626,7 @@ galeon_autocompletion_window_key_press_cb (GtkWidget *widget,
 	
 	if (dest_widget != widget)
 	{
-		//DEBUG_MSG (("Resending event\n"));
+		LOG ("Resending event");
 		
 		tmp_event = *event;
 		gtk_widget_event (dest_widget, (GdkEvent *)&tmp_event);
@@ -664,9 +637,9 @@ galeon_autocompletion_window_key_press_cb (GtkWidget *widget,
 	{
 		if (widget == GTK_WIDGET (p->tree_view))
 		{
-			//DEBUG_MSG (("on the tree view "));
+			LOG ("on the tree view");
 		}
-		//DEBUG_MSG (("Ignoring event\n"));
+		LOG ("Ignoring event");
 		return FALSE;
 	}
 	
@@ -692,6 +665,7 @@ galeon_autocompletion_window_key_press_hack (GaleonAutocompletionWindow *aw,
 		hack_tree_view_move_selection (p->tree_view, -5);
 		break;
 	case GDK_Return:
+	case GDK_KP_Enter:
 	case GDK_space:
 		if (p->selected)
 		{
