@@ -16,19 +16,22 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "galeon-dialog.h"
 #include "gul-glade.h"
 #include "gul-prefs-utils.h"
 #include "gul-gui.h"
 
 #include <gtk/gtktogglebutton.h>
-#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtkcombobox.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkdialog.h>
-
-#include <libgnomeui/gnome-color-picker.h>
+#include <gtk/gtkexpander.h>
 
 #include <string.h>
 
@@ -89,9 +92,9 @@ typedef enum
         PT_TOGGLEBUTTON,
         PT_RADIOBUTTON,
         PT_SPINBUTTON,
-        PT_COLOR,
-        PT_OPTIONMENU,
+        PT_COMBOBOX,
         PT_ENTRY,
+	PT_EXPANDER,
 	PT_UNKNOWN
 } PrefType;
 
@@ -115,6 +118,7 @@ struct GaleonDialogPrivate
 	
 	int spin_item_id;
 	GTimer *spin_timer;
+	gboolean initialized;
 };
 
 #define SPIN_DELAY 0.20
@@ -187,9 +191,9 @@ galeon_dialog_class_init (GaleonDialogClass *klass)
 static PrefType
 get_pref_type_from_widget (GtkWidget *widget)
 {
-	if (GTK_IS_OPTION_MENU (widget))
+	if (GTK_IS_COMBO_BOX (widget))
 	{
-		return PT_OPTIONMENU;
+		return PT_COMBOBOX;
 	}
 	else if (GTK_IS_SPIN_BUTTON (widget))
 	{
@@ -207,9 +211,9 @@ get_pref_type_from_widget (GtkWidget *widget)
 	{
 		return PT_ENTRY;
 	}
-	else if (GNOME_IS_COLOR_PICKER (widget))
+	else if (GTK_IS_EXPANDER (widget))
 	{
-		return PT_COLOR;
+		return PT_EXPANDER;
 	}
 
 	return PT_UNKNOWN;
@@ -368,16 +372,6 @@ prefs_spinbutton_changed_cb (GtkWidget *widget, PropertyInfo *pi)
 		       dialog);
 }
 
-static void 
-prefs_color_changed_cb (GtkWidget *widget, guint r, guint g, 
-			guint b, guint a, const PropertyInfo *pi)
-{
-	if (pi->type == PT_AUTOAPPLY)
-	{
-		gul_pu_set_config_from_color (widget,  pi->pref);
-	}
-}
-
 static void 
 prefs_entry_changed_cb (GtkWidget *widget, PropertyInfo *pi) 
 {
@@ -388,11 +382,11 @@ prefs_entry_changed_cb (GtkWidget *widget, PropertyInfo *pi)
 }
 
 static void 
-prefs_optionmenu_selected_cb (GtkWidget *widget, PropertyInfo *pi)  
+prefs_combobox_selected_cb (GtkWidget *widget, PropertyInfo *pi)  
 {
 	if (pi->type == PT_AUTOAPPLY)
 	{
-		gul_pu_set_config_from_optionmenu (widget, pi->pref);
+		gul_pu_set_config_from_combobox (widget, pi->pref);
 	}
 }
 
@@ -443,15 +437,10 @@ prefs_connect_signals (GaleonDialog *dialog)
 					  G_CALLBACK(prefs_spinbutton_changed_cb), 
 					  (gpointer)info);
 			break;
-		case PT_COLOR:
-			g_signal_connect (G_OBJECT (info->widget), "color_set", 
-					  G_CALLBACK(prefs_color_changed_cb), 
-					  (gpointer)info);
-			break;
-		case PT_OPTIONMENU:
+		case PT_COMBOBOX:
 			g_signal_connect (G_OBJECT (info->widget), 
 					  "changed", 
-					  G_CALLBACK(prefs_optionmenu_selected_cb), 
+					  G_CALLBACK(prefs_combobox_selected_cb), 
 					  (gpointer)info);
 			break;
 		case PT_ENTRY:
@@ -459,6 +448,8 @@ prefs_connect_signals (GaleonDialog *dialog)
 					  G_CALLBACK(prefs_entry_changed_cb), 
 					  (gpointer)info);
 			break;
+
+		case PT_EXPANDER:
 		case PT_UNKNOWN:
 			break;
 		}
@@ -475,6 +466,7 @@ galeon_dialog_init (GaleonDialog *dialog)
 	dialog->priv->props = NULL;
 	dialog->priv->spin_timer = NULL;
 	dialog->priv->name = NULL;
+	dialog->priv->initialized = FALSE;
 }
 
 static void
@@ -524,23 +516,22 @@ load_props (PropertyInfo *props)
 			 gul_pu_set_togglebutton_from_config (props[i].widget,
 							      props[i].pref);
 		}
+		else if (GTK_IS_EXPANDER(props[i].widget))
+		{
+			 gul_pu_set_expander_from_config (props[i].widget,
+							  props[i].pref);
+		}
 		else if (GTK_IS_EDITABLE(props[i].widget))
 		{
 			gul_pu_set_editable_from_config (props[i].widget,
 						         props[i].pref);
 		}
-		else if (GTK_IS_OPTION_MENU(props[i].widget))
+		else if (GTK_IS_COMBO_BOX(props[i].widget))
 		{
-			gul_pu_set_optionmenu_from_config (props[i].widget,
-					  	           props[i].pref);
-		}
-		else if (GNOME_IS_COLOR_PICKER(props[i].widget))
-		{
-			gul_pu_set_color_from_config (props[i].widget,
-					  	      props[i].pref);
+			gul_pu_set_combobox_from_config (props[i].widget,
+							 props[i].pref);
 		}
 	}
-
 }
 
 static void
@@ -569,20 +560,20 @@ save_props (PropertyInfo *props)
 			 gul_pu_set_config_from_togglebutton (props[i].widget,
 							      props[i].pref);
 		}
+		else if (GTK_IS_EXPANDER(props[i].widget))
+		{
+			 gul_pu_set_config_from_expander (props[i].widget,
+							  props[i].pref);
+		}
 		else if (GTK_IS_EDITABLE(props[i].widget))
 		{
 			gul_pu_set_config_from_editable (props[i].widget,
 						         props[i].pref);
 		}
-		else if (GTK_IS_OPTION_MENU(props[i].widget))
+		else if (GTK_IS_COMBO_BOX(props[i].widget))
 		{
-			gul_pu_set_config_from_optionmenu (props[i].widget,
-					  	           props[i].pref);
-		}
-		else if (GNOME_IS_COLOR_PICKER(props[i].widget))
-		{
-			gul_pu_set_config_from_color (props[i].widget,
-					  	      props[i].pref);
+			gul_pu_set_config_from_combobox (props[i].widget,
+					  	         props[i].pref);
 		}
 	}
 }
@@ -676,9 +667,23 @@ galeon_dialog_new_with_parent (GtkWidget *parent_window)
 					    NULL));
 }
 
+static void
+check_connect_props (GaleonDialog *dialog)
+{
+	if (dialog->priv->props && !dialog->priv->initialized)
+	{
+		load_props (dialog->priv->props);
+		prefs_connect_signals (dialog);
+		prefs_set_sensitivity (dialog->priv->props);
+		dialog->priv->initialized = TRUE;
+	}
+}
+
 void galeon_dialog_show_embedded (GaleonDialog *dialog,
 				  GtkWidget *container)
 {
+	check_connect_props (dialog);
+
 	dialog->priv->container = container;
 	gtk_container_add (GTK_CONTAINER(container),
 			   dialog->priv->dialog);
@@ -739,6 +744,7 @@ dialog_destroy_cb (GtkWidget *widget, GaleonDialog *dialog)
 	{
 		save_props (dialog->priv->props);
 	}
+	dialog->priv->initialized = FALSE;
 }
 
 static void 
@@ -773,12 +779,6 @@ impl_construct (GaleonDialog *dialog,
 			   (GWeakNotify)dialog_destruction_notify,
 			   dialog);
 
-	if (dialog->priv->props)
-	{
-		load_props (dialog->priv->props);
-		prefs_connect_signals (dialog);
-		prefs_set_sensitivity (dialog->priv->props);
-	}
 }
 
 static void 
@@ -831,28 +831,33 @@ impl_get_value (GaleonDialog *dialog,
 	}
 	else if (GTK_IS_EDITABLE (widget))
 	{
+		gchar *text = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
 		g_value_init (value, G_TYPE_STRING);
-		g_value_set_string (value, gtk_editable_get_chars
-                			    (GTK_EDITABLE (widget), 0, -1));
+		g_value_set_string (value, text);
+		g_free (text);
 	}
-	else if (GTK_IS_OPTION_MENU (widget))
+        else if (GTK_IS_COMBO_BOX(widget))
 	{
 		int val;
-		g_value_init (value, G_TYPE_INT);
-		val = gtk_option_menu_get_history (GTK_OPTION_MENU(widget));
-		g_value_set_int (value, val);
+		g_value_init(value, G_TYPE_INT);
+		val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+		g_value_set_int(value, val);
 	}
 }
 
 static gint
 impl_run (GaleonDialog *dialog)
 {	
+	galeon_dialog_show (dialog);
+
 	return gtk_dialog_run (GTK_DIALOG(dialog->priv->dialog));
 }
 
 static void
 impl_show (GaleonDialog *dialog)
 {
+	check_connect_props (dialog);
+
 	if (dialog->priv->parent)
 	{
 		/* Make the dialog transient again, because it seems to get
