@@ -29,6 +29,12 @@ typedef gboolean (*GulFilterFunc) (gpointer item, gpointer data);
 char 		*gul_general_tmp_filename	 (const char *base,
 						  const char *extension);
 
+gboolean         gul_general_switch_temp_file (const char *filename,
+					       const char *filename_temp);
+
+gboolean         gul_general_safe_xml_save    (const char *filename,
+					       xmlDocPtr doc);
+
 gchar 		*gul_general_user_file 		 (const char *fname, gboolean critical);
 
 gchar 		*gul_general_read_line_from_file (FILE *f);
@@ -44,13 +50,6 @@ GSList *	 gul_find_file			 (const char *path,
 						  const char *fname, 
 						  gint maxdepth);
 
-xmlAttrPtr 	 xmlSetRawProp 			 (xmlNodePtr node, 
-					 	  const xmlChar *name,
-			  		 	  const xmlChar *value);
-
-xmlChar 	*xmlGetRawProp 			(xmlNodePtr node, 
-					 	 const xmlChar *name);
-
 gint 		xmlGetIntProp 			(xmlNodePtr node, 
 					 	 const gchar *attribute);
 
