@@ -25,17 +25,13 @@
 #include "gul-gestures.h"
 #include "gul-gobject-misc.h"
 #include "gul-general.h"
+#include "galeon-debug.h"
 #include "galeon-marshal.h"
 #include "stroke.h"
 #include <gtk/gtkmain.h>
 #include <gtk/gtkdnd.h>
 #include <string.h>
 
-#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-
-//#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
-
 /**
  * Private data
  */
@@ -120,7 +116,7 @@ gul_gestures_finalize_impl (GObject *o)
 	GulGestures *as = GUL_GESTURES (o);
 	GulGesturesPrivate *p = as->priv;
 
-	DEBUG_MSG (("in gul_gestures_finalize_impl\n"));
+	LOG ("in gul_gestures_finalize_impl");
 	
 	if (p->autocancel_timeout)
 	{
