@@ -28,8 +28,8 @@
 
 #include "gul-notebook.h"
 #include "gul-gui.h"
+#include "gul-string.h"
 #include "galeon-marshal.h"
-#include "eel-input-event-box.h"
 
 #include <gtk/gtklabel.h>
 #include <gtk/gtkhbox.h>
@@ -38,21 +38,22 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtkeventbox.h>
 
 #include <glib-object.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #define AFTER_ALL_TABS -1
 #define NOT_IN_APP_WINDOWS -2
 #define TAB_MIN_SIZE 60
 #define TAB_NB_MAX    8
+#define MENU_ITEM_MAX_LENGTH 40
 
 struct GulNotebookPrivate 
 {
 	GList *focused_pages;
 	GList *opened_tabs;
 	GtkPolicyType policy;
-	GtkTooltips *title_tips;
 	gboolean automatic_tab_switch;
 	
 	/* Used during tab drag'n'drop */
@@ -69,6 +70,7 @@ static void gul_notebook_finalize     (GObject *object);
 /* Local variables */
 static GdkCursor *cursor = NULL;
 static GList *notebooks  = NULL;
+static GtkTooltips *_gul_notebook_tooltips = NULL;
 
 
 /* Local functions */
@@ -86,10 +88,12 @@ enum
 	TAB_REMOVED,
 	TABS_REORDERED,
 	TAB_DETACHED,
+	TAB_DELETE,
 	LAST_SIGNAL
 };
 
 static guint gul_notebook_signals[LAST_SIGNAL] = { 0 };
+static GObjectClass *parent_class = NULL;
 
 GType 
 gul_notebook_get_type (void)
@@ -123,6 +127,8 @@ static void
 gul_notebook_class_init (GulNotebookClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	parent_class = g_type_class_peek_parent (klass);
+
 	object_class->finalize = gul_notebook_finalize;
 
 	/* init signals */
@@ -169,7 +175,16 @@ gul_notebook_class_init (GulNotebookClass *klass)
 			      G_TYPE_NONE,
 			      0);
 
-
+	gul_notebook_signals[TAB_DELETE] =
+		g_signal_new ("tab_delete",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GulNotebookClass, tab_delete),
+			      NULL, NULL,
+			      galeon_marshal_BOOLEAN__OBJECT,
+			      G_TYPE_BOOLEAN,
+			      1,
+			      GTK_TYPE_WIDGET);
 }
 
 static gboolean 
@@ -199,7 +214,7 @@ find_notebook_at_pointer (gint abs_x, gint abs_y)
 	GList *l;
 	gint x, y;
 	GdkWindow *win_at_pointer = gdk_window_at_pointer (&x, &y);
-	GdkWindow *parent_at_pointer = NULL;
+	GdkWindow *toplevel_at_pointer = NULL;
 
 	if (win_at_pointer == NULL) 
 	{
@@ -207,12 +222,8 @@ find_notebook_at_pointer (gint abs_x, gint abs_y)
 		return NULL;
 	}
 
-	gdk_window_get_toplevel (win_at_pointer);
-	/* When we are in the notebook event window, win_at_pointer will be
-	   this event window, and the toplevel window we are interested in
-	   will be its parent
-	*/
-	parent_at_pointer = gdk_window_get_parent (win_at_pointer);
+	/* Get the toplevel window for where the pointer is currently */
+	toplevel_at_pointer = gdk_window_get_toplevel (win_at_pointer);
 
 	for (l = notebooks; l != NULL; l = l->next) 
 	{
@@ -220,7 +231,7 @@ find_notebook_at_pointer (gint abs_x, gint abs_y)
 		GdkWindow *win = GTK_WIDGET (nb)->window;
 
 		win = gdk_window_get_toplevel (win);
-		if (((win == win_at_pointer) || (win == parent_at_pointer))
+		if (win == toplevel_at_pointer
 		    && is_in_notebook_window (nb, abs_x, abs_y)) 
 		{
 			return nb;
@@ -482,6 +493,12 @@ drag_start (GulNotebook *notebook)
 static void 
 drag_stop (GulNotebook *notebook)
 {
+	if (notebook->priv->drag_in_progress)
+	{
+		g_signal_emit (G_OBJECT (notebook), 
+			       gul_notebook_signals[TABS_REORDERED], 0);
+	}
+
 	notebook->priv->drag_in_progress = FALSE;
 
 	if (notebook->priv->motion_notify_handler_id != 0) 
@@ -605,6 +622,20 @@ button_press_cb (GulNotebook *notebook,
 					  G_CALLBACK (motion_notify_cb), NULL);
 	}
 
+	/* Switch tabs on right click. This means that when the popup
+	 * menu is shown, it will be clear that it acts on the tab under
+	 * the mouse. popup menu is attached by the GulNotebook user if
+	 * the default is not desired.
+	 */
+	else if ((event->button == 3) && (event->type == GDK_BUTTON_PRESS))
+	{
+		if (tab_clicked != -1)
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook),
+						      tab_clicked);
+		else
+			return TRUE;
+	}
+
 	return FALSE;
 }
 
@@ -652,11 +683,16 @@ gul_notebook_switch_page_cb (GtkNotebook *notebook,
 static void
 gul_notebook_init (GulNotebook *notebook)
 {
-	notebook->priv = g_new (GulNotebookPrivate, 1);
+	if (!_gul_notebook_tooltips)
+	{
+		_gul_notebook_tooltips = gtk_tooltips_new ();
+		g_object_add_weak_pointer (G_OBJECT (_gul_notebook_tooltips),
+					   (gpointer)&_gul_notebook_tooltips);
+	}
+	g_object_ref (_gul_notebook_tooltips);
+	gtk_object_sink (GTK_OBJECT (_gul_notebook_tooltips));
 
-	notebook->priv->title_tips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (notebook->priv->title_tips));
-	gtk_object_sink (GTK_OBJECT (notebook->priv->title_tips));
+	notebook->priv = g_new (GulNotebookPrivate, 1);
 
 	notebook->priv->drag_in_progress = FALSE;
 	notebook->priv->motion_notify_handler_id = 0;
@@ -687,9 +723,12 @@ gul_notebook_finalize (GObject *object)
 
 	g_list_free (notebook->priv->focused_pages);
 	g_list_free (notebook->priv->opened_tabs);
-	g_object_unref (notebook->priv->title_tips);
 
 	g_free (notebook->priv);
+
+	g_object_unref (_gul_notebook_tooltips);
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -697,18 +736,16 @@ tab_close_button_clicked_cb (GtkWidget *widget,
 			     GtkWidget *child)
 {
 	GulNotebook *notebook;
+	gboolean inhibited = FALSE;
 
 	notebook = GUL_NOTEBOOK (gtk_widget_get_parent (child));
-	gul_notebook_remove_page (notebook, child);
-}
 
-static void
-tab_close_button_state_changed_cb (GtkWidget	*widget,
-		                   GtkStateType	 previous_state)
-{
-	if (GTK_WIDGET_STATE(widget) == GTK_STATE_ACTIVE)
+	g_signal_emit (G_OBJECT (notebook), gul_notebook_signals[TAB_DELETE],
+		       0, child, &inhibited);
+
+	if (inhibited == FALSE)
 	{
-		gtk_widget_set_state (widget, GTK_STATE_NORMAL);
+		gul_notebook_remove_page (notebook, child);
 	}
 }
 
@@ -745,8 +782,12 @@ tab_build_label (GulNotebook *nb, GtkWidget *child)
 	gtk_widget_set_size_request (close_button, size.width, size.height);
 	gtk_container_add (GTK_CONTAINER (close_button), image);
 
+	gtk_tooltips_set_tip (_gul_notebook_tooltips, close_button,
+			      _("Close this tab"), NULL);
+
 	/* setup label */ 
-	label_ebox = eel_input_event_box_new ();
+	label_ebox = gtk_event_box_new ();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (label_ebox), FALSE);
 	label_hbox = gtk_hbox_new (FALSE, 3);
         label = gtk_label_new (_("Untitled"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.5);
@@ -774,13 +815,6 @@ tab_build_label (GulNotebook *nb, GtkWidget *child)
                           G_CALLBACK (tab_close_button_clicked_cb),
                           child);
 
-	/* work around a cosmetic bug in gtk where buttons with GTK_RELIEF_NONE
-	 * have annoyingly visible border only when in GTK_STATE_ACTIVE (ie. in
-	 * inactive tabs) */
-	g_signal_connect (G_OBJECT (close_button), "state_changed",
-                          G_CALLBACK (tab_close_button_state_changed_cb),
-                          child);
-		
 	gtk_widget_show (hbox);
 	gtk_widget_show (label_ebox);
 	gtk_widget_show (label_hbox);
@@ -938,7 +972,7 @@ gul_notebook_remove_page (GulNotebook *nb,
  	/* remove the tool tip for this ebox */
  	label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
  	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "label-ebox"));
- 	gtk_tooltips_set_tip (GTK_TOOLTIPS (nb->priv->title_tips), ebox, NULL, NULL);
+ 	gtk_tooltips_set_tip (_gul_notebook_tooltips, ebox, NULL, NULL);
  
 	/* If we don't clear the icon then for some reason it gets reset back
 	 * to an empty icon. This means that although gul_notebook_move_page() has
@@ -983,15 +1017,19 @@ gul_notebook_set_page_title (GulNotebook *nb,
 			     const char *title)
 {
 	GtkWidget *label, *ebox, *tab_label;
+	gchar *short_title;
 
 	tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
 
 	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (tab_label), "label-ebox"));
 	label = GTK_WIDGET (g_object_get_data (G_OBJECT (tab_label), "label"));
+	short_title = gul_string_shorten (title, MENU_ITEM_MAX_LENGTH);
 
 	gtk_label_set_label (GTK_LABEL (label), title);
-	gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (nb), child, title);
-	gtk_tooltips_set_tip (nb->priv->title_tips, ebox, title, NULL);
+	gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (nb), child, short_title);
+	gtk_tooltips_set_tip (_gul_notebook_tooltips, ebox, title, NULL);
+
+	g_free (short_title);
 }
 
 
