@@ -14,17 +14,30 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: gul-state.c,v 1.7 2003/08/30 13:50:36 tko Exp $
+ *  $Id: gul-state.c,v 1.10 2004/03/29 21:55:56 crispin Exp $
  */
 
 #include "gul-state.h"
 
 #include <gtk/gtkpaned.h>
 #include <gtk/gtkwindow.h>
+#include <gtk/gtkexpander.h>
 #include <libgnome/gnome-config.h>
 
 #include <string.h>
 
+#define CONFIG_SYNC_INTERVAL (60 * 5 * 1000)
+
+static guint timeout_id = 0;
+
+static gboolean
+config_periodic_sync_cb (gpointer dummy)
+{
+	gnome_config_sync ();
+	return TRUE;
+}
+
+
 /**
  * gul_state_init:
  *
@@ -48,8 +61,30 @@ gul_state_init (void)
 
 	gnome_config_push_prefix (config_prefix);
 	g_free (config_prefix);
+
+
+	timeout_id = g_timeout_add (CONFIG_SYNC_INTERVAL, 
+				    config_periodic_sync_cb, 
+				    NULL);
 }
 
+/**
+ * gul_state_shutdown:
+ *
+ * Ensures that any state changes are synced to disk
+ */
+void
+gul_state_shutdown (void)
+{
+	gnome_config_sync ();
+	if (timeout_id)
+	{
+		g_source_remove (timeout_id);
+		timeout_id = 0;
+	}
+}
+
+
 static void
 gul_state_window_load (GtkWindow  *window,
 		       const char *name,
@@ -143,7 +178,6 @@ on_window_state_event (GtkWidget           *widget,
 	if (!(event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN))
 	{
 		gul_state_window_save (GTK_WINDOW(widget), name);
-		gnome_config_sync ();
 	}
 	return FALSE;
 }
@@ -182,8 +216,6 @@ gul_state_monitor_window (GtkWidget  *window,
 	g_signal_connect_data (G_OBJECT(window), "window-state-event",
 			       G_CALLBACK(on_window_state_event),
 			       g_strdup (name), (GClosureNotify)g_free, 0);
-
-	gnome_config_sync ();
 }
 
 static void
@@ -234,6 +266,45 @@ gul_state_monitor_paned (GtkWidget  *paned,
 			       G_CALLBACK(on_size_allocate),
 			       g_strdup(key), (GClosureNotify)g_free, 
 			       G_CONNECT_AFTER);
+}
 
-	gnome_config_sync ();
+
+static void
+on_expander_toggle (GtkExpander *expander, GParamSpec *pspec, char *key)
+{
+	gnome_config_set_bool (key, gtk_expander_get_expanded (expander));
+}
+
+/** gul_state_monitor_expander:
+ *  @expander: an #CDDBDisclosure
+ *  @key: a gnome-config key
+ *  @default_visibility: the default visiblity for the expander
+ *
+ *  Monitor the visibilty of an #CDDBDisclosure and save the current
+ *  visibility using the key @key.
+ *
+ */
+void
+gul_state_monitor_expander (GtkWidget *expander,
+			    const char *key,
+			    gboolean default_visibility)
+{
+	char *tmp;
+	gboolean visible;
+
+	g_return_if_fail (GTK_IS_EXPANDER (expander));
+	g_return_if_fail (key != NULL);
+	g_return_if_fail (strchr (key, '/') != NULL);
+
+	tmp = g_strdup_printf ("%s=%s", key, default_visibility ? "true" : "false");
+	visible = gnome_config_get_bool (tmp);
+	g_free (tmp);
+
+	gtk_expander_set_expanded (GTK_EXPANDER (expander), visible);
+
+	g_signal_connect_data (G_OBJECT (expander),
+			       "notify::expanded",
+			       G_CALLBACK (on_expander_toggle),
+			       g_strdup (key), (GClosureNotify)g_free, 
+			       G_CONNECT_AFTER);
 }
