@@ -14,7 +14,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *  
- *  $Id: gul-state.h,v 1.4 2003/08/30 13:50:36 tko Exp $
+ *  $Id: gul-state.h,v 1.6 2004/02/20 23:27:29 crispin Exp $
  */
 
 #ifndef GUL_STATE_H
@@ -25,6 +25,7 @@
 G_BEGIN_DECLS
 
 void gul_state_init                     (void);
+void gul_state_shutdown                 (void);
 
 void gul_state_monitor_window           (GtkWidget  *window,
 		                         const char *name,
@@ -35,6 +36,10 @@ void gul_state_monitor_paned            (GtkWidget  *paned,
 		                         const char *key,
 					 int         default_size);
 
+void gul_state_monitor_expander         (GtkWidget *expander,
+					 const char *key,
+					 gboolean    default_visiblity);
+
 G_END_DECLS
 
 #endif /* __gul_state_H */
